#!/bin/ksh
# @(#) alarms/delivery/bin/metrica_alarmtemplateadmin.ksh alarms_r4.79.0.4:cset.000251:2:2 04/05/04 @(#)
###############################################################################
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                     ADC Metrica
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright ADC Metrica, 2001.
#
###############################################################################
#
# Administration shell script for controlling the Alarm Template command line
# tool
#
# Exit Codes:
#     0 = Successful
#     1 = Incorrect syntax
#     2 = Environment variable not set
#     3 = Directory referred to by environment variable does not exist
#     4 = JAR file was not found
#
# Created    18 June 2002
# Author     John Peckham
# Version    alarms_r4.79.0.4
#
###############################################################################

#
# Exit codes
#
exit_rc_success=0
exit_rc_syntax=1
exit_rc_bad_env=2
exit_rc_bad_dir=3
exit_rc_bad_jar=4

###############################################################################
#
# Function    : print_component_version
#
# Description : Prints the version of the component
#
# Parameters  : none
#
###############################################################################
print_component_version() {

  . $METRICA_DIR/CS/bin/metrica_version.ksh
  print_version "Alarm Template Command Line Administration" "alarms_r4.79.0.4:cset.000251:2:2"

}

###############################################################################
#
# Function    : start_app
#
# Description : Start the Alarm template command line tool application
#
# Parameters  : none
#
###############################################################################
start_app() {
    java \
    -Dvbroker.agent.port=$OSAGENT_PORT \
    -DSVCnameroot=namingservice \
    com.adc.metrica.alarms.alarmmanager.cmdlinetool.AlarmTemplateCmdLineApp $*
}

###############################################################################
#
# Function    : usage
#
# Description : Shows program syntax and arguments
#
# Parameters  : none
#
###############################################################################
usage() {
    PROGNAME=`basename $0`

    echo
    echo "Usage: "$PROGNAME {
    echo "            -h | -help | --help |"
    echo "            -v | -version | --version"
    echo "       } | {"
    echo "            -load <alarm_template_xml_file> |"
    echo "            -drop -context <context> -name <name> -version <version> |"
    echo "            -unload -context <context> -name <name> -version <version> -file <file> |"
    echo "            -list |"
    echo "            -createcontext <document_context_path> |"
    echo "            -removecontext <document_context_path> [-r] "
    echo "       }"
    echo
    echo
    echo "Options:"
    echo
    echo "    -v | -version | --version                                                Display version information"
    echo "    -h | -help | --help                                                      Display command help"
    echo "    -load   <alarm_template_xml_file>                                        Create/update a template"
    echo "    -drop -context <context> -name <name> -version <version>                 Drop a template from the database"
    echo "    -unload -context <context> -name <name> -version <version> -file <file>  Unload a template to a file"
    echo "    -list                                                                    Summary of templates"
    echo "    -createcontext <document_context_path>                                   Create a document context"
    echo "    -removecontext <document_context_path> [-r]                              Delete the specified document context"
    echo "                                                                             or subcontext if -r is specified" 
    echo
    echo "NOTE: The -load option automatically creates the document context"
    echo "      path as necessary (specified in the Alarm Template)"
    echo
}

###############################################################################
#
# Function    : addJar
#
# Description : Add a Jar to the runtime CLASSPATH. A check is made to ensure
#               the Jar file exists.
#
# Parameters  : none
#
###############################################################################
addJar() {

    # First ensure the JAR file exists
    if [ ! -f "$1" ]
    then
        bad_jar $1
    fi

    # Add it to the classpath
    CLASSPATH=$CLASSPATH:$1
}

###############################################################################
#
# Function    : setup_runtime
#
# Description : Set the CLASSPATH needed at runtime
#
# Parameters  : none
#
###############################################################################
setup_runtime() {

    # Initialise the classpath	
    CLASSPATH=''

    # SAXParser class needs to be resolved here because asrt.jar contains
    # a different verson of this class which will not work
    addJar $METRICA_DIR/ext/lib/xerces.jar

    # The org.omg.CosNaming package in vbjorb.jar needs to be resolved here
    # before the one in fusion.jar
    addJar $METRICA_DIR/ext/lib/vbjorb.jar
    addJar $METRICA_DIR/ext/lib/asrt.jar
    addJar $METRICA_DIR/ext/lib/vbsec.jar
    addJar $METRICA_DIR/ext/lib/lm.jar
    addJar $METRICA_DIR/ext/lib/xmlrt.jar

    #
    # Common Services
    #
    addJar $METRICA_DIR/CS/lib/ejbhomefactory.jar
    addJar $METRICA_DIR/CS/lib/corbautils.jar
    addJar $METRICA_DIR/CS/lib/exceptionutils.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_common.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_cache.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_client.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_persistence.jar
    addJar $METRICA_DIR/CS/lib/logservice.jar

    #
    # JAXB Runtime
    #
    addJar $METRICA_DIR/ext/lib/jax-qname.jar
    addJar $METRICA_DIR/ext/lib/namespace.jar
    addJar $METRICA_DIR/ext/lib/relaxngDatatype.jar
    addJar $METRICA_DIR/ext/lib/xsdlib.jar
    addJar $METRICA_DIR/ext/lib/jaxb-impl.jar
    addJar $METRICA_DIR/ext/lib/jaxb-api.jar
    addJar $METRICA_DIR/ext/lib/jaxb-libs.jar


    # Informix JDBC
    addJar $METRICA_DIR/ext/lib/ifxjdbc.jar
    addJar $METRICA_DIR/ext/lib/ifxjdbcx.jar

    #
    # Database services
    #
    addJar $METRICA_DIR/DBServices/lib/queryservice_connectionmanager.jar
    addJar $METRICA_DIR/DBServices/lib/queryservice_logging.jar
    addJar $METRICA_DIR/DBServices/lib/queryservice_requestmanager.jar
    addJar $METRICA_DIR/DBServices/lib/queryservice_util.jar
    addJar $METRICA_DIR/DBElements/lib/metricaudt.jar

    #
    # Alarm Manager Command Line application
    #
    addJar $METRICA_DIR/AlarmManagement/lib/alarmmanager_cmdlinetool.jar

    #
    # Alarm XML processing
    #
    addJar $METRICA_DIR/AlarmManagement/lib/alarmxml.jar

    #
    # Alarm Manager common classes
    #
    addJar $METRICA_DIR/AlarmManagement/lib/alarmmanager_common.jar

    #
    # Alarm Service persistence
    #
    addJar $METRICA_DIR/AlarmManagement/lib/alarmservice_persistence.jar
    addJar $METRICA_DIR/AlarmManagement/lib/alarmservice_server.jar

    export CLASSPATH
}

###############################################################################
#
# Function    : bad_env
#
# Description : Environment variable is not set
#
# Parameters  : Name of environment variable
#
###############################################################################
bad_env() {
    echo
    echo "Environment variable \$$1 is not set" >&2
    echo
    exit $exit_rc_bad_env
}

###############################################################################
#
# Function    : bad_dir
#
# Description : Directory pointed to by environment variable does not exist
#
# Parameters  : Name of environment variable
#
###############################################################################
bad_dir() {
    echo
    echo "Directory referred to by environment variable \$$1 does not exist" >&2
    echo
    exit $exit_rc_bad_dir
}

###############################################################################
#
# Function    : bad_jar
#
# Description : Jar file does not exist
#
# Parameters  : Full path/name of JAR file
#
###############################################################################
bad_jar() {
    echo
    echo "JAR file not found:" >&2
    echo $1 >&2
    exit $exit_rc_bad_jar
}

###############################################################################
#
# Function    : check_env
#
# Description : Ensure environment variable is set and optionally that it
#               refers to a directory that exists.
#
# Parameters  : "var" = check variable, "var_and_dir" = also check dir exists
#               Environment variable name (without "$" sign)
#               Value of the environment variable
#
###############################################################################
check_env() {
    if [ -z "$3" ] ; then
        bad_env "$2"
    fi

    if [ "$1" = "var_and_dir" -a ! -d "$3" ] ; then
        bad_dir "$2"
    fi
}

###############################################################################
#
# Function    : init
#
# Description : Perform the necessary checks to make sure the environment is
#               set up ready to run administration functions.
#
# Parameters  : none
#
###############################################################################
init() {
    check_env "var"         "OSAGENT_PORT" $OSAGENT_PORT
    check_env "var_and_dir" "METRICA_DIR" $METRICA_DIR

    setup_runtime
}

###############################################################################
###############################################################################
#
# Main program
#
###############################################################################
###############################################################################
if [ -z $1 ] ; then
    usage
    exit $exit_rc_syntax
fi

case "$1" in
    "-h"|"-help"|"--help" )
        print_component_version 
        usage
        exit $exit_rc_success
        ;;
    "-v"|"-version"|"--version" )
        print_component_version 
        exit $exit_rc_success
        ;;
    "-load"|"-drop"|"-unload"|"-list"|"-createcontext"|"-removecontext" )
        print_component_version 
        init
        start_app $*
        ;;
    * )
        print_component_version 
        usage
        exit $exit_rc_syntax
        ;;
esac

exit $exit_rc_success
