#!/bin/ksh
# @(#) CS/delivery/bin/metrica_eventserviceadmin.ksh CS_r4.79.0.3:aset.000571:1:1 02/17/04 @(#)
###############################################################################
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                     ADC Metrica
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright ADC Metrica, 2001.
#
###############################################################################
#
# Administration shell script for controlling the Event Service
#
# Exit Codes:
#     0 = Successful
#     1 = Incorrect syntax
#     2 = Environment variable not set
#     3 = Directory referred to by environment variable does not exist
#     4 = JAR file was not found
#
# Created    30 May 2002
# Author     John Peckham
# Version    CS_r4.79.0.3
#
###############################################################################

#
# Exit codes
#
exit_rc_success=0
exit_rc_syntax=1
exit_rc_bad_env=2
exit_rc_bad_dir=3
exit_rc_bad_jar=4

###############################################################################
#
# Function    : print_component_version
#
# Description : Prints the version of the component
#
# Parameters  : none
#
###############################################################################
print_component_version() {

  . $METRICA_DIR/CS/bin/metrica_version.ksh
  print_version "Event Service Administration" "CS_r4.79.0.3:aset.000571:1:1"

}

###############################################################################
#
# Function    : start_service
#
# Description : Start the Event Service
#
# Parameters  : none
#
###############################################################################
start_service() {
    java \
    -Dvbroker.agent.port=$OSAGENT_PORT \
    -DSVCnameroot=namingservice \
    -Xms5m -Xmx64m \
    com.adc.metrica.cs.eventservice.EventServiceController -start
}

###############################################################################
#
# Function    : restart_service
#
# Description : Restarts the Event Service
#
# Parameters  : none
#
# Returns     : nothing
#
###############################################################################
restart_service() {
    java \
    -Dvbroker.agent.port=$OSAGENT_PORT \
    -DSVCnameroot=namingservice \
    com.adc.metrica.cs.eventservice.EventServiceController -restart
}

###############################################################################
#
# Function    : stop_service
#
# Description : Stop the Event Service
# Parameters  : none
#
###############################################################################
stop_service() {
    java \
    -Dvbroker.agent.port=$OSAGENT_PORT \
    -DSVCnameroot=namingservice \
    com.adc.metrica.cs.eventservice.EventServiceController -stop
}

###############################################################################
#
# Function    : get_status
#
# Description : Get current status of Event Service
#
# Parameters  : none
#
###############################################################################
get_status() {
    java \
    -Dvbroker.agent.port=$OSAGENT_PORT \
    -DSVCnameroot=namingservice \
    com.adc.metrica.cs.eventservice.EventServiceController -status
}

###############################################################################
#
# Function    : usage
#
# Description : Shows program syntax and arguments
#
# Parameters  : none
#
###############################################################################
usage() {

    PROGNAME=`basename $0`
    
    echo
    echo "Usage: "$PROGNAME {
    echo "            -h | -help | --help |"
    echo "            -v | -version | --version"
    echo "       } | {"                         
    echo "            -start | -restart | -stop | -status"
    echo "       }"
    echo 
    echo
    echo "Options:"
    echo
    echo "     -v | -version | --version     Display version information" 
    echo "     -h | -help | --help           Display command help"
    echo "     -start                        Start Event Service"
    echo "     -restart                      Restart the Event Service"
    echo "     -stop                         Stop the Event Service"
    echo "     -status                       Display status of the Event Service"
    echo
    echo "NOTE: The Notification Channel Factory needs to be started before"
    echo "      the Event Service can be started"
    echo
}

###############################################################################
#
# Function    : check_cmdline
#
# Description : Validate command line arguments
#
# Parameters  : none
#
###############################################################################
check_cmdline() {
    # Make sure only one argument is passed in
    if [ $# -ne 1 ]
    then
        usage
        exit $exit_rc_syntax
    fi
}

###############################################################################
#
# Function    : addJar
#
# Description : Add a Jar to the runtime CLASSPATH. A check is made to ensure
#               the Jar file exists.
#
# Parameters  : none
#
###############################################################################
addJar() {

    # First ensure the JAR file exists
    if [ ! -f "$1" ]
    then
        bad_jar $1
    fi

    # Add it to the classpath
    CLASSPATH=$CLASSPATH:$1
}

###############################################################################
#
# Function    : setup_runtime
#
# Description : Set the CLASSPATH needed at runtime
#
# Parameters  : none
#
###############################################################################
setup_runtime() {

    # Initialise the classpath	
    # The org.omg.CosNaming package in vbjorb.jar needs to be resolved here
    # before the one in fusion.jar

    CLASSPATH=''

    addJar $METRICA_DIR/ext/lib/vbjorb.jar
    addJar $METRICA_DIR/ext/lib/asrt.jar
    addJar $METRICA_DIR/ext/lib/vbsec.jar
    addJar $METRICA_DIR/ext/lib/lm.jar
    addJar $METRICA_DIR/ext/lib/xmlrt.jar

    # OpenFusion
    addJar $METRICA_DIR/ext/lib/fusion.jar

    # Informix JDBC
    addJar $METRICA_DIR/ext/lib/ifxjdbcx.jar
    addJar $METRICA_DIR/ext/lib/ifxjdbc.jar

    #
    # Database services
    #
    addJar $METRICA_DIR/DBServices/lib/queryservice_connectionmanager.jar
    addJar $METRICA_DIR/DBServices/lib/queryservice_requestmanager.jar
    addJar $METRICA_DIR/DBServices/lib/queryservice_logging.jar
    addJar $METRICA_DIR/DBServices/lib/queryservice_util.jar
    addJar $METRICA_DIR/DBElements/lib/metricaudt.jar


    #########################################################################
    #
    # Common Services archives follow
    #
    #########################################################################

    #
    # Event Service classes
    #
    addJar $METRICA_DIR/CS/lib/eventservice.jar

    #
    # Notification Framework classes
    #
    addJar $METRICA_DIR/CS/lib/nfservice.jar

    #
    # Utility classes
    #
    addJar $METRICA_DIR/CS/lib/corbautils.jar
    addJar $METRICA_DIR/CS/lib/exceptionutils.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_cache.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_client.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_common.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_persistence.jar
    addJar $METRICA_DIR/CS/lib/logservice.jar
    addJar $METRICA_DIR/CS/lib/sysmanager.jar

    export CLASSPATH

}

###############################################################################
#
# Function    : bad_env
#
# Description : Environment variable is not set
#
# Parameters  : Name of environment variable
#
###############################################################################
bad_env() {
    echo
    echo "Environment variable \$$1 is not set" >&2
    echo
    exit $exit_rc_bad_env
}

###############################################################################
#
# Function    : bad_dir
#
# Description : Directory pointed to by environment variable does not exist
#
# Parameters  : Name of environment variable
#
###############################################################################
bad_dir() {
    echo
    echo "Directory referred to by environment variable \$$1 does not exist" >&2
    echo
    exit $exit_rc_bad_dir
}

###############################################################################
#
# Function    : bad_jar
#
# Description : Jar file does not exist
#
# Parameters  : Full path/name of JAR file
#
###############################################################################
bad_jar() {
    echo
    echo "JAR file not found:" >&2
    echo $1 >&2
    exit $exit_rc_bad_jar
}

###############################################################################
#
# Function    : check_env
#
# Description : Ensure environment variable is set and optionally that it
#               refers to a directory that exists.
#
# Parameters  : "var" = check variable, "var_and_dir" = also check dir exists
#               Environment variable name (without "$" sign)
#               Value of the environment variable
#
###############################################################################
check_env() {
    if [ -z "$3" ] ; then
        bad_env "$2"
    fi

    if [ "$1" = "var_and_dir" -a ! -d "$3" ] ; then
        bad_dir "$2"
    fi
}

###############################################################################
#
# Function    : init
#
# Description : Perform the necessary checks to make sure the environment is
#               set up ready to run administration functions.
#
# Parameters  : none
#
###############################################################################
init() {
    check_env "var"         "OSAGENT_PORT" $OSAGENT_PORT
    check_env "var_and_dir" "METRICA_DIR" $METRICA_DIR

    setup_runtime
}

###############################################################################
###############################################################################
#
# Main program
#
###############################################################################
###############################################################################
check_cmdline $*

case "$1" in
    "-h"|"-help"|"--help" )
        print_component_version 
        usage
        exit $exit_rc_success
        ;;
    "-start" )
        print_component_version 
        init
        start_service
        ;;
    "-restart" )
        init
        restart_service
        ;;
    "-stop" )
        init
        stop_service
        ;;
    "-status" )
        init
        get_status
        ;;
    "-v"|"-version"|"--version" )
        print_component_version 
        exit $exit_rc_success
        ;;
    * )
        print_component_version 
        usage
        exit $exit_rc_syntax
        ;;
esac

exit $exit_rc_success
