#!/bin/ksh
# @(#) CS/delivery/bin/metrica_version.ksh CS_r4.79.0.3:cset.000937:6:5 04/06/05 @(#)
################################################################################
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                     Vallent
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# Copyright  2005 Vallent Software Systems UK. All rights reserved
#
###############################################################################
#

#################################################################
# Description : Set version of awk to use 
# Parameters  : none
#################################################################
env_setup() {
   if [ "$MACHINE" = "" ] ; then
      MACHINE=`metrica_machtype`
      export MACHINE
#       . metrica_machtype
   fi
   case $MACHINE in
      hpux_parisc )
        M_AWK=awk ;;
      solaris_sparc )
        M_AWK=/usr/xpg4/bin/awk ;;
      linux_ia32 )
        M_AWK=awk ;;
      * )
        M_AWK=nawk ;;
   esac
}


##################################################################
# Description : show version information 
# Parameters  : $1 - Component name
#               $2 - Version string (SCCS keywords)
##################################################################
show_version() {

  # Show blank line
 
  print

  # Show product name
  print "Metrica Performance Manager"

  # Show copyright message
  print "Copyright  2005 Vallent Software Systems UK. All rights reserved"

  # Print the title of the component
  print -n $1 " "

  # Display Interpreted Truechange ID
  echo $2 | $M_AWK -F _ '
  BEGIN {
    major  = "?"
    minor1 = "?"
    minor2 = "?"
    minor3 = "?"
    state = "unknown"
  }

  function decode_tcref( tcref )
  {
    prefix = tolower(substr(tcref, 1, 1))
    rest = substr(tcref, 2, index(tcref, ":")-2)
    fields = split(rest, nums, ".")
    if (fields != 4) {
      print "Error interpreting version numbers " rest
      return
    }
    major  = nums[1]
    minor1 = nums[2]
    minor2 = nums[3]
    minor3 = nums[4]
    if (prefix == "d") {
      state = "Development"
    } else if (prefix == "a") {
      state = "Development testing"
    } else if (prefix == "b") {
      state = "Functional testing"
    } else if (prefix == "r") {
      state = "Released product"
    } else {
      state = "Invalid state"
    }
  }

  {
    if (NF == 2) {
      # The simpler two argument version
      decode_tcref($2)
      printf("%s Version %s.%s Patch %s (Checkpoint: %s, State: %s)\n",
             $1, major, minor1, minor2, minor3, state);
    } else if (NF == 3) {
      # The 3 argument "branched" variation
      decode_tcref($3)
      printf("%s (%s branch) Version %s.%s Patch %s (Checkpoint: %s, State: %s)\n",
             $1, toupper($2), major, minor1, minor2, minor3, state);
    } else {
      print $0 " - Version unknown."
    }
  }'
  echo

}


####################################################################
# Description : Print Version info
# Parameters  : $1 - Component name
#               $2 - Version string (SCCS keywords)
####################################################################
print_version() {
  
  #Setup awk version to use
  env_setup

  #Show the version
  show_version "$1" "$2"

}


