#!/bin/ksh
###############################################################################
#
#  DBservices_r4.79.0.3:aset.037400:1:1
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2006.
#
# Title:        metrica_offliner.ksh
# Author:       Dominic O'Toole
#
# Description:  Script to stop and report on the status of the offliner 
#               
#
###############################################################################
#
# PROGRAMS AND SUBROUTINES
#
##########################################################################

SHUTDOWN_TIMEOUT=300  #Timeout when shutting down the Offliner ( Seconds ) 
PS_COMMAND="ps -ef"
OFFLINER_SCRIPT="metrica_offliner.pm"
RUNNING="running"
NOT_RUNNING="not_running"

#
# SUB PROGRAM  main - Main Routine
#

main() {

# using utilities in common script
. metrica_install_common.ksh
PROGNAME=`basename $0`

ROOT_DIR=$METRICA_DIR/DBServices/bin/  
LOG_DIR=$METRICA_DIR/logs

valenv
env_setup

run
}


# Setup the correct verion of PS in the env  
env_setup() {                              
   MACH=`metrica_machtype`           
   case $MACH in                        
      hpux_parisc )                       
        PS_COMMAND="ps -efx" ;;                       
      solaris_sparc )                      
        if [ -f /usr/ucb/ps ];then
           PS_COMMAND="/usr/ucb/ps -augxww"
        fi;;
      * )                                  
        PS_COMMAND="ps -ef" ;;                      
   esac                                    
}                                          

#
# Sub Program prtcopyrt - Print Vallent Copyright
#

prtcopyrt() {
print_to_screen""
print_to_screen "##############################################################"
print_to_screen "#"
print_to_screen "#                 Vallent Installation "
print_to_screen "# "
print_to_screen "#                   COPYRIGHT NOTICE "
print_to_screen "# "
print_to_screen "#     This file remains the intellectual property of: "
print_to_screen "# "
print_to_screen "#                 Vallent Software Systems "
print_to_screen "#                    Spencer House "
print_to_screen "#                    23 Sheen Road "
print_to_screen "#                 Richmond, Surrey, UK "
print_to_screen "# "
print_to_screen "# All rights reserved. Copyright Vallent Software Systems UK 2006"
print_to_screen "# "
print_to_screen "##############################################################"
print_to_screen " "
}
  

#
# Sub Program prtusage - Print Help         
#

prtusage() {

prtcopyrt
print_to_screen " "
print_to_screen " "
print_to_screen "  Usage: $PROGNAME [options]"
print_to_screen " "
print_to_screen "  options:"
print_to_screen "		[ -h | -help | --help ]"
print_to_screen "		| [ -v | -version | --version ]"
print_to_screen "		| [ -stop ]"
print_to_screen "		| [ -status ]"
print_to_screen " "
}

offliner_status() { 

        JOB_COUNT=`$PS_COMMAND | grep $OFFLINER_SCRIPT | grep -v grep | wc | awk '{print $1}'`                                                                                                                                                                                  
	 if [ "$JOB_COUNT" -eq "0" ]                                                                                                          
           then 
		   print_to_screen  $NOT_RUNNING
	   else
		   print_to_screen  $RUNNING
	   fi
}

# Waits for a particular job to finish 
wait_for_shutdown()
{
 DELAY_TOTAL=0
 DELAY=2
 TIMED_OUT=0

 #Args passed into method
 TASK_NAME=$1
 SCRIPT_NAME=$2
 
 print_to_screen " "
 $ECHO  -n "Waiting for the $TASK_NAME to shutdown ."

 FLAG=`offliner_status`

 while [ "$FLAG" = $RUNNING ]                                                                                                                      
 do                                                                                                                                           
   let DELAY_TOTAL+=$DELAY
   sleep $DELAY
   [[ $DELAY_TOTAL -ge $SHUTDOWN_TIMEOUT ]] && TIMED_OUT=1 && break

   $ECHO  -n "."
   FLAG=`offliner_status`
 done

 if [[ $TIMED_OUT -eq 1 ]] ; then
   $ECHO  -n " $TASK_NAME shutdown timed out after $SHUTDOWN_TIMEOUT seconds"
 else
   $ECHO  -n " $TASK_NAME shutdown complete."
 fi
 
 print_to_screen " "
 print_to_screen " "
}

# Check the input params
run() {                                                                                                                                              
	case "$I1" in
	  "-stop" )
	         stop_offliner && exit 1
          ;;
    "-status" )
           print_status  && exit 1
          ;;

	  "-h" | "-help" | "--help" )
	         clear
	         prtusage && exit 1
          ;;
	  
	 "-v" | "-version" | "--version" )
	         clear
		 prtcopyrt                                                                                                              
	         prt_version && exit 1
         ;;
	  * )
	         prtusage && exit 1
       ;;
     esac
}                                                                                                                                                         
print_status(){
  
  FLAG=`offliner_status`

  print_to_screen " "
	
 if [ "$FLAG" = "$RUNNING" ]; then
     print_to_screen "Offliner running"
 else
     print_to_screen "Offliner not running"
 fi

 print_to_screen " "

}

stop_offliner(){
     print_to_screen " " 
     print_to_screen "Checking Offliner status" 
     print_status

     FLAG=`offliner_status`

     if [ "$FLAG" = "$RUNNING" ]; then
          print_to_screen "Shutting down the Offliner"
          $METRICA_DIR/DBServices/bin/$OFFLINER_SCRIPT -stop
          wait_for_shutdown "Offliner" $OFFLINER_SCRIPT
     fi
}


valenv() {
if [ -z $METRICA_DIR ];then
   print_to_screen "\nERROR: Environment variable METRICA_DIR not set\n"
         exit
fi
}

print_to_screen() {
  $ECHO  $1
}

prt_version() {

   . $METRICA_DIR/CS/bin/metrica_version.ksh
   print_version "Metrica Offliner " DBservices_r4.79.0.3:aset.037400:1:1 
   exit 0

}

## - code entry point
#Preserve Input parameters
I1=$1           
I2=$2           
I3=$3           
I4=$4           
I5=$5           
I6=$6           
I7=$7           
I8=$8           
I9=$9           

main
