#!/usr/bin/ksh
#@(#) DBservices/delivery/unix/bin/metrica_summ.ksh DBservices_r4.79.0.3:cset.039770:17:16 04/15/05 @(#)
#
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                 Vallent Software Systems UK 
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# Copyright @ 2005 Vallent Software Systems UK. All rights reserved. 
#
# Title:        metrica_summ.ksh
# Author:       Micheal Spillane
# Description:  This runs the command line tool for the Summariser.
#
###############################################################################

#
# Exit codes
#
exit_rc_success=0
exit_rc_syntax=1
exit_rc_file_incorrect=3


###############################################################################
#
# Function    : addJar 
#
# Description : Add a Jar to the runtime CLASSPATH. A check is made to ensure
#               the Jar file exists.
#
# Parameters  : none
#
###############################################################################
addJar() {

    # First ensure the JAR file exists
    if [ ! -f "$1" ] ; then
        bad_jar $1
    fi

    # Add it to the classpath
    CP=$CP:$1
}


###############################################################################
#
# Function    : bad_jar
#
# Description : Jar file does not exist
#
# Parameters  : Full path/name of JAR file
#
###############################################################################
bad_jar() {
    echo
    echo "JAR file not found:" >&2
    echo $1 >&2
    exit $exit_rc_file_incorrect
}



# SAXParser class needs to be resolved here because asrt.jar contains
# a different verson of this class which will not work
CP=$METRICA_DIR/ext/lib/xerces.jar

# The org.omg.CosNaming package in vbjorb.jar needs to be resolved here
# before the one in fusion.jar
addJar $METRICA_DIR/ext/lib/vbjorb.jar
addJar $METRICA_DIR/ext/lib/asrt.jar
addJar $METRICA_DIR/ext/lib/vbsec.jar
addJar $METRICA_DIR/ext/lib/lm.jar
addJar $METRICA_DIR/ext/lib/xmlrt.jar

# Informix JDBC
addJar $METRICA_DIR/ext/lib/ifxjdbc.jar
addJar $METRICA_DIR/ext/lib/ifxjdbcx.jar

#
# Database services
#
addJar $METRICA_DIR/DBServices/lib/queryservice_connectionmanager.jar
addJar $METRICA_DIR/DBServices/lib/queryservice_requestmanager.jar
addJar $METRICA_DIR/DBServices/lib/queryservice_logging.jar
addJar $METRICA_DIR/DBServices/lib/queryservice_util.jar
addJar $METRICA_DIR/DBElements/lib/metricaudt.jar


# 
# Summariser libraries
#      
addJar $METRICA_DIR/DBServices/lib/dbserv_utils.jar
addJar $METRICA_DIR/DBServices/lib/summariser_server.jar
addJar $METRICA_DIR/DBServices/lib/summariser_ui.jar


# Property Service
addJar $METRICA_DIR/CS/lib/propertyservice_cache.jar
addJar $METRICA_DIR/CS/lib/propertyservice_client.jar
addJar $METRICA_DIR/CS/lib/propertyservice_common.jar
addJar $METRICA_DIR/CS/lib/propertyservice_persistence.jar

# Log Service
addJar $METRICA_DIR/CS/lib/logservice.jar

#
# Utility classes
#
addJar $METRICA_DIR/CS/lib/corbautils.jar
addJar $METRICA_DIR/CS/lib/exceptionutils.jar
addJar $METRICA_DIR/CS/lib/timeutils.jar
addJar $METRICA_DIR/CS/lib/ejbhomefactory.jar
addJar $METRICA_DIR/CS/lib/sysmanager.jar



###############################################################################
#
# Function    : run_command
#
# Description : Passes the command line args to SummariserUIController
#
# Parameters  : Command line arguments
#
###############################################################################
run_command(){

  #
  # OSAGENT_ADDR needs to be defined only if the summariser server 
  # and the client running metrica_summ.sh are in different subnets.
  #

  java -Xms4m -Xmx64m \
    -Dvbroker.agent.port=$OSAGENT_PORT \
    -Dvbroker.agent.addr=$OSAGENT_ADDR \
    -DSVCnameroot=namingservice \
    -Dorg.omg.CORBA.ORBClass=com.inprise.vbroker.orb.ORB \
    -Dorg.omg.CORBA.ORBSingletonClass=com.inprise.vbroker.orb.ORB \
    -Djavax.rmi.CORBA.StubClass=com.inprise.vbroker.rmi.CORBA.StubImpl \
    -Djavax.rmi.CORBA.UtilClass=com.inprise.vbroker.rmi.CORBA.UtilImpl \
    -Djavax.rmi.CORBA.PortableRemoteObjectClass=com.inprise.vbroker.rmi.CORBA.PortableRemoteObjectImpl \
    -classpath $CP com.adc.metrica.dbserv.summariser.ui.SummariserUIController "$@"
  #  -verbose -Xbootclasspath/a:$CP -jar $METRICA_DIR/DBServices/lib/summariser_ui.jar "$@"  

  exitcode=$?
  exit $exitcode

}


###############################################################################
#
# Function    : version_summariser_noheader
#
# Description : Displays the summariser version without component and copyright
#
# Parameters  : None
#
###############################################################################
version_summariser_noheader(){

  if [ $noheader -eq 0 -a $numargs -ne 1 ]
  then
    print_version_usage
  elif [ $noheader -eq 1 -a $numargs -ne 2 ]
  then
    print_version_usage
  fi
  
  . $METRICA_DIR/DBServices/bin/metrica_dbserv_version.ksh
  print_version_noheader "DBservices_r4.79.0.3:cset.039770:17:16"  
  
}

###############################################################################
#
# Function    : print_version_usage
#
# Description : Displays usage for -version command
#
# Parameters  : None
#
###############################################################################
print_version_usage(){
  
  PROGNAME=`basename $0`
  echo "The command failed."
  echo "$PROGNAME -v | -version | --version [-noheader]" 
  exit $exit_rc_syntax
}

###############################################################################
#
# Function    : print_header
#
# Description : Displays component name and copyright
#
# Parameters  : None
#
###############################################################################
print_header(){
  
  . $METRICA_DIR/DBServices/bin/metrica_dbserv_copyright.ksh "Metrica Summariser"
  
}



###############################################################################
#
# Function    : main
#
# Description : Process command line args 
#
# Parameters  : Command line arguments
#
###############################################################################


numargs=$#
noheader=0

for cmd in $@ 
do
  if [ "$cmd" = "-noheader" ]
    then noheader=1
  fi
done

if [ $noheader -eq 0 ]
  then print_header
fi
  

# Process command line parameters
case "$1" in
    "-v"|"-version"|"--version" ) version_summariser_noheader ;;
    *) 
esac

run_command "$@"

exit $exit_rc_success

