#!/bin/ksh
#@(#) DBservices/delivery/unix/bin/metrica_summariseradmin.ksh DBservices_r4.79.0.3:cset.042195:34:34 11/12/07 @(#)
#
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                 Vallent Software Systems UK 
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# Copyright @ 2005 Vallent Software Systems UK. All rights reserved. 
#
# Title:        metrica_summariseradmin.ksh
# Author:       Shane O'Rourke
# Description:  Start and stop summariser component.
#
###############################################################################
 
#
# Exit codes
#
exit_rc_success=0
exit_rc_usage=1
exit_rc_var_incorrect=2
exit_rc_dir_incorrect=3
exit_rc_file_incorrect=3

#
# Status codes
#
RUNNING=0
STARTING=1
NONFUNCTIONAL=3
STOPPED=4
SUMMARISER_STATUS=$STOPPED
SUMMARISER_STATUS_STRING="summariser Stopped"


##################
#     Usage      #
##################
usage() {
    echo "Usage:   `basename $0` {-status | -start | -stop | -restart | -preparetostop} | 
	                             {-v | -version | --version} |
	                             {-h | -help | --help}"
    echo ""
    exit
}


###############################################################################
#
# Function    : var_incorrect
#
# Description : Environment variable is set incorrectly
#
# Parameters  : Name of environment variable
#
###############################################################################
var_incorrect() {
    echo "Environment variable \$"$1" is not set correctly"
    exit $exit_rc_var_incorrect
}


###############################################################################
#
# Function    : dir_incorrect
#
# Description : Directory environment variable is set incorrectly
#
# Parameters  : Name of directory environment variable
#
###############################################################################
dir_incorrect() {
    echo "Directory environment variable \$"$1" does not exist"
    exit $exit_rc_dir_incorrect
}


###############################################################################
#
# Function    : file_incorrect
#
# Description : File environment variable is set incorrectly
#
# Parameters  : Name of file environment variable
#
###############################################################################
var_incorrect() {
    echo "File environment variable \$"$1" is not set correctly"
    exit $exit_rc_file_incorrect
}


###############################################################################
#
# Function    : check_env
#
# Description : Ensure environment variables, file and directory variables are
#               set incorrectly, and quits accordingly if not.
#
# Parameters  : Name of environment variable
#
###############################################################################
check_env() {

  # METRICA_DIR DIRECTORY ENVIRONMENT VARIABLE
  #
  if [ ! -d "$METRICA_DIR" ] 
  then
    dir_incorrect "METRICA_DIR"
  fi
  
  # NOTIFICATION_DIR  ENVIRONMENT VARIABLE
  #
  if [ ! -d "$NOTIFICATION_DIR" ] 
  then
     dir_incorrect "NOTIFICATION_DIR"
  fi
  
  # OSAGENT_PORT SERVER ENVIRONMENT VARIABLE
  #
  if [ ! "$OSAGENT_PORT" ] 
  then
     var_incorrect "OSAGENT_PORT"
  fi 
}

###############################################################################
#
# Function    : addJar
#
# Description : Add a Jar to the runtime CLASSPATH. A check is made to ensure
#               the Jar file exists.
#
# Parameters  : none
#
###############################################################################
addJar() {

    # First ensure the JAR file exists
    if [ ! -f "$1" ] ; then
        bad_jar $1
    fi

    # Add it to the classpath
    CP=$CP:$1
}


###############################################################################
#
# Function    : bad_jar
#
# Description : Jar file does not exist
#
# Parameters  : Full path/name of JAR file
#
###############################################################################
bad_jar() {
    echo
    echo "JAR file not found:" >&2
    echo $1 >&2
    exit $exit_rc_file_incorrect
}


###############################################################################
#
# Function    : set_env
#
# Description : Checks and set environment variables
#
# Parameters  : Name of file environment variable
#
###############################################################################
set_env(){

# check environment first
check_env

# set vars that need setting


# SAXParser class needs to be resolved here because asrt.jar contains
# a different verson of this class which will not work
CP=$METRICA_DIR/ext/lib/xerces.jar

# The org.omg.CosNaming package in vbjorb.jar needs to be resolved here
# before the one in fusion.jar
addJar $METRICA_DIR/ext/lib/vbjorb.jar
addJar $METRICA_DIR/ext/lib/asrt.jar
addJar $METRICA_DIR/ext/lib/vbsec.jar
addJar $METRICA_DIR/ext/lib/lm.jar
addJar $METRICA_DIR/ext/lib/xmlrt.jar

# OpenFusion
addJar $METRICA_DIR/ext//lib/fusion.jar

# Informix JDBC
addJar $METRICA_DIR/ext/lib/ifxjdbc.jar
addJar $METRICA_DIR/ext/lib/ifxjdbcx.jar

#
# Database services
#
addJar $METRICA_DIR/DBServices/lib/queryservice_connectionmanager.jar
addJar $METRICA_DIR/DBServices/lib/queryservice_requestmanager.jar
addJar $METRICA_DIR/DBServices/lib/queryservice_logging.jar
addJar $METRICA_DIR/DBServices/lib/queryservice_util.jar
addJar $METRICA_DIR/DBElements/lib/metricaudt.jar

# 
#  Summariser libraries
#      
addJar $METRICA_DIR/DBServices/lib/dbserv_utils.jar
addJar $METRICA_DIR/DBServices/lib/summariser_server.jar
addJar $METRICA_DIR/DBServices/lib/summariser_ui.jar
addJar $METRICA_DIR/DBServices/lib/summariser_admin.jar

# 
#  SP Registry
#      

addJar $METRICA_DIR/SPRegistry/lib/sprcommon.jar
addJar $METRICA_DIR/SPRegistry/lib/sprentity.jar
addJar $METRICA_DIR/SPRegistry/lib/dbspregistry.jar

# 
#  MetaLayer
#      

addJar $METRICA_DIR/MetaLayer/lib/mlcommon.jar
addJar $METRICA_DIR/MetaLayer/lib/mlentitybean.jar
addJar $METRICA_DIR/MetaLayer/lib/mlcache.jar
addJar $METRICA_DIR/MetaLayer/lib/mlmetaservice.jar
addJar $METRICA_DIR/MetaLayer/lib/mlmetaeditor.jar
addJar $METRICA_DIR/MetaLayer/lib/mlstartstop.jar
addJar $METRICA_DIR/MetaLayer/lib/gsmetaquery.jar
addJar $METRICA_DIR/MetaLayer/lib/rmqcommon.jar
addJar $METRICA_DIR/MetaLayer/lib/reportmetaquery.jar
addJar $METRICA_DIR/MetaLayer/lib/servicemanagermq.jar


#
# Common Services 
#

# Notification Channel Factory
addJar $METRICA_DIR/CS/lib/nfservice.jar

# Event Service
addJar $METRICA_DIR/CS/lib/eventservice.jar

# Property Service
addJar $METRICA_DIR/CS/lib/propertyservice_cache.jar
addJar $METRICA_DIR/CS/lib/propertyservice_client.jar
addJar $METRICA_DIR/CS/lib/propertyservice_common.jar
addJar $METRICA_DIR/CS/lib/propertyservice_persistence.jar

# Log Service
addJar $METRICA_DIR/CS/lib/logservice.jar

#
# Utility classes
#
addJar $METRICA_DIR/CS/lib/corbautils.jar
addJar $METRICA_DIR/CS/lib/exceptionutils.jar
addJar $METRICA_DIR/CS/lib/timeutils.jar
addJar $METRICA_DIR/CS/lib/ejbhomefactory.jar
addJar $METRICA_DIR/CS/lib/sysmanager.jar

# Scheduler
addJar $METRICA_DIR/CS/lib/schedulingservice_valuetypes.jar
addJar $METRICA_DIR/CS/lib/schedulingservice_client.jar
addJar $METRICA_DIR/CS/lib/schedulingservice_server.jar
addJar $METRICA_DIR/CS/lib/schedulingservice_persistence.jar


#export CP

}

###############################################################################
#
# Function    : stop_summariser
#
# Description : Stops the summariser
#
# Parameters  : None
#
###############################################################################
stop_summariser(){

    set_env

       #
       # OSAGENT_ADDR needs to be defined only if the summariser server 
       # and the client running metricasumm.sh are in different subnets.
       #
       
    stop_command="java -Xms4m -Xmx64m \
       -Dvbroker.agent.port=$OSAGENT_PORT \
       -Dvbroker.agent.addr=$OSAGENT_ADDR \
       -DSVCnameroot=namingservice \
       -Dorg.omg.CORBA.ORBClass=com.inprise.vbroker.orb.ORB \
       -Dorg.omg.CORBA.ORBSingletonClass=com.inprise.vbroker.orb.ORB \
       -Djavax.rmi.CORBA.StubClass=com.inprise.vbroker.rmi.CORBA.StubImpl \
       -Djavax.rmi.CORBA.UtilClass=com.inprise.vbroker.rmi.CORBA.UtilImpl \
       -Djavax.rmi.CORBA.PortableRemoteObjectClass=com.inprise.vbroker.rmi.CORBA.PortableRemoteObjectImpl \
       -classpath $CP com.adc.metrica.dbserv.summariser.admin.SummariserAdminController -stop"
#       -verbose -Xbootclasspath/a:$CP -jar $METRICA_DIR/DBServices/lib/summariser_admin.jar -stop"

           
    status_summariser
    if [[ $SUMMARISER_STATUS != "$STOPPED" ]]
    then
  	$stop_command
	if (( $? != 0 ))
        then 
	    exit $exit_rc_failure
        fi
    fi
}

###############################################################################
#
# Function    : status
#
# Description : Displays Summariser Server status
#
# Parameters  : None
#
###############################################################################

status(){
   status_summariser
   echo $SUMMARISER_STATUS_STRING
   exit $exit_rc_success
}

###############################################################################
#
# Function    : status_summariser
#
# Description : Parses Summariser Server status
#
# Parameters  : None
#
###############################################################################

status_summariser(){

  set_env
  status_command="java -Xms4m -Xmx64m \
                       -Dvbroker.agent.port=$OSAGENT_PORT \
                       -Dvbroker.agent.addr=$OSAGENT_ADDR \
                       -DSVCnameroot=namingservice \
		       -Dorg.omg.CORBA.ORBClass=com.inprise.vbroker.orb.ORB \
                       -Dorg.omg.CORBA.ORBSingletonClass=com.inprise.vbroker.orb.ORB \
                       -Djavax.rmi.CORBA.StubClass=com.inprise.vbroker.rmi.CORBA.StubImpl \
		       -Djavax.rmi.CORBA.UtilClass=com.inprise.vbroker.rmi.CORBA.UtilImpl \
  -Djavax.rmi.CORBA.PortableRemoteObjectClass=com.inprise.vbroker.rmi.CORBA.PortableRemoteObjectImpl \
  -classpath $CP com.adc.metrica.dbserv.summariser.admin.SummariserAdminController -status"
#  -verbose -Xbootclasspath/a:$CP -jar $METRICA_DIR/DBServices/lib/summariser_admin.jar -status"

  STATUS_STRING=`$status_command`
  if [[ -n `echo $STATUS_STRING | fgrep -e 'Running'` ]]
  then
	SUMMARISER_STATUS_STRING="summariser Running"
	SUMMARISER_STATUS=$RUNNING
  elif [[ -n `echo $STATUS_STRING | fgrep -e 'Stopped'` ]]
  then
  	SUMMARISER_STATUS_STRING="summariser Stopped"
	SUMMARISER_STATUS=$STOPPED
  else 
	SUMMARISER_STATUS_STRING="summariser Nonfunctional"
	SUMMARISER_STATUS=$NONFUNCTIONAL
  fi
}

###############################################################################
#
# Function    : help_summariser
#
# Description : Displays the summariser start stop administration usage
#
# Parameters  : None
#
###############################################################################
help_summariser(){
  print_version
  usage
}

###############################################################################
#
# Function    : version_summariser
#
# Description : Displays the summariser version
#
# Parameters  : None
#
###############################################################################
version_summariser(){
  print_version  
}

###############################################################################
#
# Function    : print_version
#
# Description : Displays version information
#
# Parameters  : None
#
###############################################################################
print_version(){
  set_env
   
  . $METRICA_DIR/CS/bin/metrica_version.ksh
  print_version "Metrica Summariser" "DBservices_r4.79.0.3:cset.042195:34:34"   
}

###############################################################################
#
# Function    : preparetostop_summariser
#
# Description : Prepares to stop the summariser
#
# Parameters  : None
#
###############################################################################
preparetostop_summariser(){
  set_env
  exit $exit_rc_success
}


###############################################################################
#
# Function    : start_summariser
#
# Description : Starts the summariser if not already started
#
# Parameters  : None
#
###############################################################################

start_summariser(){
  set_env
  
  status_summariser
  if [[ $SUMMARISER_STATUS = "$STOPPED" ]]
  then
    java -Xms8m -Xmx2048m \
    -Dvbroker.agent.port=$OSAGENT_PORT \
    -Dvbroker.agent.addr=$OSAGENT_ADDR \
    -DSVCnameroot=namingservice \
    -Dorg.omg.CORBA.ORBClass=com.inprise.vbroker.orb.ORB \
    -Dorg.omg.CORBA.ORBSingletonClass=com.inprise.vbroker.orb.ORB \
    -Djavax.rmi.CORBA.StubClass=com.inprise.vbroker.rmi.CORBA.StubImpl \
    -Djavax.rmi.CORBA.UtilClass=com.inprise.vbroker.rmi.CORBA.UtilImpl \
    -Djavax.rmi.CORBA.PortableRemoteObjectClass=com.inprise.vbroker.rmi.CORBA.PortableRemoteObjectImpl \
    -classpath $CP com.adc.metrica.dbserv.summariser.server.SummaryServerApp "$@" 
#    -verbose -Xbootclasspath/a:$CP -jar $METRICA_DIR/DBServices/lib/summariser_server.jar "$@" 
    if (( $? != 0 ))
    then 
        exit $exit_rc_failure
    fi
  elif [[ $SUMMARISER_STATUS = "$NONFUNCTIONAL" ]]
  then
    java -Xms8m -Xmx2048m \
    -Dvbroker.agent.port=$OSAGENT_PORT \
    -Dvbroker.agent.addr=$OSAGENT_ADDR \
    -DSVCnameroot=namingservice \
    -Dorg.omg.CORBA.ORBClass=com.inprise.vbroker.orb.ORB \
    -Dorg.omg.CORBA.ORBSingletonClass=com.inprise.vbroker.orb.ORB \
    -Djavax.rmi.CORBA.StubClass=com.inprise.vbroker.rmi.CORBA.StubImpl \
    -Djavax.rmi.CORBA.UtilClass=com.inprise.vbroker.rmi.CORBA.UtilImpl \
    -Djavax.rmi.CORBA.PortableRemoteObjectClass=com.inprise.vbroker.rmi.CORBA.PortableRemoteObjectImpl \
    -classpath $CP com.adc.metrica.dbserv.summariser.server.SummaryServerApp "$@" 
#    -verbose -Xbootclasspath/a:$CP -jar $METRICA_DIR/DBServices/lib/summariser_server.jar "$@" 
    if (( $? != 0 ))
    then 
        exit $exit_rc_failure
    fi
  fi
}



###############################################################################
#
# Function    : restart_summariser
#
# Description : Stops the summariser
#               Starts the summariser
#
# Parameters  : None
#
###############################################################################
restart_summariser(){
  
  status_summariser
  if [[ $SUMMARISER_STATUS = "$RUNNING" ]]
    then
    stop_summariser
    start_summariser
  elif [[ $SUMMARISER_STATUS = "$NONFUNCTIONAL" ]]
    then
    start_summariser
  elif [[ $SUMMARISER_STATUS = "$STOPPED" ]]
    then
    start_summariser
  fi
  exit $exit_rc_success
}


###############################################################################
#
# Function    : main
#
# Description : Process command line args 
#
# Parameters  : Command line arguments
#
###############################################################################

# Validate command line parameters
if [ $# -ne 1 ]
then
    usage
    exit $exit_rc_usage
fi

# Process command line parameters
while [[ $1 = -* ]]; do
    case $1 in
        -v) version_summariser ;;
        -version) version_summariser ;;
        --version) version_summariser ;;
        -h) help_summariser ;;
        -help) help_summariser ;;
        --help) help_summariser ;;
        -start) start_summariser 
	          exit $exit_rc_success;;
        -stop) stop_summariser 
	          exit $exit_rc_success;;
        -preparetostop) preparetostop_summariser ;;
        -restart) restart_summariser 
	          exit $exit_rc_success;;
        -status) status ;;
        *) usage
    esac
    shift
done

exit $exit_rc_usage
