#!/bin/ksh
###############################################################################
# @(#) SysManagement/delivery/unix/bin/clean_server.ksh SysManagement_r4.79.0.9:cset.005491:2:2 04/04/05 @(#)
#
#                   COPYRIGHT NOTICE
#
#   This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
# 
#   Copyright  2005 Vallent Software Systems UK. All rights reserved
#
# Title:        clean_server.ksh
# Authors:      Patrick Wood / Fergus Kelledy / Shane O'Rourke
# Description:  Script to clean out a previous installation
#               of Performance Manager
#
# WARNING:      This is an unsupported script. Use at your own risk.
#
###############################################################################



# Get common definitions
#
. metrica_install_common.ksh


THIS_HOST=`uname -n`
DFLT_APPSERVER_DIR=/apps/BES5.1
DFLT_APPSERVER_NAME=bes-$THIS_HOST
DFLT_EJBPARTITION=standard
DFLT_INFORMIXDIR=/apps/informix/ids.2000
DFLT_INFORMIXSERVER=$THIS_HOST
DFLT_INFORMIXSQLHOSTS=\$INFORMIXDIR/etc/sqlhosts.\$INFORMIXSERVER
DFLT_ONCONFIG=onconfig.\$INFORMIXSERVER
DFLT_METRICA_PERFORMANCE_DB=metrica_performance
DFLT_METRICA_SYSTEM_DB=metrica_system
DFLT_METRICA_ADMINISTRATION_DB=metrica_administration

[[ $1 = -auto ]] && AUTO_MODE=true

# First warn the user ininteractive mode
#
if [[ $AUTO_MODE != true ]]; then
  $ECHO
  $ECHO "WARNING: running this script will delete any existing installation of Metrica Performance Manager."
  $ECHO "DTDs, ejb and library jars, ears, datasources, wars, jds files and logs from the ejbcontainer,"
  $ECHO "the Metrica performance, administration and system databases and the Metrica datablade will all"
  $ECHO "be removed."
  $ECHO
  $ECHO -n "Are you sure you wish to proceed? [y/n] "
  read
  [[ $REPLY != "y" && $REPLY != "Y" ]] && exit 0
fi



for VAR in APPSERVER_DIR APPSERVER_NAME EJBPARTITION INFORMIXDIR INFORMIXSERVER INFORMIXSQLHOSTS ONCONFIG METRICA_PERFORMANCE_DB METRICA_SYSTEM_DB METRICA_ADMINISTRATION_DB; do
  eval THE_VAR=\$$VAR
  eval eval THE_DFLT=\$DFLT_$VAR
  if [[ -z $THE_VAR && $AUTO_MODE != true ]]; then
    eval echo "$VAR not defined"
    eval echo "\"Enter $VAR ($THE_DFLT): \\c\""
    eval read $VAR
  fi
  eval $VAR=\${$VAR:-$THE_DFLT}
done


CURRENT=$APPSERVER_DIR/var/servers/$APPSERVER_NAME
PREV_DIR=`pwd`

DIR=$CURRENT/partitions/$EJBPARTITION
cd $DIR
[ $? -ne 0 ] && echo "Cannot change dir to $DIR" && exit 1



# Delete DTDs
#
echo "Deleting DTDs"
[ -f *.dtd ] && rm -f *.dtd

# Delete all ejb jars
#
DIR=$CURRENT/partitions/$EJBPARTITION/ejb_jars
echo "Deleting EJB JARs in $DIR"
cd $DIR
[ $? -ne 0 ] && echo "Cannot change dir to $DIR" && exit 1
[ -f *.jar ] && rm -f *.jar

# Delete all libraries
#
DIR=$CURRENT/partitions/$EJBPARTITION/lib
echo "Deleting library JARs from $DIR"
cd $DIR
[ $? -ne 0 ] && echo "Cannot change dir to $DIR" && exit 1
[ -f *.jar ] && rm -f *.jar

# delete datasources
#
DIR=$CURRENT/partitions/$EJBPARTITION/dars
echo "Deleting Datasource DARs from $DIR"
cd $DIR
[ $? -ne 0 ] && echo "Cannot change dir to $DIR" && exit 1
[ -f *.dar ] && rm -f *.dar

# delete WARs
#
DIR=$CURRENT/partitions/$EJBPARTITION/wars
echo "Deleting WARs from $DIR"
cd $DIR
[ $? -ne 0 ] && echo "Cannot change dir to $DIR" && exit 1
[ -f *.war ] && rm -f *.war

# delete JDataStore files
#
DIR=$CURRENT/partitions/$EJBPARTITION
echo "Deleting (backing up) jds files and logs from $DIR"
cd $DIR
[ $? -ne 0 ] && echo "Cannot change dir to $DIR" && exit 1
mkdir -p OLD-JDS
[ -f *.jds ] && mv *.jds OLD-JDS # BES has ejbcontainer.jds and jss_storage.jds
[ -f *_LOGA_* ] && mv *_LOGA_* OLD-JDS

# Drop databases and remove blade
# Ensure dbaccess is on the path
#
export PATH=$PATH:$INFORMIXDIR/bin

echo "Deleting databases"
echo "drop database $METRICA_PERFORMANCE_DB; drop database $METRICA_SYSTEM_DB; drop database $METRICA_ADMINISTRATION_DB" | dbaccess sysmaster

# Delete blade
#
echo "Deleting blade"

# DataBlade names are expected to be of the format
#   ${METRICA_INSTALL_INSTANCE}_MetricaBlade.*
#
rm -rf $INFORMIXDIR/extend/${METRICA_INSTALL_INSTANCE}_MetricaBlade.*

(( $?!=0 )) && echo "WARNING: failed to delete blade directory from $INFORMIXDIR/extend. The blade directory should be removed manually"
