###############################################################################	
#
# @(#) SysManagement/delivery/utils/bin/DBColType/metrica_change_column_type_installer.pm SysManagement_r4.79.0.9:cset.005286:4:4 01/24/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#		 ADC Software Systems UK
#			Spencer House
#			23 Sheen Road
#		  Richmond, Surrey, UK
#
# All rights reserved. Copyright ADC Software Systems UK, 2000-2003.
#
# Title:	    metrica_change_column_type.pm
# Author:	    Tom Randles       
#
# Description:      Selects tables with column to alter and submit threads
#
##############################################################################

package metrica_change_column_type;

use strict;
use Getopt::Long;
use File::Basename;
use File::Find;


my ($database_name, $l_file, @column_names, $from_type,$to_type, $thread_count);
my ($log_filehandle, $table_name, @results, @table_list, %hash_of_tables);
my ($total_table_count, $no_tables_per_process); 

$Getopt::Long::ignorecase=1;
$Getopt::Long::autoabbrev=1;

if (scalar @ARGV < 6 ) {
    print "Error: Wrong number of arguments received.\n";
    usage();
}

my $result = GetOptions(
    "database=s"    => \$database_name,
    "log_file=s"    => \$l_file,
    "column_name=s" => \@column_names,
    "from_type=s"   => \$from_type,
    "to_type=s"     => \$to_type,
    "no_threads=i"  => \$thread_count,
    "<>"            => \&process_unknown_arguments);

# Print out options

log_msg("Database name : $database_name \n");
log_msg("Log file      : $l_file \n");
log_msg("Column names   : @column_names  \n");
log_msg("From type     : $from_type \n");
log_msg("To type       : $to_type \n");
log_msg("No threads    : $thread_count  \n");
log_msg("\n\n");

my $INFORMIXDIR = $ENV{INFORMIXDIR};
log_msg("Informix directory : $INFORMIXDIR ");
#print("Informix directory : $INFORMIXDIR \n");

if ( $from_type eq $to_type ) {
   print "Error: From and to types are identical. Exiting ...\n";
   exit 0;
}

# Initialising the log file.
if ( defined($l_file) ) {
    open(LOG_FILE, ">> $l_file") || die "Could not open log file $l_file\n";
}


# Tasks to accomplish 

# Get list of tables with column names of interest.
get_tables_of_interest();


# Parse table and divide up into chunks for processing. 
parse_table_list();

# Spawn sub processes and allocate tables to each one.
distribute_tasks_to_perform_updates();

exit 0;

sub chomp_values {
    my (@data) = @_;

    return grep(!/^\s*$/, @data);
}

sub execute {
   my ($database, $sql) = @_;
   my $command;

   $command = sprintf "/usr/bin/echo \"%s\" | \$INFORMIXDIR/bin/dbaccess %s 2>&1", $sql, $database;
   my @lines = `$command`;

   return chomp_values(@lines);
}

sub get_tables_of_interest {
   my $column_names="('".join("','",@column_names)."')";

  if ( $from_type eq "utime" ) {
   @results = execute($database_name, "SELECT tabname,'END' FROM systables st, syscolumns sc, sysxtdtypes xt WHERE st.tabname NOT LIKE 'sys%' AND st.tabname NOT LIKE 'bld%' AND st.tabid > 99 AND st.tabid  =  sc.tabid AND sc.colname IN $column_names AND st.tabtype = 'T' AND xt.name = 'utime' AND sc.extended_id=xt.extended_id ;");
        }
  else {
   @results = execute($database_name, "SELECT tabname,'END' FROM systables st, syscolumns sc WHERE st.tabname NOT LIKE 'sys%' AND st.tabname NOT LIKE 'bld%' AND st.tabid > 99 AND st.tabid  =  sc.tabid AND sc.colname IN $column_names AND st.tabtype = 'T' ;");
       }

}

sub parse_table_list {

   my @tabname;
   foreach my $line ( @results) {
      # Necessary for ".." operator
      $_ = $line;

      if ($line =~ /^tabname\s+/ .. /^\(constant\)\s+/) {
         if ($line =~ /^tabname\s+(.*)$/) {
            @tabname = ($1);
         }
         elsif ($line =~ /^\s+(.*)$/) {
            push @tabname, $1;
         }
         elsif ($line =~ /^\(constant\)\s+/) {
            push (@table_list, join("",@tabname));
         }
      }
   }
   $total_table_count = scalar(@table_list);
   $no_tables_per_process = ($total_table_count/$thread_count + 1);

   $no_tables_per_process = int($no_tables_per_process);
   log_msg("Thread count : $thread_count ");
   log_msg("Total table count : $total_table_count ");
   log_msg("Number of tables per process : $no_tables_per_process");
   #print("Thread count : $thread_count \n");
   #print("Total table count : $total_table_count \n");
   #print("Number of tables per process : $no_tables_per_process \n");

   #Divide up tables for processing
   my $cnt = 0;
   my $batch_count = 1;
   foreach my $table_name (@table_list) {
      push (@{$hash_of_tables{$batch_count}}, $table_name);
      $cnt++; 
      if ( $cnt == $no_tables_per_process ) {
         $batch_count++;
         $cnt = 0;
      } 
   }
   my $size = scalar(keys %hash_of_tables);
   log_msg("size of hash_of_tables is : $size ");
   #print("size of hash_of_tables is : $size \n");
   foreach my $batch_no (sort keys %hash_of_tables ) { 
     foreach my $table_name (@{$hash_of_tables{$batch_no}}) {
        log_msg("Batch number $batch_no : table : $table_name ");
        #print("Batch number $batch_no : table : $table_name \n");
     }
  }
}

sub distribute_tasks_to_perform_updates {

   log_msg("distribute_tasks_to_perform_updates");
   #print("distribute_tasks_to_perform_updates \n");
   my ($arg_list, $rc); 

   my $cnt=1;
   my $column_name;
   my $tables ;

   foreach (@column_names) {
      $column_name .= "-column=$_ ";
   }
   #print "Column name : $column_name \n";
   foreach my $batch_no ( keys %hash_of_tables ) {
     
      foreach (@{$hash_of_tables{$batch_no}}) {
         $tables .= "-table=$_ ";
      }
      $arg_list = "perl $ENV{METRICA_DIR}/Installer/bin/metrica_change_column_type_installer.pl -database=$database_name -log_file=$l_file"."_thread_"."$cnt $column_name -from_type=$from_type -to_type=$to_type $tables &";

      log_msg ("About to execute exec for batch $batch_no $arg_list");
      #print ("About to execute exec for batch $batch_no $arg_list \n");
      $rc = 0xffff & system ( $arg_list );
      log_msg("Return code from system : $rc");
      #print("Return code from system : $rc \n");
      $cnt++;
      undef $tables;
   }
}
sub log_msg {
   my $msg = shift;

   my $msg_time = scalar(localtime(time));
   print LOG_FILE "$msg_time - $msg \n";
}

sub process_unknown_arguments {
    my $unknown = shift;

    print "Unknown argument '$unknown'\n";
    usage();

}

sub usage {

    print join "",
    "Usage: $0 [mandatory-options] \n",
    "    Where 'mandatory-options' are:\n",
    "    --database_name            Name of database.\n",
    "    --log_file                 A file where log messages are written\n",
    "    --column_name              Name of column whose type will be changed.\n",
    "    --from_type                Existing type to change.  \n",
    "    --to_type                  New column type.  \n",
    "    --no_threads               Number of independent threads of execution. \n",
    "\n";

    exit 0;
}

