###############################################################################	
#
# @(#) SysManagement/delivery/utils/bin/EndStamp/metrica_add_endstamp.pm SysManagement_r4.79.0.9:cset.007769:6:6 08/25/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#			 Vallent Software Systems UK
#							Spencer House
#			         23 Sheen Road
#		         Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:	   metrica_add_endstamp.pm 
# Author:	    Dominic O'Toole       
#
# Description:      Selects tables without an endstamp column, adds 
#                   the endstamp(int) col and updates any index where 
#                   containing a timestamp
#
##############################################################################

package metrica_add_endstamp;

use strict;
use Getopt::Long;
use File::Basename;
use File::Find;
use FileHandle;
use lib "$ENV{METRICA_DIR}/Installer/bin/lib";
use Application;

my $add_flag=-1;
my $update_flag=-1;

my ($database_name, $l_file, $thread_count, $admin_db);
my ($log_filehandle, $table_name, @results, @table_list, %hash_of_tables);
my ($total_table_count, $no_tables_per_process); 
my $ROOTDIR="$ENV{METRICA_DIR}/Installer/bin";
my $LOG_DIR="$ENV{METRICA_DIR}/logs";

my $WORKINGDIR="/tmp";
my $ENDSTAMP_COL="endstamp";
# -- bugfix 50585 - changing default from 2147483647 to 0
my $ENDSTAMP_DEFAULT=0;
my $ENDSTAMP_MAX=2147483647;
my $USEFUL_FNS = new Application();

# -- the expected number of table rows processed in a minute by 1 update task thread
my $EXPECTED_UPDATE_RATE=8000;
# -- the expected number of table rows processed in a minute by 1 add task thread
my $EXPECTED_ADD_RATE=70000;

# -- if this threshold is exceeded, then a message is written to STDOUT
# -- informing user of expected duration of task in minutes
my $DURATION_THRESHOLD=2; 

my $tab_list_file_prefix="$WORKINGDIR/table_list_thread_";
my $NO_TABLES_RET_CODE=2 ; # Return code checked by the calling script that shows no processing is taking place


$Getopt::Long::ignorecase=1;
$Getopt::Long::autoabbrev=1;

if (scalar @ARGV < 4 ) {
    print "Error: Wrong number of arguments received.\n";
    usage();
}

my $result = GetOptions(
    "add"       => \$add_flag,
    "update"       => \$update_flag,
    "database=s"       => \$database_name,
    "log_file=s"       => \$l_file,
    "no_threads=i"     => \$thread_count,
    "admin_database=s" => \$admin_db,
    "<>"               => \&process_unknown_arguments);

# Check the --add or --update flag have been passed in
if($add_flag ne 1 && $update_flag ne 1){
 log_msg("--update or --add must be passed as an argument to this script");	
 exit;
}
if($add_flag ==  1 && $update_flag == 1){
 log_msg("Either --update or --add must be passed as an argument to this script, not both");	
 exit;
}


# Print out options
if($add_flag == 1 ) {
 log_msg(" \n\n***Starting Add Task - adding $ENDSTAMP_COL column and re-creating indexes \n\n");
 print " \n***Starting Add Task - adding $ENDSTAMP_COL column and re-creating indexes \n\n";
}
if($update_flag == 1 ) {
 log_msg( "\n\n***Starting Update Task - updating $ENDSTAMP_COL column for reparented objects\n\n");
 print "\n***Starting Update Task - updating $ENDSTAMP_COL column for reparented objects\n\n";
}


log_msg("Database name    : $database_name \n");
log_msg("Main log file    : $l_file \n");
print "Main Log file    : $l_file \n\n";
log_msg("Max threads      : $thread_count  \n");
log_msg("Admin Database   : $admin_db  \n\n");

my $INFORMIXDIR = $ENV{INFORMIXDIR};
log_msg("Informix directory : $INFORMIXDIR \n");
log_msg("\n");


# Initialising the log file.
if ( defined($l_file) ) {
    open(LOG_FILE, ">> $l_file") || die "Could not open log file $l_file\n";
}


## add flag is set
if($add_flag == 1 ) {

 # Tasks to accomplish 
 # Get list of tables with column names of interest.
 get_nc_tables_with_no_endstamp();

 extract_tables_from_results();

 # Return if there are no tables to process
 if(0 == scalar(@table_list)) { 
  printf("Column \'%s\' present in all nc tables, no further processsing needed\n\n", $ENDSTAMP_COL); 
  exit $NO_TABLES_RET_CODE; 
	
 }

}
else { 
 # -- get list of NC tables from systables, checking that
 # -- each table's endstamp column has:
 # -- either not yet been updated by previous run of script
 # -- or has no rows for update (no re-parenting).
 GetNCtables($database_name, "timestamp", $ENDSTAMP_COL, $admin_db);

 # -- Return if there are no tables to process
 if(0 == scalar(@table_list)) { 
  printf("Column \'%s\' has already been successfully processed for all nc tables, no further processsing needed\n\n", $ENDSTAMP_COL); 
  exit $NO_TABLES_RET_CODE;
 } 
 
}

# Parse table and divide up into chunks for processing. 
parse_table_list();

# Spawn sub processes and allocate tables to each one.
distribute_tasks_to_perform_updates();

exit 0;



 ## functions
 #
 #
 #

 sub chomp_values {
    my (@data) = @_;

    return grep(!/^\s*$/, @data);
}

sub execute {
   my ($database, $sql) = @_;
   my $command;

   $command = sprintf "/usr/bin/echo \"%s\" | \$INFORMIXDIR/bin/dbaccess %s 2>&1", $sql, $database;
   my @lines = `$command`;

   return chomp_values(@lines);
}

sub get_nc_tables_with_no_endstamp {

   @results = execute($database_name, "select tabname, nrows from  systables where tabname like 'nc_%' and tabid not in (select tabid from syscolumns where colname ='$ENDSTAMP_COL') and tabtype = 'T' order by nrows");
}


# Query results will be in the form of :
# tabname nc_cell
# tabname nc_bsc
#
# X rows returned
#
# Filter and extract the table name
#
sub extract_tables_from_results {

	my $row_count=0;
	my $total_count=0;

	foreach my $line ( @results) {         
		$_=$line;                            
		if(/tabname (.*)/)             # search for any line containing 'tabname'      
		{                                   
			my $table=$1;
			$table =~ s/^\s+//;
			$table =~ s/\s+$//;
			push (@table_list, join("",$table)); 
		} elsif (/nrows (.*)/) {        # search for any line containing 'nrows' 
			my $nrows=$1;
			$nrows =~ s/^\s+//;
			$nrows =~ s/\s+$//;  
			$row_count=$nrows;
			$total_count+=$row_count;
		}
	}
	
	InformUserOfDuration($total_count, $EXPECTED_ADD_RATE, $thread_count, "add task");
}


#***************************************************************************
## get "nc_*" tables
##
## Note the order by clause is to ensure we distribute the tables to threads
## according to size - a rough load-balancing effort.
##
## bugfix 50585 -	replacing duplicates, EndstampDuplicatesExist(), check 
##					with basic count(*) from NC tables.
#
sub GetNCtables {
	my ($database, $timestamp_col, $endstamp_col, $admin_db) = @_;

	log_msg("Retrieving NC tables...");

	# -- bugfix 50585 - added subselect, nrows only required for order 
	my $nc_tables_SQL = "select tabname, nrows from systables where tabname like 'nc_%' and tabtype = 'T' and tabid in (select tabid from syscolumns where colname='$endstamp_col') order by nrows"; 

	my @lines = execute($database, $nc_tables_SQL);
	my $table = "";
	my $row_count=0;
	my $total_count=0;

	# -- clear the global table_list, to be sure to be sure
	@table_list = ();

	# -- get access keys for all nc tables
	my $nc_access_keys_REF = GetAccessKeys($admin_db);

	# -- Get each table name and add it to list
	foreach my $line ( @lines ) {
		if ($line =~ /^tabname\s+(.*)$/) {
			$table = $1;
			# -- if no access key found in lc_relations,
			# -- then try and get it from index on table
			# -- that contains timestamp column.
			if ( !$nc_access_keys_REF->{$table} ) {
				log_msg("Looking for [$table]'s access key in sys catalog");

				log_msg("WARNING: Could not find the access key for $table in lc_relations, searching for an index with a timestamp\n");

				$nc_access_keys_REF->{$table} = GetAccessKeyFromSysCatalog($database, $table, "timestamp");
				if ( !$nc_access_keys_REF->{$table} ) {

					  my $message= "ERROR:   Cannot find the information needed to update endstamp column on table [$table].  See the Technology Pack Installation Notes for details of how to resolve this issue.\n";
					  print $message;
					  log_msg($message);

					# -- get next line
					#
				        my $sql = "update $table set $endstamp_col=$ENDSTAMP_MAX";

					log_msg("Updating table with the following SQL: $sql \n");

				        $USEFUL_FNS->execute($database,$sql);					
					next;
				} else {
					log_msg("Found access key [$nc_access_keys_REF->{$table}] for table [$table] in sys catalog");	
				}
			}
			# -- add table to list, only if endstamp values
			# -- have not already been updated or there is
			# -- no reparenting of object rows (duplicates).
			# -- bugfix 50585 - remove EndstampDuplicatesExist() call
			if ( $nc_access_keys_REF->{$table} ) {
				
				# -- bugfix 50585 - get row count 
				my $row_count_SQL = "select count(*) as rowcount from $table where $endstamp_col=$ENDSTAMP_DEFAULT";
				log_msg("About to execute SQL:\n$row_count_SQL");
				my @ret_lines=execute($database, $row_count_SQL);
				log_msg("@ret_lines");
				my $count_token=0;
				$row_count=0;
				foreach my $r_line ( @ret_lines ) {
					if ($r_line =~ /rowcount/) {
						$count_token=1;
					} elsif ( $count_token && ($r_line =~/([0-9]*)$/) ) {
						$row_count = $1;
						$total_count += $row_count;
						# -- break out
						last;
					}
				}
				if ( $row_count ) {
					log_msg("Adding table [$table] to list of tables to be updated");
					push(@table_list, join("",$table));
				} else {
					log_msg("Table [$table] does not require updating. No reparented rows.");
				}
			} else {
				# -- should never be reached. this message will be sent to stdout earlier.
				log_msg("Table [$table] cannot be updated. No access key.");
			}
		} 
	
	}   # -- end get each table...
	
	# -- bugfix 50585 
	# -- get list of NC tables that do not have endstamp column
	# -- this may happen if the add task did not succed for that table.
	my $invalid_nc_tables_SQL = "select tabname from systables where tabname like 'nc_%' and tabtype='T' and tabid not in (select tabid from syscolumns where colname='$endstamp_col')";

	log_msg ("About to execute SQL:\n$invalid_nc_tables_SQL");
	
	# -- clear the lines
	@lines = ();

	@lines = execute($database, $invalid_nc_tables_SQL);
	my @invalid_table_list=();
	my $invalid_tab="";

	# -- bugfix 50585 - Get each invalid-table name and add it to list
	foreach my $line ( @lines ) {
		if ($line =~ /^tabname\s+(.*)$/) {
			$invalid_tab = $1;
			push(@invalid_table_list, join("",$invalid_tab));
		}
	}
	
	# -- if we have tables without endstamp col 
	if(0 != scalar(@invalid_table_list)) {
		log_msg ("The following NC tables do not have an $endstamp_col column:\n@invalid_table_list");
		print "The following NC tables do not have an $endstamp_col column:\n@invalid_table_list\n";
	}

	InformUserOfDuration($total_count, $EXPECTED_UPDATE_RATE, $thread_count, "update task");

	log_msg("Finished retrieving NC tables...");
}

# -- Tell user expected duation of update processes
sub InformUserOfDuration
{
	my ($rows, $expected_rate, $num_threads, $task) = @_;

	my $duration = ($rows / $expected_rate) / $num_threads;
	if ( $duration >= $DURATION_THRESHOLD ) {
		log_msg("The $task will take approximately $duration minutes");
		printf "***The %s will take approximately %d minutes\n", $task, $duration;
	}
}

sub EndstampDuplicatesExist
{
   my ($database, $table, $access_key, $timestamp_col, $endstamp_col) = @_;

	my $duplicates_SQL = "select $access_key, $endstamp_col from $table group by $access_key, $endstamp_col having count(*) > 1";

	my @lines = execute($database, $duplicates_SQL);
	my $heading_found = 0;
	my $endstamp_val = 0;
	my $id = "";
	foreach my $line ( @lines ) {
		if ($line =~ /^$access_key\s*/) {
			$heading_found = 1;
			next;	
		} elsif ( $heading_found ) {
			# -- get the value of the endstamp column
			# -- if there is any value here we know
			# -- at least one duplicate row was returned.
			# -- a duplicate row will always have endstamp default val 
			($id, $endstamp_val) = split (/\s+/,$line,3);
			if ( $endstamp_val != $ENDSTAMP_DEFAULT) {	
				$endstamp_val = 0;
			} else {	
				$endstamp_val = 1;
				# -- end processing
				last;
			}
		} 
	}

	return $endstamp_val;
}


#***************************************************************************
## Get the access key for nc_table
#
#  Returns a reference to a hash of NC table access keys
#
sub GetAccessKeys {
	my ($admin_database) = @_;

	log_msg("Getting access keys for NC tables from database [$admin_database]\n");

	# -- Get the contents of the lc_relations table
	my $relations_SQL = "select unique nc_tabname,access_key from lc_relations where nc_tabname like 'nc_%'";

	my @lines = execute($admin_database, $relations_SQL);
	my $tab="";
	my $access="";
        my %nc_table_access = ();

	foreach my $line ( @lines ) {
		if ($line =~ /^nc_tabname\s+(.*)$/) {
			$tab = $1;
		} elsif ($line =~ /^access_key\s+(.*)$/) {
			$access = $1;
		}
		if ( $tab && $access) {
			$nc_table_access{$tab} = $access;
			$tab="";
			$access="";
		} 
	}

	if (%nc_table_access) {
		log_msg("Retrieved access keys for NC tables\n");
	}

	return \%nc_table_access;
}


#***************************************************************************
## Get the access key for a particular nc_table.
## This sub is used when the access key is not present in lc_relations.
#
#  Returns a string representing access keys
#
sub GetAccessKeyFromSysCatalog {
	my ($database, $table, $timestamp_col) = @_;
	
	# -- Get the indexes associated with this table
	my ($idxtype, $idxcolumns_REF, $idxdbspace) = $USEFUL_FNS->getIndexDetails($database, $table);

	# -- return value
	my $access_key = "";

	# -- Iterate indexes until find one with timestamp in it
	# -- Then return the column in index that is not called
	# -- timestamp or endstamp
	foreach my $index ( keys %{$idxcolumns_REF} ) {
		if (grep(/^$timestamp_col\s*/, @{$idxcolumns_REF->{$index}}) > 0) {
			foreach my $col ( @{$idxcolumns_REF->{$index}} ) {
				if ( ($col ne $ENDSTAMP_COL) && ($col ne $timestamp_col) ) {
					# -- found access key
					$access_key = $col;
				}
			}
			if ( $access_key ) {
				#-- no  need to process any more indexes for this table
				last;
			}
		}
	}

	# -- return
	return $access_key;
}


sub parse_table_list {

   
   $total_table_count = scalar(@table_list);
   $no_tables_per_process = ($total_table_count/$thread_count + 1);

   $no_tables_per_process = int($no_tables_per_process);
 
   log_msg("Thread count : $thread_count ");
   log_msg("Total table count : $total_table_count ");
   log_msg("Number of tables per process : $no_tables_per_process\n");

   #Divide up tables for processing
   my $batch_count = 1;
   my $batch_total = 0; 
   if ( $total_table_count > $thread_count ) {
      $batch_total = $thread_count;
   } else {
      $batch_total = $total_table_count;
   }
   foreach my $table_name (@table_list) {
      push (@{$hash_of_tables{$batch_count}}, $table_name);
      $batch_count++; 
      if ( $batch_count > $batch_total ) {
         $batch_count = 1;
      } 
   }
   my $size = scalar(keys %hash_of_tables);
   log_msg("size of hash_of_tables is : $size ");
   
   foreach my $batch_no (sort keys %hash_of_tables ) { 
	   
    my $file_name="$tab_list_file_prefix$batch_no";

    my $table_list_file = new FileHandle("$file_name", "w")  or die "Cannot open table file $file_name";

     foreach my $table_name (@{$hash_of_tables{$batch_no}}) {
        log_msg("Batch number $batch_no : table : $table_name\n ");
	print $table_list_file $table_name;
	print $table_list_file "\n";
     }
  }
}

sub distribute_tasks_to_perform_updates {

   log_msg("distribute_tasks_to_perform_updates");
   my ($arg_list, $rc); 

   my $cnt=1;
   my $tables ;

   foreach my $batch_no ( keys %hash_of_tables ) {

	   
    if($add_flag == 1 ) {
	     $arg_list = "perl $ROOTDIR/metrica_add_endstamp_task.pl -database=$database_name -log_file=$l_file"."_add_thread_"."$cnt -table=$tab_list_file_prefix$batch_no;"; 
     }
     else
     {
         $arg_list = "perl $ROOTDIR/metrica_update_endstamp_task.pl -database=$database_name -log_file=$l_file"."_update_thread_"."$cnt -table=$tab_list_file_prefix$batch_no -admin_database $admin_db;";                                                                                                                                                                                                
     }

      log_msg ("About to execute exec for batch $batch_no $arg_list");
      $rc = 0xffff & system ( "(" . $arg_list . ")&" );
      log_msg("Return code from system : $rc");

      $cnt++;
      undef $tables;
   }

}

sub log_msg {
   my $msg = shift;
   #printf("$msg ");

   my $msg_time = scalar(localtime(time));
   print LOG_FILE "$msg_time - $msg \n";
}

sub process_unknown_arguments {
    my $unknown = shift;

    print "Unknown argument '$unknown'\n";
    usage();

}

sub usage {

    print join "",
    "Usage: $0 [mandatory-options] \n",
    "    Where 'mandatory-options' are:\n",
    "    --database_name            Name of performance database.\n",
    "    --log_file                 A file where log messages are written.\n",
    "    --no_threads               Number of independent threads of execution.\n",
    "    --admin_database           Name of administration database.\n",
    "\n";

    exit 0;
}

