###############################################################################	
#
# @(#) SysManagement/delivery/utils/bin/EndStamp/metrica_add_endstamp_task.pl SysManagement_r4.79.0.9:cset.007747:7:7 08/24/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#		 Vallent Software Systems UK
#			     Spencer House
#			     23 Sheen Road
#		     Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:	    metrica_add_endstamp_task.pl      
# Author:	    Domininc O'Toole 
#
# Description:      A generic task that adds and endstamp column to a table and
# recreates the indexes that use timestamp to include endstamp.
#
##############################################################################

#---------------------------------------------------------------------------
#       PACKAGE NAME
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#   Core Perl Libraries
#---------------------------------------------------------------------------
use strict;
use FileHandle;
use Getopt::Long;
use lib "$ENV{METRICA_DIR}/Installer/bin/lib";
use Application;

# Force output after every line (keeps stderr and stdout in sync)
autoflush STDOUT 1;
autoflush STDERR 1;

#---------------------------------------------------------------------------
#   Program Libraries
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PRIVATE GLOBAL CONSTANTS
#---------------------------------------------------------------------------
my $OK       = "OK";
my $ENDSTAMP       = "endstamp";
# -- bugfix 50585 - changed default from 2147483647 to 0
my $ENDSTAMP_DEFAULT = 0;
my $USEFUL_FNS = new Application();

#---------------------------------------------------------------------------
#       PUBLIC GLOBAL CONSTANTS
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PRIVATE GLOBAL VARIABLES
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PUBLIC GLOBAL VARIABLES
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PUBLIC SUBROUTINES
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       CLASS METHOD SUBROUTINES
#---------------------------------------------------------------------------


my %param = ();
main(\%param);

#***************************************************************************

=head2 new 

 DESCRIPTION:
   This constructor parses the configuration file and initialises the 
   object data structures

 PARAMETERS:
   $class    - The name of the class (implicit parameter).

 RETURNS:
   The class object.

=cut

sub new
{
    my $class = shift;
    my $self = {};
    bless $self, $class;

    return $self;
}


#***************************************************************************


=head2  createIndexSQL

 DESCRIPTION:
   This method is used to recover the information regarding all table
   indexes involving this column.
   2 list references are returned. One list contains SQL statements to
   drop the indexes, the second contains SQL statements to create the
   indexes.

 PARAMETERS:
   $self     - The class object (implicit parameter).
   $database - database name
   $table    - Table name
   $column   - column name

 RETURNS:
   1) Array reference to "DROP INDEX" SQL statements
   2) Array reference to "CREATE INDEX" SQL statements

=cut

sub createIndexSQL
{
   #my $self = shift;
   my ($log,$database, $table, $column) = @_;
   my %indexData;
   my @drop = ();
   my @create = ();

   #
   # Get the indexes associated with this table
   #
   my ($idxtype, $idxcolumns, $idxdbspace) = $USEFUL_FNS->getIndexDetails($database, $table);


   foreach my $index ( keys %{$idxcolumns} ) {
      if (grep(/^$column\s*/, @{$idxcolumns->{$index}}) > 0) {
	       my @indexcolumns = @{$idxcolumns->{$index}};
	       push @indexcolumns, $ENDSTAMP;
       
         push @drop,   sprintf "DROP INDEX %s;", $index;

         push @create, "SET PDQPRIORITY 80;";
	 
	 log_msg($log, "AUDIT: Adding column $ENDSTAMP to index $index on table $table...");
         print "AUDIT: Adding column $ENDSTAMP to index $index on table $table...\n";
	 push @create, sprintf "CREATE %s INDEX %s ON %s (%s) IN %s;", $idxtype->{$index}, $index, $table, join(",",@indexcolumns), $idxdbspace->{$index};
      }
   }

   return (\@drop, \@create);
}

#***************************************************************************

=head2  main

 DESCRIPTION:
   Main routine for the application.

 PARAMETERS:
   $self - The class object (implicit parameter).
   $ref  - A Reference to a hash of parameters

 RETURNS:
   $OK

=cut

sub main
{

   my ($database, $logFile, $table_list_file); 

   my $args = GetOptions(                            
	     "database=s"    => \$database,             
		   "log_file=s"    => \$logFile,                    
       "table=s"  => \$table_list_file);


   my $table_list_file_handle = new FileHandle($table_list_file, "r") or die "Cannot open logfile: $table_list_file";

   my @tables   = $table_list_file_handle->getlines();
	
 	 #
   # Open the log file
   #
   my $log = new FileHandle($logFile, "w") or die "Cannot open logfile: $logFile";
   
	 log_msg($log,"Starting Thread:\n");                                         

   #
   # Process each table
   #
   foreach my $table ( @tables ) {
   
	  # remove white space
    $table =~ s/^\s+//;
	  $table =~ s/\s+$//;

	  log_msg($log,"processing $table\n");
     
      if (!$USEFUL_FNS->tableExists($database, $table)) {
         log_msg($log, "ERROR: Table does not exist : $table\n");
         printf("ERROR: Table does not exist : %s\n", $table);
      }
      else {
	       #
	       # Fetch the Index details
	       #
               print "AUDIT: Adding column $ENDSTAMP to table $table....\n";
	       log_msg($log, "AUDIT: Adding column $ENDSTAMP to table $table....\n");
               
	       my ($dropRef, $createRef) = createIndexSQL($log,$database, $table,"timestamp");

	       #
	       # Construct a list of upgrade SQL
	       #
	       my @sql;
		   push @sql, "SET LOCK MODE TO WAIT;";
		   push @sql, "ALTER table $table ADD $ENDSTAMP integer default $ENDSTAMP_DEFAULT not null;";
	       push @sql, "SET LOCK MODE TO WAIT;";
		   push @sql, @{$dropRef};
		   push @sql, "SET LOCK MODE TO WAIT;";
	       push @sql, @{$createRef};

	       my $sqlStr = join("\n",@sql);

	       #
	       # Log message
	       #
	       log_msg($log, "Upgrade SQL:\n");
	       log_msg($log, "$sqlStr\n");

	       #
	       # Execute a log the upgrade SQL
	       #
	       log_msg($log, "\nLog:\n");

	       foreach my $line ( $USEFUL_FNS->execute($database, $sqlStr) ) {
                  log_msg($log, "$line\n");
	       }
      }
   }

   #
   # Cleanup
   #
   log_msg($log, "Finished Thread.");
   $log->close;

   return $OK;
}


sub log_msg {
	my ($file_handle, $msg) = @_;

	#printf("$msg ");

	my $msg_time = scalar(localtime(time));
	$file_handle->print("$msg_time - $msg");
}

#---------------------------------------------------------------------------
#       PRIVATE SUBROUTINES
#---------------------------------------------------------------------------

#---------------------------------------------------------------------------
#       PACKAGE INITIALISATION
#---------------------------------------------------------------------------
1;


#
#       End (of template.pm)
#
