#!/bin/ksh
###############################################################################
#
# @(#) SysManagement/delivery/utils/bin/DBIndexCreate/metrica_add_name_indexes.ksh SysManagement_r4.79.0.9:cset.007770:3:3 08/26/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_add_name_indexes.ksh
# Author:       Eamonn Larkin,Dominic O'Toole
#
# Description:  Add index to certain columns in nc tables 
#               
#
###############################################################################
#
# PROGRAMS AND SUBROUTINES
#
##########################################################################

ALL_TABLES_HAVE_INDEX_RET_CODE=2
PS_COMMAND="ps -ef"

#
# SUB PROGRAM  main - Main Routine
#

main() {

# using utilities in common script
. metrica_install_common.ksh
PROGNAME=`basename $0`


ROOT_DIR=$METRICA_DIR/Installer/bin/  
LOG_DIR=$METRICA_DIR/logs
NC_FILE_NAME=/tmp/nc_names

## Set the Log File 
LOG_FILE="$LOG_DIR/metrica_add_name_indexes.log"
rm -f $LOG_FILE* > /dev/null
rm -f $NC_FILE_NAME > /dev/null

valparms
valenv
valdb
env_setup

THREADS=$I2

call_mladmin
add_index

}


# Setup the correct verion of PS in the env  
env_setup() {                              
   MACH=`metrica_machtype`           
   echo `date`" - The metrica_machtype has been determined as: $MACH" >> $LOG_FILE
   case $MACH in                        
      hpux_parisc )                       
        PS_COMMAND="ps -efx" ;;                       
      solaris_sparc )                      
        if [ -f /usr/ucb/ps ];then
           PS_COMMAND="/usr/ucb/ps -augxww"
        fi;;
      * )                                  
        PS_COMMAND="ps -ef" ;;                      
   esac                                    
}                                          

#
# Sub Program prtcopyrt - Print Vallent Copyright
#

prtcopyrt() {
echo""
echo "##############################################################"
echo "#"
echo "#                 Vallent Installation "
echo "# "
echo "#                   COPYRIGHT NOTICE "
echo "# "
echo "#     This file remains the intellectual property of: "
echo "# "
echo "#                 Vallent Software Systems "
echo "#                    Spencer House "
echo "#                    23 Sheen Road "
echo "#                 Richmond, Surrey, UK "
echo "# "
echo "# All rights reserved. Copyright Vallent Software Systems UK 2005"
echo "# "
echo "##############################################################"
echo " "
}
  

#
# Sub Program prtusage - Print Help         
#

prtusage() {

prtcopyrt
echo " "
echo " "
echo "  Usage: $PROGNAME [options]"
echo " "
echo "  options:"
echo "		[ -h | -help | --help ]"
echo "		| [ -v | -version | --version ]"
echo "		| [ -threads <number of threads (between 1 and 10> ]"
echo " "
}

# add the endstamp column and wait for all the jobs to finish

add_index() {

 perl $ROOT_DIR/metrica_add_name_indexes.pm --name_columns_file=$NC_FILE_NAME --database=$METRICA_PERFORMANCE_DB --log_file=$LOG_FILE --no_threads=$THREADS --admin_database=$METRICA_ADMINISTRATION_DB 

 #The script returns this is all nc tables already have the endstamp column

 if [ "$?" -eq "$ALL_TABLES_HAVE_INDEX_RET_CODE" ]; then                                                                                                  
	return;                                                                                                                                      
 fi                                                                                                                                           
 THREAD_LOG_SUFFIX="_add_index_thread_"
 THREAD_LOG_FILE=$LOG_FILE$THREAD_LOG_SUFFIX
 TASK_NAME="metrica_add_name_indexes.pl"
 TASK_TYPE="Addition"

 wait_for_job_to_finish  $TASK_NAME $TASK_TYPE $THREAD_LOG_SUFFIX $THREAD_LOG_FILE

}
 

# Waits for a particular job to finish and check the logs for errors
wait_for_job_to_finish()
{

 TASK_NAME=$1
 TASK_TYPE=$2
 THREAD_LOG_SUFFIX=$3
 THREAD_LOG_FILE=$4

 echo `date`" - Using this command to find running sub-processes: $PS_COMMAND" >> $LOG_FILE

 FLAG="n"                                  
 while [ "$FLAG" = "n" ]                                                                                                                      
 do                                                                                                                                           
        JOB_COUNT=`$PS_COMMAND | grep $TASK_NAME | grep -v grep | wc | awk '{print $1}'`                                                                                                                                                                                  
        if [ "$JOB_COUNT" -eq "0" ]                                                                                                          
        then                                                                                                                                 
               echo " "                                                                                                                     
               echo "$TASK_TYPE Complete"                                                                                           
               echo " "                                                                                                                     
               #echo " $THREAD_LOG_SUFFIX "

               #echo $LOG_DIR 
	       ACTUAL_THREADS=`ls $LOG_DIR | grep "$THREAD_LOG_SUFFIX" | wc | awk '{print $1}'`                                               
															                                                                                                                                             
               if [ "$ACTUAL_THREADS" -eq 0 ]; then                                                                                         
                      echo "No conversion threads were required"                                                                           
               else                                                                                                                         
   		      echo $ACTUAL_THREADS" out of "$THREADS" possible threads were required and have finished"                            
      		      echo " "                                                                                                             
		      CNT0="0" 
                      while [ "$CNT0" -lt "$THREADS" ]
                      do    
		          CNT0=`expr $CNT0 + 1`                                                                                        
 
		         if [ -f $THREAD_LOG_FILE$CNT0 ]; then 

              			 ERROR_COUNT=`grep -i error $THREAD_LOG_FILE$CNT0 | grep -vi error_ | wc | awk '{print $1}'`       
                                 echo `date`" - \c" >> $LOG_FILE
                                 echo $ERROR_COUNT" error messages detected in log file $THREAD_LOG_FILE$CNT0" | tee -a $LOG_FILE 
              			 
				 #ERROR_COUNT=`grep WARNING $THREAD_LOG_FILE$CNT0 | grep -vi error_ | wc | awk '{print $1}'`       
                                 #echo `date`" - \c" >> $LOG_FILE
                                 #echo $ERROR_COUNT" warning messages detected in log file $THREAD_LOG_FILE$CNT0" | tee -a $LOG_FILE 
                         fi                                                                                                           
                      done                                                                                                                 
               fi                                                                                                                           
               echo " "                                                                                                                     
	       FLAG="y"
	       
        else                                                                                                                                 
               sleep 10  
        fi
 done
}

#Call metrica_mladmin.ksh script to generate the names of tables and there columns that require index added

call_mladmin() {

  if [ -f $METRICA_DIR/MetaLayer/bin/metrica_mladmin.ksh ]; then 
   $METRICA_DIR/MetaLayer/bin/metrica_mladmin.ksh -getnamecolumns -file $NC_FILE_NAME
  else
    echo "WARNING: $METRICA_DIR/MetaLayer/bin/metrica_mladmin.ksh does not exist, skipping this step"
  fi
}


# Check the input params
valparms() {                                                                                                                                              
 if [ -z "$I1" ]; then                                                                                                                                     
    prtusage                                                                                                                                              
	  echo " "                                                                                                                                                
    echo "ERROR: No arguments given"                                                                                                                      
	  echo " "                                                                                                                                                
	  exit                                                                                                                                                    
 fi                                                                                                                                                        
 if [ "$I1" = "-h" -o "$I1" = "-help" -o "$I1" = "--help" ]                                                                                              
 then 
  clear                                                                                                                                                 
  prtusage                                                                                                                                              
  exit                                                                                                                                                  
 fi                                                                                                                                                      

 if [ "$I1" = "-v" -o "$I1" = "-version" -o "$I1" = "--version" ]                                                                                        
  then                                                                                                                                                    
  clear                                                                                                                                                 
  prtcopyrt                                                                                                                                             
  echo "$PROGNAME version SysManagement_r4.79.0.9:cset.007770:3:3"
  echo ""
  exit                                                                                                                                                  
 fi                                                                                                                                                      

 if [ "$I1" != "-threads" ]                                     
  then                                                                                                                                                    
   prtusage                                                                                                                                              
   echo " "                                                                                                                                              
   echo "ERROR: Invalid Arguments Passed to program"                                                                                                     
   echo " "                                                                                                                                              
   exit                                                                                                                                                  
 fi                                                                                                                                                    
}                                                                                                                                                         

# check the environment
valenv() {
if [ -z $METRICA_PERFORMANCE_DB ];then
   echo "\nERROR: Environment variable METRICA_PERFORMANCE_DB not set\n"
	 exit
fi
if [ -z $METRICA_ADMINISTRATION_DB ];then
   echo "\nERROR: Environment variable METRICA_ADMINISTRATION_DB not set\n"
         exit
fi
if [ -z $METRICA_DIR ];then
   echo "\nERROR: Environment variable METRICA_DIR not set\n"
         exit
fi
}

valdb() {                                                                                                

echo 'select count(*) from sysdatabases where name ="'$METRICA_PERFORMANCE_DB'";' | dbaccess sysmaster > $LOG0        2>&1   
DB_COUNT=`cat $LOG0 | grep -v count | grep -v Database | grep -v row`                                    
rm -rf $LOG0
if [ "$DB_COUNT" -eq "0" ]                                                                               
then                                                                                                     
  prtusage                                                                                               
  echo " "                                                                                               
  echo "ERROR: "$METRICA_PERFORMANCE_DB" is not a valid database on this Informix Server"                                    
  echo " "                                                                                               
  exit                                                                                                   
fi

}

## - code entry point


#Preserve Input parameters
LOG0=/tmp/mcc$$ 
I1=$1           
I2=$2           
I3=$3           
I4=$4           
I5=$5           
I6=$6           
I7=$7           
I8=$8           
I9=$9           


main
