#!/bin/ksh
###############################################################################
#
# @(#) SysManagement/delivery/utils/bin/DBColType/metrica_change_column_type_installer.ksh SysManagement_r4.79.0.9:cset.007628:10:10 07/06/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                ADC Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright ADC Software Systems UK, 2000-2003.
#
# Title:        metrica_change_column_type.ksh
# Author:       Barry Deignan    
#
# Description:  Alter all tables with specified column 
#               and column type                          
#
###############################################################################
#
# PROGRAMS AND SUBROUTINES
#
##########################################################################

PS_COMMAND="ps -ef"

#
# SUB PROGRAM  main - Main Routine
#

main() {

# using utilities in common script
. metrica_install_common.ksh

## Validate Input Parms
valparms

#Setup the PS command
env_setup

##  Validate Database Name                      
valdb
## Create the View Re-creation scripts
view_scr

## Set the Log File                                 
LOG_FILE="$METRICA_DIR/logs/metrica_change_column_type.log"
rm -f $LOG_FILE* > /dev/null

## Validate the Column Name File                             
valcl

## Validate FROM_TYPE                              
TYPE=$I6
valtype
FROM_TYPE=$I6

## Get the TO_TYPE                              
TYPE=$I7
valtype
TO_TYPE=$I7

## Validate the number of Concurrent Threads         
valthreads
#THREADS="4"


## Present Selection Summary & Confirmation
#clear
#echo ' '
#echo "METRICA COLUMN CONVERTER"                                      

#echo ' '
#echo "You have entered the following parameters"
#echo ""
#echo "	Admin Database                   	:	"$THE_DATABASE
#echo "	Log File                         	:	"$LOG_FILE     
#echo "	Column Name                      	:	"$DISPLAY_COLUMN_NAME
#echo "	From Type                        	:	"$FROM_TYPE   
#echo "	To Type                          	:	"$TO_TYPE      
#echo "	Number of Concurrent Threads     	:	"$THREADS      
#echo ""
#echo ""
#	mkmesg
	conv_column  
}
  
                                                                                     
# Setup the correct verion of PS in the env                                          
env_setup() {                                                                        
   MACH=`metrica_machtype`                                                           
   echo `date`" - The metrica_machtype has been determined as: $MACH" >> $LOG_FILE   
   case $MACH in                                                                     
       hpux_parisc )                                                                  
           PS_COMMAND="ps -efx" ;;                                                      
       solaris_sparc )                                                                
           if [ -f /usr/ucb/ps ];then                                                   
               PS_COMMAND="/usr/ucb/ps -augxww"                                          
           fi;;                                                                         
       * )                                                                            
           PS_COMMAND="ps -ef" ;;                                                       
   esac                                                                              
}                                                                                    

#
# Sub Program mkmesg - Display Configuration Completion Message
#

mkmesg() {

#clear
echo " "
echo "METRICA COLUMN CONVERTER"
echo " "
echo " "
echo "   You have just submitted "$THREADS" perl jobs "                  
echo "   to alter all tables with columns "$DISPLAY_COLUMN_NAME
echo "   in database "$THE_DATABASE                         
echo "   from type "$FROM_TYPE" to type "$TO_TYPE           
echo " "
echo "   Monitor their progress by checking the log files " 
echo " "
echo " 		" $LOG_FILE
echo "      		and"
echo " 		" $LOG_FILE"_thread_1 to "$THREADS
echo " "
}
 
#
# Sub Program valparms - Validate Input Parameters
#

valparms() {
if [ -z "$I1" ]; then
  	prtusage
	echo " "	
  	echo "ERROR: No arguments given"
	echo " "	
	exit
fi

#while [[ -n $I1 ]]
#do
  #case "$I1" in
    ## Help options
    #"-h"|"-help"|"--help"|"-?")
      #prtusage
    #;;

    ## Version options
    #"-v"|"-version"|"--version")
	#prtcopyrt
	#echo "SysManagement_r???"
	#exit
    #;;
#esac
#done

	if [ "$I1" = "-h" -o "$I1" = "-help" -o "$I1" = "--help" ]
	then
		clear
		prtusage
		exit
	fi

	if [ "$I1" = "-v" -o "$I1" = "-version" -o "$I1" = "--version" ]
	then
		clear
		prtcopyrt
		echo "SysManagement_r???"
		exit
	fi

	if [ "$I1" != "-database" -o "$I3" != "-columnlist" -o "$I5" != "-from_to_types" -o "$I8" != "-threads" ]
	#if [ "$I1" != "-database" -o "$I3" != "-columnlist" -o "$I5" != "-from_to_types" ]
	then
		prtusage
		echo " "	
		echo "ERROR: Invalid Arguments Passed to program"
		echo " "	
		exit
	fi
}

#
# Sub Program prtcopyrt - Print ADC Copyright
#

prtcopyrt() {

echo "##############################################################"
echo "#"
echo "#                 Metrica Installation "
echo "# "
echo "#                   COPYRIGHT NOTICE "
echo "# "
echo "#     This file remains the intellectual property of: "
echo "# "
echo "#                 Vallent Software Systems "
echo "#                    Spencer House "
echo "#                    23 Sheen Road "
echo "#                 Richmond, Surrey, UK "
echo "# "
echo "# All rights reserved. Copyright Vallent Software Systems UK, 2005"
echo "# "
echo "##############################################################"
echo " "
}
  

#
# Sub Program prtusage - Print Help         
#

prtusage() {

prtcopyrt
echo " "
echo " "
echo "  Usage: metrica_change_column_type.ksh [options]"
echo " "
echo "  options:"
echo "		[ -h | -help | --help ]"
echo "		| [ -v | -version | --version ]"
echo "		| [ -database <database_name> ]"
echo "		| [ -columnlist <filename> ]"
echo "		| [ -from_to_types <from_columntype> <to_columntype>]"
echo "		| [ -threads <number of threads (between 1 and 10> ]"
echo " "
}

 
#
# Sub Program valitem - Validate an item entered
#

valitem() {

	if [ "$ITEM" = "" ]
	then
		#clear
		prtusage
		echo ''
		echo 'ERROR : You Entered a blank parameter - Try Again'
		echo ''
		exit
	fi

}

#
# Sub Program valdb - Validate Database Name Entered
#

valdb() {

ITEM=$I2
valitem
	
echo 'select count(*) from sysdatabases where name ="'$I2'";' | dbaccess sysmaster > $LOG0        2>&1 
DB_COUNT=`cat $LOG0 | grep -v count | grep -v Database | grep -v row`

if [ "$DB_COUNT" -eq "0" ]
then
	prtusage
	echo " "
	echo "ERROR: "$I2" is not a valid database on this Informix Server"
	echo " "
	exit
else
	THE_DATABASE=$I2
fi	


}

#
# Generate Re-Create View Scripts
#
view_scr()
{

echo " "
echo "Creating Scripts to preserve view definitions"
echo " "

OS=`uname -a|cut -d" " -f1`
if [ "$OS" = "HP-UX" ] ; then
	TAIL="tail -f -n"
else
	TAIL="tail"
fi

VIEW_DROP=/tmp/metrica_view_drop$$.sql
VIEW_CREATE=/tmp/metrica_view_create$$.sql
cat /dev/null >$VIEW_DROP
cat /dev/null >$VIEW_CREATE

TAB_LIST=`echo "output to pipe 'cat ' without headings SELECT tabname FROM systables WHERE tabid>99 AND tabtype='V' AND tabname not like 'bld%' ORDER BY 1;"|dbaccess $THE_DATABASE 2>/dev/null | grep -v '^$'`

for TAB in $TAB_LIST
do
	echo "drop view $TAB;\n" >>$VIEW_DROP
	dbschema -d $THE_DATABASE -t $TAB | $TAIL +20 >>$VIEW_CREATE
done

TAB_LIST=`echo "output to pipe 'cat ' without headings SELECT tabname FROM systables WHERE tabid>99 AND tabtype='S' and tabname not like 'bld%' ORDER BY 1;"|dbaccess $THE_DATABASE 2>/dev/null | grep -v '^$'`

for TAB in $TAB_LIST
do
	echo "drop synonym $TAB;" >>$VIEW_DROP
done
dbschema -d $THE_DATABASE -s all | $TAIL +5 >>$VIEW_CREATE

}
					   

#
# Generate Re-Create View Scripts
#
view_recreate()
{

echo " "
echo "Re-creating Views"
echo " "

dbaccess $THE_DATABASE $VIEW_DROP > /dev/null 2>&1
dbaccess $THE_DATABASE $VIEW_CREATE > /dev/null 2>&1

}

#
# Sub Program valcl - Validate Column List           
#

valcl() {

ITEM=$I4
valitem

if [ ! -f $I4 ]
then
	prtusage
	echo " "
	echo "ERROR: The Column list file "$I4" does not exist"                                        
	echo " "
	exit
fi
	
DISPLAY_COLUMN_NAME=""
PASS_COLUMN_NAME=""

for COLUMN_NAME in `cat $I4`
do
	#echo 'select count(*) from syscolumns where colname ="'$COLUMN_NAME'";' | dbaccess $I2 > $LOG0        2>&1 
	#CL_COUNT=`cat $LOG0 | grep -v count | grep -v Database | grep -v row`

	#if [ "$CL_COUNT" -eq "0" ]
	#then
		#prtusage
	#	LOG "WARNING: "$COLUMN_NAME" is an invalid column name - Tables do not contain this column, techpacks may not be installed, check the contents of the column list file "$I4
        #        LOG "Column conversion will not be run"
	#exit
	#fi	

        DISPLAY_COLUMN_NAME=$DISPLAY_COLUMN_NAME"'"$COLUMN_NAME"',"
        PASS_COLUMN_NAME=$PASS_COLUMN_NAME"--column_name "$COLUMN_NAME" "
done

DISPLAY_COLUMN_NAME="("$DISPLAY_COLUMN_NAME")"

}

#
# Sub Program valtype - Validate Column Type           
#

valtype() {

ITEM=$TYPE
valitem
	

if [ "$TYPE" = "utime" -o "$TYPE" = "integer" ]
then
	OK="y"
else
	prtusage
	echo " "
	echo "ERROR: "$TYPE" is an invalid column type only utime or integer allowed"
	echo " "
	exit
fi

}

#
# Sub Program valtthreads - Validate Number of Threads     
#

valthreads() {

ITEM=$I9
valitem
	

if [ "$I9" -gt "0" -a "$I9" -lt "11" ]
then
	THREADS=$I9
else
	prtusage
	echo " "
	echo "ERROR: Invalid number of threads specified - Must be between 1 and 10"
	echo " "
	exit
fi

}


#
# Sub Program conv_column            
#

conv_column() {

perl $METRICA_DIR/Installer/bin/metrica_change_column_type_installer.pm --database=$THE_DATABASE --log_file=$LOG_FILE $PASS_COLUMN_NAME --from_type=$FROM_TYPE --to_type=$TO_TYPE --no_threads=$THREADS 

echo `date`" - Using this command to find running sub-processes: $PS_COMMAND" >> $LOG_FILE

FLAG="n"
while [ "$FLAG" = "n" ]
do
        JOB_COUNT=`$PS_COMMAND | grep "metrica_change_column_type_installer.pl" | grep -v grep | wc | awk '{print $1}'`

        if [ "$JOB_COUNT" -eq "0" ]
        then
                echo " "
                echo "Column Conversion Completed"
                echo " "
                ACTUAL_THREADS=`ls $METRICA_DIR/logs | grep "metrica_change_column_type.log_thread" | wc | awk '{print $1}'`

                if [ "$ACTUAL_THREADS" -eq 0 ]; then
                        echo "No conversion threads were required"
                else
			view_recreate
                        echo $ACTUAL_THREADS" out of "$THREADS" possible threads were required and have finished"
                        echo " "
                        CNT0="0"
                        while [ "$CNT0" -lt "$THREADS" ]
                        do
                                CNT0=`expr $CNT0 + 1`
                                if [ -f $LOG_FILE"_thread_"$CNT0 ]; then
                                        ERROR_COUNT=`grep -i error $LOG_FILE"_thread_"$CNT0 | grep -vi error_ | wc | awk '{print $1}'`
                                        echo "  "$ERROR_COUNT" messages detected in log file "$LOG_FILE"_thread_"$CNT0

                                fi
                        done
                fi
                echo " "
                FLAG="y"
        else
                sleep 10
        fi
done
	
}

#Preserve Input parameters
LOG0=/tmp/mcc$$
I1=$1
I2=$2
I3=$3
I4=$4
I5=$5
I6=$6
I7=$7
I8=$8
I9=$9
main
