#!/bin/ksh
###############################################################################
#
# @(#) SysManagement/delivery/utils/bin/DBAnumber/metrica_create_tab_a_number.ksh SysManagement_r4.79.0.9:cset.007825:4:4 11/03/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_recreate_tab_a_number.ksh
# Author:       Fran Gannon
#
# Description:  Re-create and reload data for tables that has columns that
#               contained a_numbers.
#
###############################################################################
#
# PROGRAMS AND SUBROUTINES
#
##########################################################################

#
# SUB PROGRAM  main - Main Routine
#

main() {
. metrica_install_common.ksh 
PROGNAME=`basename $0`


ROOT_DIR=$METRICA_DIR/Installer/bin
LOG_DIR=$METRICA_DIR/logs

valparms
valenv

Create_tab_load_data

}


                        

#
# Sub Program prtcopyrt - Print Vallent Copyright
#
prtcopyrt() {
echo""
echo "##############################################################"
echo "#"
echo "#                 Vallent Installation "
echo "# "
echo "#                   COPYRIGHT NOTICE "
echo "# "
echo "#     This file remains the intellectual property of: "
echo "# "
echo "#                 Vallent Software Systems "
echo "#                    Spencer House "
echo "#                    23 Sheen Road "
echo "#                 Richmond, Surrey, UK "
echo "# "
echo "# All rights reserved. Copyright Vallent Software Systems UK 2005"
echo "# "
echo "##############################################################"
echo " "
}
  

#
# Sub Program prtusage - Print Help         
#

prtusage() {

prtcopyrt
echo " "
echo " "
echo "  Usage: $PROGNAME [options]"
echo " "
echo "  options:"
echo "		[ -h | -help | --help ]"
echo "		| [ -v | -version | --version ]"
echo " "
}

# Create the tables and load the data for the tables that were dropped.
Create_tab_load_data() {

 ADMIN_DIR="/tmp/upd/$METRICA_ADMINISTRATION_DB"
 if [ -f $ADMIN_DIR/tabname$METRICA_ADMINISTRATION_DB.ixd ]
 then
   LOG_FILE="$LOG_DIR/metrica_create_a_number_$METRICA_ADMINISTRATION_DB.log"
   rm -f $LOG_FILE* > /dev/null 
   perl $ROOT_DIR/metrica_load_a_number_tab.pm --database=$METRICA_ADMINISTRATION_DB --log_file=$LOG_FILE
   if [ $? -ne "0" ]
     then
     echo "ERROR : Running script $ROOT_DIR/metrica_load_a_number_tab.pm failed , ensure that all the"
     echo "ERROR : tables that were dropped when running $ROOT_DIR/metrica_unload_a_number_tab.pm exist"
     echo "ERROR : in $METRICA_ADMINISTRATION_DB. A full list of the tables that were dropped is contained"
     echo "ERROR : in file $METRICA_DIR/Installer/upd/$METRICA_ADMINISTRATION_DB/tabname$METRICA_ADMINISTRATION_DB," 
     echo "ERROR : also all the ixd and SQL files to recreate the tables exist in this directory."
     echo "ERROR : Ensure all tables exsist and contain their data before proceeding."
     exit 2
   fi
   echo "Script $ROOT_DIR/metrica_load_a_number_tab.pm has being run successfully on $METRICA_ADMINISTRATION_DB ".
 fi
 
 PERF_DIR="/tmp/upd/$METRICA_PERFORMANCE_DB"
 if [ -f $PERF_DIR/tabname$METRICA_PERFORMANCE_DB.ixd ]
 then
   LOG_FILE="$LOG_DIR/metrica_create_a_number_$METRICA_PERFORMANCE_DB.log"
   rm -f $LOG_FILE* > /dev/null  
   perl $ROOT_DIR/metrica_load_a_number_tab.pm --database=$METRICA_PERFORMANCE_DB --log_file=$LOG_FILE
   if [ $? -ne "0" ]
     then
     echo "ERROR : Running script $ROOT_DIR/metrica_load_a_number_tab.pm failed , ensure that all the"
     echo "ERROR : tables that were dropped when running $ROOT_DIR/metrica_unload_a_number_tab.pm exist"
     echo "ERROR : in $METRICA_PERFORMANCE_DB. A full list of the tables that were dropped is contained"
     echo "ERROR : in file $METRICA_DIR/Installer/upd/$$METRICA_PERFORMANCE_DB/tabname$METRICA_PERFORMANCE_DB,"
     echo "ERROR : also all the ixd and SQL files to recreate the tables exist in this directory."
     echo "ERROR : Ensure all tables exsist and contain their data before proceeding."
   fi
   echo "Script $ROOT_DIR/metrica_load_a_number_tab.pm has being run successfully on $METRICA_PERFORMANCE_DB ". 
 fi
  
 return;                  

 }
 
# Check the input params
valparms() {                                                                                                                                              
  if [ "$I1" = "-v" -o "$I1" = "-version" -o "$I1" = "--version" ]; 
    then                                                                                                                                                    
    clear                                                                                                                                                 
    prtcopyrt                                                                                                                                             
    echo "$PROGNAME version SysManagement_r4.79.0.9:cset.007825:4:4"
    echo ""
    exit                                                                                                                                                
  fi
 
  if [ "$I1" = "-h" -o "$I1" = "-help" -o "$I1" = "--help" ]                                                                                              
    then 
    clear                                                                                                                                                 
    prtusage                                                                                                                                              
    exit                                                                                                                                                  
  fi 
}

# check the environment
valenv() {
if [ -z $METRICA_PERFORMANCE_DB ];then
   echo "\nERROR: Environment variable METRICA_PERFORMANCE_DB not set\n"
	 exit
fi
if [ -z $METRICA_ADMINISTRATION_DB ];then
   echo "\nERROR: Environment variable METRICA_ADMINISTRATION_DB not set\n"
         exit
fi
if [ -z $METRICA_DIR ];then
   echo "\nERROR: Environment variable METRICA_DIR not set\n"
         exit
fi
} 

I1=$1
LOG0=/tmp/admin$$
LOG1=/tmp/perf$$

main 
