#!/bin/ksh
###############################################################################
#
# @(#) SysManagement/delivery/utils/bin/DBAnumber/metrica_drop_tab_a_number.ksh SysManagement_r4.79.0.9:cset.007828:10:10 11/04/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_drop_tab_a_number.ksh
# Author:       Fran Gannon
#
# Description:  Get sql definitions , unload data and drop tables that
#               contain columns of type a_number.
#
###############################################################################
#
# PROGRAMS AND SUBROUTINES
#
##########################################################################

#
# SUB PROGRAM  main - Main Routine
#

main() {
. metrica_install_common.ksh 
PROGNAME=`basename $0`

ROOT_DIR=$METRICA_DIR/Installer/bin
LOG_DIR=$METRICA_DIR/logs

valparms
valenv


Get_sql_unload_data
}                        

#
# Sub Program prtcopyrt - Print Vallent Copyright
#

prtcopyrt() {
echo""
echo "##############################################################"
echo "#"
echo "#                 Vallent Installation "
echo "# "
echo "#                   COPYRIGHT NOTICE "
echo "# "
echo "#     This file remains the intellectual property of: "
echo "# "
echo "#                 Vallent Software Systems "
echo "#                    Spencer House "
echo "#                    23 Sheen Road "
echo "#                 Richmond, Surrey, UK "
echo "# "
echo "# All rights reserved. Copyright Vallent Software Systems UK 2005"
echo "# "
echo "##############################################################"
echo " "
}
  

#
# Sub Program prtusage - Print Help         
#

prtusage() {

prtcopyrt
echo " "
echo " "
echo "  Usage: $PROGNAME [options]"
echo " "
echo "  options:"
echo "		[ -h | -help | --help ]"
echo "		| [ -v | -version | --version ]"
echo " "
}

# Get SQL definitions , unload data and drop all tables contain a_number columns

Get_sql_unload_data() {

  echo "select extended_id from sysxtdtypes where name = 'a_number'" | dbaccess $METRICA_ADMINISTRATION_DB > $LOG2 2>&1
  EXTENDED_ID=`cat $LOG2 | grep -v extended_id | grep -v Database | grep -v row` 
  rm -rf $LOG2
  if [ "$EXTENDED_ID" -gt "0" ]
  then 

    echo "select count(*) from syscolumns a,systables b where a.tabid = b.tabid and extended_id = "$EXTENDED_ID"" | dbaccess $METRICA_ADMINISTRATION_DB > $LOG0 2>&1
    DB_COUNT=`cat $LOG0 | grep -v count | grep -v Database | grep -v row`                                    
    rm -rf $LOG0
    if [ "$DB_COUNT" -eq "0" ] 
    then
      echo " " 
      echo "No tables detected in $METRICA_ADMINISTRATION_DB that contain columns of type a_number"
      echo " " 
    fi
  
    if [ "$DB_COUNT" -ne "0" ] 
    then
      echo " "
      echo "Tables containing columns of type a_number have to be dropped and recreated in order"
      echo "to register the new datablade, the data will be reloaded into these tables after the"
      echo "datablade is registered successfully."
      echo " "      
      ## Set the Log File 
      mkdir -p /tmp/upd/$METRICA_ADMINISTRATION_DB
      mkdir -p $METRICA_DIR/Installer/upd/$METRICA_ADMINISTRATION_DB
      LOG_FILE="$LOG_DIR/metrica_drop_a_number_$METRICA_ADMINISTRATION_DB.log"
      rm -f $LOG_FILE* > /dev/null
      RESULT=0;
      perl $ROOT_DIR/metrica_unload_a_number_tab.pm --database=$METRICA_ADMINISTRATION_DB --log_file=$LOG_FILE  --extended_id="$EXTENDED_ID"
      RESULT=$?;
      if [ "$RESULT" -eq "0" ]
      then 
        echo "Script $ROOT_DIR/metrica_unload_a_number_tab.pm was run successfully on database $METRICA_ADMINISTRATION_DB"
      elif [ "$RESULT" -eq "5" ]
      then
        echo "ERROR : Droping a table that contained an a_number column Failed.Will now need to manually check that "
        echo "ERROR : all tables in directory /tmp/upd/$METRICA_ADMINISTRATION_DB are present in $METRICA_ADMINISTRATION_DB "
        echo "ERROR : and that they contain the data that was unloaded from them."
        exit 1
      else
        echo "ERROR : Proccessing tables that contained a_numbers Failed , check log files in $METRICA_DIR/logs."
        exit 1
      fi
    fi
  fi

  echo "select extended_id from sysxtdtypes where name = 'a_number'" | dbaccess $METRICA_PERFORMANCE_DB > $LOG2 2>&1
  EXTENDED_ID_PERF=`cat $LOG2 | grep -v extended_id | grep -v Database | grep -v row` 
  rm -rf $LOG2
  if [ "$EXTENDED_ID_PERF" -gt "0" ]
  then  
   
    echo "select count(*) from syscolumns a,systables b where a.tabid = b.tabid and extended_id = "$EXTENDED_ID_PERF"" | dbaccess $METRICA_PERFORMANCE_DB > $LOG1 2>&1
    DB_COUNT=`cat $LOG1 | grep -v count | grep -v Database | grep -v row` 
    rm -rf $LOG1
    if [ "$DB_COUNT" -eq "0" ] 
    then
      echo " "
      echo "No tables detected in $METRICA_PERFORMANCE_DB that contain columns of type a_number."
      echo " "
    fi
  
    if [ "$DB_COUNT" -ne "0" ] 
    then 
      echo " "
      echo "Tables containing columns of type a_number have to be dropped and recreated in order"
      echo "to register the new datablade, the data will be reloaded into these tables after the"
      echo "datablade is registered successfully."
      echo " "  
      ## Set the Log File 
      mkdir -p /tmp/upd/$METRICA_PERFORMANCE_DB
      mkdir -p $METRICA_DIR/Installer/upd/$METRICA_PERFORMANCE_DB
      LOG_FILE="$LOG_DIR/metrica_drop_a_number_$METRICA_PERFORMANCE_DB.log"
      rm -f $LOG_FILE* > /dev/null
      RESULT=0;
      perl $ROOT_DIR/metrica_unload_a_number_tab.pm --database=$METRICA_PERFORMANCE_DB --log_file=$LOG_FILE  --extended_id="$EXTENDED_ID_PERF"
      RESULT=$?;
      if [ "$RESULT" -eq "0" ]
      then 
        echo "Script $ROOT_DIR/metrica_unload_a_number_tab.pm was run successfully on database $METRICA_PERFORMANCE_DB"
      elif [ "$RESULT" -eq "5" ]
      then
        echo "ERROR : Droping a table that contained an a_number column Failed.Will now need to manually check that "
        echo "ERROR : all tables in directory /tmp/upd/$METRICA_PERFORMANCE_DB are present in $METRICA_PERFORMANCE_DB "
        echo "ERROR : and that they contain the data that was unloaded from them."
        exit 1
      else
        echo "ERROR : Proccessing tables that contained a_numbers Failed , check log files in $METRICA_DIR/logs."
        exit 1
      fi
    fi
    return;
  fi

 }
 
# Check the input params
valparms() {                                                                                                                                              
  if [ "$I1" = "-v" -o "$I1" = "-version" -o "$I1" = "--version" ]; 
    then                                                                                                                                                    
    clear                                                                                                                                                 
    prtcopyrt                                                                                                                                             
    echo "$PROGNAME version SysManagement_r4.79.0.9:cset.007828:10:10"
    echo ""
    exit                                                                                                                                                
  fi
 
  if [ "$I1" = "-h" -o "$I1" = "-help" -o "$I1" = "--help" ]                                                                                              
    then 
    clear                                                                                                                                                 
    prtusage                                                                                                                                              
    exit                                                                                                                                                  
  fi 
}

# check the environment
valenv() {
if [ -z $METRICA_PERFORMANCE_DB ];then
   echo "\nERROR: Environment variable METRICA_PERFORMANCE_DB not set\n"
	 exit
fi
if [ -z $METRICA_ADMINISTRATION_DB ];then
   echo "\nERROR: Environment variable METRICA_ADMINISTRATION_DB not set\n"
         exit
fi
if [ -z $METRICA_DIR ];then
   echo "\nERROR: Environment variable METRICA_DIR not set\n"
         exit
fi
}


I1=$1
LOG0=/tmp/admin$$
LOG1=/tmp/perf$$
LOG2=/tmp/extend$$
main 
