#!/bin/ksh
###############################################################################
# @(#) SysManagement/delivery/unix/bin/metrica_info.ksh SysManagement_r4.79.0.9:cset.005520:5:5 04/06/05 @(#)
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#               Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2002-2003.
#
# Title:        metrica_info.ksh
# Author:       Patrick Wood
# Description:  Script to list the installed components
#               of Performance Manager and their versions
#
###############################################################################

PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9



# Check environment variables
#
build_environment()
{
  if [[ -z "$METRICA_DIR" ]] ; then
      echo "METRICA_DIR not defined"
      exit 1
  fi
  REGFILE=$METRICA_DIR/etc/admin_registry
}



# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}



# Show correct command-line usage
#
usage()
{
show_info
cat <<END

  Usage: $PROGNAME [options]

	options:    [ -h | -help | --help ]
		  | [ -v | -version | --version ]

  When run with no options, $PROGNAME lists the 
  names and versions of all installed packages

END
}



version()
{
  if [ -f $METRICA_DIR/Installer/bin/metrica_version.ksh ]; then
    . $METRICA_DIR/Installer/bin/metrica_version.ksh
    print_version "$PROGNAME" "SysManagement_r4.79.0.9:cset.005520:5:5"
  else
    $ECHO "SysManagement_r4.79.0.9"
  fi
  exit 0
}



# Routine to print the registry entries
#
print_regentries()
{
  regfile=$1
  iters="1 2"
  for i in $iters; do
      let length=0
      for line in `cut -f$i -d" " $regfile`; do
	  (( $length<${#line} )) && length=${#line}
      done
      let len[$i-1]=$length+5
  done

  printf "%${len[0]}s    %-${len[1]}s\n" "NAME" "VERSION"

  let j=-1
  while (( j=$j+1 )); (( $j<${len[0]}+4+${len[1]} )); do
      spacer=$spacer-
  done

  echo $spacer

  while read nm ver; do
      printf "%${len[0]}s    %-${len[1]}s\n" $nm $ver
  done < $regfile
}



# List the packages
#
if (( $#==0 ))
then
  show_info
  build_environment
  if [[ ! -f $REGFILE ]] ; then
    echo "Cannot find $REGFILE"
    exit 1
  else
    echo "The following packages are installed:"
    echo
    print_regentries $REGFILE
    exit 0
  fi
fi

case $1 in
  # Help options
    "-h" | "-help" | "--help")
	usage
	exit 0
	;;

  # Version options
    "-v" | "-version" | "--version")
	version
	exit 0
	;;

  # Any other option
    *)
	usage
	exit 1
	;;
esac

exit 0
