##################################################################
# IBM Confidential
#
# OCO Source Materials
#
# 5724-N93
# Copyright IBM Corp. 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what
# been deposited with the U.S. Copyright Office.
#
##################################################################
# Created by: sbrooks - 22 Jan 2008 - 14:10:00
# Module: @(#) SysManagement/delivery/unix/bin/metrica_install_common.ksh SysManagement_r4.79.0.9:cset.008551:8:8
# Version: SysManagement_r4.79.0.9:cset.008551:8:8
# Last Changed: 08/01/23 11:33:11
# Version Control: "@(#) SysManagement_r4.79.0.9:cset.008551:8:8 08/01/23 11:33:11 @(#) SysManagement/delivery/unix/bin/metrica_install_common.ksh SysManagement_r4.79.0.9:cset.008551:8:8 @(#)"
#
# Change Activity:
# _trk_ __Date__ _PGMR_ _Description_________________________
#     1 22/01/08        Created from Vallent metrica_install_common.ksh
#     2 22/01/08  68072 Changed TMP_DIR from /tmp to $METRICA_DIR for
#                       performance improvement
###################################################################
#
# Description:
#
#   Shell script to assist in setting up databases schemas
#
###################################################################

# For more debugging set M_DEBUG=1

# This flag is to stop clients from re-sourcing this file
#
M_COMMON_DEFINED=1
export M_COMMON_DEFINED



# Common exit handler
#
m_exit()
{
  rm -f $tmp_file
  if [ "$1" = "0" ]; then
    exit 0
  fi
  $ECHO
  $ECHO -n "$PROGNAME exited with errors"
  if [ "$2" != "" ]; then
    $ECHO -n ": $2"
  fi
  if [ "$1" != "" ]; then
    $ECHO " ($1)"
  fi
  $ECHO
  exit $1
}



# Retrieves table name from an EIXD file.
# Parameters:
#   FILE
# Returns:
#   RETURN - string containing name of table
#
get_eixd_table()
{
  FILE="$1"

  # In an EIXD file, comments are denoted with #, and
  # the first uncommented line must be something like:
  #    TABLE gsm_cell_voice_traffic_raw
  # Blank lines are also valid in an EIXD file.

  # Remove comments and blank lines from EIXD file.
  grep -v ^# $FILE | grep -v ^$ > $TMP_DIR/get_eixd_table.eixd

  # Get the table name and create a Korn shell script
  # which can be sourced in order to set an environment
  # variable to the name of the table.

  # Make way for a Korn shell script in $TMP_DIR.
  tmpkshfile=$TMP_DIR/get_eixd_table.ksh
  rm -f $tmpkshfile > /dev/null 2>&1

  # As comments and blank lines have been removed,
  # the 'TABLE ' line must be first line in the EIXD file.
  $M_AWK '

  {
    if (NR == 1) {
      print "export RETURN=" tolower($2)
      exit
    }
  }

  ' $TMP_DIR/get_eixd_table.eixd > $tmpkshfile

  . $tmpkshfile
  rm -f $tmpkshfile > /dev/null 2>&1
}



# Retrieves IXD file from an EIXD file.
# Parameters:
#   FILE
# Returns:
#   RETURN - IXD file
#
eixd_to_ixd()
{
  FILE="$1"

  # In an EIXD file, comments are denoted with #, and
  # the first uncommented line must be something like:
  #    TABLE gsm_cell_voice_traffic_raw
  # Blank lines are also valid in an EIXD file.

  # Remove comments and blank lines from EIXD file.
  grep -v ^# $FILE | grep -v ^$ > $TMP_DIR/eixd_to_ixd.eixd

  # Create an IXD file containing all but the first line.

  # Make way for an IXD file in $TMP_DIR
  tmpixdfile=$TMP_DIR/eixd_to_ixd.ixd
  rm -f $tmpixdfile > /dev/null 2>&1

  $M_AWK '
  {
    if (NR == 1) {
      next
    }

    { print $0 }
  }
  ' $TMP_DIR/eixd_to_ixd.eixd > $tmpixdfile

  RETURN=$tmpixdfile
}



# Function to get the name of whatever an SQL file creates.
# File is assumed to contain one 'create' statement,
# of which the first line is 'create something name'.
#
get_creation_name()
{
  file=$1

  $M_AWK '
    /^create / {
      sub("^create +.* +(?",
    }
  '
}

# Log a warning message to console and to disk
#
WARNING()
{
   MESSAGE=$1
   $ECHO "[`$DATE`] WARNING: " $MESSAGE
   $ECHO "[`$DATE`] WARNING: " $MESSAGE >> $METRICA_DIR/etc/install.log
}

# Log a debug message to console and to disk
#
DEBUG()
{
   if [ -n "$M_DEBUG" ]; then 
      MESSAGE=$1
      $ECHO "[`$DATE`] DEBUG: " $MESSAGE
      $ECHO "[`$DATE`] DEBUG: " $MESSAGE >> $METRICA_DIR/etc/install.log
   fi
}

# Log a debug file to console and to disk
#
DEBUG_CAT()
{
   if [ -n "$M_DEBUG" ]; then 
      FILENAME=$1
      $ECHO "[`$DATE`] FILE: " $FILENAME
      cat $FILENAME

      $ECHO "[`$DATE`] FILE: " $FILENAME >> $METRICA_DIR/etc/install.log
      cat $FILENAME                      >> $METRICA_DIR/etc/install.log
   fi
}

# Log a message to console and to disk
#
LOG()
{
  MESSAGE=$1
  $ECHO "[`$DATE`]" $MESSAGE
  $ECHO "[`$DATE`]" $MESSAGE >> $METRICA_DIR/etc/install.log
}

disk_log_string()
{
  MESSAGE=$1
  $ECHO "[`$DATE`]" $MESSAGE >> $METRICA_DIR/etc/install.log
}

disk_log_file()
{
  FILE=$1
  disk_log_double_bar
  disk_log_string "$FILE"
  disk_log_single_bar
  #
  # BUG-38649
  # Connect string can contain a user's database password
  # Filter out this line when saving the SQL file to install log
  #
  cat $FILE | grep -v "CONNECT TO" >> $METRICA_DIR/etc/install.log
  disk_log_single_bar
}

disk_log_double_bar()
{
  disk_log_string "========================================"
}

disk_log_single_bar()
{
  disk_log_string "----------------------------------------"
}

# A replacement echo function that copes with the -n issue
#
echo_n()
{
  if [ "$_M_ECHO" = "" ] ; then
    _M_ECHO=echo
  fi
  if [ "$1" = "-n" ] ; then
    shift
    $_M_ECHO "$*\c";
  else
    $_M_ECHO "$*"
  fi
}



# Set mechine-specific environment variables
#
set_mach_vars()
{
  if [ "$MACHINE" = "" ] ; then
    . metrica_machtype > /dev/null
    export MACHINE
    if [ -z "$MACHINE" ] ; then
      echo "ERROR: Unable to locate script metrica_machtype."
    fi
  fi

  case $MACHINE in
    hpux_parisc )
      opts_cpio="-phu"
      opts_cp="-p" ;;
    solaris_sparc )
      ECHO=/usr/ucb/echo
      M_AWK=/usr/xpg4/bin/awk
      opts_cpio="-Lpu"
      opts_cp="-p" ;;
    linux_ia32 )
      DATE=/bin/date
      ECHO=/bin/echo
      M_AWK=/bin/awk
      opts_cpio="--quiet -Lpu"
      opts_cp="-Lp" ;;
    * )
      opts_cpio="-pu"
      opts_cp="-p" ;;
  esac

  # Set up the defaults
  #
  [[ -z "$ECHO" ]] && ECHO=/usr/bin/echo
  if [ "X`$ECHO -n`" = "X-n" ] ; then
    _M_ECHO=$ECHO
    export _M_ECHO
    ECHO=echo_n
  fi
  export ECHO

  [[ -z "$DATE" ]] && DATE=/usr/bin/date
  export DATE

  [[ -z "$M_AWK" ]] && M_AWK=/usr/bin/awk 
  if [ ! -x $M_AWK ] ; then
    m_exit 1 "$M_AWK is not executable!"
  else
    METRICA_NAWK=$M_AWK
    export M_AWK
    export METRICA_NAWK
  fi
}



# Creates an empty file that will not be overwritten.
#
get_tmp_file()
{
  SEED="$1"

  # tr is used to prevent arguments to $DATE being expanded into SCCS macros.
  TMP_FILE=$TMP_DIR/F$SEED`$DATE +"%Y %m %d %H %M %S" | tr -d ' '`
  for x in $TMP_FILE*; do
    if [ -a $x ] ; then
      TMP_FILE="$TMP_FILE."
    fi
  done

  $ECHO > $TMP_FILE

  RETURN="$TMP_FILE"
}



# Creates an empty directory that will not be overwritten.
#
get_tmp_dir()
{
  SEED="$1"

  # tr is used to prevent arguments to $DATE being expanded into SCCS macros.
  TMP_TMP_DIR=$TMP_DIR/D$SEED`$DATE +"%Y %m %d %H %M %S" | tr -d ' '`
  for x in $TMP_TMP_DIR*; do
    if [ -a $x ] ; then
      TMP_TMP_DIR="$TMP_TMP_DIR."
    fi
  done

  mkdir $TMP_TMP_DIR

  RETURN="$TMP_TMP_DIR"
}



# loadspl:
#     This procedure takes an SPL function name and tries to load it into the
#     Stored Procedure Registry. If this fails because the function already
#     exists, an update is done on the existing function.
#
loadspl()
{
    PROCNAME=$1

    LOG "----------------------------------------------------------------------------"
    LOG "Loading stored procedure $PROCNAME"
    LOG "----------------------------------------------------------------------------"
    DSNAME='serial://datasources/metrica/MetricaPerformance'
    metrica_spradmin.ksh -create -name $PROCNAME -category metalayer -datasource $DSNAME -function -file $DATA_DIR/$PROCNAME.sql 2>&1
    if [ "$?" != "0" ]; then
        LOG "\nFunction may already exist. Trying an update...\n"
        metrica_spradmin.ksh -update -name $PROCNAME -category metalayer -datasource serial://datasources/metrica/MetricaPerformance -function -file $DATA_DIR/$PROCNAME.sql 2>&1
        if [ "$?" != "0" ]; then
            LOG "\nFunction could not be loaded. Exiting..."
        else
            LOG "\nFunction updated successfully."
        fi
    else
        LOG "\nFunction created successfully."
    fi
}



# Function get_metalayer_name()
# parameters:
#   FILE - metalayer XML file
# returns:
#   RETURN - metalayer name
# Get the value of the metalayer-name attribute of
# the metalayer element in a metalayer XML file.
#
get_metalayer_name()
{
  FILE=$1

  $M_AWK -v file=$TMP_DIR/get_metalayer_name.ksh '

    /metalayer-name/ {
      sub(".*metalayer-name *= *\"", "", $0)
      sub("\".*$", "", $0)
      print "export RETURN=" $0 >> file
      exit
    }

  ' $FILE

  . $TMP_DIR/get_metalayer_name.ksh
}



# Copy all files from given source
# directory to given destination directory.
#
copy()
{
  SOURCE="$1"
  DEST="$2"

  cd $SOURCE
  FILE_LIST=`find . -type f`
  DIR_LIST=`find . -type d`
  cd $CUR_DIR

  mkdir -p $DEST/$dir

  for dir in $DIR_LIST
  do
    if [ ! -a $DEST/$dir ]; then
      mkdir -p $DEST/$dir
    fi
  done

  for file in $FILE_LIST
  do
    if [ -f $DEST/$file ]; then
      rm -f $DEST/$file
    fi

    cp $SOURCE/$file $DEST/$file
  done
}



# Remove all files in given source directory,
# that exist in given destination directory.
#
remove()
{
  SOURCE="$1"
  DEST="$2"

  cd $SOURCE
  FILE_LIST=`find . -type f`
  cd $CUR_DIR

  for file in $FILE_LIST
  do
    if [ -f $DEST/$file ]; then
      LOG "Deleting $DEST/$file..."
      rm -f $DEST/$file
    fi
  done
}



function create_connect_statement
{
  DATABASE="$1"
  USER_ID="$2"
  VALIDATION_VAR="$3"

  # If a database is provided, then
  # create a CONNECT statement.
  #
  if [ "$DATABASE" ] ; then
    CONNECT_STATEMENT="CONNECT TO '$DATABASE'"

    # If a user ID is provided,
    # then add a USER clause.
    #
    if [ "$USER_ID" ] ; then
      CONNECT_STATEMENT="$CONNECT_STATEMENT USER '$USER_ID' USING '$VALIDATION_VAR'"
    fi

    # Terminate SQL CONNECT statement with a semi-colon.
    #
    CONNECT_STATEMENT="$CONNECT_STATEMENT;"
  fi

  RETURN="$CONNECT_STATEMENT"
}



function query_string
{
  QUERY="$1"
  DATABASE="$2"
  USER_ID="$3"
  VALIDATION_VAR="$4"

  DEBUG "Entering: query_string"
  DEBUG "QUERY = $DATABASE"
  DEBUG "DATABASE = $DATABASE"
  DEBUG "USER_ID = $USER_ID"
  #DEBUG "VALIDATION_VAR = $VALIDATION_VAR"

  get_tmp_file

  # 'dbaccess <database> <filename>' does not
  # work unless <filename> has a .sql extension.
  #
  QUERY_FILE="$RETURN.sql"

  create_connect_statement $DATABASE $USER_ID $VALIDATION_VAR
  CONNECT_STATEMENT="$RETURN"

  $ECHO "$CONNECT_STATEMENT" > $QUERY_FILE
  $ECHO "$QUERY" >> $QUERY_FILE

  if [ "$M_DEBUG" = "1" ] ; then
    LOG "About to run dbaccess..."
  fi

  disk_log_file $QUERY_FILE

  dbaccess $DATABASE $QUERY_FILE >> $METRICA_DIR/etc/install.log 2>&1

  return $?
}



function query_file
{
  # TODO: File may contain many SQL statements. Run each statement separately
  # and write dbaccess output to log. This will make analysis of the running of
  # the SQL file a lot easier as each statement will be followed by its
  # dbaccess output.

  QUERY="$1"
  DATABASE="$2"
  USER_ID="$3"
  VALIDATION_VAR="$4"

  get_tmp_file

  # 'dbaccess <database> <filename>' does not
  # work unless <filename> has a .sql extension.
  #
  QUERY_FILE="$RETURN.sql"

  create_connect_statement $DATABASE $USER_ID $VALIDATION_VAR
  CONNECT_STATEMENT="$RETURN"

  $ECHO "$CONNECT_STATEMENT" > $QUERY_FILE
  cat $QUERY >> $QUERY_FILE

  disk_log_file $QUERY_FILE

  dbaccess $DATABASE $QUERY_FILE >> $METRICA_DIR/etc/install.log 2>&1

  return $?
}



# Set up some common variables
#
set_mach_vars

#TMP_DIR="/tmp/metrica_install/`$DATE +'%H %M %S' | tr -d ' '`"
#mkdir -p $TMP_DIR 2> /dev/null
if [ -z "$TMP_DIR" ]; then 
   export TMP_DIR="$METRICA_DIR/metrica_install/`$DATE +'%H %M %S' | tr -d ' '`"
   mkdir -p $TMP_DIR 2> /dev/null
fi 

