###############################################################################	
#
# @(#) SysManagement/delivery/utils/bin/DBAnumber/metrica_load_a_number_tab.pm SysManagement_r4.79.0.9:cset.007826:6:6 11/04/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#			 Vallent Software Systems UK
#				Spencer House
#			         23 Sheen Road
#		             Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:	    metrica_load_a_number_tab.pm 
# Author:	    Fran Gannon       
#
# Description:      Creates and loads data for tables that had columns of
# 		    type a_number.
#
##############################################################################

package metrica_load_a_number_tab;

use Getopt::Long;
use lib "$ENV{METRICA_DIR}/Installer/bin/lib";
use Application;

my $EXIT_RC_SUCCESS="0";
my $EXIT_RC_SYNTAX="1";
my $EXIT_RC_BAD_ENV="2";
my $EXIT_RC_OPEN_FILE="3";
my $EXIT_RC_SQL_ERR="4";

my ($database_name, $l_file);

my @results;

my $result = GetOptions(
  "database=s"       => \$database_name,              
  "log_file=s"       => \$l_file,                             
  "<>"               => \&process_unknown_arguments); 

if (!defined($database_name)) {
  exit "$EXIT_RC_SYNTAX";
}

if (!defined($l_file)) {
  exit "$EXIT_RC_SYNTAX";
}

my $WORKINGDIR="/tmp/upd/$database_name";
my $tabname_ixd="$WORKINGDIR/tabname$database_name.ixd";    

# Initialising the log file.
if ( defined($l_file) ) {
  open(LOG_FILE, ">> $l_file") || die "Could not open log file $l_file\n";
}

log_msg("Database name    : $database_name \n"); 
log_msg("Main log file    : $l_file \n");

my $INFORMIXDIR = $ENV{INFORMIXDIR};
if (!defined($INFORMIXDIR)) {
  exit "$EXIT_RC_BAD_ENV";
}


log_msg("Informix directory : $INFORMIXDIR \n");
log_msg("\n");

my $USEFUL_FNS = new Application();

create_tables();

load_data();

# Create the tables that were dropped before the datablade was registered
#

sub create_tables{

  open TABNAMES, $tabname_ixd or die "cannot open file 
                                        $tabname_ixd \n";  
  while (<TABNAMES>) {
    my ($tabname) = split (/\|/,$_,0);
    log_msg("Creating Table    : $tabname \n");
    my $filename_sql = $tabname . ".sql";

    if ( defined($filename_sql) ) {
      open(FILENAME_SQL, ">> $WORKINGDIR/$filename_sql") || 
           die "Could not open log file $WORKINGDIR/$filename_sql\n";
    }   
    @args = "\$INFORMIXDIR/bin/dbaccess $database_name < $WORKINGDIR/$filename_sql";
    if (system (@args) != 0) {
      log_msg("Error creating table $tabname from file $WORKINGDIR/$filename_ixd\n");
      print "ERROR : Error creating table $tabname from file $WORKINGDIR/$filename_sql";
      print "ERROR : Check if the table already exists in database $database_name , and that ";
      print "ERROR : if it contained any data that the data is present in the the table.";
      exit "$EXIT_RC_SQL_ERR";
    } else {
    unlink "$WORKINGDIR/$filename_sql";
    }
  }
  close TABNAMES;

}

# Load the data into the tables that were dropped.

sub load_data {

  open TABNAMES, $tabname_ixd or die "cannot open file $tabname_ixd \n";
  while (<TABNAMES>) {
    my ($tabname) = split (/\|/,$_,0);
    my $filename_ixd = $tabname . ".ixd";
    log_msg("loading data from $filename_ixd into Table $tabname \n");
    my $return_value = execute($database_name,
    "load from $WORKINGDIR/$filename_ixd
     insert into $tabname");
    if (return_value == -1) {
      log_msg("Error loading data into table $tabname1 from file $WORKINGDIR/$filename_ixd\n");
      print "ERROR : Error loading data into table $tabname from file $WORKINGDIR/$filename_ixd";
      print "ERROR : Check that the table exists in the database $database_name, check there is";
      print "ERROR : free space in the database.";
      exit "$EXIT_RC_SQL_ERR";
    } else {
      unlink "$WORKINGDIR/$filename_ixd";
    }
  }
  close TABNAMES;
  unlink $tabname_ixd;
}

sub log_msg {                              
  my $msg = shift;                        
  #printf("$msg ");                       
                                           
  my $msg_time = scalar(localtime(time)); 
  print LOG_FILE "$msg_time - $msg \n";   
}                                          

sub execute {                                                                                       
  my ($database, $sql) = @_;                                                                       
  my $command;                                                                                     
                                                                                                    
  $command = sprintf "/usr/bin/echo \"%s\" | \$INFORMIXDIR/bin/dbaccess %s 2>&1", $sql, $database; 
  my @lines = `$command`;
                                                                                                    
  my $return_value = $?>>8;
  
  if ($return_value != 0){
    log_msg("The following command failed ,$command\n");
    return -1;
  } else {
    return 0;
  }
}



 sub chomp_values {               
   my (@data) = @_;              
                                 
   return grep(!/^\s*$/, @data); 
}
