#!/bin/ksh
##########################################################################
# @(#) SysManagement/delivery/unix/bin/metrica_start_install.ksh SysManagement_r4.79.0.9:cset.005417:6:6 03/02/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                ADC Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
# 
# All rights reserved. Copyright ADC Software Systems UK, 2001-2003.
#
# Title:        metrica_start_install.ksh
# Author:       Alan Greasley (original), modified by Fergus Kelledy
#		modified further by Patrick Wood
#               (username/password capture and validation etc)
# Description:  Sets up the installation and runtime environment scripts
#
##########################################################################


# Setup Variables
#
. metrica_install_common.ksh

DIRNAME=`dirname $0`
FILE_VERSION="@(#) SysManagement_r4.79.0.9:cset.005417:6:6 03/02/05 @(#)"
MPM_PRODUCT_NAME="Metrica Performance Manager"
MSM_PRODUCT_NAME="Metrica Service Manager"
INST_SETUP_FILE=install_setup.ksh
TEMP_INST_SETUP_FILE=install_setup.tmp
TEMP_SETUP_FILE=setup.tpl # template for setup.ksh
BES_AUTH_FILE=$APPSERVER_DIR/.bes_auth
ABORT_INSTALL=0

#Properties file
INSTALL_PROP_FILE=$DIRNAME/metrica_install.properties
LOG "Properties file is $INSTALL_PROP_FILE"
INSTALL_VAR_NULL=false


# Parameter
# This is the var passed from metrica_install to indicate whether
# the database layer has been selected for install. Will be y or n
DATABASE_INSTALLATION=$1


# Defaults
#
DFLT_METRICA_INSTALL_INSTANCE=${METRICA_INSTALL_INSTANCE:-"metrica"}
DFLT_TZ=${TZ:=GB}
DFLT_ADMINISTRATION_DB=${METRICA_ADMINISTRATION_DB:-metrica_administration}
DFLT_SYSTEM_DB=${METRICA_SYSTEM_DB:-metrica_system}
DFLT_PERFORMANCE_DB=${METRICA_PERFORMANCE_DB:-metrica_performance}
DFLT_DFLT_DATABASE_DBSPACE=${METRICA_DFLT_DATABASE_DBSPACE:-dbspace1}
DFLT_DB_HOST_NAME=${DB_HOST_NAME:-`uname -n`}
DFLT_DB_PORT=${DB_PORT:-112}


# Offliner defaults
#
DFLT_DBSERV_OFFLINER_EXTRACT_DIR=${METRICA_DBSERV_OFFLINER_EXTRACT_DIR:-$METRICA_DIR/DBServices/Offliner/extract}
DFLT_DBSERV_OFFLINER_DATABASE=${METRICA_DBSERV_OFFLINER_DATABASE:-$METRICA_PERFORMANCE_DB}
DFLT_DBSERV_OFFLINER_RESTORE_DIR=${METRICA_DBSERV_OFFLINER_RESTORE_DIR:-$METRICA_DIR/DBServices/Offliner/restore}
DFLT_DBSERV_OFFLINER_LOG_DIR=${METRICA_DBSERV_OFFLINER_LOG_DIR:-$METRICA_DIR/logs}
DFLT_DBSERV_OFFLINER_LOG_FILE=${METRICA_DBSERV_OFFLINER_LOG_FILE:-offliner.log}
DFLT_DBSERV_OFFLINER_COMPRESSION=${METRICA_DBSERV_OFFLINER_COMPRESSION:-compress}
DFLT_DBSERV_OFFLINER_UNCOMPRESSION=${METRICA_DBSERV_OFFLINER_UNCOMPRESSION:-uncompress}

#Get values from properties file if they exist

#Function to try and find the correct Property for install variables
#
function get_property {
VAR_NAME=$1
FILE_NAME=$2
PROP_NAME=`awk -F"=" '
      /^'"$VAR_NAME"'[ \t]*=/ {
      print $2
      exit
      }
    ' $FILE_NAME | xargs echo`

  if [[ (-z "$PROP_NAME") && ("$INSTALL_VAR_NULL" = "false") ]]; then
    if [[ -f $METRICA_DIR/install_setup.ksh ]]; then
       INSTALL_VAR_NULL=true
       get_property $1 $METRICA_DIR/install_setup.ksh
       return
    fi
  fi
  INSTALL_VAR_NULL=false
  echo $PROP_NAME
}


#get properties using get property function
PROP_METRICA_INSTALL_INSTANCE="`get_property METRICA_INSTALL_INSTANCE $INSTALL_PROP_FILE`"
PROP_TZ="`get_property TZ $INSTALL_PROP_FILE`"
PROP_METRICA_ADMISTRATION_DB="`get_property METRICA_ADMINISTRATION_DB $INSTALL_PROP_FILE`"
PROP_METRICA_SYSTEM_DB="`get_property METRICA_SYSTEM_DB $INSTALL_PROP_FILE`"
PROP_METRICA_PERFORMANCE_DB="`get_property METRICA_PERFORMANCE_DB $INSTALL_PROP_FILE`"
PROP_METRICA_DFLT_DATABASE_DBSPACE="`get_property METRICA_DEFAULT_DBSPACE $INSTALL_PROP_FILE`"
#extra check as METRICA_DEFAULT_DBSPACE is not comtained in install setup
if [[ -z $PROP_METRICA_DFLT_DATABASE_DBSPACE ]]; then
  # we ned to check install setup for this
  PROP_METRICA_DFLT_DATABASE_DBSPACE="`get_property  METRICA_ADMINISTRATION_DBSPACE $INSTALL_PROP_FILE`"
fi
PROP_DB_HOST_NAME="`get_property DB_HOST_NAME $INSTALL_PROP_FILE`"
PROP_DB_PORT="`get_property DB_PORT $INSTALL_PROP_FILE`"
PROP_METRICA_DBSERV_OFFLINER_EXTRACT_DIR="`get_property METRICA_DBSERV_OFFLINER_EXTRACT_DIR $INSTALL_PROP_FILE`"
PROP_METRICA_DBSERV_OFFLINER_DATABASE="`get_property METRICA_DBSERV_OFFLINER_DATABASE $INSTALL_PROP_FILE`"
PROP_METRICA_DBSERV_OFFLINER_COMPRESSION="`get_property METRICA_DBSERV_OFFLINER_COMPRESSION $INSTALL_PROP_FILE`"
PROP_METRICA_DBSERV_OFFLINER_UNCOMPRESSION="`get_property METRICA_DBSERV_OFFLINER_UNCOMPRESSION $INSTALL_PROP_FILE`"
PROP_DBSERV_OFFLINER_RESTORE_DIR="`get_property METRICA_DBSERV_OFFLINER_RESTORE_DIR $INSTALL_PROP_FILE`"

#Special case. Offliner path
# Need to check prop file first. If not here will take DIR and LOG together
# from install_setup.ksh file

PROP_DBSERV_OFFLINER_LOG_FILE=`awk '/METRICA_DBSERV_OFFLINER_LOG/' $INSTALL_PROP_FILE | awk -F= '{print $2}' | awk '{print $1}'`

PROP_DBSERV_OFFLINER_LOG_DIR=`awk '/METRICA_DBSERV_OFFLINER_DIR/' $INSTALL_PROP_FILE | awk -F= '{print $2}' | awk '{print $1}'`

# Remove the temporary installation setup file 
#
abort() {
  rm -f $METRICA_DIR/$TEMP_INST_SETUP_FILE 2> /dev/null
  # Make sure terminal echoing is on, in case script has been
  # interrupted during get_metrica_password
  stty echo
  ABORT_INSTALL=1
  exit 1
}



# Move the temporary installation setup file to the final version
#
commit() {
  [[ $ABORT_INSTALL -ne 1 ]] && mv $METRICA_DIR/$TEMP_INST_SETUP_FILE $METRICA_DIR/$INST_SETUP_FILE 2> /dev/null
}



# Trap a few signals
#
trap commit 0
trap abort 1 2 3 15



##########################################################################
# Setup Functions
#
write_header() {
    FILE=$1
    TITLE=$2
    DESC=$3
    cat > $METRICA_DIR/$FILE <<!
#!/bin/ksh
#
# $FILE_VERSION
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                ADC Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
# 
# All rights reserved. Copyright ADC Software Systems UK, 2001.
#
# Title:        $TITLE
# Author:       Alan Greasley/Patrick Wood
# Description:  $DESC
#
##########################################################################
# 

!
}



append()
{
  TEXT=$1 && shift
  while [ -n "$1" ]; do
    $ECHO "$TEXT" >> $METRICA_DIR/$1
    shift
  done
}



# Function to get appserver management username and password
#
get_appserver_credentials() {

  $ECHO -n "Enter Appserver Management Username (<Enter> for default - \"$DFLT_APPSERVER_MGMT_USER\")> "
  read APPSERVER_MGMT_USER
  [ -z "$APPSERVER_MGMT_USER" ] && APPSERVER_MGMT_USER=$DFLT_APPSERVER_MGMT_USER

  RESULT=1
  export PASSWORD_ENTRY=
  while [[ $RESULT -ne 0 ]]; do
      get_password "Appserver Management"
      RESULT=$?
  done
  APPSERVER_MGMT_PASSWORD=$PASSWORD_ENTRY

  [[ -f $BES_AUTH_FILE ]] && $ECHO "Appserver auth file exists - replacing ..." && rm -f $BES_AUTH_FILE
  $ECHO "2" > $BES_AUTH_FILE
  $ECHO "$APPSERVER_MGMT_USER" >> $BES_AUTH_FILE
  $ECHO "$APPSERVER_MGMT_PASSWORD" >> $BES_AUTH_FILE
}


	
##########################################################################
# Setup

[[ -z $METRICA_DIR ]] && $ECHO "METRICA_DIR undefined" && exit 1

[ ! -d "$METRICA_DIR/utils" ] && mkdir -p $METRICA_DIR/utils
[ ! -d "$METRICA_DIR/utils/bin" ] && mkdir -p $METRICA_DIR/utils/bin
[ ! -d "$METRICA_DIR/logs" ] && mkdir -p $METRICA_DIR/logs
[ ! -d "$METRICA_DIR/ext/lib" ] && mkdir -p $METRICA_DIR/ext/lib
[ ! -d "$METRICA_DIR/etc" ] && mkdir -p $METRICA_DIR/etc
[ ! -d "$METRICA_DIR/etc/start" ] && mkdir -p $METRICA_DIR/etc/start
[ ! -d "$METRICA_DIR/etc/stop" ] && mkdir -p $METRICA_DIR/etc/stop



# Execute third-party product set-up scripts.
#
. $METRICA_DIR/setup_appserver.ksh
. $METRICA_DIR/setup_informix.ksh
. $METRICA_DIR/setup_java.ksh
. $METRICA_DIR/setup_openfusion.ksh

if [[ -f $METRICA_DIR/$TEMP_INST_SETUP_FILE ]]
then
    $ECHO "" > $METRICA_DIR/$TEMP_INST_SETUP_FILE
else
    touch $METRICA_DIR/$TEMP_INST_SETUP_FILE
fi

chmod 700 $METRICA_DIR/$TEMP_INST_SETUP_FILE

if [[ -f $METRICA_DIR/$TEMP_SETUP_FILE ]]
then
    $ECHO "" > $METRICA_DIR/$TEMP_SETUP_FILE
fi



# Setup the file headers
#
write_header $TEMP_INST_SETUP_FILE $INST_SETUP_FILE "Sets the installation environment"

append "export METRICA_DIR=$METRICA_DIR" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE



# Set $TEMP_INST_SETUP_FILE and $INST_SETUP_FILE
# to execute third-party product set-up scripts.
#
append ". $METRICA_DIR/setup_appserver.ksh" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE
append ". $METRICA_DIR/setup_informix.ksh" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE
append ". $METRICA_DIR/setup_java.ksh" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE
append ". $METRICA_DIR/setup_openfusion.ksh" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

append "set -a" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE



# Ask the installer for all the required environment values

#Install Instance
if [[ -n "$PROP_METRICA_INSTALL_INSTANCE" ]]; then
   LOG  "METRICA_INSTALL_INSTANCE from properties file is $PROP_METRICA_INSTALL_INSTANCE"
   METRICA_INSTALL_INSTANCE=$PROP_METRICA_INSTALL_INSTANCE
else
   $ECHO -n "Enter name of Install Instance (<Enter> for default - \"$DFLT_METRICA_INSTALL_INSTANCE\")> "
   read METRICA_INSTALL_INSTANCE
   [ -z "$METRICA_INSTALL_INSTANCE" ] && METRICA_INSTALL_INSTANCE=$DFLT_METRICA_INSTALL_INSTANCE
fi
append "METRICA_INSTALL_INSTANCE=$METRICA_INSTALL_INSTANCE" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE


#Set TIME ZONE
if [[ -n "$PROP_TZ" ]]; then
   LOG  "TZ from properties file is $PROP_TZ"
   TZ=$PROP_TZ
else
  $ECHO -n "Enter timezone (<Enter> for default - \"$DFLT_TZ\")> "
  read TZ
  [ -z "$TZ" ] && TZ=$DFLT_TZ
fi
append "TZ=$TZ" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

# Set ADMIN DB
if [[ -n "$PROP_METRICA_ADMISTRATION_DB" ]]; then
   LOG  "METRICA_ADMISTRATION_DB from properties file is $PROP_METRICA_ADMISTRATION_DB"
   METRICA_ADMINISTRATION_DB=$PROP_METRICA_ADMISTRATION_DB
else
  $ECHO -n "Enter Administration DB Name (<Enter> for default - \"$DFLT_ADMINISTRATION_DB\")> "
  read METRICA_ADMINISTRATION_DB
  [ -z "$METRICA_ADMINISTRATION_DB" ] && METRICA_ADMINISTRATION_DB=$DFLT_ADMINISTRATION_DB
fi
append "METRICA_ADMINISTRATION_DB=$METRICA_ADMINISTRATION_DB" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

#Set System DB
if [[ -n "$PROP_METRICA_SYSTEM_DB" ]]; then
   LOG  "METRICA_SYSTEM_DB from properties file is $PROP_METRICA_SYSTEM_DB"
   METRICA_SYSTEM_DB=$PROP_METRICA_SYSTEM_DB
else
  $ECHO -n "Enter System DB Name (<Enter> for default - \"$DFLT_SYSTEM_DB\")> "
  read METRICA_SYSTEM_DB
  [ -z "$METRICA_SYSTEM_DB" ] && METRICA_SYSTEM_DB=$DFLT_SYSTEM_DB
fi
append "METRICA_SYSTEM_DB=$METRICA_SYSTEM_DB" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

#Set Performance DB
if [[ -n "$PROP_METRICA_PERFORMANCE_DB" ]]; then
   LOG  "METRICA_PERFORMANCE_DB from properties file is $PROP_METRICA_PERFORMANCE_DB"
   METRICA_PERFORMANCE_DB=$PROP_METRICA_PERFORMANCE_DB
else
   $ECHO -n "Enter Performance DB Name (<Enter> for default - \"$DFLT_PERFORMANCE_DB\")> "
   read METRICA_PERFORMANCE_DB
   [ -z "$METRICA_PERFORMANCE_DB" ] && METRICA_PERFORMANCE_DB=$DFLT_PERFORMANCE_DB
fi
append "METRICA_PERFORMANCE_DB=$METRICA_PERFORMANCE_DB" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

#Set Default DBSpacae Name
if [[ -n "$PROP_METRICA_DFLT_DATABASE_DBSPACE" ]]; then
   LOG  "METRICA_DFLT_DATABASE_DBSPACE from properties file is $PROP_METRICA_DFLT_DATABASE_DBSPACE"
   METRICA_DFLT_DATABASE_DBSPACE=$PROP_METRICA_DFLT_DATABASE_DBSPACE
else
   $ECHO -n "Enter Default Database DBSpace Name (<Enter> for default - \"$DFLT_DFLT_DATABASE_DBSPACE\")> "
   read METRICA_DFLT_DATABASE_DBSPACE
   [ -z "$METRICA_DFLT_DATABASE_DBSPACE" ] && METRICA_DFLT_DATABASE_DBSPACE=$DFLT_DFLT_DATABASE_DBSPACE
fi

[ -z "$METRICA_ADMINISTRATION_DBSPACE" ] && METRICA_ADMINISTRATION_DBSPACE=$METRICA_DFLT_DATABASE_DBSPACE
[ -z "$METRICA_SYSTEM_DBSPACE" ] && METRICA_SYSTEM_DBSPACE=$METRICA_DFLT_DATABASE_DBSPACE
[ -z "$METRICA_PERFORMANCE_DBSPACE" ] && METRICA_PERFORMANCE_DBSPACE=$METRICA_DFLT_DATABASE_DBSPACE

append "METRICA_ADMINISTRATION_DBSPACE=$METRICA_ADMINISTRATION_DBSPACE" $TEMP_INST_SETUP_FILE
append "METRICA_SYSTEM_DBSPACE=$METRICA_SYSTEM_DBSPACE" $TEMP_INST_SETUP_FILE
append "METRICA_PERFORMANCE_DBSPACE=$METRICA_PERFORMANCE_DBSPACE" $TEMP_INST_SETUP_FILE


append "DB_SERVER_NAME=\$INFORMIXSERVER" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

#Set DB Host
if [[ -n "$PROP_DB_HOST_NAME" ]]; then
   LOG  "DB_HOST_NAME from properties file is $PROP_DB_HOST_NAME"
   DB_HOST_NAME=$PROP_DB_HOST_NAME
else
   $ECHO -n "Enter DB Host Name (<Enter> for default - \"$DFLT_DB_HOST_NAME\")> "
   read DB_HOST_NAME
   [ -z "$DB_HOST_NAME" ] && DB_HOST_NAME=$DFLT_DB_HOST_NAME
fi
append "DB_HOST_NAME=$DB_HOST_NAME" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

#Set DB Port
if [[ -n "$PROP_DB_PORT" ]]; then
   LOG  "DB_PORT from properties file is $PROP_DB_PORT"
   DB_PORT=$PROP_DB_PORT
else
   $ECHO -n "Enter DB Port Number (<Enter> for default - \"$DFLT_DB_PORT\")> "
   read DB_PORT
   [ -z "$DB_PORT" ] && DB_PORT=$DFLT_DB_PORT
fi
append "DB_PORT=$DB_PORT" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

#Set Offliner extract dir
if [[ -n "$PROP_METRICA_DBSERV_OFFLINER_EXTRACT_DIR" ]]; then
   LOG  "METRICA_DBSERV_OFFLINER_EXTRACT_DIR from properties file is $PROP_METRICA_DBSERV_OFFLINER_EXTRACT_DIR"
   METRICA_DBSERV_OFFLINER_EXTRACT_DIR=$PROP_METRICA_DBSERV_OFFLINER_EXTRACT_DIR
else
   $ECHO -n "Enter directory path to extract/offline to Server Name (<Enter> for default - \"$DFLT_DBSERV_OFFLINER_EXTRACT_DIR\")> "
   read METRICA_DBSERV_OFFLINER_EXTRACT_DIR
   [ -z "$METRICA_DBSERV_OFFLINER_EXTRACT_DIR" ] && METRICA_DBSERV_OFFLINER_EXTRACT_DIR=$DFLT_DBSERV_OFFLINER_EXTRACT_DIR
fi
append "METRICA_DBSERV_OFFLINER_EXTRACT_DIR=$METRICA_DBSERV_OFFLINER_EXTRACT_DIR" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE


# Set DB name for offliner
if [[ -n "$PROP_METRICA_DBSERV_OFFLINER_DATABASE" ]]; then
   LOG  "METRICA_DBSERV_OFFLINER_DATABASE from properties file is $PROP_METRICA_DBSERV_OFFLINER_DATABASE"
   METRICA_DBSERV_OFFLINER_DATABASE=$PROP_METRICA_DBSERV_OFFLINER_DATABASE
else
   if [[ -n "$DFLT_DBSERV_OFFLINER_DATABASE" ]]
   then
	$ECHO -n "Enter DB Name to offline from/to (<Enter> for default - \"$DFLT_DBSERV_OFFLINER_DATABASE\")> "
else
	$ECHO -n "Enter DB Name to offline from/to (<Enter> for default - \"$METRICA_PERFORMANCE_DB\")> "
fi
read METRICA_DBSERV_OFFLINER_DATABASE
if [[ -z "$METRICA_DBSERV_OFFLINER_DATABASE" ]]; then
  if [[ -z "$DFLT_DBSERV_OFFLINER_DATABASE" ]]; then
    METRICA_DBSERV_OFFLINER_DATABASE=$METRICA_PERFORMANCE_DB
  else
    METRICA_DBSERV_OFFLINER_DATABASE=$DFLT_DBSERV_OFFLINER_DATABASE
  fi
fi
fi
append "METRICA_DBSERV_OFFLINER_DATABASE=$METRICA_DBSERV_OFFLINER_DATABASE" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

#Set Offliner Restore Directory path

if [[ -n "$PROP_DBSERV_OFFLINER_RESTORE_DIR" ]]; then
   LOG  "METRICA_DBSERV_OFFLINER_RESTORE_DIR from properties file is $PROP_DBSERV_OFFLINER_RESTORE_DIR"
   METRICA_DBSERV_OFFLINER_RESTORE_DIR=$PROP_DBSERV_OFFLINER_RESTORE_DIR
else
   $ECHO -n "Enter the Offliner Restore directory path (<Enter> for default - \"$DFLT_DBSERV_OFFLINER_RESTORE_DIR\")> "
   read METRICA_DBSERV_OFFLINER_RESTORE_DIR
   [ -z "$METRICA_DBSERV_OFFLINER_RESTORE_DIR" ] && METRICA_DBSERV_OFFLINER_RESTORE_DIR=$DFLT_DBSERV_OFFLINER_RESTORE_DIR
fi
append "METRICA_DBSERV_OFFLINER_RESTORE_DIR=$METRICA_DBSERV_OFFLINER_RESTORE_DIR" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

#Set Compression tool  
if [[ -n "$PROP_METRICA_DBSERV_OFFLINER_COMPRESSION" ]]; then
   LOG  "METRICA_DBSERV_OFFLINER_COMPRESSION from properties file is $PROP_METRICA_DBSERV_OFFLINER_COMPRESSION"
   METRICA_DBSERV_OFFLINER_COMPRESSION=$PROP_METRICA_DBSERV_OFFLINER_COMPRESSION
else
   $ECHO -n "Enter the offliner compression tool name (<Enter> for default - \"$DFLT_DBSERV_OFFLINER_COMPRESSION\")> "
   read METRICA_DBSERV_OFFLINER_COMPRESSION
   [ -z "$METRICA_DBSERV_OFFLINER_COMPRESSION" ] && METRICA_DBSERV_OFFLINER_COMPRESSION=$DFLT_DBSERV_OFFLINER_COMPRESSION
fi
append "METRICA_DBSERV_OFFLINER_COMPRESSION=$METRICA_DBSERV_OFFLINER_COMPRESSION" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

#Set uncompression tool
if [[ -n "$PROP_METRICA_DBSERV_OFFLINER_UNCOMPRESSION" ]]; then
   LOG  "METRICA_DBSERV_OFFLINER_UNCOMPRESSION from properties file is $PROP_METRICA_DBSERV_OFFLINER_UNCOMPRESSION"
   METRICA_DBSERV_OFFLINER_UNCOMPRESSION=$PROP_METRICA_DBSERV_OFFLINER_UNCOMPRESSION
else
   $ECHO -n "Enter the offliner uncompression tool name  (<Enter> for default - \"$DFLT_DBSERV_OFFLINER_UNCOMPRESSION\")> "
   read METRICA_DBSERV_OFFLINER_UNCOMPRESSION
   [ -z "$METRICA_DBSERV_OFFLINER_UNCOMPRESSION" ] && METRICA_DBSERV_OFFLINER_UNCOMPRESSION=$DFLT_DBSERV_OFFLINER_UNCOMPRESSION
fi
append "METRICA_DBSERV_OFFLINER_UNCOMPRESSION=$METRICA_DBSERV_OFFLINER_UNCOMPRESSION" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE

#set Offliner log dir
if [[ -n "$PROP_DBSERV_OFFLINER_LOG_DIR" ]]; then
   LOG  "DBSERV_OFFLINER_LOG_DIR from properties file is $PROP_DBSERV_OFFLINER_LOG_DIR"
   DBSERV_OFFLINER_LOG_DIR=$PROP_DBSERV_OFFLINER_LOG_DIR
else
   if [[ ! -f $METRICA_DIR/$INST_SETUP_FILE ]]; then
      $ECHO -n "Enter the Offliner log directory path (<Enter> for default - \"$DFLT_DBSERV_OFFLINER_LOG_DIR\")> "
      read DBSERV_OFFLINER_LOG_DIR
      [ -z "$DBSERV_OFFLINER_LOG_DIR" ] && DBSERV_OFFLINER_LOG_DIR=$DFLT_DBSERV_OFFLINER_LOG_DIR
    fi
fi

#Set Offliner fileName
if [[ -n "$PROP_DBSERV_OFFLINER_LOG_FILE" ]]; then
   LOG  "DBSERV_OFFLINER_LOG_FILE from properties file is $PROP_DBSERV_OFFLINER_LOG_FILE"
   DBSERV_OFFLINER_LOG_FILE=$PROP_DBSERV_OFFLINER_LOG_FILE
else
   if [[ ! -f $METRICA_DIR/$INST_SETUP_FILE ]]; then
      $ECHO -n "Enter the Offliner log file name (<Enter> for default - \"$DFLT_DBSERV_OFFLINER_LOG_FILE\")> "
      read DBSERV_OFFLINER_LOG_FILE
      [ -z "$DBSERV_OFFLINER_LOG_FILE" ] && DBSERV_OFFLINER_LOG_FILE=$DFLT_DBSERV_OFFLINER_LOG_FILE
   fi
fi


if [[ (-z "$DBSERV_OFFLINER_LOG_DIR") || (-z "$DBSERV_OFFLINER_LOG_FILE") ]]; then
if [[ -f $METRICA_DIR/$INST_SETUP_FILE ]]; then
#If either of them are blank in properties file we get the whole thing from install.ksh if possible
METRICA_DBSERV_OFFLINER_LOG="`get_property METRICA_DBSERV_OFFLINER_LOG $METRICA_DIR/$INST_SETUP_FILE`"
append "METRICA_DBSERV_OFFLINER_LOG=$METRICA_DBSERV_OFFLINER_LOG" $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE
fi
else
append "METRICA_DBSERV_OFFLINER_LOG=$DBSERV_OFFLINER_LOG_DIR/$DBSERV_OFFLINER_LOG_FILE " $TEMP_INST_SETUP_FILE $TEMP_SETUP_FILE
fi


# Copy metrica_machtype into installation dir
#
if [[ -f $DIRNAME/metrica_machtype && ! -f $METRICA_DIR/utils/bin/metrica_machtype ]]
then
  cp $DIRNAME/metrica_machtype $METRICA_DIR/utils/bin
  chmod 755 $METRICA_DIR/utils/bin/metrica_machtype
fi

if [[ -f $DIRNAME/metrica_machtype ]]
then
  MACHINE=`$DIRNAME/metrica_machtype`
elif [[ -f $METRICA_DIR/utils/bin/metrica_machtype ]]
then
  MACHINE=`$METRICA_DIR/utils/bin/metrica_machtype`
fi



# Set up temporary install setup file
#
append "SPM_NAWK=$METRICA_NAWK" $TEMP_INST_SETUP_FILE
append "METRICA_NAWK=$METRICA_NAWK" $TEMP_INST_SETUP_FILE
append "set +a" $TEMP_INST_SETUP_FILE
