###############################################################################	
#
# @(#) SysManagement/delivery/utils/bin/DBAnumber/metrica_unload_a_number_tab.pm SysManagement_r4.79.0.9:cset.007830:10:10 11/04/05 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#			 Vallent Software Systems UK
#				Spencer House
#			         23 Sheen Road
#		               Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:	    metrica_unload_a_number_tab.pm 
# Author:	    Fran Gannon       
#
# Description:      Selects tables with columns of type a_number, then 
#                   unloads the data from the tables and re-creates them.
#
##############################################################################

package metrica_unload_a_number_tab;


use Getopt::Long;
use lib "$ENV{METRICA_DIR}/Installer/bin/lib";
use Application;
use File::Basename; 
use File::Copy;

my $EXIT_RC_SUCCESS="0";
my $EXIT_RC_SYNTAX="1";
my $EXIT_RC_BAD_ENV="2";
my $EXIT_RC_OPEN_FILE="3";
my $EXIT_RC_SQL_ERR="4";
my $EXIT_RC_DROP_TAB="5";

my ($database_name, $l_file);

my $result = GetOptions(                                                   
    "database=s"       => \$database_name,              
    "log_file=s"       => \$l_file,
    "extended_id=s"    => \$a_number_extended_id,
    "<>"               => \&process_unknown_arguments);
    
my $WORKINGDIR="/tmp/upd/$database_name";
my $tabname_ixd="$WORKINGDIR/tabname$database_name.ixd";
my $temp_table="a_number_temp_table"; 

my @results;


# Initialising the log file.
if ( defined($l_file) ) {
  open(LOG_FILE, ">> $l_file") || exit "$EXIT_RC_OPEN_FILE";
}

if (!defined($database_name)) {
  exit "$EXIT_RC_SYNTAX";
}

if (!defined($a_number_extended_id)) {
  exit "$EXIT_RC_SYNTAX";
}

log_msg("Database name    : $database_name \n"); 
log_msg("Main log file    : $l_file \n");

my $INFORMIXDIR = $ENV{INFORMIXDIR};
if (!defined($INFORMIXDIR)) {
  exit "$EXIT_RC_BAD_ENV";
}

log_msg("Informix directory : $INFORMIXDIR \n");
log_msg("\n");

my $METRICA_DIR = $ENV{METRICA_DIR};
if (!defined($METRICA_DIR)) {
  exit "$EXIT_RC_BAD_ENV";
}

my $BACKUP_DIR="$METRICA_DIR/Installer/upd/$database_name";

my $USEFUL_FNS = new Application();

create_temp_table();

add_table_names();

unload_table_names();

get_sql_for_tables();

unload_data_from_tabs();

copy_files();

drop_tables();

# create temp table to get all the tables that contain columns of type 
# a_number 
#

sub create_temp_table{

  if ($USEFUL_FNS->tableExists($database_name , $temp_table)) {
    log_msg("Table $temp_table allready exists in $database_name will delete all data from this table before proceeding");
    my $return_value = execute($database_name,
    "delete from $temp_table");
    if ($return_value == -1) {
      print "ERROR : Cannot delete from table $temp_table , exiting upgrade, check log file $l_file\n";
      exit "$EXIT_RC_SQL_ERR";
    }
  } else {
    my $ret_value = execute($database_name, 
    "create table $temp_table (tabid integer,tabname varchar(250))");
     if ($ret_value == -1) {
       print "ERROR : Cannot create table $temp_table , exiting upgrade, check log file $l_file\n";
       exit "$EXIT_RC_SQL_ERR";     
     }
  }
   
} 

# Insert table names into that contain a_numbers into temp table.
#
sub add_table_names{ 
   
  my $return_value = execute($database_name, 
  "insert into $temp_table
   select a.tabid, b.tabname from syscolumns a,systables  b where 
   a.tabid = b.tabid and extended_id = $a_number_extended_id");
    
   if ($return_value == -1) {
     log_msg("Error insert data into table $temp_table , exiting upgrade \n"); 
     print "ERROR : Exiting upgrade , error installing data into $temp_table \n";
     exit "$EXIT_RC_SQL_ERR";
   }    
    
} 

# Unload temp distict table names to ixd file.
#
sub unload_table_names{

  my $return_value = execute($database_name,
  "unload to $tabname_ixd
   select distinct(tabname) from $temp_table");
     
     
  if ($return_value == -1) {
    log_msg("Exiting upgrade, Error unloading from table $temp_table \n");
    unlink $tabname_ixd;
    print "ERROR : Exiting upgrade , Error unloading from table $temp_table check disk space\n";
    exit "$EXIT_RC_SQL_ERR";
  }
     
}


# Since the tables are going to be dropped we will need the sql
# to re-create these tables.
#
sub get_sql_for_tables{

  my $start="DBSCHEMA";
  my $end="create table";
  open TABNAMES, $tabname_ixd or die "cannot open file $tabname_ixd \n";
  while (<TABNAMES>) {
    my ($tabname) = split (/\|/,$_,0);
    log_msg("Processing Table    : $tabname \n");
    my $filename_sql = $tabname . ".sql";
  
    my $arg_list = "$INFORMIXDIR/bin/dbschema -d $database_name -t $tabname -ss >> $WORKINGDIR/$filename_sql" ;      
      
    my $rc = 0xffff & system $arg_list;
    # If an error occurs exit and clean up files created
    if ($rc){
      close TABNAMES;
      delete_files();
      log_msg("Could not get SQL definition for table : $tabname \n");
      print("ERROR : Command failed due to : $! \n");
      print "ERROR : Could not get SQL definition for table $tabname , exiting upgrade \n";
      exit "$EXIT_RC_SQL_ERR";
    }
    my $output = delete_section("$WORKINGDIR/$filename_sql",$start,$end);
    open(FILENAME_SQL, "> $WORKINGDIR/$filename_sql") || 
                die "Could not open log file $l_file\n";
    print FILENAME_SQL $output;
    close FILENAME_SQL;
  }
  close TABNAMES;

}

# Unload the data from the tables that are going to be dropped.
#
sub unload_data_from_tabs{

  open TABNAMES, $tabname_ixd or die "cannot open file 
                                          $tabname_ixd \n";
  while (<TABNAMES>) {
    my ($tabname) = split (/\|/,$_,0);
    log_msg("Unloading data from Table    : $tabname \n");
    my $filename_ixd = $tabname . ".ixd";
    my $return_value = execute($database_name,
    "unload to $WORKINGDIR/$filename_ixd
     select * from $tabname");
     # If an error occurs we need to exit and delete files created.
     if ($return_value == -1) {
       close TABNAMES;
       delete_files();
       log_msg("Exiting upgrade, Error unloading from table $tabname \n");   
       print "ERROR : Exiting upgrade , Error unloading from table $tabname,check disk space\n";
       exit "$EXIT_RC_SQL_ERR";     
    }       
  }
  close TABNAMES;
}

sub copy_files{
  
  my $file;
  my $filex;
  
  &Get_LS;
  
  chdir "$WORKINGDIR";
  
  foreach $file (@ls){
    my $filecopyto="$BACKUP_DIR/$file";
    copy($file, $filecopyto);   
  } 
}

# drop the tables
sub drop_tables{

  my $cnt = 0;
  my $count = 0;

  open TABNAMES, $tabname_ixd or die "cannot open file 
                                        $tabname_ixd \n";
  while (<TABNAMES>) {
    my ($tabname) = split (/\|/,$_,0);
    log_msg("droping Table    : $tabname \n");
    my $return_value = execute($database_name,
    "drop table $tabname");
    if ($return_value == -1) {
      close TABNAMES;
      open ERR_TABNAMES, $tabname_ixd or die "cannot open file 
                                        $tabname_ixd \n";
      while (<ERR_TABNAMES>) {
        if ($cnt < $count) {
          my ($tabname1) = split(/\|/,$_,0);
          my $filename_sql1 = $tabname1 . ".sql";
          my $filename_ixd1 = $tabname1 . ".ixd";
          @args = "\$INFORMIXDIR/bin/dbaccess $database_name < $WORKINGDIR/$filename_sql1";                                                                            
          if (system (@args) != 0) {
            print "ERROR : Running script $WORKINGDIR/$filename_sql  Failed\n";
            print "ERROR : Ensure that $tabname exists in $database_name before proceeding\n";
            print "ERROR : Also ensure that all tables that have sql files in $WORKINGDIR exist\n";
            print "ERROR : in $database_name , and that they contain the data that is in there \n";
            print "ERROR : tabname.ixd file\n";
          }
          log_msg("Created table $tabname1 from sql file $WORKINGDIR/$filename_sql1\n");
          my $return_value = execute($database_name,
          "load from $WORKINGDIR/$filename_ixd1
          insert into $tabname1");
          log_msg("Loaded data from ixd file $WORKINGDIR/$filename_ixd1, into table $tabname1\n");
          if (return_value == -1) {
            log_msg("Error loading data into table $tabname from file $WORKINGDIR/$filename_ixd\n");
            print "ERROR : Loading data into table $tabname from file $WORKINGDIR/$filename_ixd\n";
            print "ERROR : Ensure $tabname exists in $database_name and that it contains the data that was unloaded\n";
            exit "$EXIT_RC_DROP_TAB"; 
          }
          $cnt = $cnt + 1;
        }
     }
     close ERR_TABNAMES;                                                                                         
     log_msg("Exiting upgrade, Error droping table $tabname \n");   
     print "ERROR : Exiting upgrade, Error droping table $tabname \n";
     print "ERROR : Ensure that only the database and appserver are running \n";
     exit "$EXIT_RC_DROP_TAB";
   }      
   $count = $count + 1;    
  }
  close TABNAMES;
}


sub log_msg {                              
  my $msg = shift;                        
  #printf("$msg ");                       
                                           
  my $msg_time = scalar(localtime(time)); 
  print LOG_FILE "$msg_time - $msg \n";   
}                                          

sub execute {                                                                                       
  my ($database, $sql) = @_;                                                                       
  my $command;                                                                                     
                                                                                                    
  $command = sprintf "/usr/bin/echo \"%s\" | \$INFORMIXDIR/bin/dbaccess %s 2>&1", $sql, $database; 
  my @lines = `$command`;
                                                                                                    
  my $return_value = $?>>8;
   
  if ($return_value != 0){
    log_msg("The following command failed ,$command\n");
    return -1;
  } else {
    return 0;
  }
}

 sub chomp_values {               
  my (@data) = @_;              
                                 
  return grep(!/^\s*$/, @data); 
}


sub delete_section
{
   my $file=$_[0];
   my $start=$_[1];
   my $end=$_[2];
        
   my $copy = slurp_file($file);        
   @result = split ( /$start(.|\n)*?($end)/, $copy );
   return "$result[2] $result[3]";
}


sub slurp_file
{
  my $infile=$_[0];
  open(SLURP, "<$infile") or return '';
  my $line = '';
  my $all = '';
  while($line = <SLURP>) { $all .= $line; }
    close(SLURP);
    return $all || '';
}


sub delete_files{
  
  my $file;
  
  &Get_LS;
  
  chdir "$WORKINGDIR";
  
  foreach $file (@ls){
    unlink basename($file);
  }
  
}


sub Get_LS {

# Get a list of full path names into array @ls

  chdir "$WORKINGDIR";
  $DIR=$WORKINGDIR;

  local(@localls)=`ls -R1`;

  local($item,$DIR);

  foreach $item (@localls) {
  #  print "$item\n";
    if ($item =~ /:$/) { 
      $DIR=$item; 
      chop($DIR);
      $DIR =~ s/:$/\//;
    }
    else {
      chop($item);
      $item = $DIR.$item;
      if ($item !~ /^\s*$/) { push(@ls, $item); }
    }
  }
  @localls=();

}  # sub Get_LS

  



