#!/bin/ksh

###############################################################################
# @(#) SysManagement/delivery/unix/bin/metrica_upgrade.ksh SysManagement_r4.79.0.9:cset.008571:90:90 02/25/08 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
# Copyright  2005 Vallent Software Systems UK. All rights reserved 
#
# Title:        metrica_upgrade.ksh
# Author:       Alan Greasley
#               
# Description:  PM 2.2.13 FP1 to 2.2.13 FP2 Upgrade Wrapper
#
###############################################################################
# Base Vars
#

SCRIPT_NAME=$(basename $0)
DIRNAME=$(dirname $0)
SCRIPT_TITLE="Metrica $NEWPMVER Upgrade Script"
AUTO_MODE=false
BASE_DIR=$(pwd)
ECHO=print  # works on all platforms under ksh
TMPFILE=met_upg_out.$$
LOGFILE=
ERRFILE=

#pattern for Grouping service refresh that could be disabled during upgrade
CRON_GS_SERVICE_PATTERN="metrica_gsadmin.ksh -refresh"
#patterns for other services that will be silently disabled during upgrade
CRON_SERVICES_PATTERN_LIST[1]="metrica_systemserviceadmin.ksh -collect"
#other services that have to be silently disabled during upgrade, could be added here 
#CRON_SERVICES_PATTERN_LIST[2]="metrica_other"


# Products/Versions
BES="BES 5.2.1"

if [ -f ${DIRNAME}/metrica_release.ksh ]; then
    . ${DIRNAME}/metrica_release.ksh
else
    echo "Could not find file ${DIRNAME}/metrica_release.ksh"
    exit 1
fi

OLDPMVER="Performance Manager ${PREV_MPM_RELEASE}"
NEWPMVER="Performance Manager ${MPM_RELEASE}"
NEW_JAVA="Java 1.4.2"


############################
# Error handling
ERR_OK=0
ERR_USAGE=1
ERR_ABORT=2
ERR_NO_TMP_FILE=3
ERR_NO_METRICA_DIR=5
ERR_NO_ADMIN_DB=6
ERR_NO_BES_DIR=7
ERR_IFX_NOT_UP=9
ERR_NO_INST_PACKAGE=10
ERR_NO_DBEL_PACKAGE=11
#
ERR_LICENSE_NOT_ACCEPTED=20
ERR_JAVA_NOT_142=21
ERR_JAVA_NOT_JDK=22
ERR_BES_NOT_521=26
ERR_BES_NOT_UPDATED=27
#
ERR_BES_START_FAILED=37
ERR_BES_STOP_FAILED=38
ERR_PROPERTIES_FILE_EXISTS=40
ERR_BES_RESTART_FAILED=41
#
############################
ERRTXT_0="Normal Exit"
ERRTXT_1= # Nothing, usage has been printed
ERRTXT_2="Abnormal termination"
ERRTXT_3="Cannot create temp file in $(pwd), /tmp or /usr/tmp"
# env
ERRTXT_5="METRICA_DIR not defined, aborting"
ERRTXT_6="METRICA_ADMINISTRATION_DB not defined, aborting"
ERRTXT_7="APPSERVER_DIR not defined, aborting"
ERRTXT_9="Informix is down"
ERRTXT_10="Installer package not found"
ERRTXT_11="DBElements package not found"
# pre-upgrade
ERRTXT_20="License not accepted, aborting"
ERRTXT_21="Current JAVA_HOME is not $NEW_JAVA"
ERRTXT_22="Current JAVA_HOME is not a JDK"
ERRTXT_26="Current APPSERVER_DIR is not $BES"
ERRTXT_27="BES not updated for multi partitioning"
#ERRTXT_27="BES 5.1 instance is still running (osagent present)"
# Upgrade errors 
#
ERRTXT_37="Appserver startup failed. Check that the new Appserver instance\nis up and running and re-run the upgrade from this point"
ERRTXT_38="Appserver stop failed. Check that the new Appserver instance\nis fully shutdown and re-run the upgrade from this point"
ERRTXT_40="metrica_install.properties file not deleted correctly, exiting upgrade"
ERRTXT_41="Appserver Restart failed" 
############################


###############################################################################
# Convenience Functions
#

# log function
#   Options:
#     S  Screen
#     L  Log File
#     E  Error File
#
#     C  with leading CR
#     N  without trailing CR (-n print/echo option)
#
log()
{
    ARGS=$1
	SMSG="$2"
	LMSG="[$(now long)] $SMSG"
    NARG=

	[ $($ECHO $ARGS | grep -c C) -gt 0 ] && SMSG="\n$SMSG" && LMSG="\n$LMSG"
	[ $($ECHO $ARGS | grep -c N) -gt 0 ] && NARG=-n

    # Log to screen
	[ $($ECHO $ARGS | grep -c S) -gt 0 ] && $ECHO $NARG "$SMSG"
    
    # Only log to file if we have one, else to screen if no screen events above
    if [ $($ECHO $ARGS | grep -c L) -gt 0 ]; then
        if [ -n "$LOGFILE" ]; then
            $ECHO $NARG "$LMSG" >> $LOGFILE
        elif [ -z "$LOGFILE" -a $($ECHO $ARGS | grep -c S) -eq 0 ]; then
            $ECHO $NARG "$SMSG"
        fi
    fi

    # Log to err file or to screen if no err file and no screen/log events above
    if [ $($ECHO $ARGS | grep -c E) -gt 0 ]; then
        if [ -n "$ERRFILE" ]; then
            $ECHO $NARG "$LMSG" >> $ERRFILE
        elif [ -z "$ERRFILE" -a $($ECHO $ARGS | egrep -c 'S|L') -eq 0 ]; then
            $ECHO $NARG "$SMSG"
        fi
    fi
}

# general cleanup on exit
#
cleanup() {
    rm -f $TMPFILE
}

# stopped on signal - extra logging/cleanup before cleanup() above
#
abort(){
    eval MSG_ABORT=\$ERRTXT_$ERR_ABORT
    log CSE "$MSG_ABORT"
    exit $ERR_ABORT
}

# print handy timestamp for file backups etc.
#
now() {
    if [ "$1" = "long" ]; then
        date '+%b %e %Y %T'
    else
        date '+%d%b_%H:%M'
    fi
}

# Prompt
#
ask() {
	PROMPT="$1"
	[ "$AUTO_MODE" = "on" ] && log CSL "Auto: ${PROMPT}" && return 0
    FULL_PROMPT="${PROMPT}?\n[ Hit (y/Y/Enter) to confirm or any other key to skip ] > "
	log CSLN "$FULL_PROMPT"
    read ANSWER
    $ECHO "$ANSWER" >> $LOGFILE
    
	[ "$ANSWER" = "y" -o "$ANSWER" = "Y" -o "$ANSWER" = "" ] && return 0
	return 1
}

# Script intro
#
intro() {
    log CS "$SCRIPT_TITLE\n"
    log S "This script will run through the steps necessary to upgrade your"
    log S " $OLDPMVER installation to"
    log S " $NEWPMVER\n"
    log S "It assumes you have installed the necessary pre-requisites:"
    log S " - Server software and configuration updates as stated in the Deployment Guide"
    log S "   (particularly the correct release of the Java Development Kit version 1.4.2)"
    [ "$MACHINE" = "hpux_parisc" ] && log S "   (and an installation of the Java Development Kit version 1.3.1)"
    log S " - An installation of the Appserver: $BES"
    log S "   (this will have updated your Metrica setup files)"
     
    # if [ -z "$TURN_OFF_SUMM_GEN" ]; then
    #     log CSL""
    #     log CSL "Note: Summary generation will be performed at the end of this upgrade"
    #     log CSL""
    # fi
}

# Script usage
#
usage() {
    log S "\nUsage:   $SCRIPT_NAME <options>\n"
    log S "  [ -auto ]"
    log S "      Auto-confirm each prompt\n"
    exit $ERR_USAGE
}

# Fatal error handler
#
die_on_failure() {
    RETCODE=$1
    eval MSG=\"\$ERRTXT_$RETCODE\"
    log CSE "FATAL ERROR $RETCODE: $MSG"
    exit $RETCODE
}

# Check env/args
#
check_env_args() {

    log S ""

    # Check Metrica dir
    [ -z "$METRICA_DIR" ] && return $ERR_NO_METRICA_DIR

    # Check Metrica log dir
    if [ -w "$METRICA_DIR/logs" ]; then
        LOGFILE=$METRICA_DIR/logs/upgrade.log
        ERRFILE=$METRICA_DIR/logs/upgrade.err
    else
        log S "Cannot access: $METRICA_DIR/logs"
        LOGFILE=$BASE_DIR/upgrade.log
        ERRFILE=$BASE_DIR/upgrade.err
    fi

    # Check main log
    if touch $LOGFILE 2>/dev/null; then
        log S "Using main log file: $LOGFILE"
    else
        log S "Warning: Cannot create main log file: $LOGFILE"
        LOGFILE=
    fi

    # Check error log
    if touch $ERRFILE 2>/dev/null; then
        log S "Using error log file: $ERRFILE"
    else
        log S "Warning: Cannot create error log file: $ERRFILE"
        ERRFILE=
    fi

    # Check tmp file
    if ! touch $TMPFILE 2>/dev/null; then
        TMPFILE=/tmp/$TMPFILE
        if ! touch $TMPFILE 2>/dev/null; then
            TMPFILE=/usr$TMPFILE
            if ! touch $TMPFILE 2>/dev/null; then
                return $ERR_NO_TMP_FILE
            fi
        fi
    fi

    # Check we have admin db set
    [ -z "$METRICA_ADMINISTRATION_DB" ] && return $ERR_NO_ADMIN_DB

    # Check we have Appserver basic env
    [ -z "$APPSERVER_DIR" ] && return $ERR_NO_BES_DIR


    return 0
}

remove_install_properties() {

# Removing metrica_install.properties file in upgrade scenario

    if [ -f $METRICA_DIR/Installer/bin/metrica_install.properties ]; then
        rm -f $METRICA_DIR/Installer/bin/metrica_install.properties 2> /dev/null
    fi

    ##Also check locally to make sure we delete this on
    if [ -f $BASE_DIR/metrica_install.properties ]; then
        rm -f $BASE_DIR/metrica_install.properties 2> /dev/null
    fi

        ## Make sure all conceivable metrica_install.properties files are deleted
        if ([ -f $METRICA_DIR/Installer/bin/metrica_install.properties ] || [ -f $BASE_DIR/metrica_install.properties ]); then
            return $ERR_PROPERTIES_FILE_EXISTS
        fi
}

# Needed when init KPI groups or Meta context list
reporting_metacontext_initialisation_off() {
   # 2.2.6 /2.2.10 This will prevent server starts from prematurely initialising the metacontext cache because of a call from Reporting (Uiweb package). 

   log CSL "Turning metacontext initialisation off ..."
   echo "update propertyservice_property set value='no' where property_name = 'reporting_metacontext_initialisation' ;"| dbaccess $METRICA_ADMINISTRATION_DB

   return 0
}
# Needed when init KPI groups or Meta context list
reporting_metacontext_initialisation_on() {
   #2.2.6 /2.2.10  This will allow the meta context cache initialisation to happen as expected on server starts.
   
	 log CSL "Turning metacontext initialisation on ..."  
   echo "update propertyservice_property set value='yes' where property_name = 'reporting_metacontext_initialisation' ;"| dbaccess $METRICA_ADMINISTRATION_DB
   return 0
}

post_install_metalayer_update() {
	 
    log CSL  "Starting summary generation"
    # 1) Reset summary generation flag
    #######################################################

    log CL  "Reset summary generation flag"
    echo "update ml_kpi_group set summary_generation='t' where system='t';"| dbaccess $METRICA_ADMINISTRATION_DB >> $LOGFILE 2>&1

    metrica_mladmin.ksh -start
    
    log CSL "Started summary generation, this may take some time. Verify that summary" 
    log SL "generation has completed by running metrica_mladmin.ksh -summarygenerationstatus "
    log SL "and inspecting that the summary generation has been run for each of the KPI groups."
		log CSL ""
   
    return 0
}

# this function is required for GS 2.2.9 or 2.2.11 changes made to the schema
clear_down_and_start_grouping() {

   log CSL "INFO: Grouping cleardown is now commencing , this may take some time";
   if [ ! -d $METRICA_DIR/GroupingService/saved ]; then 
     	mkdir $METRICA_DIR/GroupingService/saved
   fi
   
   if [ -f $METRICA_DIR/GroupingService/bin/metrica_gsadmin.ksh ]; then
      STATIC_GROUP_XML_FILE=$METRICA_DIR/GroupingService/saved/2211_upgrade_bkup_`date +%y%m%d%H_%M_%S`.xml
      log CSL "INFO: unloading static groups to backup file $STATIC_GROUP_XML_FILE"
      $METRICA_DIR/GroupingService/bin/metrica_gsadmin.ksh -staticgroups -unload $STATIC_GROUP_XML_FILE  
      $METRICA_DIR/GroupingService/bin/metrica_gsadmin.ksh -cleardown
      log CSL "INFO: Executing Grouping Service start, this may take some time.."
      $METRICA_DIR/GroupingService/bin/metrica_gsadmin.ksh -start
      log CSL "INFO: loading static groups from backup file $STATIC_GROUP_XML_FILE"
      $METRICA_DIR/GroupingService/bin/metrica_gsadmin.ksh -staticgroups  -load $STATIC_GROUP_XML_FILE
   else
      log CSL "ERROR: metrica_gsadmin.ksh not found - GS cleardown will have to be re-run manually"
      exit
   fi
}
# This function is required for adding indexes to TP that were installed pre 2.2.9
run_index_creation_script() {
	if [ -f $METRICA_DIR/Installer/bin/metrica_add_name_indexes.ksh ]; then
		$METRICA_DIR/Installer/bin/metrica_add_name_indexes.ksh -threads 8 # | tee -a $METRICA_DIR/logs/upgrade.log
	else
		log CSL "ERROR: metrica_add_name_indexes.ksh was not found - this script needs to be run manually"
	fi
}

fix_gs_dbd_file()
{

        log L "Altering $METRICA_DIR/GroupingService/data/administration.dbd file ...."

        if [ -f $METRICA_DIR/GroupingService/data/administration.dbd ]; then
                rm -rf $METRICA_DIR/GroupingService/data/administration.dbd 2> /dev/null

echo "database:\$METRICA_ADMINISTRATION_DB
user:\$METRICA_DB_USERNAME
password:\$METRICA_DB_PASSWORD
data:gs_grouping_messagecatalogue_en_gb
data:gs_grpgui_messagecatalogue_en_gb
data:gs_grpaccess_messagecatalogue_en_gb
data:gs_security_capabilities
table:gs_class
table:gs_group
table:gs_ncgroup
table:gs_staticgroup
table:gs_compoundgroup
table:gs_creatingexpression
table:gs_kpigroupid_creatingexpressionid_mapping
table:gs_trigger_status" > $METRICA_DIR/GroupingService/data/administration.dbd

        else
                log CSL "ERROR: $METRICA_DIR/GroupingService/data/administration.dbd does not exist,check this file before proceeding with upgrade"
		exit
        fi
}


# Removing "cryptix32.jar" as part of 2.2.13 IBM BlueWash
remove_jars_for_bluewash() {   
    log CSL " START Removing $METRICA_DIR/ext/lib/cryptix32.jar"

    if [ -f $METRICA_DIR/ext/lib/cryptix32.jar ]; then
        rm -f $METRICA_DIR/ext/lib/cryptix32.jar 2> /dev/null
    fi
    
    log CSL " START Removing $APPSERVER_DIR/var/servers/$APPSERVER_NAME/partitions/standard/lib/cryptix32.jar"
    
    if [ -f $APPSERVER_DIR/var/servers/$APPSERVER_NAME/partitions/standard/lib/cryptix32.jar ]; then
            rm -f $APPSERVER_DIR/var/servers/$APPSERVER_NAME/partitions/standard/lib/cryptix32.jar 2> /dev/null
    fi
    
    log CSL "END Remove COMPLETE"
}


###############################################################################
# Upgrade routines
#

# Before commencing upgrade, check Java / BES / Informix
#
pre_upgrade() {

    # Prompt the user to accept the license.  If '-auto' has been specified, we silently accept the license
    [ "$AUTO_MODE" = "true" ] && LAP_SILENT_PARAM="-silent"
    ${DIRNAME}/metrica_lap.ksh -logfile ${LOGFILE} ${LAP_SILENT_PARAM}
    [ $? -eq 1 ] && return $ERR_LICENSE_NOT_ACCEPTED

    # Check we have necessary Java installations
    CHK=$($JAVA_HOME/bin/java -version 2>&1 | grep "^java version" | awk '{print $3}' | cut -c2-6)
    [ "$CHK" != "1.4.2" ] && return $ERR_JAVA_NOT_142

    # Check we have a JDK
    if [ ! -x $JAVA_HOME/bin/jar ]; then
        return $ERR_JAVA_NOT_JDK
    fi

    # Check BES 5.2.1 has been installed and is the current Appserver
    CHK=$(grep -c "BORLAND ENTERPRISE SERVER 5.2.1" $APPSERVER_DIR/license.txt)
    [ $CHK -eq 0 ] && return $ERR_BES_NOT_521

    # Check Informix is up
    CHK=$(onstat -p | grep -c "On-Line")
    [ $CHK -eq 0 ] && return $ERR_IFX_NOT_UP

    # Check to ensure BES has been updated correctly
    CHK=$(grep -c "metrica.static.partitions.list" $APPSERVER_DIR/var/servers/$APPSERVER_NAME/adm/properties/server.properties)
    [ $CHK -eq 0 ] && return $ERR_BES_NOT_UPDATED

    # Check for Installer package
    if [ -f Installer*.tar* ]; then
        INST_PACKAGE=$(ls -1t Installer*.tar*)
    else
        return $ERR_NO_INST_PACKAGE
    fi
    
    # Check for DBElements package. If in the list then make 
    # sure its in the directory
    if [ $(grep -c DBElements $PM_UPG_PKG_LIST) -eq 1 ]; then
    	if [ -f DBElements*.tar* ]; then
    	    DBELEMENTS_PACKAGE=$(ls -1t DBElements*.tar*)
	else
    	    return $ERR_NO_DBEL_PACKAGE
	fi    
    fi   

#Set the upgrade permissions to nothing , so we can upgrade KPI groups
 #  log CSL "make the "adc." and "vallent." kpi groups read/write "
 #  UPDATE_OEMTechPackReadOnly="update propertyservice_property set value=\"\" where property_name=\"OEMTechPackReadOnly\""
 #  echo "$UPDATE_OEMTechPackReadOnly" | dbaccess $METRICA_ADMINISTRATION_DB 

    return 0
}

# Check cron for GS Refresh and other services in cron
#
check_cron() {

    $ECHO "# $SCRIPT_NAME: cron backup at $(now)" > $TMPFILE
    crontab -l >> $TMPFILE

    #Grouping service state
    CRON_GS_STATE="NONACTIVE"
    #Other - selected - services (silently disabled) - e.g. garbage collection 
    CRON_OTHER_SERVICES_STATE="NONACTIVE"

    if [ $(grep -c "^[^#].*$CRON_GS_SERVICE_PATTERN.*" $TMPFILE) -ne 0 ]; then
        log CSL "Found active Grouping service refresh record in cron"
    	CRON_GS_STATE="ACTIVE"
    fi
    
    i=1
    while [ $i -le ${#CRON_SERVICES_PATTERN_LIST[*]} ]
    do
        if [ $(grep -c "^[^#].*${CRON_SERVICES_PATTERN_LIST[$i]}.*" $TMPFILE) -ne 0 ]; then
            log CSL "Found active metrica service record in cron ($i)"
            CRON_OTHER_SERVICES_STATE="ACTIVE"
        fi
        let i=i+1
    done
    
}

# Wrap install script to avoid repeated prompts
#
install_wrap() {

    PKG_OR_PKGLIST="$*"

    MET_USER=
    MET_PASS=

    # Get Metrica DB User
    $ECHO "select value from propertyservice_property where property_name=\"cs_metricaAdminUser\"" | dbaccess $METRICA_ADMINISTRATION_DB >$TMPFILE 2>&1
    if [ $(grep -c "^value" $TMPFILE) -gt 0 ]; then
        MET_USER=$(grep "^value" $TMPFILE | awk '{print $2}')
    fi

    # Get Metrica DB User
    $ECHO "select value from propertyservice_property where property_name=\"cs_metricaAdminPassword\"" | dbaccess $METRICA_ADMINISTRATION_DB >$TMPFILE 2>&1
    if [ $(grep -c "^value" $TMPFILE) -gt 0 ]; then
        MET_PASS=$(grep "^value" $TMPFILE | awk '{print $2}')
    fi


    if [ -n "$MET_USER" -a -n "$MET_PASS" ]; then
        # Run installer, passing in entries to :
        # - 6 for All Layers
        # - metrica DB user derived from Admin DB
        # - metrica DB password derived from Admin DB (repeated)
        # - <Return> for schema vars prompt
        echo "6\n$MET_USER\n$MET_PASS\n" | metrica_install.ksh $PKG_OR_PKGLIST
    else
        metrica_install.ksh $PKG_OR_PKGLIST
    fi

}



################################################################################
################################   MAIN   ######################################
################################################################################

# Stage 1 - Check relevant pre-requisites
#
# We should have:
#
#  - Java 1.4.2 installation (and Java 1.3.1 for HP)
#  - BES 5.2.1 installation, configured and new setup files in $METRICA_DIR
#  - Logged in as metrica user with new setup run

trap cleanup 0
trap abort 1 2 15

# Check for args
while [ -n "$1" ]; do
    case "$1" in
        "-auto")	AUTO_MODE=true;;
        *)	usage
    esac
    shift
done

intro

check_env_args || die_on_failure $?

# Post-checking env e.g. when METRICA_DIR etc. have been verified
SCRIPT_START=$(now)
CRON_BAK= # The backup file to use - could be created this time, or from before
CRON_BAK_STUB=$METRICA_DIR/etc/crontab.bak
CRON_BAK_NOW=$CRON_BAK_STUB.$SCRIPT_START
PM_UPG_PKG_LIST=Performance_Manager_Upgrade_Package_List


APPSERVER_START_CMD="$METRICA_DIR/utils/bin/metrica_appserveradmin.ksh -start"
APPSERVER_STOP_CMD="$METRICA_DIR/utils/bin/metrica_appserveradmin.ksh -stop"
METRICA_SEC_INITIALISE="$METRICA_DIR/MetaLayer/bin/metrica_mladmin.ksh -initialise -security"


pre_upgrade || die_on_failure $?


# 2.2.6 /2.2.10 prevent server starts from prematurely initialising the metacontext cache because of a call from Reporting 
#reporting_metacontext_initialisation_off 

# Removing "cryptix32.jar" as part of 2.2.13 IBM BlueWash
remove_jars_for_bluewash

# Stage 2 - Ensure GS Refresh disabled in cron
#
check_cron

if [ "$CRON_OTHER_SERVICES_STATE" = "ACTIVE" ] || [ "$CRON_GS_STATE" = "ACTIVE" ]; then

    # Backup current crontab entries, and keep for re-load later
    log SL "Started cron task disable"
    CRON_BAK=$CRON_BAK_NOW
    mv $TMPFILE $CRON_BAK
    log SL "Current crontab backuped at ${CRON_BAK} - will be reloaded automaticaly at the end of upgrade"
    
    CMD_CRON_DISABLE_GS="s,^\(.*\),\1,"
    CMD_CRON_DISABLE_OTHER="s,^\(.*\),\1,"
    INSTALL_TIMESTAMP=`date +%y%m%d_%H_%M_%S`

    if [ "$CRON_GS_STATE" = "ACTIVE" ] && ask "Disable cron task for Grouping Service refresh"; then
        CMD_CRON_DISABLE_GS="s,^\([^#].*$CRON_GS_SERVICE_PATTERN.*\),#Upgrade Disable GS ${INSTALL_TIMESTAMP}#\1,"
    	CRON_GS_STATE="DISABLED"
    fi

    if [ "$CRON_OTHER_SERVICES_STATE" = "ACTIVE" ]; then
    	NL=`echo "\n "`

        CMD_CRON_DISABLE_OTHER="{ $NL"
	i=1
        while [ $i -le ${#CRON_SERVICES_PATTERN_LIST[*]} ]
        do
	    CMD_CRON_DISABLE_OTHER="${CMD_CRON_DISABLE_OTHER}s,^\([^#].*${CRON_SERVICES_PATTERN_LIST[$i]}.*\),#Upgrade Silent Disable ${INSTALL_TIMESTAMP}#\1,$NL"
            let i=i+1
	done
	CMD_CRON_DISABLE_OTHER="$CMD_CRON_DISABLE_OTHER }"
    	CRON_OTHER_SERVICES_STATE="DISABLED"
    fi
    
    # Disable GS Refresh and other relevant services in crontab
    cat $CRON_BAK | sed "$CMD_CRON_DISABLE_GS" | sed "$CMD_CRON_DISABLE_OTHER" > $TMPFILE
    # Load cron entries minus GS refresh
    crontab $TMPFILE
    
    #log SL "Finished GS refresh disable"
    log SL "Finished cron tasks disable"
fi

# 2.2.11 Fixing Corrupt administration.dbd file for GS
#fix_gs_dbd_file


# Stage 3 - Install the Installer!
#
if ask "Install the $NEWPMVER Installer (pre-requisite for following installations)"; then

    log CSL "Ensuring Appserver is started up .."
    $APPSERVER_START_CMD || die_on_failure $ERR_BES_START_FAILED

    # Removing metrica_install.properties file in upgrade scenario before upgrade
    remove_install_properties || die_on_failure $?

    # Install the Installer
    install_wrap $INST_PACKAGE 

    log SL "Finished installing the $NEWPMVER Installer"

    # Removing metrica_install.properties file in upgrade scenario

    remove_install_properties || die_on_failure $?
fi

# Stage 4  Install DBElements Package ; Only install if its in the package list
if [ $(grep -c DBElements $PM_UPG_PKG_LIST) -eq 1 ]; then
    if ask "Install $NEWPMVER DBElements (pre-requisite for following installations)"; then  
    	# Because of informix bug 422885 , before starting the upgrade we need to ensure that
	# blademgr is working OK on the DB.
	OLD_BLADE_NAME=`echo "list $METRICA_ADMINISTRATION_DB" | blademgr | grep "_MetricaBlade" | tr -d "'\t'' '"`

	if [ -n "$OLD_BLADE_NAME" ]; then
		echo "Database $METRICA_ADMINISTRATION_DB has datablade $OLD_BLADE_NAME registered\n"
	else
		echo "error running blademgr, exiting upgrade \n "
		exit 1   
	fi
 
	$METRICA_DIR/Installer/bin/metrica_drop_tab_a_number.ksh || die_on_failure $?
  
	install_wrap $DBELEMENTS_PACKAGE
  
	$METRICA_DIR/Installer/bin/metrica_create_tab_a_number.ksh || die_on_failure $?
    fi                                                                                                     
fi  

# Stage 5 - Install Patch Packages
#
if ask "Install $NEWPMVER patch packages"; then

	install_wrap -packagelist $PM_UPG_PKG_LIST    

fi

# Enter POST install change here

#2.2.12 Do not generate summaries if the env variable is set
#
# N O T E
# -------
# If you are uncommenting this section, then uncomment the associated
# output statements in the intro() function.
#
#if [ -n "$TURN_OFF_SUMM_GEN" ]; then
#    log CSL "Environment variable TURN_OFF_SUMM_GEN has been detected, skipping summary generation"
#		log CSL""
#else
#    post_install_metalayer_update
#fi

# Stage 6 - Re-enable Cron only if enabled initially 
#
#if [ -n "$CRON_BAK" -a "$CRON_BAK" != "none" ] && ask "Re-enable cron task for Grouping Service refresh"; then
if [ "$CRON_OTHER_SERVICES_STATE" = "DISABLED" ] || [ "$CRON_GS_STATE" = "DISABLED" ]; then

    #backup copy of crontab (made at the begining of installation) will be used
    #if GS was disabled on user reqest and now the user will decide not to re-enable it
    #then GS will be commented out from backup copy and such crontab will be used
    #all other crontab entries will be the same as they were before installation
    
    CMD_CRON_DISABLE_GS="s,^\(.*\),\1,"
    if [ "$CRON_GS_STATE" = "DISABLED" ]; then
    	if ask "Re-enable cron task for Grouping Service refresh"; then
	    CMD_CRON_DISABLE_GS="s,^\(.*\),\1,"
	else
	    CMD_CRON_DISABLE_GS="s,^\([^#].*$CRON_GS_SERVICE_PATTERN.*\),#Upgrade Disable GS ${INSTALL_TIMESTAMP}#\1,"
	fi
    fi
    
    # Restore crontab file from before and disable enabled GS if user decide not to re-enable them
    cat $CRON_BAK | sed "$CMD_CRON_DISABLE_GS" | crontab 2> $TMPFILE
    RC=$?
    if [ $RC -ne 0 ]; then
        log SLE "Error $RC restoring cronfile: $CRON_BAK\nNeed to manually check cron entries"
        cat $TMPFILE >> $ERRFILE
    else
        log SL "Finished restoring cron settings"
        rm -f $CRON_BAK
    fi
fi
