#!/usr/bin/perl
#
#**************************************************************************
# @(#) SysManagement/delivery/unix/bin/metrica_cloneadmin.pl SysManagement_r4.79.0.9:cset.005482:5:5 03/30/05 @(#)
#
#
#                  COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems
#                     Spencer House
#                     23 Sheen Road
#                  Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems, 2005.
#
# Title:        
# Authors:      Alan Greasley, Padraig Mc Carthy
# Description:  Perl script to clone partitions under BES 5.2.1
#
#**************************************************************************#
use strict;
use Sys::Hostname;
use File::Copy;
use File::Path;
use Cwd;
use Getopt::Long;

# Getopt settings

# turn off auto-abbreviation
$Getopt::Long::autoabbrev=0; 

# exit when errors occur
$Getopt::Long::runthrough=1; 

# turn on case sensitivity
$Getopt::Long::ignorecase=0; 

my $product_long_name = "Borland Enterprise Server";
my $product_version = "5.2.1";

my $verbose = 0;
my $auto = 1;
my $ptn_name;

my ($src_ptn, $new_ptn);

my %metrica_env = (
	bes_dir => "APPSERVER_DIR",
	svr_name => "APPSERVER_NAME"
);
my @link_dirs = qw (classes dars ears ejb_jars lib rars wars);

my $svr_dir = check_env();
my $ptn_dir = "$svr_dir/partitions";
my $props_dir = "$svr_dir/adm/properties";
my $ptn_props_dir = "$svr_dir/adm/properties/partitions";
my $stn_ptn_props = "$ptn_props_dir/standard";
my $ptn_log_dir ="$svr_dir/adm/logs/partitions";

##########################################################################
# suffix for backing up files

my ( $cfg_file, $cfg_bak, $old, $found );
        my ($sec,$min,$hr,$mday,$mon,$yr,$wday,$yday,$isdst);
        ($sec,$min,$hr,$mday,$mon,$yr,$wday,$yday,$isdst) = localtime;

        $yr += 1900;
        $mon += 1;
        $mon = "0" . $mon if ($mon < 10);
        $mday = "0" . $mday if ($mday < 10);
        $hr = "0" . $hr if ($hr < 10);
        $min = "0" . $min if ($min < 10);
        my $suffix = "$yr.$mon.$mday.$hr$min";

################################################################################
#
# Test Args
#

my ($opt_prompt,$opt_verbose,$opt_drop,$opt_clone,$opt_help);

# handle all options from the command line
my $result = GetOptions
     ("" => \$opt_help,
         "v!" => \$opt_verbose,
          "verbose!" => \$opt_verbose,
          "p!" => \$opt_prompt,
          "prompt!" => \$opt_prompt,
          "drop!" => \$opt_drop,
	  "clone!" => \$opt_clone
        );

if($opt_prompt){
   $auto = 0;
}

if($opt_verbose){
   $verbose = 1;
}

if($opt_help){
    Usage();
}

################################################################################
#
# Welcome
#
if ($verbose) {
 welcome();
}

if ($opt_drop) {
#if drop is entered we need a partition Name as the only arg
$ptn_name=$ARGV[$0];
  if(!$ptn_name){
     Usage();
     exit;
   }
drop_partition();
exit 0;
}
elsif ($opt_clone) {
   clone_partition();
   exit 0;
}
else
{
   Usage();
   exit 0;
}


################################################################################
# Subroutine name:  clone_partition
#
# Description:      Creates a new partition
#
# Arguments:        Nothing
#
# Returns:          Nothing
#

sub clone_partition {


# Check partition dir - as installed (standard only), or already one or more
# web partititions present (in which case we can clone them)
#
check_ptn_dir ();

# Now, do the clone
# 

# Create a new partition deployment dir, and link main subdirs to standard
create_ptn_dir() if (($auto) or prompt ("s", "Add partition dir"));

# Make a copy of the partition admin dir from the source partition
# NOTE: Certain MP config should have been done by the installer at this stage
#       e.g. Apache config for forwarding Metrica requests to Tomcat contexts
#       via IIOP, and configuration of the 'standard' Tomcat service to disable
#       the HTTP service and accept requests only using the IIOP service
copy_adm_dir() if (($auto) or prompt ("s", "Create new admin dir"));

# Make configuration changes to server, new and standard partitions
configure_adm() if (($auto) or prompt ("s", "Configure new partition"));


# Goodbye message
goodbye();

}

################################################################################
# Subroutine name:  drop_partition
#
# Description:      Removes a named partition
#
# Arguments:        Nothing
#
# Returns:          Nothing
#

sub drop_partition {
# drop the partition deployment dir, and remove link of main subdirs to standard
remove_ptn_dir() if (($auto) or prompt ("s", "Remove partition dir"));

# Make configuration changes to server, new and standard partitions
unconfigure_adm() if (($auto) or prompt ("s", "Unconfigure partition $ptn_name"));

# Goodbye message
goodbye();
}

################################################################################
# Subroutine name:  remove_ptn_dir
#
# Description:      Removes directories
#
# Arguments:        Nothing
#
# Returns:          Nothing
#

sub remove_ptn_dir {
my $remove_ptn_dir = "$ptn_dir/$ptn_name";
my $remove_props_dir = "$ptn_props_dir/$ptn_name";
my $remove_log_dir = "$ptn_log_dir/$ptn_name";

if (! -d $remove_ptn_dir ) {
    print "ERROR: Could not find partition dir: $remove_ptn_dir\n";
    exit 3;
    }
if (! -d $remove_props_dir ) {
    print "ERROR: Could not find partition dir: $remove_props_dir\n";
    exit 3;
    }

if (! -d $remove_log_dir ) {
    print "ERROR: Could not find partition dir: $remove_log_dir\n";
    exit 3;
    }


if ($verbose) {
        print "\n";
	print "Removing dir $remove_ptn_dir\n";
	print "Removing dir $remove_props_dir\n";
	print "Removing dir $remove_log_dir\n";
        print "\n";
     }

    system("rm -rf $remove_props_dir");
    system("rm -rf $remove_ptn_dir");
    system("rm -rf $remove_log_dir");
}

################################################################################
# Subroutine name:  unconfigure_adm_dir
#
# Description:      removes web partition from conf files
#
# Arguments:        none
#
# Returns:          nothing
#

sub unconfigure_adm {

# Modify ias server.properties

my $cfg_file = "$props_dir/server.properties" ;
my $cfg_bak = "${cfg_file}_". $suffix;

if ($verbose) {
	print "Unconfigure file $cfg_file\n";
	}

if ( ! copy $cfg_file, $cfg_bak ) {
        print "\nERROR: Cannot rename config file: $cfg_file\n";
	exit 3;
        }

unless(open FILE, '<'.$cfg_file) {
   print "Error: Unable to open file $cfg_file";
   exit 1;
}

my @input_data=<FILE>;
close FILE;

unless(open FILE, '>'.$cfg_file) {
   print "Error: Unable to open file $cfg_file";
   exit 1;
}


my $line;

  until(!@input_data) {
    $line = shift @input_data;
    chomp $line;
    $line =~ s/\=[\s]*$ptn_name[\s]+/\=/;
    $line =~ s/[\s]+$ptn_name[\s]+/ /;
    $line =~ s/[\s]+$ptn_name$//;
    print FILE "$line\n";
  }
close FILE;

my $skip = is_other_partitions();

#if removing the last web partition we need to reenable web container in 
#partition standar

if(!$skip) {

my $tomcat="tomcat4";
my $found = 0;

	$cfg_file = "$stn_ptn_props/partition.properties" ;
  $cfg_bak = "${cfg_file}_" . $suffix;

	 if ($verbose) {
		 print "Unconfigure file $cfg_file\n";
	 }

        if ( ! rename $cfg_file, $cfg_bak ) {
                print "\nERROR: Cannot rename config file: $cfg_file\n";
        }
        elsif ( ! open IN, "$cfg_bak" ) {
                print "\nERROR: Cannot open input file: $cfg_bak\n";
        }
        elsif ( ! open OUT, ">$cfg_file" ) {
                print "\nERROR: Cannot open new config file: $cfg_file\n";
        }
	else {
                $found = 0;
                while(<IN>) {
                        if ( m,^server.service.autostart=, ) {
                                $found = 1;
                                s,$, $tomcat\n,;
                        }
                        print OUT;
                }

                close IN;
                close OUT;

                if ( ! $found ) {
                        print "WARNING: Substitution pattern not found in $cfg_file\n";
                }
        }

}

}

################################################################################
# Subroutine name:  prompt
#
# Description:      All purpose prompt routine
#
# Arguments:        $flags   - s - skip option - "s" or "n"
#                   $pstr    - 0 or more prompt strings
#
# Returns:          1 (true) = affirmative i.e. continue
#                   0 (false) to skip this time
#

sub prompt {

	my $flags = shift(@_);
	my $skip = 0;
	my $ans;
	my $pstr;
	my $estr = "";

	print "\n";
	foreach $pstr (@_) {
		print "$pstr\n";
	}

	return 1 if ($auto);

	if ( $flags =~ /s/ ) {
		$skip = 1;
		$estr = " \"s\" or \"n\" to skip,";
	}

	print "Hit <Enter> to continue,$estr or any other key to quit> ";

	die "\nPrompt Error\n\n" unless ( ($ans = <STDIN>) and (chomp $ans) );
	return 0				if ( $skip and (($ans eq "s") or ($ans eq "n")) );
	die "\nExiting installation. Delete any directories created during this installation before running the installation script again.\n\n"	if ($ans ne "");
	
	return 1;
}

################################################################################
# Subroutine name:  check_env
#
# Description:      Check Metrica environment variables
#
# Arguments:        
#
# Returns:          nothing (void)
#
sub check_env {

	my $env_var;

	foreach $env_var ( qw (bes_dir svr_name) ) {
		if ($ENV{$metrica_env{$env_var}} eq "") {
			# Env not right
			print "\nERROR: Metrica environment not setup correctly: Cannot read $metrica_env{$env_var}\n\n";
			exit 2;
		}
	}

	return "$ENV{$metrica_env{bes_dir}}/var/servers/$ENV{$metrica_env{svr_name}}";

}

################################################################################
# Subroutine name:  check_ptn_dir
#
# Description:      Checks the appserver partitions dir for previously cloned
#                   web partition dirs - returns the primary "web1" if available
#                   else "standard"
#
# Arguments:        
#
# Returns:          nothing (void)
#
sub check_ptn_dir {

	my (@ptn_dirs, $pdir, $idx);
	
	opendir (DIR, $ptn_dir) or 
		die "Cannot open BES partitions dir: $ptn_dir\n";

	@ptn_dirs = grep /^web(\d)*$/, readdir(DIR);

	closedir( DIR );

	if (@ptn_dirs == 0) {
		$src_ptn = "standard";
		$new_ptn = "web1";
	}
	else {
		$src_ptn = $ptn_dirs[0];
		$idx = 1;
		foreach $pdir ( sort alphanumeric @ptn_dirs ) {
			$pdir =~ s,^web,, ;
			last if ($pdir != $idx);
            $idx++;
		}
		$new_ptn = "web$idx";
	}
}

################################################################################
# Subroutine name:  create_ptn_dir
#
# Description:      
#
# Arguments:        
#
# Returns:          nothing (void)
#
sub create_ptn_dir {

	my $subdir;
	my $newdir = "$ptn_dir/$new_ptn";

    if($verbose) {
    	print "New dir: $newdir\n";
    }

	if ( ! mkdir $newdir, 0755 ) {
    		print "ERROR: Could not create new partition dir: $newdir\n";
		exit 3;
	}

	if ( ! chdir ($newdir) ) {
    		print "ERROR: Could not change directory to new partition dir: $newdir\n";
		exit 3;
	}

	foreach $subdir (@link_dirs) {
		symlink "../standard/$subdir", "$subdir";
	}

}

################################################################################
# Subroutine name:  copy_adm_dir
#
# Description:      
#
# Arguments:        
#
# Returns:          nothing (void)
#
sub copy_adm_dir {

	if ( ! mkdir "$ptn_props_dir/$new_ptn", 0755 ) {
    	     print "ERROR: Could not create new partition adm dir: $ptn_props_dir/$new_ptn\n";
	     exit 3;
	}

	if ( ! chdir ("$ptn_props_dir/$src_ptn") ) {
    		print "ERROR: Could not change directory to partition adm dir: $ptn_props_dir/$src_ptn\n";
		exit 3;
	}


	system("tar cf - * | (cd ../$new_ptn; tar xf -)");

}

################################################################################
# Subroutine name:  configure_adm_dir
#
# Description:      
#
# Arguments:        
#
# Returns:          nothing (void)
#
sub configure_adm {

	# Modify ias server.properties
	$cfg_file = "$props_dir/server.properties" ;
	$cfg_bak = "${cfg_file}_" . $suffix ;

	if ($verbose) {
		print "Configure file $cfg_file\n";
	}

	if ( ! rename $cfg_file, $cfg_bak ) {
		print "\nERROR: Cannot rename config file: $cfg_file\n";
	}
	elsif ( ! open IN, "$cfg_bak" ) {
		print "\nERROR: Cannot open input file: $cfg_bak\n";
	}
	elsif ( ! open OUT, ">$cfg_file" ) {
		print "\nERROR: Cannot open new config file: $cfg_file\n";
	}
	else {
		$found = 0;
		while(<IN>) {
			if ( m,^server.service.autostart=, ) {
				$found = 1;
				s,$, $new_ptn,;
			}
			 if ( m,^metrica.static.partitions.list=, ) {
				 $found = 1;
				 s,$, $new_ptn,;
			 }

			#Need to ensure standard partition starts first
			if (m,^server.partition.startorder=, ){
       				print OUT "server.partition.startorder=standard\n";
       			}else {
      				print OUT;
      			}
		}

		close IN;
		close OUT;
					
		if ( ! $found ) {
			print "WARNING: Substitution pattern not found in $cfg_file\n";
		}
	}
	
	# Modify new web service.properties
	$cfg_file = "$ptn_props_dir/$new_ptn/service.properties" ;
	$cfg_bak = "${cfg_file}_" . $suffix ;

	if ($verbose) {
		print "Configure file $cfg_file\n";
	}

	if ( ! rename $cfg_file, $cfg_bak ) {
		print "\nERROR: Cannot rename config file: $cfg_file\n";
	}
	elsif ( ! open IN, "$cfg_bak" ) {
		print "\nERROR: Cannot open input file: $cfg_bak\n";
	}
	elsif ( ! open OUT, ">$cfg_file" ) {
		print "\nERROR: Cannot open new config file: $cfg_file\n";
	}
	else {
		$found = 0;
		while(<IN>) {
			if ( m,^service.name=, ) {
				$found++;
				print OUT "service.name=$new_ptn\n";
			}
			elsif ( m,^service.description=, ) {
				$found++;
				print OUT "service.description=$new_ptn\n";
			}
			else {
				print OUT;
			}
		}

		close IN;
		close OUT;
					
		if ( $found != 2 ) {
			print "WARNING: Substitution pattern(s) not found in $cfg_file\n";
		}
	}

	# Modify new web partition.properties
	$cfg_file = "$ptn_props_dir/$new_ptn/partition.properties" ;
	$cfg_bak = "${cfg_file}_" . $suffix ;

	if ($verbose) {
		print "Configure file $cfg_file\n";
	}

	if ( ! rename $cfg_file, $cfg_bak ) {
		print "\nERROR: Cannot rename config file: $cfg_file\n";
	}
	elsif ( ! open IN, "$cfg_bak" ) {
		print "\nERROR: Cannot open input file: $cfg_bak\n";
	}
	elsif ( ! open OUT, ">$cfg_file" ) {
		print "\nERROR: Cannot open new config file: $cfg_file\n";
	}
	else {
		$found = 0;
		while(<IN>) {
			if ( m,^server.description=, ) {
				$found++;
				print OUT "server.description=$new_ptn\n";
			}
			elsif ( m,^server.service.autostart=, ) {
				$found++;
				print OUT "server.service.autostart=interceptor partitionagent tomcat4\n";
			}
			else {
				print OUT;
			}
		}

		close IN;
		close OUT;
					
		if ( $found != 2 ) {
			print "WARNING: Substitution pattern(s) not found in $cfg_file\n";
		}
	}

	# Modify standard partition.properties, if necessary
        $cfg_file = "$stn_ptn_props/partition.properties" ;
	$cfg_bak = "${cfg_file}_" . $suffix ;

 my $skip = is_other_partitions();

#if there is another web partition created ensure that the 
#standard partition.properties is updated to not start its 
#web container

 if ($skip) {

	if ($verbose) {
		print "Configure file $cfg_file\n";
	}

	if ( ! rename $cfg_file, $cfg_bak ) {
		print "\nERROR: Cannot rename config file: $cfg_file\n";
	}
	elsif ( ! open IN, "$cfg_bak" ) {
		print "\nERROR: Cannot open input file: $cfg_bak\n";
	}
	elsif ( ! open OUT, ">$cfg_file" ) {
		print "\nERROR: Cannot open new config file: $cfg_file\n";
	}
	else {
		$found = 0;
		while(<IN>) {
			if ( m,^server.service.autostart=, ) {
			   $found = 1;
			   print OUT "server.service.autostart=interceptor partitionagent visinaming jts jss ejbcontainer\n";
			}else {
			print OUT;
			}
		}

		close IN;
		close OUT;
					
		if ( ! $found ) {
            if ( ! rename $cfg_bak, $cfg_file ) {
                print "\nERROR: No change in config - tried to restore original file but failed: $cfg_file\n";
            }
		}
	}

 }


}

sub alphanumeric {
    my $first = $a;
    $first =~ s,web,,;
    my $second = $b;
    $second =~ s,web,,;
    $first <=> $second;
}

sub Usage {
    welcome();
    exit 0;
}

sub goodbye {
if ($verbose) {
        print "\n";
        print "****************\n";
        print "Cloning Finished\n";
        print "****************\n";
        print "\n";
     }
}

sub welcome {
    print "\n";
    print "*********************************************************\n";
    print "Metrica Clone Admin Utility for $product_long_name $product_version\n";
    print "*********************************************************\n";
    print "\n\n";
    print "Warning : Please do not use this script directly. Use the\n";
    print "metrica_appserveradmin.ksh script to clone and drop partitions\n";
    print "as needed.\n";
}

################################################################################
# Subroutine name:  is_other_partitions
#
# Description:      finds out if there are any web partitions 
#
# Arguments:        None
#
# Returns:          1 if there is a web partition 
#		    0 otherwise
#

sub is_other_partitions
{
my $cfg_file = "$props_dir/server.properties" ;
my $value = "metrica.static.partitions.list=standard";
my $partition_exists=0;

unless(open FILE, '<'.$cfg_file) {
  print "Error: Unable to open file $cfg_file\n";
    }
    my @input_data=<FILE>;
    close FILE;
    my $line;
    until(!@input_data) {
     $line = shift @input_data;
    if ($line =~ /^[ \t]*metrica\.static\.partitions\.list[ \t]*\=[ \t]*/)  {
     my @result = split ( /^[ \t]*metrica\.static\.partitions\.list[ \t]*\=[ \t]*/, $line );
 
     if ($result[1] =~ /^standard[ \t]*$/) {
       $partition_exists = 0 ;
     } else {
     $partition_exists = 1  ;
   }
  }
}

return $partition_exists;
}

