#!/bin/ksh
##################################################################
# IBM Confidential
#
# OCO Source Materials
#
# 5724-N93
# Copyright IBM Corp. 2008
# The source code for this program is not published or otherwise
# divested of its trade secrets, irrespective of what
# been deposited with the U.S. Copyright Office.
#
##################################################################
# Created by: sbrooks - 22 Jan 2008 - 14:38:00
# Module: @(#) SysManagement/delivery/unix/bin/metrica_dbdef.ksh SysManagement_r4.79.0.9:cset.008565:18:18
# Version: SysManagement_r4.79.0.9:cset.008565:18:18
# Last Changed: 08/02/06 13:54:21
# Version Control: "@(#) SysManagement_r4.79.0.9:cset.008565:18:18 08/02/06 13:54:21 @(#) SysManagement/delivery/unix/bin/metrica_dbdef.ksh SysManagement_r4.79.0.9:cset.008565:18:18 @(#)"
#
# Change Activity:
# _trk_ __Date__ _PGMR_ _Description_________________________
#     1 22/01/08        Created from Vallent metrica_dbdef.ksh
#     2 23/01/08  68072 Performance improvements to TP upgrade
#     3 23/01/08        Bug 57388 incorrectly identifying columns
#                       to drop
#     4 24/01/08        Bug 58307 Table upgrade with date stamped
#                       markup causes errors on HP
###################################################################
#
# Description:
#
# Shell script to assist in setting up databases schemas
#
###################################################################

unalias -a

#---------------------------------
#       Defaults
#---------------------------------
DIRECTORY="`pwd`"

#---------------------------------
#      Global variables
#---------------------------------
PROGNAME=`basename $0`    
VERSION="SysManagement_r4.79.0.9:cset.008565:18:18"
SHORT_VERSION="SysManagement_r4.79.0.9"
functionname="_go"


#---------------------------------
#      Exit codes
#---------------------------------
exit_rc_success=0
exit_rc_usage=1
exit_rc_var_incorrect=2
exit_rc_dir_incorrect=3
exit_rc_file_incorrect=3

###################################################################
#
# Function    : create_common
#
# Description : Creates based on dbd file and directory files.
#
###################################################################

function create_common
{
  MY_FILE=$1
  MY_DB=$2
  MY_USER=$3
  MY_PASS=$4

  name=`basename $MY_FILE .sql`
  if [ `cat $MY_FILE | grep -ic "create function"` -gt 0 ]; then
    LOG "Creating SPL function $name..."
  elif [ `cat $MY_FILE | grep -ic "create procedure"` -gt 0 ]; then
    LOG "Creating SPL procedure $name..."
  elif [ `cat $MY_FILE | grep -ic "create table"` -gt 0 ]; then
    LOG "Creating table $name..."
  elif [ `cat $MY_FILE | grep -ic "create view"` -gt 0 ]; then
    LOG "Creating view $name..."
  else
    # Warn but try the command anyway...
    $ECHO "$MY_FILE: Unsupported SQL command" >> $LOGNAME
  fi

  query_file $MY_FILE $MY_DB $MY_USER $MY_PASS

  if [ $? != "0" ]; then
    LOG "Failed"
  else
    LOG "Succeeded"
  fi
}

# Get name of view.
#
# Output is a string, RETURN.
#
function get_tdef_view
{
  TDEF="$1"

  get_tmp_file 1
  TMP_FILE_1="$RETURN"

  # Get second field of view line out of TDEF.
  #
  $M_AWK -F";" '

    { $1 = tolower($1) }

    /^view/ {
      print "RETURN=\"" $2 "\""

      exit
    }

  ' $TDEF > $TMP_FILE_1

  . $TMP_FILE_1
}

# Get name of table.
#
# Output is a string, RETURN.
#
function get_tdef_table
{
  TDEF="$1"

  # Get second field of table line out of TDEF
  # Get last field from comma-delimited lines.
  #
  RETURN=`grep -i ^table $TDEF | cut -d";" -f2 | $M_AWK -F"," '{print $NF}'`
}

# Get list of table names, sorted in file order.
#
# Output is file, $RETURN.
#
function get_tdef_tables
{
  TDEF="$1"

  get_tmp_file 5
  TMP_FILE_2="$RETURN"

  DEBUG "5% TDEF file"
  DEBUG_CAT $TDEF

  # Get second field of table lines out of TDEF
  # Get last field from comma-delimited lines.
  #
  grep -i ^table $TDEF | cut -d";" -f2 | $M_AWK -F"," '{print $NF}' > $TMP_FILE_2

  DEBUG "5% TMP file"
  DEBUG_CAT $TMP_FILE_2

  RETURN="$TMP_FILE_2"
}

###################################################################
#
# Function    : tdef_to_sql
#
# Description : Converts TDEF file to SQL file.
#               Parameters:
#                 FILE
#               Returns:
#                 RETURN - SQL file
#
###################################################################

tdef_to_sql() {
  DEBUG "6% In tdef_to_sql!"

  FILE="$1"

  DEBUG "6% TDEF file:"
  DEBUG_CAT $FILE

  # Remove comments and blank lines from TDEF file.
  #
  grep -v ^# $FILE | grep -v ^$ > $TMP_DIR/tdef_to_sql.tdef

  get_tmp_file 6
  TABLE_OR_VIEW="$RETURN"

  # Determine whether TDEF is for a table or a view.
  #
  $M_AWK -v file=$TABLE_OR_VIEW '

    BEGIN {
      FS = ";"
    }

    {
      $1 = tolower($1)
    }

    $1 ~ /table/ {
      print "export RETURN='TABLE'" > file
      exit
    }

    $1 ~ /view/ {
      print "export RETURN='VIEW'" > file
      exit
    }

  ' $TMP_DIR/tdef_to_sql.tdef

  . $TABLE_OR_VIEW
  TABLE_OR_VIEW="$RETURN"

  # Make way for an SQL file in $TMP_DIR
  #
  tmpsqlfile=$TMP_DIR/tdef_to_sql.sql
  rm -f $tmpsqlfile > /dev/null 2>&1

  if [ $TABLE_OR_VIEW = "TABLE" ]; then

    DEBUG "8% Is table!"

    # Initialize temporary files.
    #
    $ECHO > $TMP_DIR/tdef_to_sql_table.sql
    $ECHO > $TMP_DIR/tdef_to_sql_extra.sql
    $ECHO > $TMP_DIR/tdef_to_sql_error.txt

    get_tmp_file 7
    TMP_FILE_1="$RETURN"

    # Check schema_variables file (produced from DB Sizing tool and passed to
    # metrica_dbdef.ksh with -schema_variables argument) and DBD file, for table
    # and index dbspaces, and extent and next extent sizes.
    #
    # Table and index dbspaces can be specified in the DBD file and in the
    # schema_variables file. Extent and next extent sizes can be specified in
    # the DBD file, the schema_variables file, and in the TDEF file. The order
    # of precedence, from greatest to least, is:
    #
    #   schema_variables                            table and index dbspaces and
    #                                               extent and next extent sizes
    #
    #   TDEF                                        extent and next extent sizes
    #
    #   DBD                                         table and index dbspaces and
    #                                               extent and next extent sizes

    # If schema_variables file is input, then get line for table from
    # schema_variables file.
    #
    # Line in schema_variables file is expected to be like:
    #
    #   TableName|%dbspace%|%initial%|%next%|
    #   %indexspace1%|%indexspace2%|..|%indexspacen%|
    #
    # E.g.
    #
    #   gsm_msc_resource_mobility_day|dbdata1|2000|200|
    #   dbindex1|dbindex1|dbindex1|
    #
    # These fields are placed into environment variables as:
    #
    #   TABLE_NAME|SCHEMA_VARIABLES_TABLE_DBSPACE|
    #   SCHEMA_VARIABLES_INITIAL_EXTENT_SIZE|SCHEMA_VARIABLES_NEXT_EXTENT_SIZE|
    #   SCHEMA_VARIABLES_INDEX_DBSPACES
    #
    set -A TABLE_NAME
    set -A SCHEMA_VARIABLES_TABLE_DBSPACE
    set -A SCHEMA_VARIABLES_INITIAL_EXTENT_SIZE
    set -A SCHEMA_VARIABLES_NEXT_EXTENT_SIZE
    set -A SCHEMA_VARIABLES_INDEX_DBSPACES

    get_tdef_table $FILE
    TABLE_NAME="$RETURN"

    if [ $SCHEMA_VARIABLES_FILE ]; then
      DEBUG "9% schema_variables file present."
      get_tmp_file 8
      TMP_FILE_2="$RETURN"

      grep -i "^$TABLE_NAME|" "$SCHEMA_VARIABLES_FILE" > $TMP_FILE_1

      # If schema_variables file contains an entry to
      # the table of this TDEF file, then read line.
      #
      if [ $? = 0 ]; then
        $M_AWK -F"|" '

          {
            print "SCHEMA_VARIABLES_TABLE_DBSPACE=" $2
            print "SCHEMA_VARIABLES_INITIAL_EXTENT_SIZE=" $3
            print "SCHEMA_VARIABLES_NEXT_EXTENT_SIZE=" $4

            for(i = 5; i <= NF; i++)
              SCHEMA_VARIABLES_INDEX_DBSPACES = SCHEMA_VARIABLES_INDEX_DBSPACES " " $i

            print "SCHEMA_VARIABLES_INDEX_DBSPACES=\"" SCHEMA_VARIABLES_INDEX_DBSPACES "\""
          }

        ' $TMP_FILE_1 > $TMP_FILE_2

        . $TMP_FILE_2
      else
        LOG "WARNING: No entry for table $TABLE_NAME in $SCHEMA_VARIABLES_FILE."
      fi
    fi

    DEBUG "9%"
    DEBUG_CAT $TMP_FILE_1
    DEBUG_CAT $TMP_FILE_2

    # Environment variables SCHEMA_VARIABLES_TABLE_DBSPACE,
    # SCHEMA_VARIABLES_INITIAL_EXTENT_SIZE, SCHEMA_VARIABLES_NEXT_EXTENT_SIZE,
    # and SCHEMA_VARIABLES_INDEX_DBSPACES, are now either set, or are not set.
    # These environment variables are passed to the awk script that parses the
    # TDEF file, and will be used to override any table and index dbspaces, and
    # initial and next extent sizes, set by the DBD file or the TDEF file.
    #
    # Environment variables dbd_dbspace, dbd_indexspace, dbd_extentsize, and
    # dbd_nextsize, are also either set or not set. These environment variables
    # are also passed to the awk script that parses the TDEF file, and are used
    # where neither the schema_variables file, nor the TDEF file, specify a
    # value for initial or next extent sizes, or table or index dbspaces.

    export TABLE_NAME
    export dbd_dbspace
    export dbd_indexspace
    export dbd_extentsize
    export dbd_nextsize
    export SCHEMA_VARIABLES_TABLE_DBSPACE
    export SCHEMA_VARIABLES_INITIAL_EXTENT_SIZE
    export SCHEMA_VARIABLES_NEXT_EXTENT_SIZE
    export SCHEMA_VARIABLES_INDEX_DBSPACES

    $M_AWK -F";" \
      -v tablesqlfile="$TMP_DIR/tdef_to_sql_table.sql" \
      -v extrasqlfile="$TMP_DIR/tdef_to_sql_extra.sql" \
      -v errorsfile="$TMP_DIR/tdef_to_sql_error.txt" '

      BEGIN {
        ORS = ""

         num_SCHEMA_VARIABLES_INDEX_DBSPACES = split(ENVIRON["SCHEMA_VARIABLES_INDEX_DBSPACES"], SCHEMA_VARIABLES_INDEX_DBSPACES_array, " ")

         SCHEMA_VARIABLES_INDEX_DBSPACES_counter = 1
      }

      { $1 = tolower($1) }

      # Already have table name, so no processng of table line in TDEF file
      # needed.
      #
      $1 ~ /table/ { next }

      $1 ~ /column/ {
        num_renames = split($2, column_renames, ",")

        latest_name = column_renames[num_renames]

        cols = cols "   " latest_name " " $3 ",\n"

        next
      }

      # primary key TDEF LINE IS DEPRECATED.
      #
      $1 ~ /primary key/ {
        cols = cols "   PRIMARY KEY " $2 ",\n"
	print "-- primary key TDEF LINE IS DEPRECATED\n" >> tablesqlfile

        next
      }

      $1 ~ /extent size/ {
        tdef_initial_extent_size = $2

        next
      }

      $1 ~ /next size/ {
        tdef_next_extent_size = $2

        next
      }

      $1 ~ /lock mode/ {
        lockmode = lockmode "\nLOCK MODE " $2

        next
      }

      $1 ~ /index/ {
        # Construct CREATE INDEX statement.
        #
        # If schema_variables file has been input, then use next index dbspace
        # in list of index dbspaces for this table, in order to construct the
        # storage options.
        #
        # If schema_variables file has not been input, but an index dbspace has
        # been specified in the DBD file, then use this index dbspace to
        # construct the storage options.
        #
        # If neither a schema_variables file has been input, nor an index
        # dbspace has been specified in the DBD file, then do not include
        # storage options in CREATE INDEX statement.
        #
	idx_renames = split($2, idx_history, ",")

        if (ENVIRON["SCHEMA_VARIABLES_INDEX_DBSPACES"] != "") {
          print "CREATE " $3 " INDEX " idx_history[idx_renames] " ON " ENVIRON["TABLE_NAME"] " " $4 " IN " SCHEMA_VARIABLES_INDEX_DBSPACES_array[SCHEMA_VARIABLES_INDEX_DBSPACES_counter] ";\n" >> extrasqlfile

          # Increment SCHEMA_VARIABLES_INDEX_DBSPACES_counter by 1, or reset to
          # 1 if end of SCHEMA_VARIABLES_INDEX_DBSPACES has been reached.
          #
          if (SCHEMA_VARIABLES_INDEX_DBSPACES_counter == num_SCHEMA_VARIABLES_INDEX_DBSPACES)
            SCHEMA_VARIABLES_INDEX_DBSPACES_counter = 1
          else
            SCHEMA_VARIABLES_INDEX_DBSPACES_counter++
        } else
          if (ENVIRON["dbd_indexspace"] != "")
            print "CREATE " $3 " INDEX " idx_history[idx_renames] " ON " ENVIRON["TABLE_NAME"] " " $4 " IN " ENVIRON["dbd_indexspace"] ";\n" >> extrasqlfile
          else
            print "CREATE " $3 " INDEX " idx_history[idx_renames] " ON " ENVIRON["TABLE_NAME"] " " $4 ";\n" >> extrasqlfile

        next
      }

      $1 ~ /constraint/ {
	cons_renames = split($2, cons_history, ",")

        print "ALTER TABLE " ENVIRON["TABLE_NAME"] " ADD CONSTRAINT (" $3 " constraint " cons_history[cons_renames] ");\n" >> extrasqlfile

        next
      }

      $1 ~ /revoke/ {
        print "REVOKE " $2 " ON " ENVIRON["TABLE_NAME"] " FROM " $3 ";\n" >> extrasqlfile

        next
      }

      $1 ~ /synonym/ {
        for (field = 2; field <= NF; field++) print "CREATE SYNONYM " $field " FOR " ENVIRON["TABLE_NAME"] ";\n" >> extrasqlfile

        next
      }

      { print "ERROR: SQL command not handled: " $2 "\n" >> errorsfile }

      END {
        # Decide which table and index dbspaces, and which initial and next
        # extent sizes to use.

        # Initial extent size.
        #
        if (ENVIRON["SCHEMA_VARIABLES_INITIAL_EXTENT_SIZE"] != "")
          extentsize = "\nEXTENT SIZE " ENVIRON["SCHEMA_VARIABLES_INITIAL_EXTENT_SIZE"]
        else
          if (tdef_initial_extent_size != "")
            extentsize = "\nEXTENT SIZE " tdef_initial_extent_size
          else
            if (ENVIRON["dbd_extentsize"] != "")
              extentsize = "\nEXTENT SIZE " ENVIRON["dbd_extentsize"]

        # Next extent size.
        #
        if (ENVIRON["SCHEMA_VARIABLES_NEXT_EXTENT_SIZE"] != "")
          nextsize = "\nNEXT SIZE " ENVIRON["SCHEMA_VARIABLES_NEXT_EXTENT_SIZE"]
        else
          if (tdef_next_extent_size != "")
            nextsize = "\nNEXT SIZE " tdef_next_extent_size
          else
            if (ENVIRON["dbd_nextsize"] != "")
              nextsize = "\nNEXT SIZE " ENVIRON["dbd_nextsize"]

        # Table dbspace.
        #
        if (ENVIRON["SCHEMA_VARIABLES_TABLE_DBSPACE"] != "")
          dbspace = "\nIN " ENVIRON["SCHEMA_VARIABLES_TABLE_DBSPACE"]
        else
          if (ENVIRON["dbd_dbspace"] != "")
            dbspace = "\nIN " ENVIRON["dbd_dbspace"]


        print "CREATE TABLE " ENVIRON["TABLE_NAME"] " (\n" >> tablesqlfile

	# Remove the last comma in cols.

	# Assuming that the last 2 characters in cols are a comma and a newline,
	# replace the last 2 characters in cols with a newline.
	#
        cols = substr(cols, 1, length(cols) - 2) "\n"

        print cols >> tablesqlfile

        print ")" >> tablesqlfile

        print "\n" dbspace >> tablesqlfile

        print extentsize >> tablesqlfile

        print nextsize >> tablesqlfile

        print lockmode >> tablesqlfile

        print ";" >> tablesqlfile
      }

    ' $TMP_DIR/tdef_to_sql.tdef

    # cat tablesqlfile and extrasqlfile together, and rename the file
    # $TABLE_NAME.sql.
    #
    cat $TMP_DIR/tdef_to_sql_table.sql $TMP_DIR/tdef_to_sql_extra.sql > $TMP_DIR/$TABLE_NAME.sql

    RETURN="$TMP_DIR/$TABLE_NAME.sql"

  else

    $M_AWK -v file=$tmpsqlfile -F";" '

      { $1 = tolower($1) }

      /^view/ {
        if ($3 ~ /[(]/)
           print "CREATE VIEW " $2 " " $3 " AS" > file
        else
	   if ($3 ~ /[a-zA-Z]/)
              print "CREATE VIEW " $2 " (" $3 ") AS" > file
	   else 
              print "CREATE VIEW " $2 " AS" > file
        next
      }

      { print $0 >> file }

    ' $TMP_DIR/tdef_to_sql.tdef

    RETURN="$tmpsqlfile"
  fi

  disk_log_string "TDEF converted to SQL:"
}

# Put each line in given file into an array. NB lines are assumed not to
# contain spaces.
#
function file_to_array
{
  FILE="$1"

  # Initialize RETURN.
  #
  set -A RETURN

  let LINECOUNT=0

  for LINE in `cat $FILE`
  do
    RETURN[$LINECOUNT]=$LINE
    LINECOUNT=`expr $LINECOUNT + 1`
  done
}

###################################################################
#
# Function    : tdef_to_delta_sql
#
# Description : Creates delta SQL from two given TDEF file
#               (i.e. SQL to upgrade table produced from the first
#               TDEF file, to the table produced from the second
#               TDEF file.)
#
###################################################################

function tdef_to_delta_sql
{
  # Get chronologically-sorted, unduplicated list of dates used in renaming
  # columns from given TDEF file.
  #
  # Output is file, $RETURN.
  #
  function get_tdef_dates
  {
    TDEF="$1"

    DEBUG "16% get_tdef_dates"
    DEBUG "16% TDEF:"
    DEBUG_CAT $TDEF

    get_tmp_file 9
    TMP_FILE_2="$RETURN"

    # Get second field of column lines out of TDEF
    # Get comma-delimited fields out
    # Get dates from comma-delimited fields.
    # Sort chronologically and remove duplicates.
    #
    grep -i ^column $TDEF | cut -d";" -f2 | $M_AWK -F"," '{for(i = 1; i <= NF; i++) print $i}' | cut -f2 -d"-" -s | sort -d -u -o $TMP_FILE_2

    RETURN="$TMP_FILE_2"
  }

  # Get list of column names, sorted in order that they appear in table.
  #
  # Output is file, $RETURN.
  function get_tdef_cols
  {
    TDEF="$1"

    DEBUG "19% get_tdef_cols"
    DEBUG "19% TDEF:"
    DEBUG_CAT $TDEF

    get_tmp_file 11
    TMP_FILE_2="$RETURN"

    # Get second field of column lines out of TDEF
    # Get last field from comma-delimited lines.
    #
    grep -i ^column $TDEF | cut -d";" -f2 | $M_AWK -F"," '{print $NF}' > $TMP_FILE_2

    DEBUG "19% TMP_FILE_2:"
    DEBUG_CAT $TMP_FILE_2

    RETURN="$TMP_FILE_2"
  }

  # Get list of column types, sorted in order of columns in table (i.e. first
  # line is type of first column, second line is type of second column, etc.).
  #
  # Output is file, $RETURN.
  #
  function get_tdef_coltypes
  {
    TDEF="$1"

    get_tmp_file 12
    TMP_FILE_1="$RETURN"

    # Get third field of column lines out of TDEF and place into another file.
    #
    $M_AWK -F";" '

      { $0 = tolower($0) }

      /^column/ {
        gsub(" ", "#", $3)
        print $3
      }

    ' $TDEF > $TMP_FILE_1

    RETURN="$TMP_FILE_1"
  }

  function get_tdef_lock_mode
  {
    TDEF="$1"

    get_tmp_file 13
    TMP_FILE_1="$RETURN"

    # Initialize RETURN.
    #
    set -A RETURN

    # Get second field of lock mode line out of TDEF.
    #
    $M_AWK -F";" '

      { $1 = tolower($1) }

      /^lock mode/ {
        print "RETURN=\"" $2 "\""
        exit
      }

    ' $TDEF > $TMP_FILE_1

    . $TMP_FILE_1
  }

  function get_tdef_revoke_sql
  {
    TDEF="$1"

    DEBUG "21% TDEF = $TDEF"

    get_tmp_file 14
    TMP_FILE_1="$RETURN"

    DEBUG "21%"
    DEBUG_CAT $TMP_FILE_1

    get_tdef_table $TDEF
    TAB_NAME="$RETURN"

    DEBUG "21% TAB_NAME = $TAB_NAME"

    $M_AWK -F";" -v table="$TAB_NAME" '

      { $1 = tolower($1) }

      /^revoke/ { print "REVOKE " $2 " ON " table " FROM " $3 ";" }

    ' $TDEF > $TMP_FILE_1

    RETURN="$TMP_FILE_1"
  }

  function get_tdef_constraints
  {
    TDEF="$1"

    get_tmp_file 15
    TMP_FILE_1="$RETURN"

    $M_AWK -F";" '

      { $1 = tolower($1) }

      /^constraint/ { print $2 }

    ' $TDEF > $TMP_FILE_1

    DEBUG "22%"
    DEBUG_CAT $TDEF
    DEBUG_CAT $TMP_FILE_1

    RETURN="$TMP_FILE_1"
  }

  function get_tdef_constraint_sql
  {
    TDEF="$1"

    get_tmp_file 16
    TMP_FILE_1="$RETURN"

    get_tdef_table $TDEF
    TAB_NAME="$RETURN"

    $M_AWK -F";" -v table="$TAB_NAME" '

      { $1 = tolower($1) }

      /^constraint/ { 
         cons_renames = split($2, cons_history, ",")

         print "ALTER TABLE " table " ADD CONSTRAINT (" $3 " CONSTRAINT " cons_history[cons_renames] ");" 
      }

    ' $TDEF > $TMP_FILE_1

    RETURN="$TMP_FILE_1"
  }

  function get_tdef_synonyms
  {
    TDEF="$1"

    get_tmp_file 17
    TMP_FILE_1="$RETURN"

    $M_AWK -F";" '

      { $1 = tolower($1) }

      /^synonym/ { for (field = 2; field <= NF; field++) print $field }

    ' $TDEF > $TMP_FILE_1

    RETURN="$TMP_FILE_1"
  }

  TDEF1="$1"
  TDEF2="$2"

  get_tmp_file 18
  TMPFILE="$RETURN"

  get_tmp_file 19
  SQL_FILE="$RETURN"

  # Old name of table is latest table name of TDEF1.
  #
  get_tdef_table $TDEF1
  OLD_TAB_NAME="$RETURN"

  # New name of table is latest table name of TDEF2.
  #
  get_tdef_table $TDEF2
  NEW_TAB_NAME="$RETURN"

  DEBUG "24% OLD_TAB_NAME = $OLD_TAB_NAME"
  DEBUG "24% NEW_TAB_NAME = $NEW_TAB_NAME"

  cat $TDEF_DIFFERENCES | while read LINE
  do
     case $LINE in
     "TABLE")

        #############
        # TABLE NAME
        #############

        $ECHO "-- Rename table." >> $SQL_FILE

        if [ $OLD_TAB_NAME != $NEW_TAB_NAME ]; then
          $ECHO "RENAME TABLE $OLD_TAB_NAME TO $NEW_TAB_NAME;" >> $SQL_FILE
        fi

        DEBUG "24% SQL_FILE - After table rename"
        DEBUG_CAT $SQL_FILE
     ;;

     "COLUMN")

        ##########
        # COLUMNS
        ##########

        # Initialize environment variables.
        #
        DEBUG "24% TDEF2_NEW_COLS1 = $TDEF2_NEW_COLS"
        set -A TDEF2_NEW_COLS
        DEBUG "24% TDEF2_NEW_COLS2 = $TDEF2_NEW_COLS"

        # Create space-delimited, chronologically ordered lists of dates used in TDEF1
        # and TDEF2 e.g.
        #   TDEF1_DATES: 02/12/15
        #   TDEF2_DATES: 02/12/15 03/01/28 03/02/20
        #
        get_tdef_dates $TDEF1
        TDEF1_DATES="$RETURN"

        DEBUG "24% TDEF1_DATES:"
        DEBUG_CAT $TDEF1_DATES

        get_tdef_dates $TDEF2
        TDEF2_DATES="$RETURN"

        DEBUG "24% TDEF2_DATES:"
        DEBUG_CAT $TDEF2_DATES

        # Delete dates in TDEF1_DATES from TDEF2_DATES e.g.
        #   TDEF2_DATES: 03/01/28 03/02/20
        #
        comm -13 $TDEF1_DATES $TDEF2_DATES > $TMPFILE
        mv $TMPFILE $TDEF2_DATES

        DEBUG "25% Have removed dates from TDEF2_DATES:"
        DEBUG_CAT $TDEF2_DATES

        # Convert TDEF1_DATES and TDEF2_DATES into arrays.
        #
        file_to_array $TDEF1_DATES
        TDEF1_DATES="${RETURN[*]}"

        file_to_array $TDEF2_DATES
        TDEF2_DATES="${RETURN[*]}"
        
        DEBUG "25% TDEF1_DATES=$TDEF1_DATES"
        DEBUG "25% TDEF2_DATES=$TDEF2_DATES"

        # Construct string containing space-delimited
        # list of columns in TDEF1. E.g.
        #   TDEF1_COLS: b a c
        #
        get_tdef_cols $TDEF1
        TDEF1_COLS="$RETURN"

        file_to_array $TDEF1_COLS
        TDEF1_COLS="${RETURN[*]}"

        # Get list of types of columns in TDEF1. E.g.
        #   TDEF1_COLTYPES: integer integer float
        #
        get_tdef_coltypes $TDEF1
        TDEF1_COLTYPES="$RETURN"

        file_to_array $TDEF1_COLTYPES
        TDEF1_COLTYPES="${RETURN[*]}"

        DEBUG "25% TDEF1_COLTYPES=$TDEF1_COLTYPES"

        # Take a copy of TDEF1_COLS and call it TDEF2_COLS. E.g.
        #   TDEF2_COLS: b a c
        #
        TDEF2_COLS=$TDEF1_COLS

        # Take a copy of TDEF1_COLTYPES and call it TDEF2_COLYTPES. E.g.
        #   TDEF2_COLTYPES: integer integer float
        #
        TDEF2_COLTYPES=$TDEF1_COLTYPES

        DEBUG "26% TDEF1_COLTYPES = $TDEF1_COLTYPES"
        DEBUG "26% TDEF2_COLTYPES = $TDEF2_COLTYPES"

        get_tmp_file 20
        RENAMED_COLS="$RETURN"

        # For each date in the list of dates in TDEF2_DATES (starting from the
        # earliest date),
        #
        for TDEF_DATE in $TDEF2_DATES
        do

          DEBUG "26% TDEF_DATE = $TDEF_DATE"

          $ECHO > $RENAMED_COLS

          # Get old and new column names at date of rename, and latest column name,
          # and type, from TDEF2.
          #
          export TDEF_DATE

          $M_AWK -F";" '

            { $1 = tolower($1) }

            /^column/ && $2 ~ ENVIRON["TDEF_DATE"] {

              # num_renames is number of times this column has been renamed, indicated
              # by the number of comma-separated fields in the second field of the
              # semi-colon separated fields.
              #
              num_renames = split($2, rename_history, ",")

              # Find the field for which the column was renamed at this date.
              #
              for (rename in rename_history) {
                if (rename_history[rename] ~ ENVIRON["TDEF_DATE"]) {
                  # Text in field, before dash, is old column name.
                  #
                  dash_pos = match(rename_history[rename], "-")

                  old_col_name = substr(rename_history[rename], 0, dash_pos - 1)

                  # Text in next field, before dash, is new column name.
                  #
                  rename++

                  dash_pos = match(rename_history[rename], "-")

                  if (dash_pos > 0)
                    new_col_name = substr(rename_history[rename], 0, dash_pos - 1)
                  else
                    new_col_name = rename_history[rename]

                  break
                }
              }

              # Latest field is assumed to not have a rename in it.
              #
              latest_col_name = rename_history[num_renames]

              col_type = $3
              # replace spaces with a # in the column type field, so that output can
              # be processed line-by-line with a for command.
              #
              gsub(" ", "\#", col_type)

              print old_col_name ";" new_col_name ";" latest_col_name ";" col_type
            }

          ' $TDEF2 > $RENAMED_COLS

          DEBUG "28% RENAMED_COLS:"
          DEBUG_CAT $RENAMED_COLS

          # For each column renamed with date in TDEF2,
          #
          for RENAMED_COL in `cat $RENAMED_COLS`
          do

            # RENAMED_COL looks something like:
            #   cell;cell_id;cell_id;varchar(30)#not#null

            # Put values of fields into environment variables.
            #
            $ECHO "$RENAMED_COL" | $M_AWK -F";" '

              {
                print "OLD_COL_NAME=\"" $1 "\""
                print "NEW_COL_NAME=\"" $2 "\""
                print "LATEST_COL_NAME=\"" $3 "\""
                print "COL_TYPE=\"" $4 "\""
              }

            ' > $TMPFILE

            DEBUG "28%"
            DEBUG_CAT $TMPFILE

            . $TMPFILE

            # Now have environment variables OLD_COL_NAME, NEW_COL_NAME,
            # LATEST_COL_NAME, and COL_TYPE set to values such as:
            #   OLD_COL_NAME="cell"
            #   NEW_COL_NAME="cell_id"
            #   LATEST_COL_NAME="cell_id"
            #   COL_TYPE="varchar(30)#not#null"

            DEBUG "29% TDEF2_COLS = $TDEF2_COLS"
            DEBUG "29% TDEF2_COLTYPES = $TDEF2_COLTYPES"
            DEBUG "29% OLD_COL_NAME = $OLD_COL_NAME"
            DEBUG "29% NEW_COL_NAME = $NEW_COL_NAME"
            DEBUG "29% LATEST_COL_NAME = $LATEST_COL_NAME"
            DEBUG "29% COL_TYPE = $COL_TYPE"

            # If OLD_COL_NAME is present in TDEF2_COLS, then
            #   Rename OLD_COL_NAME to NEW_COL_NAME in TDEF2_COLS.
            #     NEW_COL_NAME in TDEF2_COLS is prefixed with a
            #     'rename-' (prefix removed afterwards).
            #   Update type in TDEF2_COLTYPES to be COL_TYPE.
            # Otherwise,
            #   Add NEW_COL_NAME to TDEF2_NEW_COLS.
            #
            export TDEF2_COLTYPES
            export OLD_COL_NAME
            export NEW_COL_NAME
            export LATEST_COL_NAME
            export COL_TYPE
            export TDEF2_NEW_COLS

            $ECHO "$TDEF2_COLS" | $M_AWK ' 
              BEGIN {
                new_TDEF2_COLS = ""
                new_TDEF2_COLTYPES = ""
                is_new_col = "TRUE"
              }

              {
                num_TDEF2_COLS_array     = split($0, TDEF2_COLS_array, " ")
                num_TDEF2_COLTYPES_array = split(ENVIRON["TDEF2_COLTYPES"], TDEF2_COLTYPES_array, " ")

                # NB Elements in array are processed from right to left.
                #
                for (col=num_TDEF2_COLS_array ; col>=1 ; col--) {
                  if (TDEF2_COLS_array[col] == ENVIRON["OLD_COL_NAME"]) {
                    is_new_col = "FALSE"

                    new_TDEF2_COLS = "rename-" ENVIRON["NEW_COL_NAME"] " " new_TDEF2_COLS
                    new_TDEF2_COLTYPES = ENVIRON["COL_TYPE"] " " new_TDEF2_COLTYPES
                  } else {
                    new_TDEF2_COLS = TDEF2_COLS_array[col] " " new_TDEF2_COLS
                    new_TDEF2_COLTYPES = TDEF2_COLTYPES_array[col] " " new_TDEF2_COLTYPES
                  }
                }
              }

              END {
                print "TDEF2_COLS=\"" new_TDEF2_COLS "\""
                print "TDEF2_COLTYPES=\"" new_TDEF2_COLTYPES "\""

                if (is_new_col == "TRUE") {
                  split(ENVIRON["TDEF2_NEW_COLS"], TDEF2_NEW_COLS_array, " ")

                  is_LATEST_COL_NAME_already_in_TDEF2_NEW_COLS = "FALSE"

                  for(col in TDEF2_NEW_COLS_array)
                    if (TDEF2_NEW_COLS_array[col] == ENVIRON["LATEST_COL_NAME"])
                      is_LATEST_COL_NAME_already_in_TDEF2_NEW_COLS = "TRUE"

                  if (is_LATEST_COL_NAME_already_in_TDEF2_NEW_COLS == "FALSE")
                    print "TDEF2_NEW_COLS=\"$TDEF2_NEW_COLS " ENVIRON["LATEST_COL_NAME"] "\""
                }
              }

            ' > $TMPFILE

            DEBUG "31% TDEF2_COLS:"
            DEBUG "31% TDEF2_COLTYPES:"
            DEBUG_CAT  $TMPFILE

            . $TMPFILE

          done

          DEBUG "31% TDEF2_COLS=$TDEF2_COLS"
          DEBUG "31% TDEF2_COLTYPES=$TDEF2_COLTYPES"

          # Any columns which have been dropped at this date, but which, at the same
          # time, another column was renamed to the name of the dropped column, will
          # now be present in TDEF2_COLS along with the renamed column which will
          # have the same name, but preceeded by a 'rename-'.
          #
          # For example, column a has been renamed to b, column b has been renamed to
          # c, and the original column c is to be dropped:
          #
          #   TDEF2_COLS: rename-b rename-c c

          # For each column name in TDEF2_COLS that is preceeded with a 'rename-',
          # search for the same column name in TDEF2_COLS that is not preceeded with a
          # 'rename-'. Any such column names are prefixed with 'drop-'.
          #
          $ECHO "$TDEF2_COLS" | $M_AWK '

            {
              num_columns = split($0, TDEF2_COLS_array, " ")

              for (column in TDEF2_COLS_array)
                # If TDEF2_COLS_array[column] is prefixed with a rename-.
                #
                if (TDEF2_COLS_array[column] ~ /^rename-/) {
                  actual_column_name = TDEF2_COLS_array[column]
                  sub("^rename-", "", actual_column_name)

#DEBUG#                  print "#DEBUG: actual_column_name = " actual_column_name

                  for (column2 in TDEF2_COLS_array)
                    # If TDEF2_COLS_array[column2] does not contain
                    # a - (i.e. does not begin with rename- or drop-).
                    #
                    if (TDEF2_COLS_array[column2] !~ /-/) {
#DEBUG#                      print "#DEBUG: TDEF2_COLS_array[column2] = " TDEF2_COLS_array[column2]

                      if (actual_column_name == TDEF2_COLS_array[column2])
                        TDEF2_COLS_array[column2] = "drop-" TDEF2_COLS_array[column2]
                    }
                }

              # Reconstruct TDEF2_COLS from TDEF2_COLS_array.
              #
              # NB "for(i=lower; i<=upper; i++)" format used in order to process
              # TDEF2_COLS_array from left to right (i.e. from first column to last
              # column in table).
              #
              for (column = 1; column <= num_columns; column++)
                TDEF2_COLS_string = TDEF2_COLS_string TDEF2_COLS_array[column] " "

              print "TDEF2_COLS=\"" TDEF2_COLS_string "\""
            }

          ' > $TMPFILE

          DEBUG "32% TDEF2_COLS:"
          DEBUG_CAT $TMPFILE

          . $TMPFILE


          # Remove 'rename-' in front of column names in TDEF2_COLS.
          #
          NEW_TDEF2_COLS=`$ECHO $TDEF2_COLS | sed 's/rename-//g'`
          TDEF2_COLS="$NEW_TDEF2_COLS"

        done

        DEBUG "32% TDEF2_COLS=$TDEF2_COLS"
        DEBUG "32% TDEF2_COLTYPES=$TDEF2_COLTYPES"

        # Construct space-delimited list of columns in TDEF2. E.g.
        #   TDEF2_ACTUAL_COLS:  a b d
        #
        get_tdef_cols $TDEF2
        TDEF2_ALL_COLS="$RETURN"

        DEBUG "33% TDEF2_ALL_COLS:"
        DEBUG_CAT $TDEF2_ALL_COLS

        file_to_array $TDEF2_ALL_COLS
        TDEF2_ALL_COLS="${RETURN[*]}"

        DEBUG "33% TDEF2_ALL_COLS = "
        for  COL in $TDEF2_ALL_COLS
        do
           DEBUG $COL
        done

        get_tdef_coltypes $TDEF2
        TDEF2_ALL_COLTYPES="$RETURN"

        file_to_array $TDEF2_ALL_COLTYPES
        TDEF2_ALL_COLTYPES="${RETURN[*]}"

        # TDEF2_NEW_COLS only contains columns that have once been renamed (i.e.
        # columns added since TDEF1, but before TDEF2). Now add columns added in
        # TDEF2, that have never been renamed. These will be columns present in
        # TDEF2_ALL_COLS, but not in TDEF2_COLS or TDEF2_NEW_COLS.
        #
        # Also, TDEF2_COLTYPES has not been updated with the types of columns that
        # have not been renamed since TDEF1. If a column in TDEF2_ALL_COLS is found in
        # TDEF2_COLS, then update TDEF2_COLTYPES with the type of the column in
        # TDEF2_ALL_COLTYPES.
        # 
        DEBUG "34% TDEF2_ALL_COLTYPES=$TDEF2_ALL_COLTYPES"
        DEBUG "34% TDEF2_COLS=$TDEF2_COLS"
        DEBUG "34% TDEF2_COLS=$TDEF2_COLS"
        DEBUG "34% TDEF2_COLTYPES=$TDEF2_COLTYPES"
        DEBUG "34% TDEF2_NEW_COLS=$TDEF2_NEW_COLS"
        DEBUG "--"
        DEBUG "34% TDEF2_ALL_COLS=$TDEF2_ALL_COLS"
        DEBUG "34% NEW_TAB_NAME=$NEW_TAB_NAME"

        export TDEF2_ALL_COLTYPES
        export TDEF2_COLS
        export TDEF2_COLTYPES
        export TDEF2_NEW_COLS

        $ECHO "$TDEF2_ALL_COLS" | $M_AWK -v table="$NEW_TAB_NAME" '

          {
            split($0, TDEF2_ALL_COLS_array, " ")
            split(ENVIRON["TDEF2_ALL_COLTYPES"], TDEF2_ALL_COLTYPES_array, " ")
            split(ENVIRON["TDEF2_COLS"], TDEF2_COLS_array, " ")
            num_cols = split(ENVIRON["TDEF2_COLTYPES"], TDEF2_COLTYPES_array, " ")
            split(ENVIRON["TDEF2_NEW_COLS"], TDEF2_NEW_COLS_array, " ")

            for (col in TDEF2_ALL_COLS_array) {
#DEBUG#              print "#DEBUG: TDEF2_ALL_COLS = " $0
#DEBUG#              print "#DEBUG: TDEF2_ALL_COLS_array[col] = " TDEF2_ALL_COLS_array[col]

              column_found = "FALSE"

              for (col2 in TDEF2_COLS_array)
                if (TDEF2_ALL_COLS_array[col] == TDEF2_COLS_array[col2]) {
                  column_found = "TRUE"

                  # Update type in TDEF2_COLS if necessary.
                  #
                  TDEF2_COLTYPES_array[col2] = TDEF2_ALL_COLTYPES_array[col]

                  break
                }

              if (column_found != "TRUE") {
                for (col2 in TDEF2_NEW_COLS_array)
                  if (TDEF2_ALL_COLS_array[col] == TDEF2_NEW_COLS_array[col2]) {
                    column_found = "TRUE"

                    break
                  }

                if (column_found != "TRUE")
                  # A column in TDEF2_ALL_COLS has been found
                  # that is not in TDEF2_COLS or TDEF2_NEW_COLS.
                  #
                  print "TDEF2_NEW_COLS=\"$TDEF2_NEW_COLS " TDEF2_ALL_COLS_array[col] "\""
              }
            }

            # Reconstruct TDEF2_COLTYPES from TDEF2_COLTYPES_array.
            # NB "for(i=lower; i<=upper; i++)" format used in order to process
            # TDEF2_COLTYPES_array from left to right (i.e. from first column to last
            # column in table).
            #
            for (col = 1; col <= num_cols; col++)
              TDEF2_COLTYPES_string = TDEF2_COLTYPES_string TDEF2_COLTYPES_array[col] " "

            print "TDEF2_COLTYPES=\"" TDEF2_COLTYPES_string "\""
          }

        ' > $TMPFILE

        DEBUG "35% TDEF2_NEW_COLS:"
        DEBUG "35% TDEF2_COLTYPES:"
        DEBUG_CAT $TMPFILE

        . $TMPFILE

        # TDEF1_COLS, TDEF1_COLTYPES, TDEF2_COLS, TDEF2_COLTYPES, and TDEF2_NEW_COLS
        # now look something like:
        #   TDEF1_COLS:         b a c
        #   TDEF1_COLTYPES:     integer#not#null integer float
        #   TDEF2_COLS:         a b c
        #   TDEF2_COLTYPES:     float#not#null integer integer
        #   TDEF2_NEW_COLS:     d

        DEBUG "36% TDEF1_COLS = $TDEF1_COLS"
        DEBUG "36% TDEF1_COLTYPES = $TDEF1_COLTYPES"
        DEBUG "36% TDEF2_COLS = $TDEF2_COLS"
        DEBUG "36% TDEF2_COLTYPES = $TDEF2_COLTYPES"
        DEBUG "36% TDEF2_NEW_COLS = $TDEF2_NEW_COLS"

        # For each column in TDEF1_COLS and TDEF2_COLS,
        #   If column name is different,
        #     Create SQL to rename column in TDEF1_COLS to name in TDEF2_COLS,
        #     prefixed with a '-'.
        #   If type in TDEF1_COLTYPES is different from TDEF2_COLTYPES,
        #     Create SQL to change the type of the column to that of TDEF2_COLTYPES.
        #
        # Create SQL to rename columns prefixed with a '_' to remove the '_'.
        #
        export TDEF1_COLTYPES
        export TDEF2_COLS
        export TDEF2_COLTYPES

        $ECHO "$TDEF1_COLS" | $M_AWK -v table="$NEW_TAB_NAME" '

          {
            num_TDEF1_COLS_array     = split($0, TDEF1_COLS_array, " ")
            num_TDEF1_COLTYPES_array = split(ENVIRON["TDEF1_COLTYPES"], TDEF1_COLTYPES_array, " ")
            num_TDEF2_COLS_array     = split(ENVIRON["TDEF2_COLS"], TDEF2_COLS_array, " ")
            num_TDEF2_COLTYPES_array = split(ENVIRON["TDEF2_COLTYPES"], TDEF2_COLTYPES_array, " ")

            print "-- Rename columns (prefixing new column names with an underscore)."

            #
            # Ensure the renames occur in the same order both HP and Solaris
            #
            for (col=1 ; col<=num_TDEF1_COLS_array ; col++)
              # If column name is different,
              #
              if (TDEF2_COLS_array[col] ~ /^drop-/) {
                # drop- is five characters long, so name of column to drop is
                # TDEF2_COLS_array[col] from the sixth character onwards.
                #
                actual_column_name = substr(TDEF2_COLS_array[col], 6)

                print "ALTER TABLE " table " DROP " actual_column_name ";"
              } else {
                if (TDEF1_COLS_array[col] != TDEF2_COLS_array[col])
                  # Create SQL to rename column in TDEF1_COLS to name in TDEF2_COLS,
                  # prefixed with an underscore.
                  #
                  print "RENAME COLUMN " table "." TDEF1_COLS_array[col] " TO _" TDEF2_COLS_array[col] ";"
              }

            print "-- Remove temporarily prefixed underscores."

            # Remove the temporarily prefixed '_' on renamed columns.
            # Ensure the renames occur in the same order both HP and Solaris
            #
            for (col=1 ; col<=num_TDEF2_COLS_array ; col++)
              if (TDEF2_COLS_array[col] ~ /^drop-/)
                continue
              else
                if (TDEF1_COLS_array[col] != TDEF2_COLS_array[col])
                  print "RENAME COLUMN " table "._" TDEF2_COLS_array[col] " TO " TDEF2_COLS_array[col] ";"

            print "-- Modify column types."

            for (col in TDEF1_COLS_array)
              if (TDEF2_COLS_array[col] ~ /^drop-/)
                continue
              else
                # If type in TDEF1_COLTYPES is different from TDEF2_COLTYPES,
                #
                if (TDEF1_COLTYPES_array[col] != TDEF2_COLTYPES_array[col]) {
                  # Create SQL to change the type of the column to that of
                  # TDEF2_COLTYPES.

                  # Remove hashes from column type string.
                  #
                  gsub("#", " ", TDEF2_COLTYPES_array[col])

                  print "ALTER TABLE " table " MODIFY (" TDEF2_COLS_array[col] " " TDEF2_COLTYPES_array[col] ");"
                }
          }

        ' >> $SQL_FILE

        DEBUG "37% SQL_FILE - After column renames, modify types"
        DEBUG_CAT $SQL_FILE

        # Construct SQL to delete columns in TDEF2_COLS that are not in
        # TDEF2_ALL_COLS, and construct SQL to create new columns listed in
        # TDEF2_NEW_COLS, using TDEF2_ALL_COLS to determine column positions.
        #

        DEBUG "38% TDEF2_ALL_COLS = $TDEF2_ALL_COLS"
        DEBUG "38% TDEF2_ALL_COLTYPES = $TDEF2_ALL_COLTYPES"
        DEBUG "38% TDEF2_NEW_COLS = $TDEF2_NEW_COLS"
        DEBUG "38% TDEF2_COLS = $TDEF2_COLS"

        export TDEF2_ALL_COLS
        export TDEF2_ALL_COLTYPES
        export TDEF2_NEW_COLS
        export TDEF2_COLS

        $ECHO "" | $M_AWK -v table="$NEW_TAB_NAME" '

          {
            num_tdef2_all_cols     = split(ENVIRON["TDEF2_ALL_COLS"], TDEF2_ALL_COLS_array, " ")
            num_tdef2_all_coltypes = split(ENVIRON["TDEF2_ALL_COLTYPES"], TDEF2_ALL_COLTYPES_array, " ")
            num_tdef2_new_cols     = split(ENVIRON["TDEF2_NEW_COLS"], TDEF2_NEW_COLS_array, " ")
            num_tdef2_cols         = split(ENVIRON["TDEF2_COLS"], TDEF2_COLS_array, " ")

            print "-- Delete old columns."

            for (col in TDEF2_COLS_array) {
              column_found = "FALSE"

              if (TDEF2_COLS_array[col] ~ /^drop-/)
                column_found = "TRUE"

              for (col2 in TDEF2_ALL_COLS_array)
                if (TDEF2_COLS_array[col] == TDEF2_ALL_COLS_array[col2])
                  column_found = "TRUE"

              if (column_found == "FALSE")
                # A column has been found that is listed in TDEF2_COLS, but not in
                # TDEF2_ALL_COLS, so it must be an old column to be deleted.
                #
                print "ALTER TABLE " table " DROP " TDEF2_COLS_array[col] ";"
            }

            print "-- Add new columns."

            # This algorithm depends on the fact that the for command iterates through
            # an array from right to left (i.e. from last column to first column of
            # table).
            #
            for (col=num_tdef2_all_cols ; col>=1 ; col--) {
              for (col2=num_tdef2_new_cols ; col2>=1 ; col2--) {
                if (TDEF2_ALL_COLS_array[col] == TDEF2_NEW_COLS_array[col2]) {
                  # Remove hashes from column type string.
                  #
                  gsub("#", " ", TDEF2_ALL_COLTYPES_array[col])

                  print "ALTER TABLE " table " ADD (" TDEF2_ALL_COLS_array[col] " " TDEF2_ALL_COLTYPES_array[col] BEFORE_clause ");"
                }
              }
              BEFORE_clause = " BEFORE " TDEF2_ALL_COLS_array[col]
            }
          }

        ' >> $SQL_FILE

        DEBUG "39% SQL_FILE:"
        DEBUG_CAT $SQL_FILE
     ;;

        ##############
        # PRIMARY KEY
        ##############

        # Primary key is not updated.

        ##############
        # EXTENT SIZE
        ##############

        # Extent size is not updated.

        ############
        # NEXT SIZE
        ############

        # Next extent size is not updated.

     "LOCK MODE")

        ############
        # LOCK MODE
        ############

        get_tdef_lock_mode $TDEF1
        TDEF1_LOCK_MODE="$RETURN"

        get_tdef_lock_mode $TDEF2
        TDEF2_LOCK_MODE="$RETURN"

        DEBUG "40% TDEF1_LOCK_MODE = \"$TDEF1_LOCK_MODE\""
        DEBUG "40% TDEF2_LOCK_MODE = \"$TDEF2_LOCK_MODE\""

        $ECHO "-- Change lock mode." >> $SQL_FILE

        if [[ -n $TDEF2_LOCK_MODE && $TDEF1_LOCK_MODE != $TDEF2_LOCK_MODE ]]; then
          $ECHO "ALTER TABLE $NEW_TAB_NAME LOCK MODE ($TDEF2_LOCK_MODE);" >> $SQL_FILE
        fi
     ;;

     "INDEX")

        ########
        # INDEX
        ########

        get_tmp_file 21
        TDEF1_INDEXES="$RETURN"

        get_tmp_file 22
        TDEF2_INDEXES="$RETURN"

        # Get index lines from TDEF1 and TDEF2.
        # Remove spaces so that lines in files can be processed with a for command.
        #
        grep -i "^index" $TDEF1 | sed 's/ /#/g' > $TDEF1_INDEXES
        grep -i "^index" $TDEF2 | sed 's/ /#/g' > $TDEF2_INDEXES

        DEBUG "41% TDEF1_INDEXES:"
        DEBUG_CAT $TDEF1_INDEXES

        DEBUG "41% TDEF2_INDEXES:"
        DEBUG_CAT $TDEF2_INDEXES

        # Get index lines that are in TDEF1 but not in TDEF2.
        #
        get_tmp_file 23
        INDEXES_TO_DROP="$RETURN"

        comm -23 $TDEF1_INDEXES $TDEF2_INDEXES > $INDEXES_TO_DROP

        # Construct SQL to drop indexes in INDEXES_TO_DROP.
        #
        get_tmp_file 24
        DROP_INDEX_SQL="$RETURN"

        $ECHO "-- Drop old indexes." >> $SQL_FILE

        $M_AWK -F";" '{print "DROP INDEX " $2 ";"}' $INDEXES_TO_DROP > $DROP_INDEX_SQL

        DEBUG "42% SQL_FILE:"
        DEBUG_CAT $SQL_FILE

        DEBUG "42% DROP_INDEX_SQL:"
        DEBUG_CAT $DROP_INDEX_SQL

        cat $SQL_FILE $DROP_INDEX_SQL > $TMPFILE
        mv $TMPFILE $SQL_FILE

        DEBUG "42% new SQL_FILE:"
        DEBUG_CAT $SQL_FILE

        $ECHO "-- Create latest indexes." >> $SQL_FILE

        # Create list of dbspaces - one for each index in TDEF2_INDEXES.
        #
        get_tmp_file 25
        INDEX_DBSPACES="$RETURN"

        # If there is a schema_variables file, then use dbspaces in schema_variables
        # file. If there are more indexes in TDEF2_INDEXES than there are index
        # dbspaces in schema_variables file, then use reuse index dbspaces.
        #
        DEBUG "43% SCHEMA_VARIABLES_FILE = $SCHEMA_VARIABLES_FILE"

        set -A IS_SCHEMA_VARIABLES_ENTRY

        if [ "$SCHEMA_VARIABLES_FILE" ]; then
          SCHEMA_VARIABLES_LINE=`grep -i "^$NEW_TAB_NAME|" "$SCHEMA_VARIABLES_FILE"`

          if [ $? = 0 ]; then
            IS_SCHEMA_VARIABLES_ENTRY="TRUE"
          else
            LOG "WARNING: No entry for table $NEW_TAB_NAME in $SCHEMA_VARIABLES_FILE."
          fi
        fi

        if [ "$IS_SCHEMA_VARIABLES_ENTRY" ]; then
          export SCHEMA_VARIABLES_LINE

          $M_AWK '

            BEGIN { i = 5 }

            {
              num_fields = split(ENVIRON["SCHEMA_VARIABLES_LINE"], SCHEMA_VARIABLES_LINE_array, "|")

              print SCHEMA_VARIABLES_LINE_array[i]

              if (i == num_fields)
                i = 5
              else
                i++
            }

          ' $TDEF2_INDEXES > $INDEX_DBSPACES

        # If there isn't a schema_variables file, then use existing dbspaces of
        # indexes for this table.
        #
        # If there are more indexes in TDEF2 than there are in TDEF1, then use index
        # dbspace specified in DBD file for remaining indexes.
        #
        # If there isn't an index dbspace specified in DBD file, then do not add any
        # more index dbspaces.
        #
        else
          # Get index names from index lines, and place into an SQL query list
          # (brackets added in query) e.g.
          #
          #   "index1", "index2", "index3"
          #
          INDEX_LIST=`$M_AWK -F";" '

            { $1 = tolower($1) }

            /^index/ { indexes = indexes " " $2 }

            END {
              num_indexes = split(indexes, index_array, " ")

              for (i = 1; i <= num_indexes; i++) {
                output = output "\"" index_array[i] "\""

#DEBUG#                print "(DEBUG: i = " i "; num_indexes = " num_indexes ";)"

                if (i < num_indexes)
                  output = output ", "
              }

              print output
            }

          ' $TDEF1_INDEXES`

          # SQL syntax requires list to be two double quotes rather than a zero length
          # string.
          #
          [ -z "$INDEX_LIST" ] && INDEX_LIST="\"\""

          DEBUG "44% INDEX_LIST = $INDEX_LIST"

          # Query database to be index dbspaces.
          #
          query_string "output to '$TMPFILE'
                        select dbspace
                          from sysfragments
                         where indexname in ($INDEX_LIST);" $DATABASE $METRICA_DB_USERNAME $METRICA_DB_PASSWORD

          # TMPFILE looks something like:
          #
          #   dbspace  indexspace1
          #
          #   dbspace  indexspace2
          #
          #   dbspace  indexspace3
          #
          $M_AWK 'NF == 2 {print $2}' $TMPFILE > $INDEX_DBSPACES
        fi

        # INDEX_DBSPACES now looks something like:
        #
        #   indexspace1
        #   indexspace2
        #   indexspace3

        DEBUG "45% INDEX_DBSPACES:"
        DEBUG_CAT $INDEX_DBSPACES

        # For all idex lines that are present in TDEF2_INDEXES but not in
        # TDEF1_INDEXES,
        #
        file_to_array $TDEF1_INDEXES
        TDEF1_INDEXES="${RETURN[*]}"

        file_to_array $INDEX_DBSPACES
        INDEX_DBSPACES="${RETURN[*]}"

        DEBUG "45% calling awk to CREATE INDEX:"
        DEBUG "$TDEF1_INDEXES"
        DEBUG "$INDEX_DBSPACES"
        DEBUG "$dbd_indexspace"
        DEBUG "$NEW_TAB_NAME"

        export TDEF1_INDEXES
        export INDEX_DBSPACES
        export dbd_indexspace
        export NEW_TAB_NAME

        $M_AWK  '

          BEGIN {
            split(ENVIRON["TDEF1_INDEXES"], TDEF1_INDEXES_array, " ")
            num_dbspaces = split(ENVIRON["INDEX_DBSPACES"], INDEX_DBSPACES_array, " ")
          }

          {
            index_found = "FALSE"

            for (i in TDEF1_INDEXES_array) {
#DEBUG#              print "--DEBUG: TDEF1_INDEXES_array[i] = " TDEF1_INDEXES_array[i]
#DEBUG#              print "--DEBUG: $0 = " $0
#DEBUG#              hello = $0
              if (TDEF1_INDEXES_array[i] == $0)
                index_found = "TRUE"
            }

            if (index_found == "TRUE") next

            gsub("#", " ")

            split($0, tdef2_index_line, ";")
            num_renames = split(tdef2_index_line[2], rename_history, ",")

            if (NR <= num_dbspaces)
              print "CREATE " tdef2_index_line[3] " INDEX " rename_history[num_renames] " ON " ENVIRON["NEW_TAB_NAME"] " " tdef2_index_line[4] " IN " INDEX_DBSPACES_array[NR] ";"
            else
              if (ENVIRON["dbd_indexspace"] != "")
                print "CREATE " tdef2_index_line[3] " INDEX " rename_history[num_renames] " ON " ENVIRON["NEW_TAB_NAME"] " " tdef2_index_line[4] " IN " ENVIRON["dbd_indexspace"] ";"
              else
                print "CREATE " tdef2_index_line[3] " INDEX " rename_history[num_renames] " ON " ENVIRON["NEW_TAB_NAME"] " " tdef2_index_line[4] ";"
          }

        ' $TDEF2_INDEXES >> $SQL_FILE

        DEBUG "47% SQL_FILE:"
        DEBUG_CAT $SQL_FILE
     ;;

     "CONSTRAINT")

        #############
        # CONSTRAINT
        #############

        # Drop all constraints in TDEF1.
        #
        $ECHO "-- Drop existing constraints." >> $SQL_FILE

        get_tdef_constraints $TDEF1
        TDEF1_CONSTRAINTS="$RETURN"

        for CONSTRAINT in `cat $TDEF1_CONSTRAINTS`
        do
          $ECHO "ALTER TABLE $OLD_TAB_NAME DROP CONSTRAINT $CONSTRAINT;" >> $SQL_FILE
        done

        # Add all constraints in TDEF2.
        #
        $ECHO "-- Add latest constraints." >> $SQL_FILE

        get_tdef_constraint_sql $TDEF2
        TDEF2_CONSTRAINT_SQL="$RETURN"

        cat $SQL_FILE $TDEF2_CONSTRAINT_SQL > $TMPFILE
        mv $TMPFILE $SQL_FILE
     ;;

     "REVOKE")

        #########
        # REVOKE
        #########

        # Apply all revoke statements in TDEF2.
        #
        $ECHO "-- Revoke privileges." >> $SQL_FILE

        get_tdef_revoke_sql $TDEF2
        REVOKE_SQL="$RETURN"

        cat $SQL_FILE $REVOKE_SQL > $TMPFILE
        mv $TMPFILE $SQL_FILE
     ;;

     "SYNONYM")

        ##########
        # SYNONYM
        ##########

        # Drop all synonyms in TDEF1.
        #
        $ECHO "-- Drop existing synonyms." >> $SQL_FILE

        get_tdef_synonyms $TDEF1
        TDEF1_SYNONYMS="$RETURN"

        for SYNONYM in `cat $TDEF1_SYNONYMS`
        do
          $ECHO "DROP SYNONYM $SYNONYM;" >> $SQL_FILE
        done

        # Add all synonyms in TDEF2.
        #
        $ECHO "-- Add latest synonyms." >> $SQL_FILE

        get_tdef_synonyms $TDEF2
        TDEF2_SYNONYMS="$RETURN"

        DEBUG "48% TDEF2_SYNONYMS"
        DEBUG_CAT $TDEF2_SYNONYMS

        for SYNONYM in `cat $TDEF2_SYNONYMS`
        do
          $ECHO "CREATE SYNONYM $SYNONYM FOR $NEW_TAB_NAME;" >> $SQL_FILE
        done
     ;;
     esac
  done

  RETURN="$SQL_FILE"
}

###################################################################
#
# Function    : CREATE_go
#
# Description : Creates DB if necessary, registers BLADE if set,
#               creates based on dbd file and directory files.
#
###################################################################

function CREATE_go
{
  # Try to create performance DB - if successful, try to register Blade
  if $ECHO "" | dbaccess $DATABASE > /dev/null 2>&1; then
    LOG;
  else
    if [ -z "$dbd_dbspace" ]; then
       LOG "create database $DATABASE;"
       dbaccess - >> $METRICA_DIR/etc/install.log 2>&1 << END
create database $DATABASE;
END
    else
       LOG "create database $DATABASE in $dbd_dbspace;"
       dbaccess - >> $METRICA_DIR/etc/install.log 2>&1 << END
create database $DATABASE in $dbd_dbspace;
END
    fi

    if [ $? != "0" ]; then
      LOG "Return code $exit_rc_usage :  Failed to create database $DATABASE"
    else
      LOG "Database $DATABASE created"
    fi
   
    if [  "$METRICA_BLADE_NAME" != "" ]; then
      LOG "register $METRICA_BLADE_NAME $DATABASE"
      get_tmp_file 26
      (blademgr | tee $RETURN) >> $METRICA_DIR/etc/install.log 2>&1 << END
register $METRICA_BLADE_NAME $DATABASE

exit
END
      if [ `cat $RETURN | grep -ic "was successfully registered in database"` -gt 0 ]; then
        LOG "Registered datablade $METRICA_BLADE_NAME"
      else
        LOG "Failed to register datablade $METRICA_BLADE_NAME"
      fi
    else
      LOG "No METRICA_BLADE_NAME set, so blade will not be registered"
    fi
  fi

  if [ -z "$DBD_FILE" ]; then
    for x in `ls $DIRECTORY/*.sql 2> /dev/null`
    do
      create_common $x $DATABASE
    done 
    for x in `ls $DIRECTORY/*.tdef 2> /dev/null`
    do
      tdef_to_sql $x
      x="$RETURN"

      create_common $x $DATABASE
    done
  else
    for x in ${tables[*]}
    do
      if [ ! -f $DIRECTORY/$x.@(sql|tdef) ]; then
        LOG "Return code $exit_rc_file_incorrect :  $x.sql or $x.tdef in $DIRECTORY not found"
      else
        # Set x.
        # x should be an SQL file.
        if [ -f $DIRECTORY/$x.sql ]; then
          x="$DIRECTORY/$x.sql"
        else # So there must be a file $DIRECTORY/$x.tdef.
          # Convert TDEF to SQL.
          tdef_to_sql $DIRECTORY/$x.tdef
          x="$RETURN"
        fi

        if [ "$dbd_user" ]; then
          create_common $x $DATABASE $dbd_user $dbd_pword
        else
          create_common $x $DATABASE
        fi
      fi
    done 
  fi
}

###################################################################
#
# Function    : CLEAR_go
#
# Description : If a .dbd file is not present, then all rows are
#               deleted from the tables of any SQL files present,
#               where table name is filename without extension.
#               If .dbd file is present, then data in EIXD files
#               listed in .dbd file, is deleted from the database.
#
###################################################################

function CLEAR_go
{
  # If a .dbd file is not present.
  #
  if [ -z "$DBD_FILE" ]; then
    for x in `ls $DIRECTORY/*.sql 2> /dev/null`
    do
      tabname=`basename $x .sql`
      if [ `cat $x | grep -ic "create table"` -gt 0 ]; then
        LOG "Found \"create table\" SQL statement: `basename $x`" 
      else
        LOG "$x: Unsupported SQL command" 
        continue
      fi

      LOG "Clearing table: $tabname"
      query_string "delete from $tabname" $DATABASE

      if [ $? != "0" ]; then
        LOG "Failed to clear table $tabname"
      else
        LOG "Table $tabname cleared"
      fi    
    done
  else
  # If a .dbd file is present.
  #
    for file in ${data[*]}
    do
      if [ ! -f $DIRECTORY/$file.eixd ]; then
        LOG "Return code $exit_rc_file_incorrect :  $file.eixd in $DIRECTORY not found"
      else
        get_eixd_table $DIRECTORY/$file.eixd
        TABLE_NAME="$RETURN"

        LOG "Deleting data in $file.eixd from $TABLE_NAME in $DATABASE..."

        get_tmp_file 27
        UNIQUE_COLUMNS="$RETURN"

        get_tmp_file 28
        TMPFILE="$RETURN"

        case $TABLE_NAME in
          "offline_tables")
            # offline_tables does not have unique columns, so the column to use
            # in the WHERE clause of the DELETE statement is hardcoded here.
            # This code can be removed once this table is given unique columns.
            #
            $ECHO "1" > $UNIQUE_COLUMNS 
	    ;;
          "lc_relations")
            # lc_relations does not have unique columns, so the columns to use
            # in the WHERE clause of the DELETE statement are hardcoded here.
            # This code can be removed once this table is given unique columns.
            #
            $ECHO "1" > $UNIQUE_COLUMNS
            $ECHO "2" >> $UNIQUE_COLUMNS
            $ECHO "3" >> $UNIQUE_COLUMNS
            $ECHO "4" >> $UNIQUE_COLUMNS 
	    ;;
          *)
            # Get value of sysindices.indexkeys for the table.
            #
            get_tmp_file 29
            INDEXKEYS_PLAIN_TEXT="$RETURN"

            query_string "output to '$INDEXKEYS_PLAIN_TEXT'
                          select indexkeys
                            from sysindices
                           where tabid = (select tabid
                                            from systables
                                           where tabname = '$TABLE_NAME') 
                             and idxtype = 'U';" $DATABASE $dbd_user $dbd_pword

            # INDEXKEYS_PLAIN_TEXT now looks something like:
            #   indexkeys  1 [1], -3 [1]
            #   
            #   indexkeys  2 [1], -3 [1]
            #   
            #   indexkeys  3 [1], 2 [1]
            #   
            #              ^      ^
            # These are the numbers of columns in which values must be unique.
            # The presence of a '-' indicates that the index on this column is
            # descending.
            DEBUG "53% INDEXKEYS_PLAIN_TEXT:"
	    DEBUG_CAT $INDEXKEYS_PLAIN_TEXT

            # Process INDEXKEYS_PLAIN_TEXT to get names
            # of columns in which values must be unique.

            # Remove blank lines, spaces, dashes, and occurrences of
            # 'indexkeys'.
            #
            grep -v ^$ $INDEXKEYS_PLAIN_TEXT > $TMPFILE
            cat $TMPFILE | tr -d 'indexkeys' | tr -d ' ' | tr -d '-' > $INDEXKEYS_PLAIN_TEXT
            DEBUG "53% INDEXKEYS_PLAIN_TEXT:"
	    DEBUG_CAT $INDEXKEYS_PLAIN_TEXT

            # INDEXKEYS_PLAIN_TEXT now looks something like:
            #   1[1],3[1]
            #   2[1],3[1]
            #   3[1],2[1]

            # Turn INDEXKEYS_PLAIN_TEXT into a list of unique columns.
            #
            $M_AWK '

              BEGIN { FS = "," }

              {
                for (i = 1; i <= NF; i++) {
                  gsub("\[.*\]", "", $i)
                  print $i
                }
              }

            ' $INDEXKEYS_PLAIN_TEXT > $UNIQUE_COLUMNS

            DEBUG "54% UNIQUE_COLUMNS:"
	    DEBUG_CAT $UNIQUE_COLUMNS

            # UNIQUE_COLUMNS now looks something like:
            #   1
            #   3
            #   2
            #   3
            #   3
            #   2

            sort -u $UNIQUE_COLUMNS > $TMPFILE
            mv $TMPFILE $UNIQUE_COLUMNS 

	    #
            # Check to ensure table has unique constraints
	    #
            if [ `wc -l $UNIQUE_COLUMNS | awk '{print $1}'` -eq 0 ]; then
               LOG "Skipping... Table does not have unique constraints: $file" 
               continue
            fi
	    ;;
        esac

        DEBUG "54% UNIQUE_COLUMNS:"
	DEBUG_CAT $UNIQUE_COLUMNS

        # UNIQUE_COLUMNS now looks something like:
        #   1
        #   2
        #   3

        # Turn UNIQUE_COLUMNS into a comma-separated list.
        #
        UNIQUE_COLUMNS_CSV=`$M_AWK '

          BEGIN {
            OFS = ""
            ORS = ""
            output = ""
          }

          { output = output $0 ", " }

          END {
            gsub(", $", "", output)
            print output
          }

        ' $UNIQUE_COLUMNS`

        DEBUG "55% UNIQUE_COLUMNS_CSV:"
	DEBUG $UNIQUE_COLUMNS_CSV

        # UNIQUE_COLUMNS_CSV now looks something like:
        #   1, 2, 3

        # Run query to get column names of columns.
        #
        get_tmp_file 30
        COL_NAME_NUM="$RETURN"

        query_string "output to '$COL_NAME_NUM'
                      select colname || ', ' || colno
                        from syscolumns
                       where tabid = (select tabid
                                        from systables
                                       where tabname = '$TABLE_NAME')
                         and colno in ($UNIQUE_COLUMNS_CSV)" $DATABASE $dbd_user $dbd_pword

        # COL_NAME_NUM now looks something like:
        #  (expression)  lac_id, 1
        #  
        #  (expression)  nc_id, 2
        #  
        #  (expression)  day, 3
        DEBUG "56% COL_NAME_NUM:"
	DEBUG_CAT $COL_NAME_NUM

        # Remove blank lines, spaces, and occurrences of '(expression)'.
        #
        grep -v ^$ $COL_NAME_NUM > $TMPFILE
        cat $TMPFILE | sed 's/(expression)//g' | tr -d ' ' > $COL_NAME_NUM

        # COL_NAME_NUM now looks something like:
        #   lac_id,1
        #   nc_id,2
        #   day,3
        DEBUG "56% COL_NAME_NUM:"
	DEBUG_CAT $COL_NAME_NUM

        # Derive IXD file from EIXD file, and process with AWK_TO_WRITE_SQL.
        #
        eixd_to_ixd $DIRECTORY/$file.eixd
        IXD_FILE="$RETURN"

	#
	# Get a temporary file to write SQL
	#
        get_tmp_file 32
        SQL_FILE="$RETURN"

	#
        # The following awk script generates a series of DELETE 
	# statements based on the column constraints
	#
	# The column constraints are read from a file in the following format
	#
	# a,1
	# b,2
	# c,3
	#
	# Processing the following IXD file
	#
	# 10|test||40|50|
	# 11|test||40|50|
	#
	# Produces the following SQL commands
	#
	# DELETE FROM tab WHERE a = "10" AND b = "test" AND c IS NULL
	# DELETE FROM tab WHERE a = "11" AND b = "test" AND c IS NULL
	#
	# The double quotes above seem to cover both numeric and character
	# columns.
	#
	$M_AWK -F"|" -v table="$TABLE_NAME" -v colfile="$COL_NAME_NUM" '
           BEGIN {
              while ((getline line < colfile ) > 0) {
                 split(line, line_array, ",")
                 column[line_array[1]] = line_array[2]
              }
           }
           {
              printf "DELETE FROM %s", table
              sep = "WHERE"
              for (idx in column) {
                 if ($column[idx] == "") {
                    printf " %s %s IS NULL", sep, idx
                 }
	         else {
                    printf " %s %s = \"%s\"", sep, idx, $column[idx]
	         }
	         sep = "AND"
              }
              printf ";\n"
           }
	' $IXD_FILE > $SQL_FILE

	#
	# Run the generated DELETE statements
	#
        query_file $SQL_FILE $DATABASE $dbd_user $dbd_pword

        if [ $? -eq 0 ]; then
          LOG "Data deleted."
        else
          LOG "ERROR: Query to delete data returned errors. See $METRICA_DIR/etc/install.log."
        fi
      fi
    done
  fi
}


###################################################################
#
# Function    : DROP_go
#
# Description : Check the environment vars and sets dependent vars.
#
###################################################################

function DROP_go
{
  for x in `ls $DIRECTORY/*.sql 2> /dev/null`
  do 
    name=`basename $x .sql`
    LOG "============================================================================" >> $LOGNAME
    LOG "+++++++++++++++++++++++ $tabname ++++++++++++++++++++++++++++" >> $LOGNAME    
    if [ `cat $x | grep -ic "create function"` -gt 0 ]; then
      contents=function
    elif [ `cat $x | grep -ic "create procedure"` -gt 0 ]; then
      contents=procedure
    elif [ `cat $x | grep -ic "create table"` -gt 0 ]; then
      contents=table
    else
      LOG "$x: Unsupported SQL command" >> $LOGNAME
      continue
    fi

    LOG "Dropping $contents $name..."

    if [ "$contents" = "table" ]; then
      query_string "DROP TABLE $name;" $DATABASE
    else
      ROUTINE_PARAMETER_TYPE=`grep -i "create" $x | $M_AWK '

        {
          # CREATE line is like:
          #   CREATE PROCEDURE gs_deleteAffectedMembers
          #   CREATE PROCEDURE gs_rewriteGroupList
          #   create function if_null(arg boolean, def varchar(255))

          $0 = tolower($0)

          # Remove the preceeding CREATE.
          #
          gsub("^create ", "")

          # Remove parameter names. Any word (i.e. not a number) following an
          # open bracket or a comma is assumed to be a parameter name.
          #
          gsub("\([a-z]* +", "\(")
          gsub(", +[a-z]* +", ", ")

          print $0
        }
      '`
      [ -z "$ROUTINE_PARAMETER_TYPE" ] && LOG "Error getting function details from $x" && continue

      DEBUG "61% DROP $ROUTINE_PARAMETER_TYPE"

      query_string "DROP $ROUTINE_PARAMETER_TYPE" $DATABASE
    fi

    if [ $? != "0" ]; then
      LOG "Failed"
    else
      LOG "Succeeded"
    fi
  done

  get_tmp_file 33
  TMP_FILE_1="$RETURN"

  get_tmp_file 34
  DROP_SQL="$RETURN"

  grep -i -h "^view" $DIRECTORY/*.tdef  > $TMP_FILE_1 2> /dev/null
  grep -i -h "^table" $DIRECTORY/*.tdef >> $TMP_FILE_1 2> /dev/null

  $M_AWK -F";" '

    { $0 = tolower($0) }

    /^table/ {
      num_renames = split($2, rename_history, ",")

      print "DROP TABLE " rename_history[num_renames] ";"
    }

    /^view/ {
      print "DROP VIEW " $2 ";"
    }

  ' $TMP_FILE_1 > $DROP_SQL

  DEBUG "61% DROP_SQL:"
  DEBUG_CAT $DROP_SQL

  query_file $DROP_SQL $DATABASE $dbd_user $dbd_pword
}

###################################################################
#
# Function    : load_common
#
# Description : Loads into the specified tables. 
#
###################################################################

function load_common
{
  MY_TABLE=$1
  MY_FILE=$2
  MY_DB=$3
  MY_USER=$4
  MY_PASS=$5

  LOG "Loading IXD $MY_FILE into $MY_TABLE ..."

  query_string "load from $MY_FILE insert into $MY_TABLE" $MY_DB $MY_USER $MY_PASS

  if [ $? != 0 ]; then
    LOG "Failed to load $MY_FILE into $MY_TABLE"
  else
    LOG "IXD file $MY_FILE was loaded into $MY_TABLE"
  fi 
}

###################################################################
#
# Function    : LOAD_go
#
# Description : Determines what ixd files to be loades. Also loads 
#               .spl files found in the specified directory
#
###################################################################

function LOAD_go
{
  if [ -z "$DBD_FILE" ]; then

    # IXD
    COUNT_IXD_FILES="`ls -l $DIRECTORY | egrep -i "\.ixd$"| wc -l`"
    if [ $COUNT_IXD_FILES -gt "0" ];then
      LOG "Loading IXD files from $DIRECTORY"
      for x in `ls $DIRECTORY/*.ixd 2> /dev/null`
      do
        tabname=`basename $x .ixd`
        tabname="${tabname%%,*}"
        load_common $tabname $x $DATABASE
      done 
    fi

    # EIXD
    COUNT_EIXD_FILES="`ls -l $DIRECTORY | egrep -i "\.eixd$"| wc -l`"
    if [ $COUNT_EIXD_FILES -gt "0" ];then
      LOG "Loading EIXD files from $DIRECTORY"
      for file in `ls $DIRECTORY/*.eixd 2> /dev/null`
      do
        get_eixd_table $file
        tabname="$RETURN"

        eixd_to_ixd $file
        x="$RETURN"

        load_common $tabname $x $DATABASE
      done 
    fi
  else
    for x in ${data[*]}
    do
      if [ ! -f $DIRECTORY/$x.@(ixd|eixd) ]; then
        LOG "Return code $exit_rc_file_incorrect :  $x.ixd or $x.eixd in $DIRECTORY not found"
      else
        # Set x and tabname.
        # x should be an IXD file (i.e. not an EIXD file).
        if [ -f $DIRECTORY/$x.ixd ]; then
          tabname="$x"
          x="$DIRECTORY/$x.ixd"
        else # So there must be a file $DIRECTORY/$x.eixd.
          get_eixd_table $DIRECTORY/$x.eixd
          tabname="$RETURN"

          eixd_to_ixd $DIRECTORY/$x.eixd
          x="$RETURN"
        fi

        if [ "$dbd_user" ]; then
          load_common $tabname $x $DATABASE $dbd_user $dbd_pword
        else
          load_common $tabname $x $DATABASE
        fi
      fi
    done 
  fi

  if [ -f "$y" ]; then
    rm $y
  fi

  COUNT_SPL_FILES="`ls -l $DIRECTORY | egrep -i "\.spl$"| wc -l`" 
  if [ $COUNT_SPL_FILES -gt "0" ] ; then
    LOG
    LOG "Loading SPL files from $DIRECTORY"
    for x in `ls $DIRECTORY/*.spl 2> /dev/null`
    do
      query_file $x $DATABASE

      if [ $? != 0 ]; then
        LOG "Failed to load $x"
      else
        LOG "SPL file $x was loaded successfully"
      fi
    done
  fi
}

###################################################################
#
# Function    : UNLOAD_go
#
# Description : Unloads from the specified tables.
#
###################################################################

function UNLOAD_go
{
  COUNT_SQL_FILES="`ls -l $DIRECTORY | egrep -i "\.sql$"| wc -l`" 
  if [ $COUNT_SQL_FILES -gt "0" ];then 
    for x in `ls $DIRECTORY/*.sql 2> /dev/null`
    do
      tabname=`basename $x .sql`
      LOG "Unloading $tabname"

      query_string "unload to $DIRECTORY/$tabname.ixd (select * from $tabname)" $DATABASE

      if [ $? != 0 ]; then
        LOG "Failed to unload from $tabname"
      else
        LOG "Unloaded from $tabname to $DIRECTORY/$tabname.ixd"
      fi    
    done
  fi
}

###################################################################
#
# Function    : DESTROY_go
#
# Description : Drops the specified database.
#
###################################################################

function DESTROY_go
{
  LOG "drop database $DATABASE"
  dbaccess - >> $METRICA_DIR/etc/install.log 2>&1 << END
drop database $DATABASE;
END

  if [ $? != 0 ]; then
     LOG "Failed to drop database $DATABASE"
  else
     LOG "Dropped database $DATABASE"
  fi    
}


function UPGRADE_go
{
  # Get chronologically-sorted, unduplicated list of
  # dates used in renaming tables from given TDEF file.
  # Output is file, $RETURN.
  #
  function get_tdef_dates
  {
    TDEF="$1"

    get_tmp_file 36
    TMP_FILE_1="$RETURN"

    # Get second field of table lines out of TDEF
    # Get comma-delimited fields from table lines
    # Get dates from comma-delimited fields.
    # Sort chronologically and remove duplicates.
    #
    grep -i ^table $TDEF | cut -d";" -f2 | $M_AWK -F"," '{for(i = 1; i <= NF; i++) print $i}' | cut -d"-" -f 2 -s | sort -d -u -o $TMP_FILE_1

    RETURN="$TMP_FILE_1"
  }

  DEBUG "68% START UPGRADE_go!"

  get_tmp_file 37
  TMPFILE="$RETURN"

  ##############################
  # Delete views in old version.
  ##############################

  for TABLE in ${VER1_TABLES[*]}
  do
    grep -i -h "^view" $VER1_DIR/$TABLE.tdef > /dev/null 2>&1

    if [ $? = 0 ]; then
      if [ -f $VER2_DIR/$TABLE.tdef ]; then

        get_tdef_view $VER2_DIR/$TABLE.tdef

        query_string "DROP VIEW $RETURN;" $DATABASE $dbd_user $dbd_pword
      fi 
    fi
  done

  ###############
  # Table TDEFs.
  ###############

  # VER1_TABLE_LINES is a file containing all the table lines in the TDEFs in
  # the old version of the package, listed in the DBD in the old version of the
  # package e.g.
  #   table;nc_region
  #   table;nc_region_sec
  #
  get_tmp_file 38
  VER1_TABLE_LINES="$RETURN"

  DEBUG "69% VER1_TABLE_LINES:"
  DEBUG_CAT $VER1_TABLE_LINES

  set -A VER1_TABLE_FILENAMES

  for TABLE in ${VER1_TABLES[*]}
  do
    if [ -f $VER1_DIR/$TABLE.tdef ]; then
      grep -i -h "^table" $VER1_DIR/$TABLE.tdef >> $VER1_TABLE_LINES

      # If the TDEF file contains a line beginning with 'table' then it must be
      # a TDEF file for a table, in which case, add the filename to
      # VER1_TABLE_FILENAMES.
      #
      [ $? = 0 ] && VER1_TABLE_FILENAMES="$VER1_TABLE_FILENAMES $VER1_DIR/$TABLE.tdef"
    fi
  done

  DEBUG "70% VER1_TABLE_LINES:"
  DEBUG_CAT $VER1_TABLE_LINES

  # VER2_TABLE_LINES is a file containing all the table lines in the TDEFs in
  # the new version of the package, listed in the DBD in the new version of the
  # package e.g.
  #   table;nc_region_sec-03/02/26,nc_region
  #   table;nc_region-03/02/26,nc_region_sec
  #
  get_tmp_file 39
  VER2_TABLE_LINES="$RETURN"

  set -A VER2_TABLE_FILENAMES

  for TABLE in ${VER2_TABLES[*]}
  do
    if [ -f $VER2_DIR/$TABLE.tdef ]; then
      grep -i -h "^table" $VER2_DIR/$TABLE.tdef >> $VER2_TABLE_LINES 2> /dev/null

      # If the TDEF file contains a line beginning with 'table' then it must be
      # a TDEF file for a table, in which case, add the filename to
      # VER2_TABLE_FILENAMES.
      #
      [ $? = 0 ] && VER2_TABLE_FILENAMES="$VER2_TABLE_FILENAMES $VER2_DIR/$TABLE.tdef"
    fi
  done

  DEBUG "71% VER2_TABLE_LINES:"
  DEBUG_CAT $VER2_TABLE_LINES

  DEBUG "71% VER2_TABLE_FILENAMES = $VER2_TABLE_FILENAMES:"

  # Create space-delimited, chronologically ordered lists of dates in
  # VER1_TABLE_LINES and VER2_TABLE_LINES e.g.
  #   VER1_TABLE_DATES: 02/12/15
  #   VER2_TABLE_DATES: 02/12/15 03/01/28 03/02/20
  #
  get_tdef_dates $VER1_TABLE_LINES
  VER1_TABLE_DATES="$RETURN"

  get_tdef_dates $VER2_TABLE_LINES
  VER2_TABLE_DATES="$RETURN"

  DEBUG "71% VER1_TABLE_DATES:"
  DEBUG_CAT $VER1_TABLE_DATES

  DEBUG "71% VER2_TABLE_DATES:"
  DEBUG_CAT $VER2_TABLE_DATES

  # Delete dates in VER1_TABLE_DATES from VER2_TABLE_DATES e.g.
  #   VER2_TABLE_DATES: 03/01/28 03/02/20
  #
  for LINE in `cat $VER1_TABLE_DATES`
  do
    grep -v $LINE $VER2_TABLE_DATES > $TMPFILE
    mv $TMPFILE $VER2_TABLE_DATES
  done

  DEBUG "71% VER1_TABLE_DATES:"
  DEBUG_CAT $VER1_TABLE_DATES

  DEBUG "71% VER2_TABLE_DATES:"
  DEBUG_CAT $VER2_TABLE_DATES

  # Convert VER1_TABLE_DATES and VER2_TABLE_DATES into arrays.
  file_to_array $VER1_TABLE_DATES
  VER1_TABLE_DATES="${RETURN[*]}"

  file_to_array $VER2_TABLE_DATES
  VER2_TABLE_DATES="${RETURN[*]}"

  DEBUG "72% VER1_TABLE_DATES = $VER1_TABLE_DATES"

  # Construct string containing space-delimited list of tables in
  # VER1_TABLE_LINES. E.g.
  #   VER1_TABLE_NAMES: nc_region nc_region_sec
  #
  get_tdef_tables $VER1_TABLE_LINES
  VER1_TABLE_NAMES="$RETURN"

  file_to_array $VER1_TABLE_NAMES
  VER1_TABLE_NAMES="${RETURN[*]}"

  # Take a copy of VER1_TABLE_NAMES and call it VER2_TABLE_NAMES. E.g.
  #   VER2_TABLE_NAMES: nc_region nc_region_sec
  #
  VER2_TABLE_NAMES=$VER1_TABLE_NAMES

  DEBUG "72% VER1_TABLE_NAMES = $VER1_TABLE_NAMES"
  DEBUG "72% VER2_TABLE_NAMES = $VER2_TABLE_NAMES"

  export VER2_TABLE_NAMES

  get_tmp_file 40
  RENAMED_TABLES="$RETURN"

  # For each date in the list of dates in VER2_TABLE_DATES (starting from the
  # earliest date),
  #
  for TDEF_DATE in $VER2_TABLE_DATES
  do

    DEBUG "73% TDEF_DATE = $TDEF_DATE"

    $ECHO > $RENAMED_TABLES

    # Get old and new table names at date of rename, and latest table name from
    # TDEF in new version.
    #
    export TDEF_DATE

    $M_AWK -F";" '

      { $1 = tolower($1) }

      /^table/ && $2 ~ ENVIRON["TDEF_DATE"] {

        # num_renames is number of times this table has been renamed, indicated
        # by the number of comma-separated fields in the second field of the
        # semi-colon separated fields.
        #
        num_renames = split($2, rename_history, ",")

        # Find the field for which the table was renamed at this date.
        #
        for (rename in rename_history) {
          if (rename_history[rename] ~ ENVIRON["TDEF_DATE"]) {
            # Text in field, before dash, is old table name.
            #
            dash_pos = match(rename_history[rename], "-")

            old_table_name = substr(rename_history[rename], 0, dash_pos - 1)

            # Text in next field, before dash, is new table name.
            #
            rename++

            dash_pos = match(rename_history[rename], "-")

            if (dash_pos > 0)
              new_table_name = substr(rename_history[rename], 0, dash_pos - 1)
            else
              new_table_name = rename_history[rename]

            break
          }
        }

        # Latest field is assumed to not have a rename in it.
        #
        latest_table_name = rename_history[num_renames]

        print old_table_name ";" new_table_name ";" latest_table_name
      }

    ' $VER2_TABLE_LINES > $RENAMED_TABLES

    DEBUG "74% RENAMED_TABLES:"
    DEBUG_CAT $RENAMED_TABLES

    # For each table renamed with date in VER2_TABLE_LINES,
    #
    for RENAMED_TABLE in `cat $RENAMED_TABLES`
    do

      # RENAMED_TABLE looks something like:
      #   nc_region_sec;nc_region;nc_region

      # Put values of fields into environment variables.
      #
      $ECHO "$RENAMED_TABLE" | $M_AWK -F";" '

        {
          print "VER1_TABLE_NAME=\"" $1 "\""
          print "VER2_TABLE_NAME=\"" $2 "\""
          print "LATEST_TABLE_NAME=\"" $3 "\""
        }

      ' > $TMPFILE

      DEBUG "74% TMPFILE1:"
      DEBUG_CAT $TMPFILE

      . $TMPFILE

      # Now have environment variables VER1_TABLE_NAME, VER2_TABLE_NAME, and
      # LATEST_TABLE_NAME, set to values such as:
      #   VER1_TABLE_NAME="nc_region_sec"
      #   VER2_TABLE_NAME="nc_region"
      #   LATEST_TABLE_NAME="nc_region"

      DEBUG "75% VER1_TABLE_NAME = $VER1_TABLE_NAME"
      DEBUG "75% VER2_TABLE_NAME = $VER2_TABLE_NAME"
      DEBUG "75% LATEST_TABLE_NAME = $LATEST_TABLE_NAME"
      DEBUG ""
      DEBUG "75% VER2_TABLE_NAMES = $VER2_TABLE_NAMES"
      DEBUG "75% VER2_NEW_TABLES_NAMES = $VER2_NEW_TABLES_NAMES"

      # If VER1_TABLE_NAME is present in VER2_TABLE_NAMES, then
      #   Rename VER1_TABLE_NAME to VER2_TABLE_NAME in VER2_TABLE_NAMES.
      #     VER2_TABLE_NAME in VER2_TABLE_NAMES is prefixed with a 'rename-'
      #     (prefix removed afterwards).
      # Otherwise,
      #   Add VER2_TABLE_NAME to VER2_NEW_TABLES_NAMES.
      #
      export VER1_TABLE_NAME
      export VER2_TABLE_NAME
      export LATEST_TABLE_NAME
      export VER2_NEW_TABLES_NAMES

      $ECHO "" | $M_AWK '

        BEGIN {
          new_VER2_TABLE_NAMES = ""
          is_new_table = "TRUE"
        }

        {
          numTables = split(ENVIRON["VER2_TABLE_NAMES"], VER2_TABLE_NAMES_array, " ")

          # NB Elements in array are processed from right to left.
          #
          for (table=numTables ; table>=1 ; table--) {
            if (VER2_TABLE_NAMES_array[table] == ENVIRON["VER1_TABLE_NAME"]) {
              is_new_table = "FALSE"

              new_VER2_TABLE_NAMES = "rename-" ENVIRON["VER2_TABLE_NAME"] " " new_VER2_TABLE_NAMES
            } else {
              new_VER2_TABLE_NAMES = VER2_TABLE_NAMES_array[table] " " new_VER2_TABLE_NAMES
            }
          }
        }

        END {
          print "VER2_TABLE_NAMES=\"" new_VER2_TABLE_NAMES "\""

          if (is_new_table == "TRUE") {
            split(ENVIRON["VER2_NEW_TABLES_NAMES"], VER2_NEW_TABLES_NAMES_array, " ")

            is_LATEST_TABLE_NAME_already_in_VER2_NEW_TABLES_NAMES = "FALSE"

            for(table in VER2_NEW_TABLES_NAMES_array)
              if (VER2_NEW_TABLES_NAMES_array[table] == ENVIRON["LATEST_TABLE_NAME"])
                is_LATEST_TABLE_NAME_already_in_VER2_NEW_TABLES_NAMES = "TRUE"

            if (is_LATEST_TABLE_NAME_already_in_VER2_NEW_TABLES_NAMES == "FALSE")
              print "VER2_NEW_TABLES_NAMES=\"$VER2_NEW_TABLES_NAMES " ENVIRON["LATEST_TABLE_NAME"] "\""
          }
        }

      ' > $TMPFILE

      DEBUG "77% TMPFILE2:"
      DEBUG_CAT $TMPFILE

      . $TMPFILE

      DEBUG "77% VER2_TABLE_NAMES2 = $VER2_TABLE_NAMES"
      DEBUG "77% VER2_NEW_TABLES_NAMES = $VER2_NEW_TABLES_NAMES"

    done

    DEBUG "77%  VER2_TABLE_NAMES = $VER2_TABLE_NAMES"

    # Any tables which have been dropped at this date, but which, at the same
    # time, another table was renamed to the name of the dropped table, will now
    # be present in VER2_TABLE_NAMES along with the renamed table which will
    # have the same name, but preceeded by a 'rename-'.
    #
    # For example, table a has been renamed to b, table b has been renamed to c,
    # and the original table c is to be dropped:
    #
    #   VER2_TABLE_NAMES: rename-b rename-c c

    # For each table name in VER2_TABLE_NAMES that is preceeded with a
    # 'rename-', search for the same table name in VER2_TABLE_NAMES that is not
    # preceeded with a 'rename-'. Any such table names are prefixed with
    # 'drop-'.
    #
    $ECHO "" | $M_AWK '

      {
        num_tables = split(ENVIRON["VER2_TABLE_NAMES"], VER2_TABLE_NAMES_array, " ")

        for (table in VER2_TABLE_NAMES_array)
          # If VER2_TABLE_NAMES_array[table] is prefixed with a rename-.
          #
          if (VER2_TABLE_NAMES_array[table] ~ /^rename-/) {
            actual_table_name = VER2_TABLE_NAMES_array[table]
            sub("^rename-", "", actual_table_name)

#DEBUG#            print "#DEBUG: actual_table_name = " actual_table_name

            for (table2 in VER2_TABLE_NAMES_array)
              # If VER2_TABLE_NAMES_array[table2] does not contain a - (i.e.
              # does not begin with rename- or drop-).
              #
              if (VER2_TABLE_NAMES_array[table2] !~ /-/) {
#DEBUG#                print "#DEBUG: VER2_TABLE_NAMES_array[table2] = " VER2_TABLE_NAMES_array[table2]

                if (actual_table_name == VER2_TABLE_NAMES_array[table2])
                  VER2_TABLE_NAMES_array[table2] = "drop-" VER2_TABLE_NAMES_array[table2]
              }
          }

        # Reconstruct VER2_TABLE_NAMES from VER2_TABLE_NAMES_array.
        # NB "for(i=lower; i<=upper; i++)" format used in order to process
        # VER2_TABLE_NAMES_array from left to right (i.e. in order of filename).
        #
        for (table = 1; table <= num_tables; table++)
          VER2_TABLE_NAMES_string = VER2_TABLE_NAMES_string VER2_TABLE_NAMES_array[table] " "

        print "VER2_TABLE_NAMES=\"" VER2_TABLE_NAMES_string "\""
      }

    ' > $TMPFILE

    DEBUG "78% (16:19): TMPFILE:"
    DEBUG_CAT $TMPFILE

    . $TMPFILE

    DEBUG "78% (A): VER2_TABLE_NAMES = $VER2_TABLE_NAMES"

    # Remove 'rename-' in front of table names in VER2_TABLE_NAMES.
    #
    # NB sed is inadequate to do this because it limits the length of the
    # output string.
    #
    $ECHO "" | $M_AWK '

      {
        tableNames = ENVIRON["VER2_TABLE_NAMES"]
        gsub("rename-", "", tableNames)
        print "VER2_TABLE_NAMES=\"" tableNames "\""
      }

    ' > $TMPFILE

    . $TMPFILE

    DEBUG "78% (B): VER2_TABLE_NAMES = $VER2_TABLE_NAMES"

  done

  # Construct string containing space-delimited list of tables in
  # VER2_TABLE_LINES. E.g.
  #   VER2_TABLE_NAMES: nc_region nc_region_sec
  #
  get_tdef_tables $VER2_TABLE_LINES
  VER2_ALL_TABLE_NAMES="$RETURN"

  file_to_array $VER2_ALL_TABLE_NAMES
  VER2_ALL_TABLE_NAMES="${RETURN[*]}"

  DEBUG "79% VER2_ALL_TABLE_NAMES  = $VER2_ALL_TABLE_NAMES"
  DEBUG "79% VER2_TABLE_NAMES      = $VER2_TABLE_NAMES"
  DEBUG "79% VER2_NEW_TABLES_NAMES = $VER2_NEW_TABLES_NAMES"

  # VER2_NEW_TABLES_NAMES only contains tables that have once been renamed (i.e.
  # tables added since VER1, but before VER2). Now add tables added in TDEF2,
  # that have never been renamed. These will be tables present in
  # VER2_ALL_TABLE_NAMES, but not in VER2_TABLE_NAMES or VER2_NEW_TABLES_NAMES.
  #
  export VER2_TABLE_NAMES
  export VER2_NEW_TABLES_NAMES
  export VER2_ALL_TABLE_NAMES


  $ECHO "" | $M_AWK '

    {
      split(ENVIRON["VER2_ALL_TABLE_NAMES"], VER2_ALL_TABLE_NAMES_array, " ")
      split(ENVIRON["VER2_TABLE_NAMES"], VER2_TABLE_NAMES_array, " ")
      split(ENVIRON["VER2_NEW_TABLES_NAMES"], VER2_NEW_TABLES_NAMES_array, " ")

#DEBUG#      print "#DEBUG: VER2_ALL_TABLE_NAMES = " ENVIRON["VER2_ALL_TABLE_NAMES"]
#DEBUG#      print "#DEBUG: VER2_TABLE_NAMES = " ENVIRON["VER2_TABLE_NAMES"]

      for (table in VER2_ALL_TABLE_NAMES_array) {

        table_found = "FALSE"

        for (table2 in VER2_TABLE_NAMES_array) {
#DEBUG#          print "#DEBUG: VER2_ALL_TABLE_NAMES_array[table] = " VER2_ALL_TABLE_NAMES_array[table]
#DEBUG#          print "#DEBUG: VER2_TABLE_NAMES_array[table2] = " VER2_TABLE_NAMES_array[table2]

          if (VER2_ALL_TABLE_NAMES_array[table] == VER2_TABLE_NAMES_array[table2]) {
            table_found = "TRUE"

            break
          }
        }

        if (table_found != "TRUE") {
          for (table2 in VER2_NEW_TABLES_NAMES_array) {
#DEBUG#            print "#DEBUG: VER2_ALL_TABLE_NAMES_array[table] = " VER2_ALL_TABLE_NAMES_array[table]
#DEBUG#            print "#DEBUG: VER2_NEW_TABLES_NAMES_array[table2] = " VER2_NEW_TABLES_NAMES_array[table2]

            if (VER2_ALL_TABLE_NAMES_array[table] == VER2_NEW_TABLES_NAMES_array[table2]) {
              table_found = "TRUE"

              break
            }
          }

          if (table_found != "TRUE")
            # A table in VER2_ALL_TABLE_NAMES has been found that is not in
            # VER2_TABLE_NAMES or VER2_NEW_TABLES_NAMES.
            #
            print "VER2_NEW_TABLES_NAMES=\"$VER2_NEW_TABLES_NAMES " VER2_ALL_TABLE_NAMES_array[table] "\""
        }
      }
    }

  ' > $TMPFILE

  DEBUG "81% TMPFILE3:"
  DEBUG_CAT $TMPFILE

  . $TMPFILE

  # VER1_TABLE_NAMES, VER2_TABLE_NAMES, and VER2_NEW_TABLES_NAMES now look
  # something like:
  #   VER1_TABLE_NAMES:      b a c
  #   VER2_TABLE_NAMES:      a b c
  #   VER2_NEW_TABLES_NAMES: d

  DEBUG "81% VER1_TABLE_NAMES = $VER1_TABLE_NAMES"
  DEBUG "81% VER2_TABLE_NAMES = $VER2_TABLE_NAMES"
  DEBUG "81% VER2_NEW_TABLES_NAMES = $VER2_NEW_TABLES_NAMES"
  DEBUG "81% VER2_ALL_TABLE_NAMES = $VER2_ALL_TABLE_NAMES"
  DEBUG "81% VER1_TABLE_FILENAMES = $VER1_TABLE_FILENAMES"
  DEBUG "81% VER2_TABLE_FILENAMES = $VER2_TABLE_FILENAMES"

  # For each table in VER2_TABLE_NAMES and VER2_ALL_TABLE_NAMES,
  #   given corresponding TDEF filenames in VER1_TABLE_FILENAMES and VER2_TABLE_FILENAMES,
  #   Produce SQL to upgrade VER1 table to VER2 table.
  #   If table is to be renamed, prefix new name with _.
  #   After all table renaming is complete, leading underscores are removed.

  get_tmp_file 41
  TDEF_PAIRS="$RETURN"

  get_tmp_file 42
  NEW_TABLES_TDEFS="$RETURN"

  # If table exists in VER2_TABLE_NAMES and VER2_ALL_TABLE_NAMES,
  #   then pair filename in VER1_TABLE_FILENAMES and VER2_TABLE_FILENAMES together.
  #
  # If table exists in VER2_NEW_TABLES_NAMES and VER2_ALL_TABLE_NAMES,
  #   then put filename into NEW_TABLES_TDEFS.
  #
  export VER2_TABLE_NAMES
  export VER2_ALL_TABLE_NAMES
  export VER2_NEW_TABLES_NAMES
  export VER1_TABLE_FILENAMES
  export VER2_TABLE_FILENAMES
  export TDEF_PAIRS
  export NEW_TABLES_TDEFS

  $ECHO "" | $M_AWK '

    {
      split(ENVIRON["VER2_TABLE_NAMES"], VER2_TABLE_NAMES_array, " ")
      split(ENVIRON["VER2_ALL_TABLE_NAMES"], VER2_ALL_TABLE_NAMES_array, " ")
      split(ENVIRON["VER2_NEW_TABLES_NAMES"], VER2_NEW_TABLES_NAMES_array, " ")
      split(ENVIRON["VER1_TABLE_FILENAMES"], VER1_TABLE_FILENAMES_array, " ")
      split(ENVIRON["VER2_TABLE_FILENAMES"], VER2_TABLE_FILENAMES_array, " ")

      for (table in VER2_ALL_TABLE_NAMES_array) {
#DEBUG#        print "#DEBUG: VER2_ALL_TABLE_NAMES_array[table] = " VER2_ALL_TABLE_NAMES_array[table]

        table_found = "FALSE"

#DEBUG#        print "#DEBUG: table_found = " table_found

        for (table2 in VER2_TABLE_NAMES_array) {
#DEBUG#          print "#DEBUG: VER2_TABLE_NAMES_array[table2] = " VER2_TABLE_NAMES_array[table2]

#DEBUG#          print "#DEBUG: Does " VER2_ALL_TABLE_NAMES_array[table] " == " VER2_TABLE_NAMES_array[table2] "?"

          if (VER2_ALL_TABLE_NAMES_array[table] == VER2_TABLE_NAMES_array[table2]) {
#DEBUG#            print "#DEBUG: VER2_ALL_TABLE_NAMES_array[table] == VER2_TABLE_NAMES_array[table2]!"

            print VER1_TABLE_FILENAMES_array[table2] ":" VER2_TABLE_FILENAMES_array[table] >> ENVIRON["TDEF_PAIRS"]

            table_found = "TRUE"

#DEBUG#            print "#DEBUG: table_found = " table_found
          }
        }

        for (table2 in VER2_NEW_TABLES_NAMES_array)
          if (VER2_ALL_TABLE_NAMES_array[table] == VER2_NEW_TABLES_NAMES_array[table2])
            print VER2_TABLE_FILENAMES_array[table] >> ENVIRON["NEW_TABLES_TDEFS"]
      }
    }

  '

  DEBUG "83% TDEF_PAIRS:"
  DEBUG_CAT $TDEF_PAIRS

  DEBUG "83% NEW_TABLES_TDEFS:"
  DEBUG_CAT $NEW_TABLES_TDEFS

  get_tmp_file 43
  SCHEMA_DELTA_SQL="$RETURN"

  get_tmp_file 44
  REMOVE_UNDERSCORE_PREFIXES_SQL="$RETURN"

  # For each pair of files in TDEF_PAIRS,
  #
  for TDEF_PAIR in `cat $TDEF_PAIRS`
  do
    VER1_TDEF=`$ECHO $TDEF_PAIR | $M_AWK -F":" '{print $1}'`
    VER2_TDEF=`$ECHO $TDEF_PAIR | $M_AWK -F":" '{print $2}'`

    # If there are no differences between the TDEFs, move on to the next pair
    # If there are differences, the type of difference (TABLE, COLUMN, etc)
    # will be put into $TDEF_DIFFERENCES so that only the relevent processing
    # and SQL generation will be done in tdef_to_delta_sql
    #
    get_tmp_file 45
    TDEF_DIFFERENCES="$RETURN"
    sdiff -s $VER1_TDEF $VER2_TDEF | sed -e "s/ *>  //" | grep ^[A-Z,a-z] | cut -f1 -d";" | uniq > $TDEF_DIFFERENCES
    if [ ! -s $TDEF_DIFFERENCES ]
    then
       rm $TDEF_DIFFERENCES > /dev/null 2>&1
       continue
    fi

    LOG "Producing SQL to upgrade $VER1_TDEF to $VER2_TDEF..."

    tdef_to_delta_sql $VER1_TDEF $VER2_TDEF
    TABLE_DELTA_SQL="$RETURN"

    DEBUG "84% TABLE_DELTA_SQL:"
    DEBUG_CAT $TABLE_DELTA_SQL

    NEW_TABLE_NAME=`grep "RENAME TABLE" $TABLE_DELTA_SQL | cut -f5 -d" " | cut -f1 -d";"`

    if [ "$NEW_TABLE_NAME" != "" ]; then
       echo "RENAME TABLE _${NEW_TABLE_NAME} TO ${NEW_TABLE_NAME};" >> $REMOVE_UNDERSCORE_PREFIXES_SQL

       sed -e "s/ ${NEW_TABLE_NAME} / _${NEW_TABLE_NAME} /g" \
           -e "s/ ${NEW_TABLE_NAME}\./ _${NEW_TABLE_NAME}\./g" \
           -e "s/ ${NEW_TABLE_NAME};/ _${NEW_TABLE_NAME};/g" $TABLE_DELTA_SQL >> $SCHEMA_DELTA_SQL
    else
       cat $TABLE_DELTA_SQL >> $SCHEMA_DELTA_SQL
    fi

    DEBUG "84% SCHEMA_DELTA_SQL:"
    DEBUG_CAT $SCHEMA_DELTA_SQL
  done

  DEBUG "84% SCHEMA_DELTA_SQL:"
  DEBUG_CAT $SCHEMA_DELTA_SQL

  cat $SCHEMA_DELTA_SQL $REMOVE_UNDERSCORE_PREFIXES_SQL > $TMPFILE
  mv $TMPFILE $SCHEMA_DELTA_SQL

  # For each table in NEW_TABLES_TDEFS
  #   produce SQL to create new table.
  #
  for TDEF in `cat $NEW_TABLES_TDEFS`
  do
    tdef_to_sql $TDEF
    CREATE_TABLE_SQL="$RETURN"

    cat $SCHEMA_DELTA_SQL $CREATE_TABLE_SQL > $TMPFILE
    mv $TMPFILE $SCHEMA_DELTA_SQL
  done

  LOG "Upgrading schema..."
  DEBUG "85% SCHEMA_DELTA_SQL:"
  DEBUG_CAT $SCHEMA_DELTA_SQL

  query_file $SCHEMA_DELTA_SQL $DATABASE $dbd_user $dbd_pword

  ##############################
  # Create views in new version.
  ##############################

  for TABLE in ${VER2_TABLES[*]}
  do
    grep -i -h "^view" $VER2_DIR/$TABLE.tdef > /dev/null 2>&1

    if [ $? = 0 ]; then
      tdef_to_sql $VER2_DIR/$TABLE.tdef

      query_file $RETURN $DATABASE $dbd_user $dbd_pword
    fi
  done
}

###################################################################
#
# Function    : showversion
#
# Description : Displays the Truechange version information.
#
###################################################################

function showversion
{
  if [ -f $METRICA_DIR/Installer/bin/metrica_version.ksh ]; then
    . $METRICA_DIR/Installer/bin/metrica_version.ksh
    print_version "Metrica database schema setup script" $VERSION
  else
    $ECHO $SHORT_VERSION
  fi
}

###################################################################
#
# Function    : also
#
# Description : Displays help prompt.
#
###################################################################

function also
{
  $ECHO ""
  $ECHO "SEE ALSO"
  $ECHO "     $PROGNAME  < -help | --help > for more detail"
  $ECHO ""
}

###################################################################
#
# Function    : syntax
#
# Description : Displays usage information.
#
###################################################################

function syntax
{
  $ECHO ""
  $ECHO "     $PROGNAME  < -create | -load | -unload | -clear | -drop | -destroy >"
  $ECHO "                [-directory <dir>]" 
  $ECHO "                [-database <database_name>]"
  $ECHO "     $PROGNAME < -v | -version | --version> "
  $ECHO "     $PROGNAME < -h | -help | --help> "
}

###################################################################
#
# Function    : shorthelp
#
# Description : Displays short help information.
#
###################################################################

function shorthelp
{
  $ECHO ""
  $ECHO "NAME"
  $ECHO "     $PROGNAME  -  database schema script"
  $ECHO ""
  $ECHO "SYNOPSIS"
  syntax
  $ECHO ""
  $ECHO "     More than one option can be specified, the options"
  $ECHO "     will be processed in the order they appear on the"
  $ECHO "     command line. A working directory can be specified"
  $ECHO "     with the '-directory' option. A database can be"
  $ECHO "     specified with the '-database'. If a .dbd file"
  $ECHO "     exists in the specified directory is used to."
  $ECHO "     specify schema actions."
  $ECHO ""
}

###################################################################
#
# Function    : longhelp
#
# Description : Displays detailed help information.
#
###################################################################

function longhelp
{
  shorthelp
  $ECHO "DESCRIPTION"
  $ECHO "     This program is used to create database schemas :"
  $ECHO "     - create and drop tables"
  $ECHO "     - create and destroy databases"
  $ECHO "     - load and delete data from tables"
  $ECHO ""
  $ECHO "     The log file $LOGNAME is created in the"
  $ECHO "     current directory. It contains the output of all the"
  $ECHO "     database operations."
  $ECHO ""
  $ECHO "     -create  : Creates the database if it does not already"
  $ECHO "                exist. If a database is created and the"
  $ECHO "                METRICA_BLADE_NAME enviroment variable is set,"
  $ECHO "                the blade will be registered with the database."
  $ECHO "                Also creates the table as specified by the .sql" 
  $ECHO "                files in the specified directory or as listed in" 
  $ECHO "                the .dbd file in the specified directory."
  $ECHO "     -load    : Loads data from the .idx and .eixd files in the"
  $ECHO "                specified directory into the tables whose names"
  $ECHO "                match those of the .idx files or are specified"
  $ECHO "                in the .eixd files. File names with the names" 
  $ECHO "                <tablename>.ixd or <tablename>,<any-name>.ixd"
  $ECHO "                will be loaded from the specified directory."
  $ECHO "                Alternatively loads files <tablename>.ixd"
  $ECHO "                for tables as specified in the .dbd file."
  $ECHO "     -unload  : Unloads data from the tables specified by the"
  $ECHO "                .sql files into .idx files in the specified"
  $ECHO "                directory."
  $ECHO "     -clear   : Deletes the data from the tables whose names"
  $ECHO "                match the names of the *.sql files in the"
  $ECHO "                specified directory."
  $ECHO "     -drop    : Drops the tables whose names match the names"
  $ECHO "                of the .sql files in the specified directory"
  $ECHO "     -destroy : Drops the specified database." 
  $ECHO ""
  $ECHO "     [-directory <dir>]"         
  $ECHO "              : A directory can be specified. If this option is" 
  $ECHO "                omitted, the current working directory is used." 
  $ECHO "                Can use full or relative pathnames." 
  $ECHO "     [-database <database_name>]"         
  $ECHO "              : A database can be specified. If this option is" 
  $ECHO "                omitted, then the .dbd file in the specified"
  $ECHO "                directory is used. If it is specified but the"
  $ECHO "                database name in the .dbd file does not match"
  $ECHO "                an error is thrown."
  $ECHO ""
  $ECHO "     Information Commands"
  $ECHO "     -h       : Displays usage information."
  $ECHO "     -help    : Displays short help information."
  $ECHO "     --help   : Displays long help information."
  $ECHO "     -v | -version |--version  : Displays version information."
  $ECHO ""
}

###################################################################
#
# Function    : check_nulldir
#
# Description : Validate that the directory param is given correctly.
#
###################################################################

function check_nulldir
{
  TMP_DIRECTORY=$1

  if $ECHO "$TMP_DIRECTORY" | egrep -i "^\-.|^\-\s*" > /dev/null; then
    LOG "Return code $exit_rc_usage : Error no directory parameter given"
    syntax
    exit $exit_rc_usage
  fi 
}

###################################################################
#
# Function    : check_relative
#
# Description : Check if the directory param is a relative
#               or a full path.
#
###################################################################

function check_relative
{
  TMP_DIRECTORY=$1

  if $ECHO "$TMP_DIRECTORY" | egrep -i "^\/." > /dev/null; then
    TMP_DIRECTORY="$TMP_DIRECTORY"
  else
    TMP_DIRECTORY="`pwd`/$TMP_DIRECTORY"
  fi 

  if [ ! -d "$TMP_DIRECTORY" ]; then
    LOG "Return code $exit_rc_dir_incorrect : cannot find directory $TMP_DIRECTORY "
    exit $exit_rc_dir_incorrect
  fi

  RETURN="$TMP_DIRECTORY"
}

###################################################################
#
# Function    : check_nulldb
#
# Description : Validate that the database param is given correctly
#
###################################################################

function check_nulldb
{
  if $ECHO "$DATABASE" | egrep -i "^\-.|^\-\s*" > /dev/null; then
    LOG "Return code $exit_rc_usage : Error no database parameter given"
    syntax
    exit $exit_rc_usage
  fi
}

###################################################################
#
# Function    : remove_spaces
#
# Description : Removes extra spaces.
#
###################################################################

# FILLINSHANE \s

function remove_spaces
{ 
  $ECHO "$@" | tr -d ' '
} 

###################################################################
#
# Function    : process_dbdfile
#
# Description : Process .dbd file from specified directory if
#               file exists
#
###################################################################

function process_dbdfile
{
  TMP_DIRECTORY=$1

  set -A dbd_user
  set -A dbd_pword

  # read dbd file
  let linecount=0
  exec 5<&0 < "$TMP_DIRECTORY/$DBD_FILE"
  while read LINE
  do 
    key=`$ECHO $LINE | $M_AWK -F":" '{ print $1 }'`
    eval val=`$ECHO $LINE | $M_AWK -F":" '{ print $2 }'`
    key="`remove_spaces $key`"
    keys[$linecount]=$key
    val="`remove_spaces $val`"
    vals[$linecount]=$val
    linecount=`expr $linecount + 1`
  done
  exec 0<&5
  
  let mylinecount=0 
  let tablecount=0 
  let datacount=0 
  
  # read dbd file and parse
  while (( $mylinecount < ${#keys[*]} ))
  do   
    case ${keys[$mylinecount]} in
      "database")    dbd_dbase=${vals[$mylinecount]} ;;
      "user")        dbd_user=${vals[$mylinecount]} ;;
      "password")    dbd_pword=${vals[$mylinecount]} ;;
      "dbspace")     dbd_dbspace=${vals[$mylinecount]} ;;
      "indexspace")  dbd_indexspace=${vals[$mylinecount]} ;;
      "extent size") dbd_extentsize=${vals[$mylinecount]} ;;
      "next size")   dbd_nextsize=${vals[$mylinecount]} ;;
      "table")   tables[$tablecount]=${vals[$mylinecount]}
                 tablecount=`expr $tablecount + 1`;;
      "data")    data[$datacount]=${vals[$mylinecount]}
                 datacount=`expr $datacount + 1`;;
      *) LOG "Return code $exit_rc_file_incorrect :  Incorrect options in DBD file $TMP_DIRECTORY/$DBD_FILE"
         exit $exit_rc_file_incorrect 
      ;;
    esac
    mylinecount=`expr $mylinecount + 1`
  done
 
  # check that databases are valid
  if [ -z "$dbd_dbase" ]; then
    if [ -z "$DATABASE" ]; then
      LOG "Return code $exit_rc_file_incorrect :  No database specified in either DBD file or command line"
      exit $exit_rc_file_incorrect
    fi
  else
    if [ -z "$DATABASE" ]; then
      DATABASE=$dbd_dbase
    elif [ "$DATABASE" != "$dbd_dbase" ]; then
      LOG "Return code $exit_rc_file_incorrect :  Database specified in DBD file ($dbd_dbase)" 
      LOG "is different to database specified in the command line ($DATABASE)"
      exit $exit_rc_file_incorrect
    fi
  fi
}

###################################################################
#
# Function    : check_dbdfile
#
# Description : Validates if one .dbd file exists in specified
#               directory and is readable
#
###################################################################

function check_dbdfile
{
  TMP_DIRECTORY="$1"

  COUNT_FILES="`ls -l $TMP_DIRECTORY | egrep -i "\.dbd$"| wc -l`"  
  
  if [ $COUNT_FILES -lt "1" ]; then 
    LOG "No .dbd file for processing"
  elif [ $COUNT_FILES -gt "1" ]; then 
    LOG "Return code $exit_rc_file_incorrect : More than one .dbd file found in $TMP_DIRECTORY"
    exit $exit_rc_file_incorrect
  elif [ $COUNT_FILES -eq "1" ]; then 
    DBD_FILE="`ls $TMP_DIRECTORY | grep -i "\.dbd$" `"  
    # File, readable, non-empty
    if [ ! -f "$TMP_DIRECTORY/$DBD_FILE" ]; then
      LOG "Return code $exit_rc_file_incorrect :  $DBD_FILE in $TMP_DIRECTORY is not a file"
      exit $exit_rc_file_incorrect
    elif [ ! -r "$TMP_DIRECTORY/$DBD_FILE" ]; then
      LOG "Return code $exit_rc_file_incorrect :  Cannot read DBD file $DBD_FILE in $TMP_DIRECTORY"
      exit $exit_rc_file_incorrect
    elif [ ! -s "$TMP_DIRECTORY/$DBD_FILE" ]; then
      LOG "Return code $exit_rc_file_incorrect :  DBD file $DBD_FILE in $TMP_DIRECTORY is empty"
      exit $exit_rc_file_incorrect
    else
      LOG "DBD file $DBD_FILE in $TMP_DIRECTORY being processed"
      process_dbdfile $TMP_DIRECTORY
    fi
  fi
}

# Get common functions and environment variables.
. metrica_install_common.ksh

#---------------------------------
#    Process Command line args
#---------------------------------

if [ "$#" = "1" ]; then
  if [ "$1" = "-h" ]; then
    syntax
    also
    exit $exit_rc_success
  elif [ "$1" = "-help" ]; then
    shorthelp
    also
    exit $exit_rc_success
  elif [ "$1" = "--help" ]; then
    longhelp
    exit $exit_rc_success
  elif [ "$1" = "-v" ]; then
    showversion
    exit $exit_rc_success
  elif [ "$1" = "-version" ]; then
    showversion
    exit $exit_rc_success
  elif [ "$1" = "--version" ]; then
    showversion
    exit $exit_rc_success
  fi
fi

#Declare passed commands with dummy vars.
#CREATE
given[0]='-1'
#UNLOAD
given[1]='-1'
#LOAD
given[2]='-1'
#CLEAR
given[3]='-1'
#DROP
given[4]='-1'
#DESTROY
given[5]='-1'
#UPGRADE
given[6]='-1'

SCHEMA_VARIABLES=

# Ensure that parameters are given fully
#
function testopts
{
  criteria="$1"
  shift
  $ECHO "$*" | egrep -i "$criteria" >/dev/null
  return $?
}

testopts '^.*-directory[ |$]|^.*-database[ |$]|^.*-from[ |$]|^.*-to[ |$]' $*
if [ $? != 0 ] ; then
  LOG "Return code $exit_rc_usage : Error no parameter given"
  syntax ; exit $exit_rc_usage
fi

testopts '^.*-create.*$|^.*-unload.*$|^.*-load.*$|^.*-clear.*$|^.*-drop.*$|^.*-destroy.*$|^.*-upgrade.*$' $*
if [ $? != 0 ] ; then
  LOG "Return code $exit_rc_usage : Action command must be given"
  syntax ; exit $exit_rc_usage
fi

set -A DATABASE

count=0
while test $# -gt 0
do
  option=`$ECHO $1 | tr [:upper:] [:lower:]`
  case $option in
    "-directory")
      DIRECTORY=$2  

      check_nulldir $DIRECTORY

      check_relative $DIRECTORY
      DIRECTORY="$RETURN"

      check_dbdfile $DIRECTORY

      shift
      ;;
    "-from")
      VER1_DIR=$2

      check_nulldir $VER1_DIR

      check_relative $VER1_DIR
      VER1_DIR="$RETURN"

      check_dbdfile $VER1_DIR

      VER1_TABLES=${tables[*]}
      VER1_DATA=${data[*]}

      shift
      ;;
    "-to")
      VER2_DIR=$2

      check_nulldir $VER2_DIR

      check_relative $VER2_DIR
      VER2_DIR="$RETURN"

      check_dbdfile $VER2_DIR

      VER2_TABLES=${tables[*]}
      VER2_DATA=${data[*]}

      shift
      ;;
    "-database")  DATABASE=$2  
                  check_nulldb
                  shift;;
    "-create")  count=`expr $count + 1`
                given[0]="$count" ;;
    "-unload")  count=`expr $count + 1`
                given[1]="$count" ;;
    "-load")    count=`expr $count + 1`
                given[2]="$count" ;;
    "-clear")   count=`expr $count + 1` 
                given[3]="$count" ;;
    "-drop")    count=`expr $count + 1`
                given[4]="$count" ;;
    "-destroy") count=`expr $count + 1`
                given[5]="$count" ;;
    "-upgrade")
      count=`expr $count + 1`
      given[6]="$count"
      ;;
    "-schema_variables")
      SCHEMA_VARIABLES_FILE=$2
      shift
      ;;
    -*) syntax 
        exit $exit_rc_usage  ;;
  esac
  shift
done

#Validate database
if [ -z "$DATABASE" ]; then
  LOG "Return code $exit_rc_file_incorrect :  No database specified in either DBD file or command line"
  exit $exit_rc_file_incorrect
fi

DEBUG "99% DATABASE = $DATABASE"
DEBUG "99% DIRECTORY = $DIRECTORY"

LOGNAME=`basename $0`.log

# Array of commands
command[0]='CREATE'
command[1]='UNLOAD'
command[2]='LOAD'
command[3]='CLEAR'
command[4]='DROP'
command[5]='DESTROY'
command[6]='UPGRADE'

# Finds which command was issued in order 
#
let mynumcount=0
while (( $mynumcount <= ${#command[*]} ))
do   
  let mycount=0
  while (( $mycount < ${#command[*]} ))
  do
    if (( ${given[$mycount]} == $mynumcount )); then
      # Call the function (COMMAND_go)
      ${command[$mycount]}$functionname
    fi
    # Increment command order
    mycount=`expr $mycount + 1`
  done

  # Increment command
  mynumcount=`expr $mynumcount + 1`
done
