#!/bin/ksh
###############################################################################
# @(#) SysManagement/delivery/unix/bin/metrica_install.ksh SysManagement_r4.79.0.9:cset.008368:54:54 04/18/07 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
# 
# Copyright  2005 Vallent Software Systems UK. All rights reserved
#
# Title:        metrica_install.ksh
# Author:       Ed Tandi, Fergus Kelledy, Patrick Wood, Alan Greasley,
#               James Richardson
# Description:  Utility for installing packages 
# 
###############################################################################

unalias -a 

# For more debugging set M_DEBUG=1

PROGNAME=`basename $0`
DIRNAME=`dirname $0`
VERSION=`echo "SysManagement_r4.79.0.9:cset.008368:54:54" | cut -d: -f1`
export TMP_DIR=$TMP_DIR
opt_debug="0" 
SYSTEM_MANAGEMENT_PACKAGE_NAME=utils
APPSERVER_INSTALLATION=n
DATABASE_INSTALLATION=n

PASSWORD_RETRIES=0

export CUR_DIR=`pwd`
#Directory used to deploy jars and wars
DEPLOY_DIR=$CUR_DIR/install_tmp
INSTALL_TMP=install_tmp
#Flag donating whether deploying is done
DEPLOY_DONE=false
DEPLOY_DAR_DONE=false
IS_TECHPACK="f"
 
# Display version string 
# 
version() 
{ 
  if [ -f $METRICA_DIR/Installer/bin/metrica_version.ksh ]; then
    . $METRICA_DIR/Installer/bin/metrica_version.ksh
	 print_version "$PROGNAME" "SysManagement_r4.79.0.9:cset.008368:54:54"
  else
    $ECHO "$VERSION"
  fi
}



# Common non-zero return code handler 
# 
check_return_code() 
{
  return_code=$1
  error_message=$2
  if (( $return_code != 0 )); then
    LOG " ERROR: $error_message"
    m_exit $return_code "$error_message"
  fi
}



# Add an item to the system administration registry
#
ADMIN_REGISTER()
{
  add_reg_entry $1 $2 $METRICA_DIR/etc/admin_registry
}



# Check to see if an earlier version exists
#
# BUG-37722
# This function has been renamed to "check_earlier_patch_ver_exists"
# This change will ensure noone else is using this logic.
#
check_earlier_ver_exists()
{
  m_exit 1 "INSTALLER ERROR - function check_earlier_ver_exists has been called"
}

# Check to see if an earlier version exists
#
# BUG-37722
# New patch version failure handling
#
check_earlier_patch_ver_exists()
{
  old_pkg_name=$1
  old_pkg_ver=$2
  regfile=$3
  while read regname regvalue
  do
    if [ "$M_DEBUG" = "1" ] ; then
      LOG "DEBUG: regname = $regname"
      LOG "DEBUG: regvalue = $regvalue"
    fi
    [[ "$regname" = "$old_pkg_name" && "$regvalue" = "$old_pkg_ver" ]] && return 0
  done < $regfile
  # if we've got this far it's not there

  #
  # BUG-37722
  # New patch version failure handling
  #
  #m_exit 1 "Cannot install patch: version $old_pkg_ver of package $old_pkg_name not found in $regfile"
  WARNING "Dependency check failed: version $old_pkg_ver of package $old_pkg_name not found in $regfile"
}


# Update the version
#
update_earlier_version()
{
  old_pkg_name=$1
  old_pkg_ver=$2
  new_pkg_ver=$3
  regfile=$4
  output=""
  match="n"
  while read regname regvalue
  do
    if [[ "$regname" = "$old_pkg_name" && "$regvalue" = "$old_pkg_ver" ]]; then
      match="y"
      if [ -z "$output" ]; then
        output="$regname $new_pkg_ver"
      else
        output="$output\n$regname $new_pkg_ver"
      fi
    else
      if [ -z "$output" ]; then
        output="$regname $regvalue"
      else
        output="$output\n$regname $regvalue"
      fi
    fi
  done < $regfile

  #
  # BUG-37722
  #
  if [ "$match" = "n" ]; then
    if [ -z "$output" ]; then
      output="$old_pkg_name $new_pkg_ver"
    else
      output="$output\n$old_pkg_name $new_pkg_ver"
    fi
  fi

  echo "$output" > $regfile
}



# If this package is not in the registry, add it. If it is in the registry
# already, update its version, unless it's a patch, in which case add it
# again if the versions involved are different
#
add_reg_entry()
{
  LOG "Adding entry $1 $2 to admin registry $3"
  pkg_name=$1
  pkg_ver=$2
  regfile=$3
  if is_dupe_reg_entry $pkg_name $pkg_ver $regfile
  then
    LOG "Entry $1 $2 already present in $regfile"
    return 0
  fi  
  if [ ! -f $regfile ]; then
    # assume file hasn't been created yet so this entry is of course not a duplicate
    echo "$pkg_name $pkg_ver" > $regfile
    return 0
  fi
  # check if it's a patch
  if [ "`echo $pkg_name | grep [-]Patch$`" ]; then
    # it's a patch
    LOG "Updating admin registry with details of patch package"
    pkg_to_be_patched=${pkg_name%-*} # everything before last dash
    ver_to_be_patched=${pkg_ver%-*} # there should be one dash in patch ver
    # only use anything before first comma, in case there's a machine-specific indicator in the version
    ver_to_be_patched=${ver_to_be_patched%%,*} 
    ver_to_patch_to=${pkg_ver#*-} # everything after first dash
    ver_to_patch_to=${ver_to_patch_to%%,*} # and before first comma (if any)
    #
    # BUG-37722
    # New patch version failure handling
    #
    check_earlier_patch_ver_exists $pkg_to_be_patched $ver_to_be_patched $regfile # exits if not
    update_earlier_version $pkg_to_be_patched $ver_to_be_patched $ver_to_patch_to $regfile
  fi
  echo "$pkg_name $pkg_ver" >> $regfile
}



# check_admin_registry
#
check_admin_registry()
{
  package=$1
  version=$2
  isthere=`cat $METRICA_DIR/etc/admin_registry | grep "$package $version" | head -1`
  instpackage=`echo $isthere | awk '{print $1}' > /dev/null 2>&1`
  instversion=`echo $isthere | awk '{print $2}' > /dev/null 2>&1`
  if [ "$package" != "$instpackage" ]; then
    m_exit 1 "Required package ($package) is not installed"
  else
    if [ "$version" != "$instversion" ]; then
      v1=`echo $version | awk -F. '{print $1}' > /dev/null 2>&1` 
      v2=`echo $version | awk -F. '{print $2}' > /dev/null 2>&1` 
      v3=`echo $version | awk -F. '{print $3}' > /dev/null 2>&1` 
      v4=`echo $version | awk -F. '{print $4}' > /dev/null 2>&1` 

      iv1=`echo $instversion | awk -F. '{print $1}' > /dev/null 2>&1` 
      iv2=`echo $instversion | awk -F. '{print $2}' > /dev/null 2>&1` 
      iv3=`echo $instversion | awk -F. '{print $3}' > /dev/null 2>&1` 
      iv4=`echo $instversion | awk -F. '{print $4}' > /dev/null 2>&1` 

      if [ "$v1" != "$iv1" ]; then
        m_exit 1 "Required version ($version) of package ($package) is not installed"
      fi

      if [ "$iv2" > "$v2" ]; then
        LOG "Warning: Installed version ($instversion) of $package more recent that required version ($version)"
      fi
      if [ "$iv2" < "$v2" ]; then
        m_exit 1 "Required version ($version) of package ($package) is not installed"
      fi

      if [ "$iv3" > "$v3" ]; then
        LOG "Warning: Installed version ($instversion) of $package more recent that required version ($version)"
      fi
      if [ "$iv3" < "$v3" ]; then
        m_exit 1 "Required version ($version) of package ($package) is not installed"
      fi

      if [ "$v4" != "latest" ]; then
        if [ "$iv4" > "$v4" ]; then
          LOG "Warning: Installed version ($instversion) of $package more recent that required version ($version)"
        fi
        if [ "$iv4" < "$v4" ]; then
          m_exit 1 "Required version ($version) of package ($package) is not installed"
        fi
      fi
    fi
  fi
}



# Add an item to the system registry
#
REGISTER()
{
  ssn=$1
  script=$2
  # First check if it's there already
  if is_dupe_reg_entry $ssn $script $METRICA_DIR/etc/registry
  then
  return 1
  fi
  reg=`echo $script | grep metrica_[A-Za-z0-9]*admin.ksh | wc -l`
  if [ $reg -gt 0 ]; then
    $ECHO "$ssn $script" >> $METRICA_DIR/etc/registry 
    return 0
  fi
  reg=`echo $script | grep metrica_[A-Za-z0-9]*admin.sh | wc -l`
  if [ $reg -gt 0 ]; then
    $ECHO "$ssn $script" >> $METRICA_DIR/etc/registry 
    return 0
  fi
  reg=`echo $script | grep metrica_[A-Za-z0-9]*admin | wc -l`
  if [ $reg -gt 0 ]; then
    $ECHO "$ssn $script" >> $METRICA_DIR/etc/registry 
    return 0
  fi
}



# Check whether a registry entry already exists:
# return 0 if it does, 1 otherwise
#
is_dupe_reg_entry() {
  regentry="$1 $2"
  regfile=$3
  if [ ! -f $regfile ]; then
    # assume file hasn't been created yet so this entry
    # is of course not a duplicate
    return 1
  fi
  while read regname regvalue
  do
    line="$regname $regvalue"
    if [ "$line" = "$regentry" ]; then
      return 0
    fi
  done < $regfile
  return 1
}


 
# Display Metrica Banner
# 
display_metrica_banner() 
{
cat <<END
####################################################################
#
#                 Metrica Installation 
# 
#                   COPYRIGHT NOTICE 
# 
#     This file remains the intellectual property of: 
# 
#                 Vallent Software Systems 
#                    Spencer House 
#                    23 Sheen Road 
#                 Richmond, Surrey, UK 
# 
# Copyright  2005 Vallent Software Systems UK. All rights reserved 
# 
####################################################################

END
}



# Display install layers 
# 
display_layer_choice() 
{

cat <<END
  
  Select Metrica Layer(s) to install

  1) Web
  2) Application 
  3) Database 
  4) Application & Database
  5) Application & Web
  6) All

END

  $ECHO -n "Enter your choice [1-6] : " 
  read ANS
    
  rm -f $METRICA_DIR/etc/servertype > /dev/null 2>&1
  mkdir -p $METRICA_DIR/etc > /dev/null 2>&1
  
  case "$ANS" in 
    "1") 
      export INST_LAYER=WEB
      APPSERVER_INSTALLATION=y
      DATABASE_INSTALLATION=n
      $ECHO "WEB" >> $METRICA_DIR/etc/servertype ;;
    "2") 
      export INST_LAYER=APPLICATION
      APPSERVER_INSTALLATION=y
      DATABASE_INSTALLATION=n
      $ECHO "APPLICATION" >> $METRICA_DIR/etc/servertype ;;
    "3") 
      export INST_LAYER=DATABASE
      APPSERVER_INSTALLATION=n
      DATABASE_INSTALLATION=y
      $ECHO "DATABASE" >> $METRICA_DIR/etc/servertype ;;
    "4") 
      export INST_LAYER=APPLICATION,DATABASE
      APPSERVER_INSTALLATION=y
      DATABASE_INSTALLATION=y
      $ECHO "APPLICATION" >> $METRICA_DIR/etc/servertype
      $ECHO "DATABASE" >> $METRICA_DIR/etc/servertype ;;
    "5") 
      export INST_LAYER=APPLICATION,WEB
      APPSERVER_INSTALLATION=y
      DATABASE_INSTALLATION=n
      $ECHO "APPLICATION" >> $METRICA_DIR/etc/servertype
      $ECHO "WEB" >> $METRICA_DIR/etc/servertype ;;
    "6") 
      export INST_LAYER=ALL
      APPSERVER_INSTALLATION=y
      DATABASE_INSTALLATION=y
      $ECHO "APPLICATION" >> $METRICA_DIR/etc/servertype
      $ECHO "DATABASE" >> $METRICA_DIR/etc/servertype
      $ECHO "WEB" >> $METRICA_DIR/etc/servertype ;;
    *) 
      m_exit 1 "Invalid application layer selected" ;; 
  esac 
  $ECHO
  $ECHO "Metrica Application Layer(s) $INST_LAYER selected"
  $ECHO
  check_user
} 


 
# Show correct command-line usage 
# 
usage() 
{
cat <<END

  Usage: $PROGNAME [options]

  options:
           [ -h | -help | --help ] |
           [ -v | -version | --version ] |
           [ -installdir <install-directory> ]
           { <package> | -packagelist <packagelist-file> }

END
}



# check_user
#
check_user() {
  METRICA_USER_NAME="metrica"
  METRICA_GROUP_NAME="metrica"
  username=`logname`
  allgroups=`groups`
  if [[ -z "$username" || -z "$allgroups" ]]; then
    LOG "Unable to determine username and/or group"
    exit 1
  fi
  if [ "$username" != "$METRICA_USER_NAME" ]; then
    LOG "WARNING: metrica_install.ksh running as non-standard user $username"
  fi
  for group in $allgroups
  do
    if [ "$group" = "$METRICA_GROUP_NAME" ]; then
      groupname="$METRICA_GROUP_NAME"
    fi
  done
  if [ "$groupname" != "$METRICA_GROUP_NAME" ]; then
    LOG "WARNING: metrica_install.ksh user $username does not belong to $METRICA_GROUP_NAME group"
  fi
  LOG "Executing metrica_install.ksh as user=$username, group=$allgroups"
}



# create database
#
create_database() {
  dbase=$1
  dbase_space=$2
  log=$3
  LOG "Checking for existence of database : $dbase" 
  $ECHO "" | dbaccess $dbase > /dev/null 2>&1
  if [ $? -ne 0 ]; then

      if [ "$log" = "LOG" ]; then
        createstr="create database $dbase in $dbase_space WITH LOG"
      fi
      if [ "$log" = "BUFFERED" ]; then
        createstr="create database $dbase in $dbase_space WITH BUFFERED LOG"
      fi
      if [ "$log" = "nothing" ]; then
        createstr="create database $dbase in $dbase_space"
      fi
      LOG " $createstr"
      echo $createstr | dbaccess > /dev/null 2>&1;
      if [ "$?" != "0" ]; then
        m_exit 1 "Error in Metrica Database creation. Database $dbase has not been created. Aborting."
      else
        LOG " Database $dbase created in $dbase_space"
      fi
  fi
}


 
# Create editing file
#
create_editing_file()
{
  jndiname=$1
  driverjndiname=$2
  txlevel=$3
  dbname=$4

  if [ -f  $TMP_DIR/datasrc_edit_commands.tmp ]; then
    rm $TMP_DIR/datasrc_edit_commands.tmp
  fi
  echo "s/metrica_datasrcjndiname/$jndiname/g" >> \
    $TMP_DIR/datasrc_edit_commands.tmp
  echo "s/metrica_driverdatasrcjndiname/$driverjndiname/g" >> \
    $TMP_DIR/datasrc_edit_commands.tmp
  echo "s/metrica_txlevel/$txlevel/g" >> $TMP_DIR/datasrc_edit_commands.tmp
  echo "s/metrica_dbname/$dbname/g" >> $TMP_DIR/datasrc_edit_commands.tmp
}



# set all files in a dir, and any subdirs to rw by owner only
#
writeprotect_dir()
{
  base_dir=$1
  for file in `ls -1 $base_dir`
  do
    if [ ! -d $base_dir/$file ]; then
      chmod 600 $base_dir/$file
    else
      # must run recursion in a separate process to keep variables separate
      `writeprotect_dir $base_dir/$file`
    fi
  done
}



# expand_package
#
expand_package()
{
  LOG "Expanding package: $1 of type $2"
  z=`echo $1 | awk -F. '{print $NF}'` 
  pkgdir=`echo $1 | awk -F_ '{print $1}'` 
  tgtdir=`echo $pkgdir | awk -F- '{print $1}'` 
  MDV=`echo $METRICA_DIR` 
  MMDV=`echo $MDV | sed -e 's/\//\\\\\//g'`
  MDN="\$METRICA_DIR"
  export MDV=$MMDV
  export MDN=$MDN

  # Decompress if appropriate
  #
  was_compressed=0
  if [ "$z" = "Z" ]; then 
    cp $1 $pkgdir.tar.Z
    uncompress $pkgdir.tar.Z
    was_compressed=".Z"
  elif [ "$z" = "gz" ] ; then
    cp $1 $pkgdir.tar.gz
    gzip -d $pkgdir.tar.gz
    was_compressed=".gz"
  else
    cp $1 $pkgdir.tar
  fi 

  CHECK_STRING=`tar -tvf $pkgdir.tar | grep metrica_techpack.txt`

  if [[ "$CHECK_STRING" != "" ]]; then
 	IS_TECHPACK="t"
  fi 

#Check for type of uncompression required
 if [[ "$2" = "normal" ]]; then
  # Unpack tar file and remove intermediate uncompressed tar
  #
  tar -xf $pkgdir.tar
  if [ "$was_compressed" != "0" ] ; then
    rm -f $pkgdir.tar
  fi
else
#DEPLOY EXTRACTION START
TMPFILE=install_out.$$
DS_TMPFILE=install_ds.$$
#Check if our tmp dir exists.
#If not create it
[ ! -d ./$INSTALL_TMP ] && mkdir ./$INSTALL_TMP
cd ./$INSTALL_TMP 

if [[ $pkgdir.tar = "DataSources.tar" ]]; then
tar tf ../$pkgdir.tar |awk '/.dsd/' >$DS_TMPFILE 2>&1
  for ds_file in `cat ./$DS_TMPFILE | awk '{print $1}'`
  do
      tar xf ../$pkgdir.tar $ds_file
  done
fi

#Now to deploy all the jars and wars to out INSTALL_tmp dir
tar tf ../$pkgdir.tar | grep deploy | awk '/.war/ || /.jar/' >$TMPFILE 2>&1

for file_name in `cat ./$TMPFILE | awk '{print $1}'`
  do
    tar xf ../$pkgdir.tar $file_name
done

#Tmp file no longer needed
rm -f $TMPFILE
rm -f $DS_TMPFILE
cd $CUR_DIR
rm -f $pkgdir.tar
# We do not need the rest at this time
return 0
fi
#DEPLOY EXTRACTION END

  # Tar unpacks WRT to the current directory,
  # but package can be from anywhere, so fix variables...
  #
  pkgdir=`basename $pkgdir`
  if [ "$tgtdir" != "" ] ; then
    tgtdir=`basename $tgtdir`
  fi

  pki_file="./$pkgdir/package_items.txt" 
  export PKG_DIR=$pkgdir 
  export TGT_DIR=$tgtdir 


  DEST_DIR=$METRICA_DIR/$TGT_DIR


  # If the package contains a file called metrica_techpack.txt,
  # then it must be a Tech Pack package.
  # Tech Pack package names have dashes in them, to more
  # sophisticated logic in required to determine TGT_DIR.
  # Also, tech Pack package target directories are
  # located in $METRICA_DIR/TechnologyPacks.
  #
  if [ -f $PKG_DIR/metrica_techpack.txt ]; then
    # Get values for TECHNOLOGY and PKG_VERSION.
    # Format of metrica_techpack.txt is:
    #   technology:<technology>
    #   version:<version>
    #
    get_tmp_file
    TMP_FILE_1="$RETURN"

    $M_AWK -F":" '

      /^technology/ { print "TECHNOLOGY=\"" $2 "\"" }

    ' $PKG_DIR/metrica_techpack.txt > $TMP_FILE_1

    . $TMP_FILE_1

    # Tech Pack packages are typically named like:
    # GSM-Core_r2.0.0.1, or GSM-Core-Patch_r2.0.0.1-r2.1.0.1.
    #
    # According to NGN Naming Standards, release packages can
    # contain dashes in the subsystem alias, so TGT_DIR for
    # all packages could be calculated in the following way.
    #
    TGT_DIR=${PKG_DIR%-Patch}
    TGT_DIR=${TGT_DIR%-[0-9]*}

    DEST_DIR=$METRICA_DIR/TechnologyPacks/$TECHNOLOGY/$TGT_DIR

    if [ "$M_DEBUG" = "1" ] ; then
      LOG "DEBUG: DEST_DIR = $DEST_DIR"
    fi

    # Copy package to $METRICA_DIR/TechnologyPacks/.packagefiles.
    #
    [ ! -d $METRICA_DIR/TechnologyPacks/.packagefiles ] && mkdir -p $METRICA_DIR/TechnologyPacks/.packagefiles

    cp $1 $METRICA_DIR/TechnologyPacks/.packagefiles
  else
    # Copy package to $METRICA_DIR/.packagefiles.
    #
    [ ! -d $METRICA_DIR/.packagefiles ] && mkdir $METRICA_DIR/.packagefiles

    cp $1 $METRICA_DIR/.packagefiles
  fi

  export DEST_DIR
}



# Function to get metrica db username and password
# (and verify them in the case of a db layer installation)
#
get_metrica_credentials() {
  DFLT_DB_USERNAME=${METRICA_DB_USERNAME:-metrica}

  $ECHO -n "Enter Metrica DB Username (<Enter> for default - \"$DFLT_DB_USERNAME\")> "
  read METRICA_DB_USERNAME
  [ -z "$METRICA_DB_USERNAME" ] && METRICA_DB_USERNAME=$DFLT_DB_USERNAME
  if [ "$DATABASE_INSTALLATION" = "y" ]; then
    while id $METRICA_DB_USERNAME > /dev/null 2>&1 ; (( $? != 0 )) || [[ -z "$METRICA_DB_USERNAME" ]]
    do
      $ECHO -n "Username \"$METRICA_DB_USERNAME\" is not valid. Enter another Metrica DB username > "
      read METRICA_DB_USERNAME
    done
  fi

  RESULT=1
  export PASSWORD_ENTRY=
  while [[ $RESULT -ne 0 ]]
  do
    get_password_once "Metrica DB"
    RESULT=$?
  done
  METRICA_DB_PASSWORD=$PASSWORD_ENTRY

  if [ "$DATABASE_INSTALLATION" = "y" ] ; then
    verify_metrica_credentials
    if [ $? != 0 ] ; then
      echo "Password not valid for user $METRICA_DB_USERNAME and Informix server $INFORMIXSERVER"
      DFLT_DB_USERNAME=$METRICA_DB_USERNAME
      unset METRICA_DB_PASSWORD
      unset METRICA_DB_PASSWORD_CONFIRM
      if [[ $PASSWORD_RETRIES -lt 2 ]]; then
        ((PASSWORD_RETRIES+=1))
        get_metrica_credentials
      else
	m_exit 1 "ERROR: No more re-tries. Check that the username and password combination is correct for the Informix server $INFORMIXSERVER." 
      fi
    else
      # reset PASSWORD_RETRIES value to 0
      PASSWORD_RETRIES=0
    fi
  fi
}



# Function to ask for a password. once.
#
get_password_once()
{
  # Store password in PASSWORD_ENTRY - this must be exported outside this func
  # Turn off echoing to the terminal while password is typed
  stty -echo
  $ECHO -n "Enter $1 Password > "
  read PASSWORD_ENTRY
  # Turn on terminal echoing again
  stty echo
  $ECHO ""
  return 0
}


# Function to ask for a password, then ask for it again to confirm
#
get_password()
{
  # Store password in PASSWORD_ENTRY - this must be exported outside this func
  # Turn off echoing to the terminal while password is typed
  stty -echo
  $ECHO -n "Enter $1 Password > "
  read PASSWORD_ENTRY
  $ECHO ""
  $ECHO -n "Confirm $1 Password > "
  read PASSWORD_ENTRY_CONFIRM
  if [ "$PASSWORD_ENTRY" != "$PASSWORD_ENTRY_CONFIRM" ]; then
    $ECHO ""
    $ECHO "Passwords don't match"
    return 1
  fi
  # Turn on terminal echoing again
  stty echo
  $ECHO ""
  return 0
}



# Function to check that username and password are valid for Informix
#
verify_metrica_credentials()
{
  TEMP_PASSWORDCHECK_SQL_FILE=temp_passwordcheck.sql
  if [ -f $TEMP_PASSWORDCHECK_SQL_FILE ]; then
    rm $TEMP_PASSWORDCHECK_SQL_FILE
  fi
  echo "CONNECT TO '@${INFORMIXSERVER}' USER '${METRICA_DB_USERNAME}' USING '${METRICA_DB_PASSWORD}'; DISCONNECT CURRENT" > $TEMP_PASSWORDCHECK_SQL_FILE
  $INFORMIXDIR/bin/dbaccess - $TEMP_PASSWORDCHECK_SQL_FILE > /dev/null 2>&1
  RET_VAL=$?
  rm $TEMP_PASSWORDCHECK_SQL_FILE
  return $RET_VAL
}



# Source environment
#
source_environment()
{
  #Check if properties file exists. 
  #Else check if install setup exists
  #Else exit
  if [[ -f $DIRNAME/metrica_install.properties ]]; then
    [[ ! -f $DIRNAME/metrica_start_install.ksh ]] && m_exit 1 "Failed to find $DIRNAME/metrica_start_install.ksh"
    # pass DATABASE_INSTALLATION parameter to metrica_start_install
    # so that it knows not to test the metrica user with dbaccess if
    # a db install isn't being done
    $DIRNAME/metrica_start_install.ksh $DATABASE_INSTALLATION
    [[ $? -ne 0 ]] && m_exit 1 "$DIRNAME/metrica_start_install.ksh failed"
    [[ ! -f $METRICA_DIR/install_setup.ksh ]] && m_exit 1 "Failed to create $METRICA_DIR/install_setup.ksh"
    # ensure that the informix user can access the file
    chmod 755 $METRICA_DIR/install_setup.ksh > /dev/null 2>&1
    . $METRICA_DIR/install_setup.ksh > /dev/null 2>&1
else
	if [ -f $METRICA_DIR/install_setup.ksh ]; then
	. $METRICA_DIR/install_setup.ksh > /dev/null 2>&1
 else
	m_exit 1 "FAILED to find $DIRNAME/metrica_install.properties"
 fi
fi
}



# Check informix is running
# 
informix_check() {
  install_informixadmin
  $ECHO -n "Checking the status of Informix ..."
  [[ ! -f $METRICA_DIR/utils/bin/metrica_informixadmin.ksh ]] && m_exit 1 "Failed to find $METRICA_DIR/utils/bin/metrica_informixadmin.ksh"
  INFORMIX_RETURN_STRING=`$METRICA_DIR/utils/bin/metrica_informixadmin.ksh -status`
  INFORMIX_STATUS=`$ECHO "$INFORMIX_RETURN_STRING" | grep -i "running$"`
  if [ -z "$INFORMIX_STATUS" ]; then
    $ECHO ""
    $ECHO ""
    $ECHO "WARNING: $INFORMIX_RETURN_STRING"
    $ECHO ""
    $ECHO "Informix appears not to be running. Start Informix and/or alter the values of"
    if [ -f $METRICA_DIR/install_setup.ksh ]; then
      $ECHO "the INFORMIXSERVER, INFORMIXDIR, ONCONFIG and PATH environment variables"
      $ECHO "in the \$METRICA_DIR/install_setup.ksh file and begin the installation again."
    else
      $ECHO "the responses given to the prompts regarding the Informix installation directory,"
      $ECHO "the onconfig file name and the Informix server name when the installation is run again."
    fi
    $ECHO ""
    exit 1
  fi
  $ECHO " OK"
}


 
# If informixadmin script isn't present in $METRICA_DIR/utils/bin,
# copy it, register it and set its gid
# 
install_informixadmin() {
  if [ ! -f $METRICA_DIR/utils/bin/metrica_informixadmin.ksh ]; then
    if [ -f $DIRNAME/metrica_informixadmin.ksh ]; then
      cp $DIRNAME/metrica_informixadmin.ksh $METRICA_DIR/utils/bin/metrica_informixadmin.ksh > /dev/null 2>&1;
      if [ "$?" != "0" ]; then
        m_exit 1 "Error copying metrica_informixadmin.ksh as user informix"
      else
        $ECHO "metrica_informixadmin.ksh has been successfully copied"
      fi
      #set_informixadmin_permissions
      REGISTER $SYSTEM_MANAGEMENT_PACKAGE_NAME metrica_informixadmin.ksh
    else
      m_exit 1 "Cannot find script $DIRNAME/metrica_informixadmin.ksh"
    fi
  fi
}



# make informix the owner of the metrica_informixadmin.ksh script, so that
# the effective group id can be set so that the script runs as the informix
# group when executed
#
set_informixadmin_permissions() {
  # Check user informix exists
  id informix > /dev/null 2>&1
  (( $?!=0 )) && m_exit 1 "User informix does not exist"
  # Give ownership of the metrica_informixadmin admin script to informix
  chown informix:informix $METRICA_DIR/utils/bin/metrica_informixadmin.ksh 2>&1
  if [ "$?" != "0" ]; then
    m_exit 1 "Error changing ownership of $METRICA_DIR/utils/bin/metrica_informixadmin.ksh to user informix, group informix"
  else
    $ECHO "Changed ownership of $METRICA_DIR/utils/bin/metrica_informixadmin.ksh to user informix, group informix"
  fi
  # Log on as user informix, chmod the informixadmin file
  # loop if there's an error, in case the user has typed an incorrect password
  let i=0
  while set_informixadmin_id; (( $?!=0 ))
  do 
    $ECHO ""
    let i=$i+1
    if (( $i>2 ))
    then
      m_exit 1 "Failed to log in as user informix"
    else
      $ECHO "Error setting the setgid on metrica_informixadmin.ksh as user informix. Check password and try again"  
      $ECHO ""
    fi
  done
}



# chmod the informixadmin script
#
set_informixadmin_id() {
  $ECHO "Logging on as user informix"
  $ECHO "Please enter informix password"
  su informix -c "chmod 2755 $METRICA_DIR/utils/bin/metrica_informixadmin.ksh" > /dev/null 2>&1
  let STATUS=$?
  if (( $STATUS = 0 ))
  then
    $ECHO "The setuid on metrica_informixadmin.ksh has been successfully set"
    $ECHO "Logging off as user informix"
  fi
  return $STATUS
}


 
# Check appserver is running
# 
appserver_check()
{
  export APPSERVER_ADM_CMD_SCRIPT=metrica_appserveradmin.ksh
  export APPSERVER_ADM_CMD=$METRICA_DIR/utils/bin/$APPSERVER_ADM_CMD_SCRIPT

  install_appserveradmin

  $ECHO -n "Checking the status of the appserver ..."

  APPSERVER_STATUS=`$APPSERVER_ADM_CMD -status | grep -i "running$"`

  if [ -z "$APPSERVER_STATUS" ]; then
    $ECHO ""
    $ECHO ""
    $ECHO "The appserver appears not to be fully running. Start the appserver, including"
    $ECHO "the EJB container and the WEB containers, and begin the installation again."
    $ECHO ""

    exit 1
  fi

  $ECHO " OK"
}


#
# The script metrica_install_common.ksh is used by variety of
# Installer scripts in the utils bin directory
#
install_metrica_install_common() 
{
  if [ ! -f $METRICA_DIR/utils/bin/metrica_install_common.ksh ]; then
    if [ -f $DIRNAME/metrica_install_common.ksh ]; then
      LOG "Copying $DIRNAME/metrica_install_common.ksh to $METRICA_DIR/utils/bin/metrica_install_common.ksh"
      cp $DIRNAME/metrica_install_common.ksh  $METRICA_DIR/utils/bin/metrica_install_common.ksh
      if (( $?!=0 ))
      then
        m_exit 1 "Failed to copy $DIRNAME/metrica_install_common.ksh to $METRICA_DIR/utils/bin/metrica_install_common.ksh"
      fi
      chmod 755 $METRICA_DIR/utils/bin/metrica_install_common.ksh
      if (( $?!=0 ))
      then
        m_exit 1 "Failed to change the mode of $METRICA_DIR/utils/bin/metrica_install_common.ksh to executable"
      fi
    else
      m_exit 1 "Cannot find script $DIRNAME/metrica_install_common.ksh"
    fi
  fi
}
 
# If appserveradmin script isn't present in $METRICA_DIR/utils/bin,
#  copy and register it 
#
install_appserveradmin()
{
  if [ ! -f $APPSERVER_ADM_CMD ]; then
    if [ -f $DIRNAME/$APPSERVER_ADM_CMD_SCRIPT ]; then
      $ECHO "Copying $DIRNAME/$APPSERVER_ADM_CMD_SCRIPT to $APPSERVER_ADM_CMD"
      cp $DIRNAME/$APPSERVER_ADM_CMD_SCRIPT $APPSERVER_ADM_CMD
      if (( $?!=0 ))
      then
        m_exit 1 "Failed to copy $DIRNAME/$APPSERVER_ADM_CMD_SCRIPT to $APPSERVER_ADM_CMD"
      fi
      chmod 755 $APPSERVER_ADM_CMD
      if (( $?!=0 ))
      then
        m_exit 1 "Failed to change the mode of $APPSERVER_ADM_CMD to executable"
      fi
      REGISTER $SYSTEM_MANAGEMENT_PACKAGE_NAME $APPSERVER_ADM_CMD_SCRIPT
    else
      m_exit 1 "Cannot find script $DIRNAME/$APPSERVER_ADM_CMD_SCRIPT"
    fi
  fi
}



# Install information scripts
#
install_info_script()
{
  if [ -f $DIRNAME/metrica_info.ksh ]; then
    [[ -f $METRICA_DIR/utils/bin/metrica_info.ksh ]] && rm -f $METRICA_DIR/utils/bin/metrica_info.ksh > /dev/null 2>&1
    cp $DIRNAME/metrica_info.ksh $METRICA_DIR/utils/bin
    if (( $?!=0 )); then
      LOG "WARNING: failed to copy metrica_info.ksh, script for listing installed packages"
    else
      if [ ! -x $METRICA_DIR/utils/bin/metrica_info.ksh ]; then 
        chmod 755 $METRICA_DIR/utils/bin/metrica_info.ksh > /dev/null 2>&1
        if (( $?!=0 )); then
          LOG "WARNING: failed to change the mode of metrica_info.ksh, script for listing installed packages"
          return 1
        fi
      fi
      LOG "Installed $METRICA_DIR/utils/bin/metrica_info.ksh, script for listing installed packages"
    fi
  else
    LOG "could not find metrica_info.ksh, script for listing installed packages"
  fi
}



# environment_check
# 
environment_check()
{
  [[ -z "$METRICA_DIR" ]] && m_exit 1 "METRICA_DIR undefined"

  $ECHO

  source_environment

  # Set DB User & Password
  #
  [[ -z "$METRICA_DB_USERNAME" || -z "$METRICA_DB_PASSWORD" ]] && get_metrica_credentials

  $ECHO

  ###################
  # schema_variables
  ###################

  # Function to ask user if there is a schema_variables
  # file to be used as input to schema creation, and if
  # there is, to ask user for location of file.
  #
  get_schema_variables_file()
  {
	 #If SCHEMA_VARIABLES_FILE exists in the properties file use it
	 #otherwise ask the user
		PROP_SCHEMA_VARIABLES_FILE=
		INSTALL_PROP_FILE=$DIRNAME/metrica_install.properties
		if [[ -f $DIRNAME/metrica_install.properties ]]; then
		PROP_SCHEMA_VARIABLES_FILE=`awk '/^SCHEMA_VARIABLES_FILE/' $INSTALL_PROP_FILE | awk -F= '{print $2}' | awk '{print $1}'`
		fi
		if [[ -n "$PROP_SCHEMA_VARIABLES_FILE" ]]; then
		LOG  "SCHEMA_VARIABLES_DIR from properties file is $PROP_SCHEMA_VARIABLES_FILE"
		SCHEMA_VARIABLES_FILE=$PROP_SCHEMA_VARIABLES_FILE
		else
    $ECHO -n "Is a schema_variables file to be input (yes/no) (<Enter> for default - no)? "
    read IS_SCHEMA_VARIABLES_FILE

    # If IS_SCHEMA_VARIABLES_FILE contains a 'y', then grep will return a 0.
    #
    #$ECHO "$IS_SCHEMA_VARIABLES_FILE" | grep -i "y" > /dev/null
    case `$ECHO $IS_SCHEMA_VARIABLES_FILE | tr [:upper:] [:lower:]` in
      "y"|"ye"|"yes") IS_SCHEMA_VARIABLES_FILE="TRUE";;
      *) IS_SCHEMA_VARIABLES_FILE=
    esac

    if [ "$IS_SCHEMA_VARIABLES_FILE" ]; then
      DFLT_SCHEMA_VARIABLES_FILE="$METRICA_DIR/schema_variables"

      $ECHO -n "Enter location of schema_variables file (<Enter> for default - \"$DFLT_SCHEMA_VARIABLES_FILE\")> "
      read SCHEMA_VARIABLES_FILE
      fi
			fi

      [[ -z "$SCHEMA_VARIABLES_FILE" ]] && SCHEMA_VARIABLES_FILE="$DFLT_SCHEMA_VARIABLES_FILE"
  }


  get_schema_variables_file

  while [[ -n "$IS_SCHEMA_VARIABLES_FILE" && ( ! -f "$SCHEMA_VARIABLES_FILE" && ! -L "$SCHEMA_VARIABLES_FILE" ) ]]
  do
    $ECHO "$SCHEMA_VARIABLES_FILE: No such file"
    get_schema_variables_file
  done

#Check for where schema file in property file does not exist 

	if [[ -n "$PROP_SCHEMA_VARIABLES_FILE" && ( ! -f "$SCHEMA_VARIABLES_FILE" && ! -L "$SCHEMA_VARIABLES_FILE" ) ]]; then
	check_return_code 1 "Schema file $SCHEMA_VARIABLES_FILE set in $INSTALL_PROP_FILE does not exist"
	fi

  if [ "$M_DEBUG" = "1" ] ; then
      LOG "DEBUG: IS_SCHEMA_VARIABLES_FILE = $IS_SCHEMA_VARIABLES_FILE"
      LOG "DEBUG: SCHEMA_VARIABLES_FILE = $SCHEMA_VARIABLES_FILE"
  fi

  #
  # BUG-40426 : Comment out the redundent call causing problems
  #
  #install_info_script

  [[ "$DATABASE_INSTALLATION" = "y" ]] && informix_check
  [[ "$APPSERVER_INSTALLATION" = "y" ]] && appserver_check

  # Copy metrica_install_common.ksh to $METRICA_DIR/utils/bin.
  #
  #if [ -a $METRICA_DIR/utils/bin/metrica_install_common.ksh ]; then
  #  rm -f $METRICA_DIR/utils/bin/metrica_install_common.ksh
  #fi
  #cp -p $DIRNAME/metrica_install_common.ksh $METRICA_DIR/utils/bin
  install_metrica_install_common

  # check temp directory setting 
  if [ -z "$TMP_DIR" ]; then 
    export TMP_DIR=/tmp
  fi 

  # check 3pp software installation directories
  export INST_DIR=$METRICA_DIR
  if [ -z "$INFORMIXDIR" ]; then 
    m_exit 1 "Informix Server Directory not Set" 
  fi 

  if [ -z "$JAVA_HOME" ]; then 
    m_exit 1 "Java Home Directory not Set" 
  fi 

  # check server name settings
  if [ -z "$INFORMIXSERVER" ]; then 
    m_exit 1 "Informix Server Name not Set" 
  fi 

  if [ -z "$EJBPARTITION_NAME" ]; then 
    m_exit 1 "EJB Partition Name not Set" 
  fi 
  if [ -z "$WEBSERVER_NAME" ]; then 
    m_exit 1 "Webserver Name not Set" 
  fi 
  if [ -z "$DB_SERVER_NAME" ]; then 
    if [ -z "$INFORMIXSERVER" ]; then 
      m_exit 1 "Database Server Name not Set" 
    fi 
    export DB_SERVER_NAME=$INFORMIXSERVER
  fi 
  if [ -z "$DB_HOST_NAME" ]; then 
    m_exit 1 "Database Host Name not Set" 
  fi 
  if [ -z "$DB_PORT" ]; then 
    m_exit 1 "Database Port not Set" 
  fi 

  # check metrica database settings
  if [ -z "$METRICA_ADMINISTRATION_DB" ]; then 
    m_exit 1 "Metrica administration database not Set" 
  fi 
  if [ -z "$METRICA_SYSTEM_DB" ]; then 
    m_exit 1 "Metrica system database not Set" 
  fi 
  if [ -z "$METRICA_PERFORMANCE_DB" ]; then 
    m_exit 1 "Metrica performance database not Set" 
  fi 
  if [ -z "$METRICA_ADMINISTRATION_DBSPACE" ]; then 
    m_exit 1 "Metrica administration database not Set" 
  fi 
  if [ -z "$METRICA_SYSTEM_DBSPACE" ]; then 
    m_exit 1 "Metrica system database not Set" 
  fi 
  if [ -z "$METRICA_PERFORMANCE_DBSPACE" ]; then 
    m_exit 1 "Metrica performance database not Set" 
  fi 
}


 
# generate_replacement_script 
# 
generate_replacement_script()
{
  LOG "Generating replacement script" 
  rm -f $TMP_DIR/edit_dd.tmp
  MDV=`echo $METRICA_DIR` 
  MMDV=`echo $MDV | sed -e 's/\//\\\\\//g'`
  $ECHO "s/metrica_dir/$MMDV/g" >> $TMP_DIR/edit_dd.tmp
  $ECHO "s/metrica_webservername/$WEBSERVER_NAME/g" >> $TMP_DIR/edit_dd.tmp
  $ECHO "s/metrica_appservername/$APPSERVER_NAME/g" >> $TMP_DIR/edit_dd.tmp
  $ECHO "s/metrica_host/$DB_HOST_NAME/g" >> $TMP_DIR/edit_dd.tmp
  $ECHO "s/metrica_port/$DB_PORT/g" >> $TMP_DIR/edit_dd.tmp
  $ECHO "s/metrica_password/$METRICA_DB_PASSWORD/g" >> $TMP_DIR/edit_dd.tmp
  $ECHO "s/metrica_username/$METRICA_DB_USERNAME/g" >> $TMP_DIR/edit_dd.tmp
  $ECHO "s/metrica_informixservername/$DB_SERVER_NAME/g" >> $TMP_DIR/edit_dd.tmp
  $ECHO "s/metrica_systemdb/$METRICA_SYSTEM_DB/g" >> $TMP_DIR/edit_dd.tmp
  $ECHO "s/metrica_administrationdb/$METRICA_ADMINISTRATION_DB/g" >> $TMP_DIR/edit_dd.tmp
  $ECHO "s/metrica_performancedb/$METRICA_PERFORMANCE_DB/g" >> $TMP_DIR/edit_dd.tmp
}



# install_custom_environment 
#
install_custom_environment()
{
  if [ -f ./$PKG_DIR/package_environment.ksh ]; then
    LOG " Installing custom environment script ./$PKG_DIR/package_enviroment.ksh"
    mkdir -p $METRICA_DIR/etc/environment/$TGT_DIR > /dev/null 2>&1
    cp -r ./$PKG_DIR/package_environment.ksh $METRICA_DIR/etc/environment/$TGT_DIR/environment.ksh > /dev/null 2>&1
    chmod 755 $METRICA_DIR/etc/environment/$TGT_DIR/environment.ksh > /dev/null 2>&1
  fi
}



do_bin_uninstall_directory()
{
  for FILE in `ls -1 ./$PKG_DIR/$1/bin_uninstall 2> /dev/null`
  do
    rm -rf $DEST_DIR/bin/`basename $FILE` 2> /dev/null
  done

  # If there aren't any more files in the bin
  # directory, then remove the bin directory.
  #
  rmdir -p -s $DEST_DIR/bin 2> /dev/null
}



do_bin_directory()
{
  type="bin"
  ls -1 ./$PKG_DIR/$1/$type > /dev/null 2>&1
  if test $? -eq 0 ; then
    LOG " Installing from $PKG_DIR/$1/$type to $DEST_DIR/$type"
    mkdir -p $DEST_DIR/$type 
    for x in `ls -1 ./$PKG_DIR/$1/$type 2> /dev/null`
    do
      [[ -d $DEST_DIR/$type/$x || -f $DEST_DIR/$type/$x ]] && rm -rf $DEST_DIR/$type/$x 2> /dev/null
      cp -r ./$PKG_DIR/$1/$type/$x $DEST_DIR/$type 2> /dev/null
      check_return_code $? "failed to copy $PKG_DIR/$1/$type/$x to $DEST_DIR/$type"

      if [ ! -x $DEST_DIR/$type/$x ]; then
      chmod 755 $DEST_DIR/$type/$x 2> /dev/null 
         check_return_code $? "failed to make $DEST_DIR/$type/$x executable"
      fi
      REGISTER $TGT_DIR $x
    done
    # PAW 23/7/02 to ensure tools are on the path if needed by subsequent packages during the same install
    export PATH=$DEST_DIR/$type:$PATH
    LOG " Added $DEST_DIR/$type to path"
  fi
}



do_lib_uninstall_directory()
{
  for FILE in `ls -1 ./$PKG_DIR/$1/lib_uninstall 2> /dev/null`
  do
    rm -rf $DEST_DIR/lib/`basename $FILE` 2> /dev/null
  done

  # If there aren't any more files in the lib
  # directory, then remove the lib directory.
  #
  rmdir -p -s $DEST_DIR/lib 2> /dev/null
}



do_lib_directory()
{
  type="lib"
  ls -1 ./$PKG_DIR/$1/$type > /dev/null 2>&1
  if test $? -eq 0 ; then
    LOG " Installing from $PKG_DIR/$1/$type to $DEST_DIR/$type"
    mkdir -p $DEST_DIR/$type 
    for x in `ls -1 ./$PKG_DIR/$1/$type 2> /dev/null`
    do
      [[ -d $DEST_DIR/$type/$x || -f $DEST_DIR/$type/$x ]] && rm -rf $DEST_DIR/$type/$x 2> /dev/null
      cp -r ./$PKG_DIR/$1/$type/$x $DEST_DIR/$type 2> /dev/null
      check_return_code $? "failed to copy $PKG_DIR/$1/$type/$x to $DEST_DIR/$type"
    done
  fi
}



do_data()
{
  [[ ! -d ./$PKG_DIR/$1/data && ! -d ./$PKG_DIR/$1/data_uninstall ]] && return

  type="data"

  ##########################
  # Update schema and data.
  ##########################

  get_tmp_file
  TMP_DBD="$RETURN"

  get_tmp_dir
  TMP_DBD_DIR="$RETURN"

  if [ "$M_DEBUG" = "1" ] ; then
    LOG "DEBUG: TMP_DBD_DIR = $TMP_DBD_DIR"
    ls $TMP_DBD_DIR
  fi

  # Clear data in EIXDs in data_uninstall item-subdirectory from database.
  #########################################################################

  # This function is called before running install/install.ksh. Only
  # execute this code if there is a DBD file in PKG_DIR or DEST_DIR.
  #
  if [[ -f ./$PKG_DIR/$1/data_uninstall/$1.dbd || -f $DEST_DIR/data/$1.dbd ]]; then
    if [ "`ls ./$PKG_DIR/$1/data_uninstall/*.eixd 2> /dev/null`" ]; then
      # Construct temporary DBD file in PKG_DIR.
      #
      if [ -f ./$PKG_DIR/$1/data_uninstall/$1.dbd ]; then
        grep -h "^database" ./$PKG_DIR/$1/data_uninstall/$1.dbd > $TMP_DBD
        grep -h "^user" ./$PKG_DIR/$1/data_uninstall/$1.dbd >> $TMP_DBD
        grep -h "^password" ./$PKG_DIR/$1/data_uninstall/$1.dbd >> $TMP_DBD
        grep -h "^indexspace" ./$PKG_DIR/$1/data_uninstall/$1.dbd >> $TMP_DBD

        mv ./$PKG_DIR/$1/data_uninstall/$1.dbd $TMP_DBD_DIR
      else
        grep -h "^database" $DEST_DIR/data/$1.dbd > $TMP_DBD
        grep -h "^user" $DEST_DIR/data/$1.dbd >> $TMP_DBD
        grep -h "^password" $DEST_DIR/data/$1.dbd >> $TMP_DBD
        grep -h "^indexspace" $DEST_DIR/data/$1.dbd >> $TMP_DBD
      fi

      # Add data lines EIXDs in data to TMP_DBD.
      #
      for FILE in `ls ./$PKG_DIR/$1/data_uninstall/*.eixd 2> /dev/null`
      do
        $ECHO "data:`basename $FILE .eixd`" >> $TMP_DBD
      done

      cp $TMP_DBD $PKG_DIR/$1/data_uninstall/$1.dbd

      if [ "$M_DEBUG" = "1" ] ; then
	ls $PKG_DIR/$1/data_uninstall/*.dbd
	
	LOG "DEBUG: $PKG_DIR/$1/data_uninstall/$1.dbd:"
	cat $PKG_DIR/$1/data_uninstall/$1.dbd

	LOG "DEBUG: Doing a clear..."
      fi

      metrica_dbdef.ksh -directory ./$PKG_DIR/$1/data_uninstall -clear

      if [ "$M_DEBUG" = "1" ] ; then
	LOG "DEBUG: Finished clear!"
      fi

      # Put original DBD back.
      #
      mv $TMP_DBD_DIR/*.dbd ./$PKG_DIR/$1/data_uninstall
    fi
  fi

  # Drop tables of TDEFs and SQL files in data_uninstall item-subdirectory.
  ##########################################################################

  # This function is called before running install/install.ksh. Only
  # execute this code if there is a DBD file in PKG_DIR or DEST_DIR.
  #
  if [[ -f ./$PKG_DIR/$1/data_uninstall/$1.dbd || -f $DEST_DIR/data/$1.dbd ]]; then
    if [ "`ls ./$PKG_DIR/$1/data_uninstall/*.sql ./$PKG_DIR/$1/data_uninstall/*.tdef 2> /dev/null`" ]; then
      # Construct temporary DBD file in PKG_DIR.
      #
      if [ -f ./$PKG_DIR/$1/data_uninstall/$1.dbd ]; then
        grep -h "^database" ./$PKG_DIR/$1/data_uninstall/$1.dbd > $TMP_DBD
        grep -h "^user" ./$PKG_DIR/$1/data_uninstall/$1.dbd >> $TMP_DBD
        grep -h "^password" ./$PKG_DIR/$1/data_uninstall/$1.dbd >> $TMP_DBD
        grep -h "^indexspace" ./$PKG_DIR/$1/data_uninstall/$1.dbd >> $TMP_DBD

        mv ./$PKG_DIR/$1/data_uninstall/$1.dbd $TMP_DBD_DIR
      else
        grep -h "^database" $DEST_DIR/data/$1.dbd > $TMP_DBD
        grep -h "^user" $DEST_DIR/data/$1.dbd >> $TMP_DBD
        grep -h "^password" $DEST_DIR/data/$1.dbd >> $TMP_DBD
        grep -h "^indexspace" $DEST_DIR/data/$1.dbd >> $TMP_DBD
      fi

      # Add table lines for SQL files in data_uninstall
      # to UNINSTALL_DBD_TABLE_SQL_LINES.
      #
      for FILE in `ls ./$PKG_DIR/$1/data_uninstall/*.sql 2> /dev/null`
      do
        $ECHO "table:`basename $FILE .sql`" >> $TMP_DBD
      done

      # Add table lines for TDEFs in data_uninstall to TMP_DBD.
      #
      for FILE in `ls ./$PKG_DIR/$1/data_uninstall/*.tdef 2> /dev/null`
      do
        $ECHO "table:`basename $FILE .tdef`" >> $TMP_DBD
      done

      cp $TMP_DBD $PKG_DIR/$1/data_uninstall/$1.dbd

      if [ "$M_DEBUG" = "1" ] ; then
	  ls $PKG_DIR/$1/data_uninstall/*.dbd
	  LOG "DEBUG: $PKG_DIR/$1/data_uninstall/$1.dbd:"
	  cat $PKG_DIR/$1/data_uninstall/$1.dbd
	  LOG "DEBUG: Doing a drop..."
      fi

      metrica_dbdef.ksh -directory ./$PKG_DIR/$1/data_uninstall -drop

      if [ "$M_DEBUG" = "1" ] ; then
	  LOG "DEBUG: Finished drop!"
      fi

      # Put original DBD back.
      #
      mv $TMP_DBD_DIR/*.dbd ./$PKG_DIR/$1/data_uninstall
    fi
  fi

  # Create or upgrade tables.
  ############################

  # This function is called before running install/install.ksh. Only
  # execute this code if there is a DBD file in PKG_DIR or DEST_DIR.
  #
  if [[ -f ./$PKG_DIR/$1/data/$1.dbd || -f $DEST_DIR/data/$1.dbd ]]; then
    if [ "`ls ./$PKG_DIR/$1/data/*.sql ./$PKG_DIR/$1/data/*.tdef 2> /dev/null`" ]; then
      # If there is a DBD file in METRICA_DIR, then upgrade the existing
      # schema from the tables defined in the TDEFs in METRICA_DIR, listed
      # in the DBD file in METRICA_DIR, to the tables defined in the TDEFs
      # in PKG_DIR, listed in the DBD file in PKG_DIR.
      #
      # Otherwise, create the schema from the tables defined in the TDEFs
      # in PKG_DIR, listed in the DBD file in PKG_DIR.

      if [ -f $DEST_DIR/data/$1.dbd ]; then
        # There is a DBD file in METRICA_DIR,
        # so upgrade schema.

        # Construct temporary DBD file in PKG_DIR.
        #
        if [ -f ./$PKG_DIR/$1/data/$1.dbd ]; then
          grep -h "^database" ./$PKG_DIR/$1/data/$1.dbd > $TMP_DBD
          grep -h "^user" ./$PKG_DIR/$1/data/$1.dbd >> $TMP_DBD
          grep -h "^password" ./$PKG_DIR/$1/data/$1.dbd >> $TMP_DBD
          grep -h "^indexspace" ./$PKG_DIR/$1/data/$1.dbd >> $TMP_DBD

          mv ./$PKG_DIR/$1/data/$1.dbd $TMP_DBD_DIR
        else
          grep -h "^database" $DEST_DIR/data/$1.dbd > $TMP_DBD
          grep -h "^user" $DEST_DIR/data/$1.dbd >> $TMP_DBD
          grep -h "^password" $DEST_DIR/data/$1.dbd >> $TMP_DBD
          grep -h "^indexspace" $DEST_DIR/data/$1.dbd >> $TMP_DBD
        fi

        # Add table lines for TDEFs in data to TMP_DBD.
        #
        for FILE in `ls ./$PKG_DIR/$1/data/*.tdef 2> /dev/null`
        do
          $ECHO "table:`basename $FILE .tdef`" >> $TMP_DBD
        done

        cp $TMP_DBD $PKG_DIR/$1/data/$1.dbd

	if [ "$M_DEBUG" = "1" ] ; then
	  LOG "DEBUG: $PKG_DIR/$1/data/$1.dbd:"
	  cat $PKG_DIR/$1/data/$1.dbd
	fi

        # Create a temporary 'from' directory.
        #
        get_tmp_dir
        TMP_DBD_DIR_2="$RETURN"

        cp $DEST_DIR/data/$1.dbd $TMP_DBD_DIR_2
        cp $DEST_DIR/data/*.tdef $TMP_DBD_DIR_2

	if [ "$M_DEBUG" = "1" ] ; then
	  LOG "DEBUG: Upgrading a Database Table"
	fi

        if [ "$IS_SCHEMA_VARIABLES_FILE" ]; then
          metrica_dbdef.ksh -upgrade -from $TMP_DBD_DIR_2 -to ./$PKG_DIR/$1/data -schema_variables $SCHEMA_VARIABLES_FILE
        else
          metrica_dbdef.ksh -upgrade -from $TMP_DBD_DIR_2 -to ./$PKG_DIR/$1/data
        fi

	if [ "$M_DEBUG" = "1" ] ; then
	  LOG "DEBUG: Upgrade complete!"
	fi

        # Put original DBD back.
        #
        mv $TMP_DBD_DIR/*.dbd ./$PKG_DIR/$1/data
      else
        # There isn't a DBD file in METRICA_DIR,
        # so create schema.
        #
	if [ "$M_DEBUG" = "1" ] ; then
	  LOG "DEBUG: Creating a Database Table"
	fi

        if [ "$IS_SCHEMA_VARIABLES_FILE" ]; then
          metrica_dbdef.ksh -directory ./$PKG_DIR/$1/data -create -schema_variables $SCHEMA_VARIABLES_FILE
        else
          metrica_dbdef.ksh -directory ./$PKG_DIR/$1/data -create
        fi

	if [ "$M_DEBUG" = "1" ] ; then
	  LOG "DEBUG: Create complete!"
	fi
      fi
    fi
  fi

  # Load in new data.
  ####################

  # This function is called before running install/install.ksh. Only
  # execute this code if there is a DBD file in PKG_DIR or DEST_DIR.
  #
  if [[ -f ./$PKG_DIR/$1/data/$1.dbd || -f $DEST_DIR/data/$1.dbd ]]; then
    if [ "`ls ./$PKG_DIR/$1/data/*.eixd ./$PKG_DIR/$1/data/*.ixd 2> /dev/null`" ]; then
      # Construct temporary DBD file in PKG_DIR.
      #
      if [ -f ./$PKG_DIR/$1/data/$1.dbd ]; then
        grep -h "^database" ./$PKG_DIR/$1/data/$1.dbd > $TMP_DBD
        grep -h "^user" ./$PKG_DIR/$1/data/$1.dbd >> $TMP_DBD
        grep -h "^password" ./$PKG_DIR/$1/data/$1.dbd >> $TMP_DBD
        grep -h "^indexspace" ./$PKG_DIR/$1/data/$1.dbd >> $TMP_DBD
        grep -h "^table" ./$PKG_DIR/$1/data/$1.dbd >> $TMP_DBD

        mv ./$PKG_DIR/$1/data/$1.dbd $TMP_DBD_DIR
      else
        grep -h "^database" $DEST_DIR/data/$1.dbd > $TMP_DBD
        grep -h "^user" $DEST_DIR/data/$1.dbd >> $TMP_DBD
        grep -h "^password" $DEST_DIR/data/$1.dbd >> $TMP_DBD
        grep -h "^indexspace" $DEST_DIR/data/$1.dbd >> $TMP_DBD
      fi

      # Add data lines for EIXDs in data to TMP_DBD.
      #
      for FILE in `ls ./$PKG_DIR/$1/data/*.eixd 2> /dev/null`
      do
        $ECHO "data:`basename $FILE .eixd`" >> $TMP_DBD
      done

      # Add data lines for IXDs in data to TMP_DBD.
      #
      for FILE in `ls ./$PKG_DIR/$1/data/*.ixd 2> /dev/null`
      do
        $ECHO "data:`basename $FILE .ixd`" >> $TMP_DBD
      done

      cp $TMP_DBD $PKG_DIR/$1/data/$1.dbd

      if [ "$M_DEBUG" = "1" ] ; then
	LOG "DEBUG: $PKG_DIR/$1/data/$1.dbd:"
	cat $PKG_DIR/$1/data/$1.dbd
	LOG "DEBUG: Doing a load..."
      fi

      metrica_dbdef.ksh -directory ./$PKG_DIR/$1/data -load

      if [ "$M_DEBUG" = "1" ] ; then
	LOG "DEBUG: Load complete!"
      fi

      # Put original DBD back.
      #
      mv $TMP_DBD_DIR/*.dbd ./$PKG_DIR/$1/data
    fi
  fi

  ###############################
  # Update files in METRICA_DIR.
  ###############################

  # TODO: Enhance algorithm to remove old versions of TDEFs
  # of which the filename has changed between versions.

  if [ "$M_DEBUG" = "1" ] ; then
      LOG "DEBUG: Are there files in data_uninstall?"
      LOG "ls -1 ./$PKG_DIR/$1/data_uninstall"
      ls -1 ./$PKG_DIR/$1/data_uninstall
      LOG "DEBUG: Are there files in data?"
      ls -1 ./$PKG_DIR/$1/$type
      ls -1 ./$PKG_DIR/$1/data_uninstall
  fi

  for FILE in `ls -1 ./$PKG_DIR/$1/data_uninstall 2> /dev/null`
  do
    if [ "$M_DEBUG" = "1" ] ; then
	LOG "DEBUG: FILE = $FILE"
    fi

    rm -rf $DEST_DIR/data/`basename $FILE`

    # If there aren't any more files in METRICA_DIR
    # directory, then remove the empty directories.
    #
    rmdir -p -s $DEST_DIR/data 2> /dev/null
  done

  ls -1 ./$PKG_DIR/$1/$type > /dev/null 2>&1
  if test $? -eq 0 ; then
    LOG " Installing from $PKG_DIR/$1/$type to $DEST_DIR/$type"
    mkdir -p $DEST_DIR/$type 
    for x in `ls -1 ./$PKG_DIR/$1/$type 2> /dev/null`
    do
      [[ -d $DEST_DIR/$type/$x || -f $DEST_DIR/$type/$x ]] && rm -rf $DEST_DIR/$type/$x 2> /dev/null
      cp -r ./$PKG_DIR/$1/$type/$x $DEST_DIR/$type 2> /dev/null
      check_return_code $? "failed to copy $PKG_DIR/$1/$type/$x to $DEST_DIR/$type"
    done
  fi

  if [ "$M_DEBUG" = "1" ] ; then
    LOG "DEBUG: What's left in $METRICA_DIR?"
    ls -l $METRICA_DIR 2>&1 | tee -a $METRICA_DIR/etc/install.log
    #LOG "DEBUG: What's left in $METRICA_DIR/test21/data?"
    #ls $METRICA_DIR/test21/data
    
  fi

  ###############
  # Update DTDs.
  ###############

  # Update DTD files in the root directory of the current application server
  # Note: An assumption is being made here that the installer has access to the
  # appserver local file system
  # 

  PARTITION_DIR=$APPSERVER_DIR/var/servers/$APPSERVER_NAME/partitions/standard

  for DTD_FILE in `ls -1 ./$PKG_DIR/$1/data_uninstall/*.dtd 2> /dev/null`
  do
    LOG "Removing $PARTITION_DIR/`basename $DTD_FILE`..."

    rm -f $PARTITION_DIR/`basename $DTD_FILE` 2> /dev/null
  done

  for DTD_FILE in `ls -1 ./$PKG_DIR/$1/$type/*.dtd 2> /dev/null`
  do
    BASE_FILE=`basename $DTD_FILE`
    LOG " Installing $BASE_FILE from $PKG_DIR/$1/$type to $PARTITION_DIR"
    [[ -f $PARTITION_DIR/$BASE_FILE ]] && rm -f $PARTITION_DIR/$BASE_FILE 2> /dev/null
    cp -r $DTD_FILE $PARTITION_DIR 2> /dev/null
    check_return_code $? "failed to copy $BASE_FILE to $PARTITION_DIR"
  done

  #####################
  # Update properties.
  #####################

  for FILE in `ls $PKG_DIR/$1/data_uninstall/*.xml 2> /dev/null`
  do
    LOG "Unloading properties in `basename $FILE`..."
    metrica_propertyserviceadmin.ksh -unload properties -file $FILE

    if [ "$M_DEBUG" = "1" ] ; then
      LOG "DEBUG: About to delete file $DEST_DIR/data/`basename $FILE`..."
    fi

    rm -f $DEST_DIR/data/`basename $FILE`
    rmdir -ps $DEST_DIR/data
  done

  # Any .xml files contained within the /data directory will be deemed to be
  # property files and an attempt will be made to load them using the 
  # metrica_propertyserviceadmin.ksh script. NOTE: This inproduces a hardcoded
  # dependency between the generic installer and a particular named component
  # within the metrica system.
  for y in `ls -1 ./$PKG_DIR/$1/$type/*.xml 2> /dev/null`
  do
    if [ -f $METRICA_DIR/CS/bin/metrica_propertyserviceadmin.ksh ]; then
      LOG "Using $METRICA_DIR/CS/bin/metrica_propertyserviceadmin.ksh to load properties "
      LOG "--------------------------------------------------------------"
      LOG "Localising properties : $y "
      rm -f $TMP_DIR/properties.xml > /dev/null 2>&1
      sed -f $TMP_DIR/edit_dd.tmp $y > $TMP_DIR/properties.xml
      LOG "Loading properties : $y "
      $METRICA_DIR/CS/bin/metrica_propertyserviceadmin.ksh -load $TMP_DIR/properties.xml 2>&1 | tee -a $METRICA_DIR/etc/install.log
      LOG "--------------------------------------------------------------"
    else
      LOG "WARNING: Property Service not installed, you must manually load properties after installation"
    fi
    cp -r $y $DEST_DIR/$type 2> /dev/null
  done
}



# Setup deployment CLASSPATH
#
setup_classpath()
 {
  METRICA_CLASSPATH=.  # Start with current dir so it isn't a leading :

  # Generate deployment classpath
  find $METRICA_DIR -name "*.jar" | while read JARFILE; do
    METRICA_CLASSPATH=$METRICA_CLASSPATH:$JARFILE
  done
  
  # Setup special CLASSPATH for DataSource deployment
  METRICA_IFX_CLASSPATH=$METRICA_DIR/lib/ext/ifxjdbc.jar:$METRICA_DIR/lib/ext/ifxjdbcx.jar
}



do_archives()
{
  # Setup CLASSPATH for module deployment
  setup_classpath

  DEPLOY_JAR_LIST=
  DEPLOY_WAR_LIST=
  UNDEPLOY_JAR_LIST=
  UNDEPLOY_WAR_LIST=

  #####################################################
  # Get lists of JARs and WARs to deploy and undeploy.
  #####################################################

  # Get list of JARs and WARs in deploy item-subdirectories.
  #
  #Get the list from the install tmp dir.

  #Step through Array of packages given
  packageArray=$1
  let i=-1
  while (( i=$i+1 )); (( $i<$packageArray)); do
#Get Package List Item based on root for metrica_root.war 
#or all for everything else
case $2 in
	all )
        	pkgdir=`echo ${pkdArray[$i]} | awk -F_ '{print $1}'`;;
	root )
		DEPLOY_WAR_LIST=./$INSTALL_TMP/*/uiweb2/deploy/metrica_root.war;;  
	single ) 
		pkgdir=`echo ${pkdArray[$i]} | awk -F_ '{print $1}'`;;
esac

 	#Chek if that metrica_root.war is missing from install
	# In this case we do not try to deploy it.
if [[ $2 = "root" ]]; then
  #Check does metrica_root.war exist
  if [ ! -f $DEPLOY_WAR_LIST ]; then
     DEPLOY_WAR_LIST=
  fi
fi

if [[ $2 != "root" ]]; then

  	for FILE in `ls -1 ./$INSTALL_TMP/$pkgdir/*/deploy/*.jar ./$INSTALL_TMP/$pkgdir/*/deploy/*.war 2> /dev/null`
  	do
    	case ${FILE##*.} in  # Gets the suffix
      	jar)
	       	if [ -z "$DEPLOY_JAR_LIST" ]; then
        	  	DEPLOY_JAR_LIST="$FILE"
       	 	else
          		DEPLOY_JAR_LIST="$DEPLOY_JAR_LIST,$FILE"
       	 	fi
       	;;
      	war)
	if [ "$2" != "single" ]; then 	

		rootcheck=`echo $FILE | grep  "metrica_root.war"|wc -l`
	    	if [ $rootcheck -eq 0 ]; then
       	 		if [ -z "$DEPLOY_WAR_LIST" ]; then
       	   			DEPLOY_WAR_LIST="$FILE"
        		else
          			DEPLOY_WAR_LIST="$DEPLOY_WAR_LIST,$FILE"
				
			fi
		fi
	else
		if [ -z "$DEPLOY_WAR_LIST" ]; then
                                DEPLOY_WAR_LIST="$FILE"
                        else
                                DEPLOY_WAR_LIST="$DEPLOY_WAR_LIST,$FILE"

                        fi
	fi	
        ;;
    	esac
  	done


  # Get list of JARs and WARs in deploy_uninstall item-subdirectories.
  #
  WAR_JAR_LIST=`ls -1 ./$INSTALL_TMP/$pkgdir/*/deploy/*.jar ./$INSTALL_TMP/$pkgdir/*/deploy/*.war 2> /dev/null`
for FILE in `ls -1 ./$INSTALL_TMP/$pkgdir/*/deploy_uninstall/*.jar ./$INSTALL_TMP/$pkgdir/*/deploy_uninstall/*.war 2> /dev/null`
  do
    case ${FILE##*.} in  # Gets the suffix
      jar)
        if [ -z "$UNDEPLOY_JAR_LIST" ]; then
          UNDEPLOY_JAR_LIST="$FILE"
        else
          UNDEPLOY_JAR_LIST="$UNDEPLOY_JAR_LIST,$FILE"
        fi
        ;;
      war)
        if [ -z "$UNDEPLOY_WAR_LIST" ]; then
          UNDEPLOY_WAR_LIST="$FILE"
        else
          UNDEPLOY_WAR_LIST="$UNDEPLOY_WAR_LIST,$FILE"
        fi
        ;;
    esac
  done

#For each Package list we need to copy jars wars over to $METRICA_DIR
update_files $pkgdir

fi
done


  ###################################################
  # Return if no JARs or WARs to deploy or undeploy.
  ###################################################

  # If there aren't any WARs or JARs in any deploy or
  # deploy_uninstall directories, then there aren't any
  # WARs or JARs to either deploy, or undeploy, so return.
  #
  if [[ "${DEPLOY_JAR_LIST}${DEPLOY_WAR_LIST}${UNDEPLOY_JAR_LIST}${UNDEPLOY_WAR_LIST}" = "" ]]; then
   if [[ "$DEPLOY_DAR_DONE" = "false" ]]; then
       deploy_dar
       DEPLOY_DAR_DONE=true
       return;
   fi
  fi

  # There is at least a deploy or deploy_uninstall item-subdirectory
  # with one or more JARs or WARs to either deploy or undeploy.

  ###################
  # Stop partitions.
  ###################

  # Stop partitions.
  #
  if [[ -n "$DEPLOY_JAR_LIST" || -n "$UNDEPLOY_JAR_LIST" || -n "$DEPLOY_WAR_LIST" || -n "$UNDEPLOY_WAR_LIST" ]]; then
    LOG "Stopping Partitions..."

    APPSERVER_STATUS=$( $APPSERVER_ADM_CMD -stop PARTITIONS )

  fi

  ##########################
  # Undeploy JARs and WARs.
  ##########################

  # Undeploy JARs.
  #
  if [ "$UNDEPLOY_JAR_LIST" ]; then
    LOG "Undeploying JARs on $APPSERVER_NAME..."

    APPSERVER_STATUS=$( $APPSERVER_ADM_CMD -undeploy $UNDEPLOY_JAR_LIST )

    [[ $? -ne 0 ]] && LOG "WARNING: Problems undeploying JARs on $APPSERVER_NAME."
  fi

  # Undeploy WARs.
  #
  if [ "$UNDEPLOY_WAR_LIST" ]; then
    LOG "Undeploying WARs on $APPSERVER_NAME..."

    APPSERVER_STATUS=$( $APPSERVER_ADM_CMD -undeploy $UNDEPLOY_WAR_LIST )

    [[ $? -ne 0 ]] && LOG "WARNING: Problems undeploying WARs on $APPSERVER_NAME."
  fi

  ########################
  # Deploy JARs and WARs.
  ########################

  # Deploy JARs.
  #
  if [ "$DEPLOY_JAR_LIST" ]; then
    LOG "Deploying JARs on $APPSERVER_NAME..."

    APPSERVER_STATUS=$( $APPSERVER_ADM_CMD -deploy $DEPLOY_JAR_LIST $METRICA_CLASSPATH )

    [[ $? -ne 0 ]] && LOG "WARNING: Problems deploying JARs on $APPSERVER_NAME."
  fi

  # Deploy WARs.
  #
  if [ "$DEPLOY_WAR_LIST" ]; then
    LOG "Deploying WARs on $APPSERVER_NAME..."

    APPSERVER_STATUS=$( $APPSERVER_ADM_CMD -deploy $DEPLOY_WAR_LIST $METRICA_CLASSPATH )

    [[ $? -ne 0 ]] && LOG "WARNING: Problems deploying WARs on $APPSERVER_NAME."
  fi

  ####################
  # Start partitions.
  ####################

  # Stop partitions.
  #

  LOG "Ensuring Partitions are started..."
  APPSERVER_STATUS=$( $APPSERVER_ADM_CMD -start PARTITIONS )

  if [[ $2 != "root" ]]; then
	if [[ "$DEPLOY_DAR_DONE" = "false" ]]; then
	    # Create and deploy the dar file
	    deploy_dar
            DEPLOY_DAR_DONE=true
	fi
  fi
  
}

deploy_dar()
{

     ######################
     # Update DataSources.
     ######################

     # If there are any DSD (DataSource definition) files
     # in either the data_uninstall or data item-subdirectories,
     # and there are files left in DEST_DIR after files
     # have been updated (previous operation),
     # then DataSources need to be updated.
     #
     # If there are any DSD (DataSource definition) files
     # in the data_uninstall item-subdirectory, but there
     # aren't any files left in DEST_DIR after files have
     # been updated, then DataSources need to be uninstalled.
     #
     #Hardcoded destination DIR

     DATASOURCE_DEST_DIR=./$INSTALL_TMP/DataSources/item1
     INSTALL_DSD_LIST="`ls $DATASOURCE_DEST_DIR/data/*.dsd 2> /dev/null`"
     UNINSTALL_DSD_LIST="`ls $DATASOURCE_DEST_DIR/data_uninstall/*.dsd 2> /dev/null`"
     DEST_DIR_DSD_LIST="`ls $DATASOURCE_DEST_DIR/data/*.dsd 2> /dev/null`"
    
     if [[ ( -n "$INSTALL_DSD_LIST" || -n "$UNINSTALL_DSD_LIST" ) && -n "$DEST_DIR_DSD_LIST" ]]; then
        LOG "Creating and Deploying metrica-datasources.dar file"
        $APPSERVER_ADM_CMD -create_ds $DATASOURCE_DEST_DIR/data -dbusername $METRICA_DB_USERNAME -dbpassword $METRICA_DB_PASSWORD -o $DATASOURCE_DEST_DIR/data/metrica-datasources.dar
     fi
    
     if [[ -n "$UNINSTALL_DSD_LIST" && -z "$DEST_DIR_DSD_LIST" ]]; then
    	$APPSERVER_ADM_CMD -stop PARTITIONS
    
        if [ "$M_DEBUG" = "1" ] ; then
           $APPSERVER_ADM_CMD -undeploy metrica-datasources.dar
        fi
        APPSERVER_STATUS=$( $APPSERVER_ADM_CMD -undeploy metrica-datasources.dar)

        [[ $? -ne 0 ]] && LOG "WARNING: Problems undeploying metrica-datasources.dar"
        $APPSERVER_ADM_CMD -start PARTITIONS
        rm -f $DATASOURCE_DEST_DIR/data/metrica-datasources.dar
     fi

}



update_files()
{
#Need to make sure we get the right Dir
tgtdir=`echo $1 | awk -F- '{print $1}'`

if [ "$tgtdir" != "" ] ; then
		tgtdir=`basename $tgtdir`
		echo $tgtdir
fi

ARCHIVE_DIR=$METRICA_DIR/$tgtdir/lib

# Remove files in UNDEPLOY_JAR_LIST and UNDEPLOY_WAR_LIST from METRICA_DIR.
#
  for FILE in `$ECHO "${UNDEPLOY_JAR_LIST} ${UNDEPLOY_WAR_LIST}" | tr "," " "`
  	do
		#Add a check to make sure only correct jars get copied
		echo $FILE | grep  $1
		if [[ $? = 0 ]] ; then
  	rm -rf $ARCHIVE_DIR/`basename $FILE` 2> /dev/null
  	#if there aren't any more files in METRICA_DIR
  	# directory, then remove the empty directories.
  	rmdir -p -s $ARCHIVE_DIR 2> /dev/null
		fi
	done

# Copy files in DEPLOY_JAR_LIST and DEPLOY_WAR_LIST to METRICA_DIR.
#
	for FILE in `$ECHO "${DEPLOY_JAR_LIST} ${DEPLOY_WAR_LIST}" | tr "," " "`
		do
		echo $FILE | grep  $1
		if [[ $? = 0 ]] ; then
		[[ -a $ARCHIVE_DIR/`basename $FILE` ]] && rm -rf $ARCHIVE_DIR/`basename $FILE` 2> /dev/null
		mkdir -p $ARCHIVE_DIR
		cp $FILE $ARCHIVE_DIR
		fi
	done
}


#
#
install_extension_directory()
{
  item_name=$1
  extension=$2

  # Determine name of extender to use.
  # If extension ends in '_uninstall' then use
  # metrica_uninstall_<extension-without-_uninstall>.ksh.
  # Otherwise, use metrica_install_${extension}.ksh.

  # Set EXTENDER_BASENAME and EXTENDER_SUFFIX.
  #
  EXTENDER_SUFFIX=

  if [ "$M_DEBUG" = "1" ] ; then
    LOG "DEBUG: extension = $extension"
  fi

  EXTENDER_SUFFIX=`$ECHO $extension | $M_AWK ' /_uninstall$/ { print "_uninstall" } '`
  EXTENDER_BASENAME=`$ECHO $extension | $M_AWK '

    {
      sub("_uninstall", "")
      print $0
    }
  '`

  if [ "$M_DEBUG" = "1" ] ; then
    LOG "DEBUG: EXTENDER_SUFFIX = $EXTENDER_SUFFIX"
    LOG "DEBUG: EXTENDER_BASENAME = $EXTENDER_BASENAME"
  fi

  if [ -z "$EXTENDER_SUFFIX" ] ; then
    EXTENDER_SCRIPT="metrica_install_${EXTENDER_BASENAME}.ksh"
  else
    EXTENDER_SCRIPT="metrica_uninstall_${EXTENDER_BASENAME}.ksh"
  fi

  # Check whether EXTENDER_SCRIPT exists.
  #
  whence -v $EXTENDER_SCRIPT > /dev/null

  # If it does,
  #
  if [ $? -eq 0 ]; then
    # Run extender.
    #
    LOG "Executing installer extension script $EXTENDER_SCRIPT..."
    LOG "--------------------------------------------------------------"
    . $EXTENDER_SCRIPT $PKG_DIR/$item_name/$extension 2>&1 | tee -a $METRICA_DIR/etc/install.log
    LOG "--------------------------------------------------------------"
  fi
}



# build_environment
#
build_environment()
{ 
  LOG "Building Metrica environment setup.ksh"

  [[ ! -f $METRICA_DIR/setup.tpl ]] && m_exit 1 "Cannot find $METRICA_DIR/setup.tpl: metrica_start_install.ksh may not have run correctly."
  rm -f $TMP_DIR/setup1.tmp
  cat $METRICA_DIR/setup.tpl > $METRICA_DIR/setup.tmp
  $ECHO "#" >> $TMP_DIR/setup1.tmp
  $ECHO "# Metrica runtime environment setup script" >> $TMP_DIR/setup1.tmp
  $ECHO "#" >> $TMP_DIR/setup1.tmp
  $ECHO >> $TMP_DIR/setup1.tmp

  # Initialize CLASSPATH
  #
  $ECHO "CLASSPATH=:" >> $METRICA_DIR/setup.tmp
  $ECHO >> $METRICA_DIR/setup.tmp

  LOG " Generating Metrica CLASSPATH"
  rm -f $TMP_DIR/cp.tmp $TMP_DIR/cpi.tmp
  find $METRICA_DIR -name "*\.jar" -print > $TMP_DIR/cpi.tmp 2> /dev/null
  for cpi in `cat $TMP_DIR/cpi.tmp 2> /dev/null`
  do
    # $ECHO -n $cpi: >> $TMP_DIR/cp.tmp
    $ECHO "CLASSPATH=\$CLASSPATH:$cpi" >> $METRICA_DIR/setup.tmp
  done
  cp=""
  $ECHO >> $METRICA_DIR/setup.tmp

  LOG " Generating Metrica LD_LIBRARY_PATH for lib.$MACHINE"

  $ECHO "LD_LIBRARY_PATH=/usr/lib:$VISIBROKER_DIR/lib" >> $METRICA_DIR/setup.tmp

  rm -f $TMP_DIR/cp.tmp $TMP_DIR/cpi.tmp
  find $METRICA_DIR -name "lib.$MACHINE" -print > $TMP_DIR/cpi.tmp 2> /dev/null
  for cpi in `cat $TMP_DIR/cpi.tmp 2> /dev/null`
  do
    $ECHO -n "$cpi:" >> $TMP_DIR/cp.tmp
  done
  cp=""
  for cp in `cat $TMP_DIR/cp.tmp 2> /dev/null`
  do
    $ECHO > /dev/null 2>&1
  done
  $ECHO "LD_LIBRARY_PATH=\$LD_LIBRARY_PATH:$cp" >> $METRICA_DIR/setup.tmp
  $ECHO >> $METRICA_DIR/setup.tmp

  LOG " Generating Metrica PATH"
  rm -f $TMP_DIR/cp.tmp $TMP_DIR/cpi.tmp
  find $METRICA_DIR -name "bin" -print > $TMP_DIR/cpi.tmp 2> /dev/null
  for cpi in `cat $TMP_DIR/cpi.tmp 2> /dev/null`
  do
    $ECHO -n "$cpi:" >> $TMP_DIR/cp.tmp
  done
  cp=""
  for cp in `cat $TMP_DIR/cp.tmp 2> /dev/null`
  do
    $ECHO > /dev/null 2>&1
  done
  $ECHO "PATH=\$PATH:$cp" >> $METRICA_DIR/setup.tmp
  $ECHO >> $METRICA_DIR/setup.tmp

  $ECHO "#" >> $METRICA_DIR/setup.tmp
  $ECHO "# Source any Subsystem specific environment setup scripts" >> $METRICA_DIR/setup.tmp
  $ECHO "#" >> $METRICA_DIR/setup.tmp
  find $METRICA_DIR/etc/environment -name "*\.ksh" -print > $TMP_DIR/env.tmp 2> /dev/null
  for env in `cat $TMP_DIR/env.tmp 2> /dev/null`
  do
    $ECHO ". $env" >> $METRICA_DIR/setup.tmp
  done
  $ECHO "#" >> $METRICA_DIR/setup.tmp

  # unset exporting of variables, set at start of setup1.tmp
  $ECHO "set +a" >> $METRICA_DIR/setup.tmp
  # following lines replace the actual current value of METRICA_DIR with the string "$METRICA_DIR"
    #rm -f $TMP_DIR/edit_su.tmp
    #$ECHO "s/$MDV/$MDN/g" >> $TMP_DIR/edit_su.tmp
    #sed -f $TMP_DIR/edit_su.tmp $METRICA_DIR/setup.tmp > $METRICA_DIR/setup1.ksh 2> /dev/null
  cat $TMP_DIR/setup1.tmp $METRICA_DIR/setup.tmp > $METRICA_DIR/setup.ksh 2> /dev/null
  rm -f $TMP_DIR/setup1.tmp
  rm -f $METRICA_DIR/setup.tmp
    #rm -f $METRICA_DIR/setup1.ksh
  mkdir -p $HOME/.metrica

  chmod 755 $METRICA_DIR/setup.ksh > /dev/null 2>&1
  LOG "Setting up Metrica environment setup.ksh"
  $METRICA_DIR/setup.ksh > /dev/null  2>&1
  . $METRICA_DIR/setup.ksh > /dev/null  2>&1
}

#
# $METRICA_DIR/utils/bin
# ----------------------
# This directory is a link to the $METRICA_DIR/Installer/bin directory
#
link_utils_bin() 
{
   #
   # BUG-39995
   # ---------
   # The "$METRICA_DIR/utils/bin" directory exists because it is not compulsory 
   # to install the Installer package. 
   # This directory is however not normally upgraded, therefore the 
   # "$METRICA_DIR/Installer/bin" directory may in fact be more up-to-date
   #
   # This solution creates a symbolic link so that the Installer package directory
   # (if it is available at install time) will always be used.
   #
   if [[ -d $METRICA_DIR/Installer/bin &&  ! -L $METRICA_DIR/utils/bin && -d $METRICA_DIR/utils/bin ]]; then
      LOG "Creating symbolic link: FROM $METRICA_DIR/utils/bin TO $METRICA_DIR/Installer/bin"
      mv $METRICA_DIR/utils/bin $METRICA_DIR/utils/bin.orig
      ln -s $METRICA_DIR/Installer/bin $METRICA_DIR/utils/bin
   fi
}

convert_utime_to_int()
{
  LOG "Converting utime to int for $METRICA_PERFORMANCE_DB Database"
  LOG "This may take some time ... "
  LOG "See $METRICA_DIR/logs directory for conversion logs "

  $METRICA_DIR/Installer/bin/metrica_change_column_type_installer.ksh -database $METRICA_PERFORMANCE_DB -columnlist $METRICA_DIR/Installer/bin/metrica_change_column_list -from_to_types utime integer -threads 5  >> $METRICA_DIR/etc/install.log
  
  LOG "Converting utime to int for $METRICA_PERFORMANCE_DB Database completed"
} 

add_endstamp_column() {

  LOG "Adding endstamp column to all nc tables in the $METRICA_PERFORMANCE_DB Database"
  LOG "This may take some time ... "

  LOG "See $METRICA_DIR/logs directory for the associated  log files"

  $METRICA_DIR/Installer/bin/metrica_add_endstamp.ksh -threads 8  | tee -a $METRICA_DIR/etc/install.log
  
  LOG "Addition of endstamp column on all nc tables in the  $METRICA_PERFORMANCE_DB Database completed"

}

add_name_indexes() {

  LOG "Adding indexes to nc tables in the $METRICA_PERFORMANCE_DB Database"
  LOG "This may take some time ... "

  LOG "See $METRICA_DIR/logs directory for the associated log files"

  $METRICA_DIR/Installer/bin/metrica_add_name_indexes.ksh -threads 8  | tee -a $METRICA_DIR/etc/install.log

  LOG "Addition of indexes to nc tables in the $METRICA_PERFORMANCE_DB Database completed"

}
# post processing

#
post_processing()
{
  build_environment
  if [ ! -a $METRICA_DIR/utils/bin ]; then
     mkdir -p $METRICA_DIR/utils/bin
  fi
  cp ./metrica_cronwrap.ksh $METRICA_DIR/utils/bin > /dev/null 2>&1
  chmod 755 $METRICA_DIR/utils/bin/metrica_cronwrap.ksh > /dev/null 2>&1
  ln -s $METRICA_DIR/utils/bin/metrica_machtype $METRICA_DIR/utils/metrica_machtype > /dev/null 2>&1

  # Determine the name of the setup file WRT installation instance
  #
  if [[ "$METRICA_INSTALL_INSTANCE" = "" || "$METRICA_INSTALL_INSTANCE" = "metrica" ]] ; then
    HOME_SETUP=setup.ksh
  else
    HOME_SETUP=setup_$METRICA_INSTALL_INSTANCE.ksh
  fi

  if [[ -L $HOME/.metrica/$HOME_SETUP || -L $HOME/.metrica/metrica_cronwrap.ksh ]]; then
    LOG "Updating softlinks already present in $HOME/.metrica"
    rm -f $HOME/.metrica/$HOME_SETUP 2>/dev/null
    rm -f $HOME/.metrica/metrica_cronwrap.ksh 2>/dev/null
    create_softlinks $HOME_SETUP
  else
    $ECHO "The installer can place symbolic links to setup.ksh and metrica_cronwrap.ksh in your \$HOME/.metrica directory."
    $ECHO "This will make it easier to recreate the environment variables set up during this installation, and to"
    $ECHO "schedule cron jobs using this environment. See the install notes for more details."
    $ECHO -n "Do you wish to add these symbolic links in your \$HOME/.metrica directory? [y/n] " 
    read SL
    if echo $SL | grep -i "y" > /dev/null; then
      create_softlinks $HOME_SETUP
    fi
  fi
  # Create softlinks from x to bin/bin.$MACHINE.x
  find $METRICA_DIR -name "*" -print | grep "bin.$MACHINE" > $TMP_DIR/bins.tmp
  for cpi in `cat $TMP_DIR/bins.tmp 2> /dev/null`
  do
    ending=`echo $cpi | awk -F\/ '{print $NF}'` 
    if [ "$ending" = "bin.$MACHINE" ]; then
      cd $cpi
    else
      ln -s $cpi ../$ending > /dev/null 2>&1
    fi
  done
  cd $CUR_DIR
  
  link_utils_bin
 
# converting database utime to int for $METRICA_PERFORMANCE_DB database as part of the post-processing stage of install
  convert_utime_to_int
  
#  Add the endstamp column to the nc_ tables 
  add_endstamp_column
# Add indexes to nc tables if a TP is being installed

  if [[ "$IS_TECHPACK" = "t" ]]; then
     add_name_indexes
  fi 

  LOG " Removing install temporary files"
  rm -rf $TMP_DIR
  clean_install_tmp
  LOG " Install Complete"
}



# create softlinks
#
create_softlinks()
{
  LOG " Creating softlink from $HOME/.metrica/metrica_cronwrap.ksh to $METRICA_DIR/utils/bin/metrica_cronwrap.ksh"
  ln -s $METRICA_DIR/utils/bin/metrica_cronwrap.ksh $HOME/.metrica/metrica_cronwrap.ksh > /dev/null 2>&1
  LOG " Creating softlink from $HOME/.metrica/$1 to $METRICA_DIR/setup.ksh"
  ln -s $METRICA_DIR/setup.ksh $HOME/.metrica/$1 > /dev/null 2>&1
}


# temporary update for bug 37853. If the package is DBElements
# then restart the database and turn logging on
#
dbelement_special_pre_install(){
  # As the informix user, bounce the database 
  # and turn database logging on
  LOG "Restarting the DB and temporarily switching database logging on"
  LOG "NB: Please supply Informix password on request!"
  # only allow 3 tries.
  chmod a+rx $METRICA_DIR/install_setup.ksh
  
  SU_OK=false
  Num_Tries=0
  while [[ "$SU_OK" = "false" ]] && [[ $Num_Tries -lt 3 ]]; do
      su informix -c "ksh -c \" \
          . $METRICA_DIR/install_setup.ksh; \
          echo 'Restarting the Database and activating log mode...'; \
          echo 'Stopping Informix...'; \
          onmode -kyF; \
          sleep 10; \
          echo 'Starting Informix...'; \
          oninit; \
          sleep 15; \
          ondblog buf $METRICA_PERFORMANCE_DB; \
          echo 'Logging started on $METRICA_PERFORMANCE_DB'; \
          onbar -b -F; \
          echo 'Done'\""
      if [ $? -ne 0 ]; then
          if [ $Num_Tries -lt 2 ]; then
      	    LOG "Password failure (or command failed), please re-enter informix password..."
          else
      	    LOG "Password failure (or command failed)."
           fi

          ((Num_Tries+=1))
      else
          # finished
          SU_OK=true
      fi
  done

  # if the su failed then stop.
  if [ "$SU_OK" = "false" ]; then
      LOG "ERROR: DBElements pre-install failed on database restart. Check database consistency."
      exit 1;
  fi

  # check that database logging is on
  LOGGING_ON=`echo "select is_logging from sysdatabases where name = '$METRICA_PERFORMANCE_DB'" | dbaccess sysmaster 2> /dev/null | grep "1"`
  if [ -z "$LOGGING_ON" ]; then
      LOG "ERROR: Problem with DBElements pre-install. Database logging should be on at this stage!"
      exit 1;
  fi

  # check the current blade versions for both databases (if they exist)
  perf_blade=`echo "execute function get_version()" | dbaccess $METRICA_PERFORMANCE_DB 2>/dev/null | grep DBELEMENTS`
  if [ -n "$perf_blade" ]; then
      perf_blade=`echo $perf_blade | sed 's/^.*DBELEMENTS Version //g' | sed 's/ Patch /\./g' | sed 's/ (Checkpoint: /\./g' | sed 's/,.*$//g'`
      LOG "Current blade package version is $perf_blade on $METRICA_PERFORMANCE_DB"
  else
      LOG "There is no blade registered currently on $METRICA_PERFORMANCE_DB"
  fi

  adm_blade=`echo "execute function get_version()" | dbaccess $METRICA_ADMINISTRATION_DB 2>/dev/null | grep DBELEMENTS` 	    
  if [ -n "$adm_blade" ]; then
      adm_blade=`echo $adm_blade | sed 's/^.*DBELEMENTS Version //g' | sed 's/ Patch /\./g' | sed 's/ (Checkpoint: /\./g' | sed 's/,.*$//g'`
      LOG "Current blade package version is $adm_blade on $METRICA_ADMINISTRATION_DB"
  else
      LOG "There is no blade registered currently on $METRICA_ADMINISTRATION_DB"
  fi
  # end of special DBElements processing (bug 37853)
}
 

# bug 37853. If the current package is DBElements then as the 
# informix user, turn database logging off and bounce the database
#
dbelement_special_post_install(){
  LOG "Switching Performance Database logging off and restarting the database"
  LOG "NB: Please supply Informix password on request!"

  # only allow 3 tries.
  SU_OK=false
  Num_Tries=0
  while [[ "$SU_OK" = "false" ]] && [[ $Num_Tries -lt 3 ]]; do
      su informix -c "ksh -c \" \
          . $METRICA_DIR/install_setup.ksh; \
          echo 'Turning database logging off ..'; \
          ondblog nolog $METRICA_PERFORMANCE_DB; \
          echo 'Logging ended on $METRICA_PERFORMANCE_DB'; \
          onbar -b -F; \
          echo 'Stopping Informix...'; \
          onmode -kyF; \
          sleep 10; \
          echo 'Starting Informix...'; \
          oninit; \
          sleep 15; \
          echo 'Done'\""
      if [ $? -ne 0 ]; then
          if [ $Num_Tries -lt 2 ]; then
      	LOG "Password failure (or command failed), please re-enter informix password..."
          else
      	LOG "Password failure (or command failed)."
          fi

          ((Num_Tries+=1))
      else
          # finished
          SU_OK=true
      fi
  done

  # if the su failed then stop.
  if [ "$SU_OK" = "false" ]; then
      LOG "ERROR: DBElements post-install failed on database restart. Check database consistency and ensure that database logging is off."
      exit 1;
  fi

  # check that database logging is off
  LOGGING_OFF=`echo "select is_logging from sysdatabases where name = '$METRICA_PERFORMANCE_DB'" | dbaccess sysmaster 2> /dev/null | grep "0"`
  if [ -z "$LOGGING_OFF" ]; then
      LOG "ERROR: Problem with DBElements post-install. Database logging is still on!"
      exit 1;
  fi

  # check the current blade versions for both databases.
  perf_blade=`echo "execute function get_version()" | dbaccess $METRICA_PERFORMANCE_DB 2>/dev/null | grep DBELEMENTS` 	    
  if [ -n "$perf_blade" ]; then
      perf_blade=`echo $perf_blade | sed 's/^.*DBELEMENTS Version //g' | sed 's/ Patch /\./g' | sed 's/ (Checkpoint: /\./g' | sed 's/,.*$//g'`
      LOG "Current blade package version is $perf_blade on $METRICA_PERFORMANCE_DB"
  else
      LOG "ERROR: Unable to get blade package version for $METRICA_PERFORMANCE_DB."
      exit 1;
  fi

  adm_blade=`echo "execute function get_version()" | dbaccess $METRICA_ADMINISTRATION_DB 2>/dev/null | grep DBELEMENTS` 	    
  if [ -n "$adm_blade" ]; then
      adm_blade=`echo $adm_blade | sed 's/^.*DBELEMENTS Version //g' | sed 's/ Patch /\./g' | sed 's/ (Checkpoint: /\./g' | sed 's/,.*$//g'`
      LOG "Current blade package version is $adm_blade on $METRICA_ADMINISTRATION_DB"
  else
      LOG "ERROR: Unable to get blade package version for $METRICA_ADMINISTRATION_DB."
      exit 1;
  fi

  # compare blade version with DBElements version
  if [ "$package_version" != "$perf_blade" ]; then
      LOG "ERROR: Blade package version is: $perf_blade on $METRICA_PERFORMANCE_DB."
      LOG "ERROR: It should match the DBElements package version: $package_version"
      LOG "ERROR: stopping install..."
      exit 1;
  fi

  # compare blade version with DBElements version
  if [ "$package_version" != "$adm_blade" ]; then
      LOG "ERROR: Blade package version is: $adm_blade on $METRICA_ADMINISTRATION_DB."
      LOG "ERROR: It should match the DBElements package version: $package_version"
      LOG "ERROR: stopping install..."
      exit 1;
  fi

  # restart the appserver due to the database restart.
  LOG "Restarting appserver and partitions..."
  APPSERVER_STATUS=`$APPSERVER_ADM_CMD -restart`

  # end of special DBElements processing (bug 37853)
}

 
# Install package 
# 
install_package()
{
  package_name=$1

  # cut the version numbers out of the package name e.g. 'r'4.10.0.5
  # developemnt version are in the format 'd'4.10.0.5. take either version.
  rel_version=`echo $package_name | grep '_r[0-9]'`

  if [[ -n $rel_version ]]; then
    package_version=`echo ${package_name#DBElements_r} | cut -d, -f1`
  else
    package_version=`echo ${package_name#DBElements_d} | cut -d, -f1`
  fi

  # check to see if $METRICA_DIR/setup.ksh exists, if so then source it
  if [ -f $METRICA_DIR/setup.ksh ]; then
    LOG "Sourcing runtime environment $METRICA_DIR/setup.ksh"
    $METRICA_DIR/setup.ksh > /dev/null  2>&1
    . $METRICA_DIR/setup.ksh > /dev/null  2>&1
    # ensure that the informix user can access setup.ksh
    chmod a+rx $METRICA_DIR/setup* >/dev/null 2>&1
  fi

  # check to see if required packages have been previously installed
  if [ -f ./$PKG_DIR/package_deps.txt ]; then
    LOG "Checking installation $METRICA_DIR for required packages"
    for dep_record in `cat ./$PKG_DIR/package_deps.txt | awk '{print $1"_"$2}' 2> /dev/null`
    do
      dep_package=`echo $dep_record | awk -F_ '{print $1}'`
      dep_version=`echo $dep_record | awk -F_ '{print $2}'`
      check_admin_registry $dep_package $dep_version
    done
  fi

  generate_replacement_script

  need_to_create="n"
  for layer in `cat $METRICA_DIR/etc/servertype 2> /dev/null`
  do
    if [ "$layer" = "ALL" ]; then
      need_to_create="y" 
    fi
    if [ "$layer" = "DATABASE" ]; then
      need_to_create="y" 
    fi
  done

  if [ "$need_to_create" = "y" ]; then
    create_database $METRICA_ADMINISTRATION_DB $METRICA_ADMINISTRATION_DBSPACE "BUFFERED"
    create_database $METRICA_PERFORMANCE_DB $METRICA_PERFORMANCE_DBSPACE "nothing"
    create_database $METRICA_SYSTEM_DB $METRICA_SYSTEM_DBSPACE "nothing"
  fi

  expand_package $package_name normal

  reg_entry=`head -1 ./$PKG_DIR/package_id.txt | awk '{print $1, $2}' 2> /dev/null`
  if [ "$reg_entry" ]; then
    ADMIN_REGISTER $reg_entry
  fi

  if [ -f ./$PKG_DIR/package_items.txt ]; then
    LOG "Installing $1"
  else
    m_exit 1 "Invalid package archive, package_items.txt not found" 
  fi

  if [ -f ./$PKG_DIR/install_pre/install.ksh ]; then
    LOG "Executing package pre-installation script ./$PKG_DIR/install_pre/install.ksh"
    cd ./$PKG_DIR/install_pre
    ./install.ksh > temp_output_file 2>&1
    let RET_VAL=$?
    cat temp_output_file
    disk_log_file temp_output_file
    rm -f temp_output_file
    cd $CUR_DIR
    if (( $RET_VAL != 0 )); then
      LOG "Error executing pre-install script $PKG_DIR/install_pre/install.ksh"
      m_exit 1 "See install log for details"
    fi
  fi

  # Deploy and undeploy JARs and WARs in all deploy and
  # deploy_uninstall item-subdirectories in the package.
  # All jars and wars will be deployed at the beginning

  if [[ "$DEPLOY_DONE" = "false" ]]; then
	#We need to deal with single case
	#Check that pkdArray has a first element to prove that pkdlist was given
	#Otherwise it is a single tar install

	if [[ -n "${pkdArray[0]}" ]]; then
  	  do_archives ${#pkdArray[*]} all
	else
	#expand and deploy a single tar
	  set -A pkdArray
	  pkdArray[0]="$1"
	  expand_package ${pkdArray[0]} deploy
	  do_archives ${#pkdArray[*]} single
	fi
  DEPLOY_DONE=true
  fi

  for item_record in `cat ./$PKG_DIR/package_items.txt | awk '{print $1"_"$2}' 2> /dev/null`
  do
    item_name=`echo $item_record | awk -F_ '{print $1}'` 
    item_type_string=`echo $item_record | awk -F_ '{print $2}'` 
    # item_type_string is now something like APPLICATION
    # or WEB,APPLICATION or WEB,APPLICATION,DATABASE etc
    if [ "$item_name" = "@APPRESTART" ]; then 
      if [ "$APPSERVER_INSTALLATION" = "y" ]; then
        if [ "$M_DEBUG" = "1" ] ; then
	    LOG "DEBUG: NOT Restarting application server ... no need"
	fi
      fi
    else 
      need_to_install="n"
      for item_type in `echo $item_type_string | tr "," " "`
      do
        for layer in `cat $METRICA_DIR/etc/servertype 2> /dev/null`
        do
          if [ "$item_type" = "$layer" ]; then
            need_to_install="y" 
            break
          fi
        done
        [[ "$need_to_install" = "y" ]] && break
      done
      if [ "$need_to_install" = "y" ]; then
        if [ -f ./$PKG_DIR/$item_name/install/install.ksh ]; then
          LOG " Executing vanilla data for $item_name" 

          # Although there is an install/install.ksh present, do_data
          # is called is order to load property XML files, to copy
          # DTDs to APPSERVER_DIR, and to copy files to METRICA_DIR.
          #
          do_data $item_name

          LOG " Executing custom installation script ./$PKG_DIR/$item_name/install/install.ksh" 
          cd ./$PKG_DIR/$item_name/install

	  # temporary update for bug 37853. If the package is DBElements
	  # then restart restart the database and turn logging on
	  if [[ "$PKG_DIR" = "DBElements" ]]; then
	    dbelement_special_pre_install
	  fi

	  # install the package
          ./install.ksh > temp_output_file 2>&1

	  let RET_VAL=$?
          cat temp_output_file
          disk_log_file temp_output_file
          rm -f temp_output_file
          cd $CUR_DIR
          if (( $RET_VAL != 0 )); then
            LOG "Error executing custom install script $PKG_DIR/$item_name/install/install.ksh"
            m_exit 1 "See install log for details"
          fi

	  # bug 37853. If the current package is DBElements then as the 
	  # informix user, turn database logging off and bounce the database
	  if [[ "$PKG_DIR" = "DBElements" ]]; then
	    dbelement_special_post_install
	  fi

        else
          #
          # Determine whether any directories other
          # than bin, data, deploy, and lib are present.
          # If so then call the appropriate script in 
          # the relevant directory
          #
          # Execute uninstall extenders before install extenders.

          # Run any necessary uninstall extenders.
          #
          ITEM_LIST=`ls -d $PKG_DIR/$item_name/!(bin|data|deploy|lib)_uninstall 2> /dev/null`

          for ext in $ITEM_LIST
          do
            # -d option to ls doesn't seem to only list directories.
            #
            [[ ! -d $ext ]] && continue

	    if [ "$M_DEBUG" = "1" ] ; then
	      LOG "DEBUG: ext(1) = $ext"
	    fi

            install_extension_directory $item_name `basename $ext`
          done

          # Run any necessary install extenders.
          #
          ITEM_LIST=`ls -d $PKG_DIR/$item_name/!(bin|data|deploy|lib|*_uninstall) 2> /dev/null`

          for ext in $ITEM_LIST
          do
            # -d option to ls doesn't seem to only list directories.
            #
            [[ ! -d $ext ]] && continue

	    if [ "$M_DEBUG" = "1" ] ; then
	      LOG "DEBUG: ext(2) = $ext"
	    fi

            install_extension_directory $item_name `basename $ext`
          done

          cd $CUR_DIR

          do_bin_uninstall_directory $item_name
          do_lib_uninstall_directory $item_name
          do_bin_directory $item_name
          do_lib_directory $item_name
          do_data $item_name
          install_custom_environment $item_name
        fi

        # copy all start xml files to $METRICA/etc/start
        for y in `ls -1 ./$PKG_DIR/$item_name/metrica_*_startdeps.xml 2> /dev/null`
        do
          LOG "Installing start dependency file $y"
          cp -r $y $METRICA_DIR/etc/start 2> /dev/null
        done

        # copy all stop xml files to $METRICA/etc/stop
        for y in `ls -1 ./$PKG_DIR/$item_name/metrica_*_stopdeps.xml 2> /dev/null`
        do
          LOG "Installing stop dependency file $y"
          cp -r $y $METRICA_DIR/etc/stop 2> /dev/null
        done

      fi
    fi
  done

  if [ -f ./$PKG_DIR/install_post/install.ksh ]; then
    LOG "Executing package post-installation script ./$PKG_DIR/install_post/install.ksh"
    cd ./$PKG_DIR/install_post
    ./install.ksh > temp_output_file 2>&1
    let RET_VAL=$?
    cat temp_output_file
    disk_log_file temp_output_file
    rm -f temp_output_file
    cd $CUR_DIR
    if (( $RET_VAL != 0 )); then
      LOG "Error executing post-install script $PKG_DIR/install_post/install.ksh"
      m_exit 1 "See install log for details"
    fi
  fi
  rm -f $PKG_DIR.tar.Z 
  rm -f $PKG_DIR.tar 
  rm -rf $PKG_DIR

  #rm -rf META-INF 
} 


# 
# Install packages 
# 
install_packages() 
{ 
  cat $1 > /dev/null 2>&1
  if test $? -ne 0 ; then
    m_exit 1 "-packagelist file '$1' does not exist" 
  fi

	#Need to expand pakages for deploying jars wars
	expand_deploy $1

  for pkd in `cat $1 2> /dev/null`
  do
    install_package $pkd 
    if (( $?!=0 ))
    then
  m_exit 1 "Error installing package $pkd, installation aborting"
    fi
  done
	#We need to deploy metrica_root.war at the end of the install
	do_archives 1 root
} 
 
packagelist_check()
{
  if [ ! -f $1 ]; then
    m_exit 1 "-packagelist file '$1' does not exist"
  else
    for pkd in `cat $1 2> /dev/null`
    do
      package_check $pkd
    done
  fi
}

package_check()
{
  if [ ! -f $1 ]; then
    m_exit 1 "package file '$1' does not exist"
  fi
}

clean_install_tmp()
{
if [[ -d $DEPLOY_DIR ]]; then 
rm -rf $DEPLOY_DIR
fi
}

expand_deploy()
{ 
#go through the Pakage list and expand the tars
#Deploy flag ensures we only get from deploy dir
#Also add pkg to Array to use in deploying
set -A pkdArray

let LINECOUNT=0
for pkd in `cat $1 2> /dev/null`
	do
	#Add to the Array then expand.
	pkdArray[$LINECOUNT]="$pkd"
	LINECOUNT=`expr $LINECOUNT + 1`
	expand_package $pkd deploy
	done
}

#
#
#
init()
{
  [[ -z "$METRICA_DIR" ]] && m_exit 1 "METRICA_DIR undefined"
  mkdir -p $TMP_DIR 2> /dev/null
  mkdir -p $METRICA_DIR/etc > /dev/null 2>&1
  (( $?!=0 )) && m_exit 1 "Failed to create temporary directory $TMP_DIR"
  #print version of installer we are using.
  LOG "Installer Version is $VERSION"
  [[ -z "$APPSERVER_DIR" ]] && m_exit 1 "Application Server Directory not Set"
  [[ -z "$APPSERVER_NAME" ]] && m_exit 1 "Application Server Name not Set"
   CHK=$(grep -c "metrica.static.partitions.list" $APPSERVER_DIR/var/servers/$APPSERVER_NAME/adm/properties/server.properties)
    if [ $CHK -eq 0 ]; then
        LOG "BES not setup for multiple partitions"
    fi
  ${DIRNAME}/metrica_lap.ksh -logfile ${METRICA_DIR}/etc/install.log
  if [ $? -eq 1 ]; then
    m_exit 1 "License not accepted, terminating install"
  fi
}


#------------------------- Main Command-line Processing ---------------------- 

. metrica_install_common.ksh

display_metrica_banner 

set_mach_vars

# Process command-line options 
# 
if [ -z "$1" ]; then 
  usage 
  m_exit 1 "No arguments given" 
fi 
 
while [[ -n $1 ]]
do 
  case "$1" in 
    # Help options 
    "-h"|"-help"|"--help"|"-?") 
      usage 
      m_exit 0 
    ;; 

    # Version options 
    "-v"|"-version"|"--version") 
      version 
      m_exit 0 
    ;; 
 
    # installdir options 
    "-installdir") 
      export METRICA_DIR=$2
      shift 
      if [ -z "$METRICA_DIR" ]; then 
        m_exit 1 "-installdir option not correctly specified" 
      fi 
      $ECHO "Metrica directory: "$METRICA_DIR
    ;; 

    # packagelist option 
    "-packagelist") 
      init
      clean_install_tmp
      packagelist_check $2
      display_layer_choice
      environment_check
      install_packages $2 
      post_processing
      m_exit 0 
    ;; 

    # Main package options 
    *) 
      init
      clean_install_tmp
      package_check $1
      display_layer_choice
      environment_check
      $ECHO  "INSTALLING $1"
      install_package $1 
      post_processing
      m_exit 0 
    ;; 

  esac 
  shift 
done 

# ensure that the install_setup.ksh is not world or group readable as it
# contains the metrica username and passwords
chmod 700 $METRICA_DIR/install_setup.ksh > /dev/null 2>&1

m_exit 0 
