#!/bin/ksh
# @(#) reporting/delivery/commandlinetools/install/metrica_rteadmin.ksh reporting_r4.79.0.4:cset.001710:2:2 03/15/07 @(#)
###############################################################################
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                     ADC Metrica
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright ADC Metrica, 2001.
#
###############################################################################
#
# Script to start, stop and assess the status of the Report Task Executor
#
# Created    7 June 2002
# Author     Patrick Wood / John Peckham / Dave Burford
# Version    reporting_r4.79.0.4
#
###############################################################################
#
# Exit codes
#
exit_rc_success=0
exit_rc_syntax=1
exit_rc_bad_env=2
exit_rc_bad_dir=3

#
# Constants
#
PROGNAME=`basename $0`
VERNO=reporting_r4.79.0.4
NS_NAME=ReportTaskExecutor
NS_ON_OFF_FLAG=0
NS_STATUS_FLAG=2

#
# Classpath
#
cp=

###############################################################################
#
# Function    : start_server
#
# Description : Start RTE
#
# Parameters  : none
#
###############################################################################
start_server() {
    status
    if [[ $NS_STATUS_FLAG = "2" ]] ; then
	# Start the server
        echo "Starting $NS_NAME ..."
	java -cp $cp	-DORBagentPort=$OSAGENT_PORT -Xms20m -Xmx100m \
  		-DSVCnameroot=namingservice \
  		-Dorg.omg.CORBA.ORBClass=com.inprise.vbroker.orb.ORB \
  		-Dorg.omg.CORBA.ORBSingletonClass=com.inprise.vbroker.orb.ORBSingleton \
  		-Djavax.rmi.CORBA.StubClass=com.inprise.vbroker.rmi.CORBA.StubImpl \
  		-Djavax.rmi.CORBA.UtilClass=com.inprise.vbroker.rmi.CORBA.UtilImpl \
  		-Djavax.rmi.CORBA.PortableRemoteObjectClass=com.inprise.vbroker.rmi.CORBA.PortableRemoteObjectImpl \
  		com.adc.metrica.reporting.reportservice.reportserver.reporttaskexecutor.server.ReportTaskExecutorServer &

	# Wait for it to load
    	while status; [[ $NS_STATUS_FLAG != "0" ]]
	    do
	        sleep 10
	    done

	# Re-register any scheduled tasks
	echo "Reregistering scheduled reports ..."
	java -cp $cp -DORBagentPort=$OSAGENT_PORT -DSVCnameroot=namingservice com.adc.metrica.reporting.reportservice.reportserver.client.TaskReregistrationClient
    fi
    

    echo "$NS_NAME started"
}

###############################################################################
#
# Function    : restart_service
#
# Description : Restarts the Event Service
#
# Parameters  : none
#
# Returns     : nothing
#
###############################################################################
restart_service() {
	java -cp $cp 	-DORBagentPort=$OSAGENT_PORT -Xms20m -Xmx100m \
  		-DSVCnameroot=namingservice \
		com.adc.metrica.reporting.reportservice.reportserver.reporttaskexecutor.server.ReportTaskExecutorController -restart	
}

###############################################################################
#
# Function    : stop_server
#
# Description : Stop RTE
#
# Parameters  : none
#
###############################################################################
stop_server() {
    status
    if [[ $NS_STATUS_FLAG != "2" ]]; then
        echo "Stopping $NS_NAME ..."

        # If the application server is accidently restarted without a
        # 'metrica_rteadmin.ksh -stop' command being entered first, and then a
        # 'metrica_rteadmin.ksh -start' command is entered, then a second RTE
        # will begin running. ReportTaskExecutorController -stop only kills one
        # RTE at a time, so repeatedly run 'ReportTaskExecutorController -stop'
        # until status function does not find any RTEs running.
        #
        while status; [[ $NS_STATUS_FLAG != "2" ]]
        do
            java -cp $cp -DORBagentPort=$OSAGENT_PORT -Xms20m -Xmx100m \
                    -DSVCnameroot=namingservice \
                    com.adc.metrica.reporting.reportservice.reportserver.reporttaskexecutor.server.ReportTaskExecutorController -stop

            sleep 10
        done
    fi

    echo "$NS_NAME stopped"
}

###############################################################################
#
# Function    : status
#
# Description : Get status of RTE
#
# Parameters  : none
#
###############################################################################
status()
{
    NS_STATUS_STRING=`java -cp $cp -DORBagentPort=$OSAGENT_PORT -Xms20m -Xmx100m \
  		-DSVCnameroot=namingservice \
		com.adc.metrica.reporting.reportservice.reportserver.reporttaskexecutor.server.ReportTaskExecutorController -status`

	 if echo $NS_STATUS_STRING% | grep Stopped > /dev/null ; then
        # RTE is stopped
        NS_ON_OFF_FLAG=0
        NS_STATUS_FLAG=2
	 elif echo $NS_STATUS_STRING% | grep Starting > /dev/null ; then
        # RTE is starting
        NS_ON_OFF_FLAG=0
        NS_STATUS_FLAG=1
	 elif echo $NS_STATUS_STRING% | grep Running > /dev/null ; then
        # RTE is running
        NS_ON_OFF_FLAG=1
        NS_STATUS_FLAG=0
    fi

    echo $NS_STATUS_STRING
    return $NS_STATUS_FLAG
}

###############################################################################
#
# Function    : usage
#
# Description : Shows program syntax and arguments
#
# Parameters  : none
#
###############################################################################
usage() {
    cat <<END
Usage: $PROGNAME [options]

       options:
                 [ -h | -help | --help ]
               | [ -v | -version | --version ]
               | [ -start | -stop | -restart | -status ]

END
}

###############################################################################
#
# Function    : check_cmdline
#
# Description : Validate command line arguments
#
# Parameters  : none
#
###############################################################################
check_cmdline() {
    # Make sure an argument is passed in
    if [[ $# -eq 0 ]] ; then
        usage
        exit $exit_rc_syntax
    fi
}

###############################################################################
#
# Function    : bad_env
#
# Description : Environment variable is not set
#
# Parameters  : Name of environment variable
#
###############################################################################
bad_env() {
    echo
    echo "Environment variable \$$1 is not set" >&2
    echo
    exit $exit_rc_bad_env
}

###############################################################################
#
# Function    : bad_dir
#
# Description : Directory pointed to by environment variable does not exist
#
# Parameters  : Name of environment variable
#
###############################################################################
bad_dir() {
    echo
    echo "Directory referred to by environment variable \$$1 does not exist" >&2
    echo
    exit $exit_rc_bad_dir
}

###############################################################################
#
# Function    : bad_jar
#
# Description : Jar file does not exist
#
# Parameters  : Full path/name of JAR file
#
###############################################################################
bad_jar() {
    echo
    echo "JAR file not found:" >&2
    echo $1 >&2
    exit $exit_rc_bad_jar
}

###############################################################################
#
# Function    : check_env
#
# Description : Ensure environment variable is set and optionally that it
#               refers to a directory that exists.
#
# Parameters  : "var" = check variable, "var_and_dir" = also check dir exists
#               Environment variable name (without "$" sign)
#               Value of the environment variable
#
###############################################################################
check_env() {
    if [ -z "$3" ] ; then
        bad_env "$2"
    fi

    if [ "$1" = "var_and_dir" -a ! -d "$3" ] ; then
        bad_dir "$2"
    fi
}

###############################################################################
#
# Function    : addJar
#
# Description : Add a Jar to the runtime CLASSPATH. A check is made to ensure
#               the Jar file exists.
#
# Parameters  : none
#
###############################################################################
addJar() {

    # First ensure the JAR file exists
    if [ ! -f "$1" ]
    then
        bad_jar $1
    fi

    # Add it to the classpath
  	 cp=$cp:$1
}

###############################################################################
#
# Function    : setup_runtime
#
# Description : Set the CLASSPATH needed at runtime
#
# Parameters  : none
#
###############################################################################
setup_runtime() {

    #
    # Third party jars
    #
    # Appserver jars
    addJar $METRICA_DIR/ext/lib/vbjorb.jar
    addJar $METRICA_DIR/ext/lib/asrt.jar
    addJar $METRICA_DIR/ext/lib/vbsec.jar
    addJar $METRICA_DIR/ext/lib/lm.jar
    addJar $METRICA_DIR/ext/lib/xerces.jar
    addJar $METRICA_DIR/ext/lib/xmlrt.jar

    # Informix Jars
    addJar $METRICA_DIR/ext/lib/ifxjdbc.jar
    addJar $METRICA_DIR/ext/lib/ifxjdbcx.jar

    #
    # Common Services Scheduler
    #
    addJar $METRICA_DIR/CS/lib/schedulingservice_client.jar
    addJar $METRICA_DIR/CS/lib/schedulingservice_server.jar
    addJar $METRICA_DIR/CS/lib/schedulingservice_valuetypes.jar
    addJar $METRICA_DIR/CS/lib/schedulingservice_persistence.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_common.jar
    addJar $METRICA_DIR/CS/lib/propertyservice_client.jar
 	 addJar $METRICA_DIR/CS/lib/propertyservice_cache.jar
 	 addJar $METRICA_DIR/CS/lib/propertyservice_persistence.jar
 	 
    #
    # Common Services Util classes
    #
    addJar $METRICA_DIR/CS/lib/exceptionutils.jar
    addJar $METRICA_DIR/CS/lib/logservice.jar
    addJar $METRICA_DIR/CS/lib/corbautils.jar
    addJar $METRICA_DIR/CS/lib/timeutils.jar
    addJar $METRICA_DIR/CS/lib/ejbhomefactory.jar
    addJar $METRICA_DIR/CS/lib/sysmanager.jar
    
    #
    # Reporting jars
    #
    addJar $METRICA_DIR/Uiweb/lib/uiweb.jar
    addJar $METRICA_DIR/Reporting/lib/reporttaskexecutor.jar
    addJar $METRICA_DIR/Reporting/lib/reporttreeservicesecuritybackdoor.jar
    addJar $METRICA_DIR/Reporting/lib/reporttreeservicesecurityelements.jar
    addJar $METRICA_DIR/Reporting/lib/taskregistrationclient.jar
    addJar $METRICA_DIR/Reporting/lib/reportserver.jar
    addJar $METRICA_DIR/Reporting/lib/reportingutil.jar
    addJar $METRICA_DIR/Reporting/lib/reportuserdetails.jar
    addJar $METRICA_DIR/Reporting/lib/reportparameterdetails.jar
    
    #
    # QueryService jars
    #
    addJar $METRICA_DIR/DBServices/lib/queryservice_connectionmanager.jar
    addJar $METRICA_DIR/DBServices/lib/queryservice_requestmanager.jar
    addJar $METRICA_DIR/DBServices/lib/queryservice_logging.jar

    #
    # DBElements jars
    #
    addJar $METRICA_DIR/DBElements/lib/metricaudt.jar


    #
    # Other
    #
	 addJar $METRICA_DIR/Reporting/lib/commandlineutils.jar
	 addJar $METRICA_DIR/Reporting/lib/xmlutils.jar
	 addJar $METRICA_DIR/Reporting/lib/datastoreutils.jar
	 addJar $METRICA_DIR/Reporting/lib/reporttreeservice.jar
	 addJar $METRICA_DIR/Reporting/lib/rtscommandlinetool.jar
	 addJar $METRICA_DIR/Security/lib/securityservice.jar
	 addJar $METRICA_DIR/Reporting/lib/reportlibraryservice.jar
	 addJar $METRICA_DIR/Reporting/lib/rlscommandlinetool.jar
	 addJar $METRICA_DIR/Reporting/lib/reportserverstores.jar    
	 addJar $METRICA_DIR/MetaLayer/lib/rmqcommon.jar
	 addJar $METRICA_DIR/DBServices/lib/queryservice_util.jar    
	 addJar $METRICA_DIR/DBServices/lib/queryservice_xmlejb.jar
}

###############################################################################
#
# Function    : init
#
# Description : Perform the necessary checks to make sure the environment is
#               set up ready to run administration functions.
#
# Parameters  : none
#
###############################################################################
init() {
    check_env "var"         "OSAGENT_PORT" $OSAGENT_PORT
    check_env "var_and_dir" "METRICA_DIR" $METRICA_DIR
    check_env "var_and_dir" "APPSERVER_DIR" $APPSERVER_DIR

    setup_runtime
}

###############################################################################
###############################################################################
#
# Main program
#
###############################################################################
###############################################################################
check_cmdline $*

case "$1" in
    "-h"|"-help"|"--help" )
        usage
        exit $exit_rc_success
        ;;

    "-v"|"-version"|"--version" )
        . `dirname $0`/metrica_version.ksh
        print_version "Report Task Executor" "reporting_r4.79.0.4:cset.001710:2:2"
        exit $exit_rc_success
        ;;

    "-start" )
        init
        start_server
        exit $exit_rc_success
        ;;

    "-stop" )
    	init
        stop_server
        exit $exit_rc_success
        ;;

    "-restart" )
    	# The script stops and starts the server, rather than use the RTE's restart capability
    	init
        stop_server
        start_server
        exit $exit_rc_success
        ;;

    "-status" )
    	init
        status
        exit $exit_rc_success
        ;;

    * )
        usage
        exit $exit_rc_syntax
        ;;
esac

exit $exit_rc_success
