#!/bin/ksh
###############################################################################
#
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_change_column_type.ksh SysManagement_r4.79.0.9:cset.007910:3:3 01/24/06 @(#)
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_change_column_type.ksh
# Author:       Barry Deignan    
#
# Description:  Alter all tables with specified column 
#               and column type                          
#
###############################################################################
#
# PROGRAMS AND SUBROUTINES
#
##########################################################################
  
#
# SUB PROGRAM  main - Main Routine
#

main() {

# using utilities in common script
. metrica_install_common.ksh

## Validate Input Parms
valparms

##  Validate Database Name                      
valdb

## Set the Log File                                 
LOG_FILE="$METRICA_DIR/logs/metrica_change_column_type.log"
rm -f $LOG_FILE* > /dev/null

## Validate the Column Name File                             
valcl

## Validate FROM_TYPE                              
TYPE=$I6
valtype
FROM_TYPE=$I6

## Get the TO_TYPE                              
TYPE=$I7
valtype
TO_TYPE=$I7

## Validate the number of Concurrent Threads         
valthreads
#THREADS="4"


## Present Selection Summary & Confirmation
#clear
#echo ' '
#echo "METRICA COLUMN CONVERTER"                                      

#echo ' '
#echo "You have entered the following parameters"
#echo ""
#echo "	Admin Database                   	:	"$THE_DATABASE
#echo "	Log File                         	:	"$LOG_FILE     
#echo "	Column Name                      	:	"$DISPLAY_COLUMN_NAME
#echo "	From Type                        	:	"$FROM_TYPE   
#echo "	To Type                          	:	"$TO_TYPE      
#echo "	Number of Concurrent Threads     	:	"$THREADS      
#echo ""
#echo ""
#	mkmesg
	conv_column  
}
  

#
# Sub Program mkmesg - Display Configuration Completion Message
#

mkmesg() {

#clear
echo " "
echo "METRICA COLUMN CONVERTER"
echo " "
echo " "
echo "   You have just submitted "$THREADS" perl jobs "                  
echo "   to alter all tables with columns "$DISPLAY_COLUMN_NAME
echo "   in database "$THE_DATABASE                         
echo "   from type "$FROM_TYPE" to type "$TO_TYPE           
echo " "
echo "   Monitor their progress by checking the log files " 
echo " "
echo " 		" $LOG_FILE
echo "      		and"
echo " 		" $LOG_FILE"_thread_1 to "$THREADS
echo " "
}
 
#
# Sub Program valparms - Validate Input Parameters
#

valparms() {
if [ -z "$I1" ]; then
  	prtusage
	echo " "	
  	echo "ERROR: No arguments given"
	echo " "	
	exit
fi

#while [[ -n $I1 ]]
#do
  #case "$I1" in
    ## Help options
    #"-h"|"-help"|"--help"|"-?")
      #prtusage
    #;;

    ## Version options
    #"-v"|"-version"|"--version")
	#prtcopyrt
	#echo "SysManagement_r???"
	#exit
    #;;
#esac
#done

	if [ "$I1" = "-h" -o "$I1" = "-help" -o "$I1" = "--help" ]
	then
		clear
		prtusage
		exit
	fi

	if [ "$I1" = "-v" -o "$I1" = "-version" -o "$I1" = "--version" ]
	then
		clear
		prtcopyrt
		echo "$PROGNAME version @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_change_column_type.ksh SysManagement_r4.79.0.9:cset.007910:3:3 01/24/06 @(#)"
		exit
	fi

	if [ "$I1" != "-database" -o "$I3" != "-columnlist" -o "$I5" != "-from_to_types" -o "$I8" != "-threads" ]
	#if [ "$I1" != "-database" -o "$I3" != "-columnlist" -o "$I5" != "-from_to_types" ]
	then
		prtusage
		echo " "	
		echo "ERROR: Invalid Arguments Passed to program"
		echo " "	
		exit
	fi
}

#
# Sub Program prtcopyrt - Print Vallent Copyright
#

prtcopyrt() {

echo "##############################################################"
echo "#"
echo "#                 Metrica Installation "
echo "# "
echo "#                   COPYRIGHT NOTICE "
echo "# "
echo "#     This file remains the intellectual property of: "
echo "# "
echo "#                 Vallent Software Systems "
echo "#                    Spencer House "
echo "#                    23 Sheen Road "
echo "#                 Richmond, Surrey, UK "
echo "# "
echo "# All rights reserved. Copyright Vallent Software Systems UK 2005"
echo "# "
echo "##############################################################"
echo " "
}
  

#
# Sub Program prtusage - Print Help         
#

prtusage() {

prtcopyrt
echo " "
echo " "
echo "  Usage: metrica_change_column_type.ksh [options]"
echo " "
echo "  options:"
echo "		[ -h | -help | --help ]"
echo "		| [ -v | -version | --version ]"
echo "		| [ -database <database_name> ]"
echo "		| [ -columnlist <filename> ]"
echo "		| [ -from_to_types <from_columntype> <to_columntype>]"
echo "		| [ -threads <number of threads (between 1 and 10> ]"
echo " "
}

 
#
# Sub Program valitem - Validate an item entered
#

valitem() {

	if [ "$ITEM" = "" ]
	then
		#clear
		prtusage
		echo ''
		echo 'ERROR : You Entered a blank parameter - Try Again'
		echo ''
		exit
	fi

}

#
# Sub Program valdb - Validate Database Name Entered
#

valdb() {

ITEM=$I2
valitem
	
echo 'select count(*) from sysdatabases where name ="'$I2'";' | dbaccess sysmaster > $LOG0        2>&1 
DB_COUNT=`cat $LOG0 | grep -v count | grep -v Database | grep -v row`

if [ "$DB_COUNT" -eq "0" ]
then
	prtusage
	echo " "
	echo "ERROR: "$I2" is not a valid database on this Informix Server"
	echo " "
	exit
else
	THE_DATABASE=$I2
fi	


}

#
# Sub Program valcl - Validate Column List           
#

valcl() {

ITEM=$I4
valitem

if [ ! -f $I4 ]
then
	prtusage
	echo " "
	echo "ERROR: The Column list file "$I4" does not exist"                                        
	echo " "
	exit
fi
	
DISPLAY_COLUMN_NAME=""
PASS_COLUMN_NAME=""

for COLUMN_NAME in `cat $I4`
do
	#echo 'select count(*) from syscolumns where colname ="'$COLUMN_NAME'";' | dbaccess $I2 > $LOG0        2>&1 
	#CL_COUNT=`cat $LOG0 | grep -v count | grep -v Database | grep -v row`

	#if [ "$CL_COUNT" -eq "0" ]
	#then
		#prtusage
	#	LOG "WARNING: "$COLUMN_NAME" is an invalid column name - Tables do not contain this column, techpacks may not be installed, check the contents of the column list file "$I4
    #            LOG "Column conversion will not be run"
	#exit
	#fi	

        DISPLAY_COLUMN_NAME=$DISPLAY_COLUMN_NAME"'"$COLUMN_NAME"',"
        PASS_COLUMN_NAME=$PASS_COLUMN_NAME"--column_name "$COLUMN_NAME" "
done

DISPLAY_COLUMN_NAME="("$DISPLAY_COLUMN_NAME")"

}

#
# Sub Program valtype - Validate Column Type           
#

valtype() {

ITEM=$TYPE
valitem
	

if [ "$TYPE" = "utime" -o "$TYPE" = "integer" ]
then
	OK="y"
else
	prtusage
	echo " "
	echo "ERROR: "$TYPE" is an invalid column type only utime or integer allowed"
	echo " "
	exit
fi

}

#
# Sub Program valtthreads - Validate Number of Threads     
#

valthreads() {

ITEM=$I9
valitem
	

if [ "$I9" -gt "0" -a "$I9" -lt "11" ]
then
	THREADS=$I9
else
	prtusage
	echo " "
	echo "ERROR: Invalid number of threads specified - Must be between 1 and 10"
	echo " "
	exit
fi

}


#
# Sub Program conv_column            
#

conv_column() {

perl $METRICA_DIR/SystemManagement/bin/metrica_change_column_type.pm --database=$THE_DATABASE --log_file=$LOG_FILE $PASS_COLUMN_NAME --from_type=$FROM_TYPE --to_type=$TO_TYPE --no_threads=$THREADS 

FLAG="n"
while [ "$FLAG" = "n" ]
do
	JOB_COUNT=`ps -ef | grep "metrica_change_column_type.pl" | grep -v grep | wc | awk '{print $1}'`

	if [ "$JOB_COUNT" -eq "0" ]
	then
		echo " "
		echo "Column Conversion Completed"
		echo " "
		ACTUAL_THREADS=`ls $METRICA_DIR/logs | grep "metrica_change_column_type.log_thread" | wc | awk '{print $1}'`

		if [ "$ACTUAL_THREADS" -eq 0 ]; then
			echo "No conversion threads were required"
		else
			echo $ACTUAL_THREADS" out of "$THREADS" possible threads were required and have finished"
			echo " "
			CNT0="0"
			while [ "$CNT0" -lt "$THREADS" ]
			do
				CNT0=`expr $CNT0 + 1`
				if [ -f $LOG_FILE"_thread_"$CNT0 ]; then
					ERROR_COUNT=`grep -i error $LOG_FILE"_thread_"$CNT0 | grep -vi error_ | wc | awk '{print $1}'`
					echo "  "$ERROR_COUNT" messages detected in log file "$LOG_FILE"_thread_"$CNT0           
				fi
			done
		fi	
		echo " "
		FLAG="y"
	else
		sleep 10
	fi
done
	
}

#Preserve Input parameters
LOG0=/tmp/mcc$$
I1=$1
I2=$2
I3=$3
I4=$4
I5=$5
I6=$6
I7=$7
I8=$8
I9=$9
main
