#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_appserver.ksh SysManagement_r4.79.0.9:cset.007910:8:8 01/24/06 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_check_appserver.ksh
# Author:       Paul Harrington
# Description:  Script to:                 
#                 1. Check current Metrica Application Server Status
#                 2. Check for error messages in App Server logs
# Options:
#
###############################################################################

PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9
DATEXT=`date +%d-%b-%Y:%H:%M'`
integer RUNNING=0
integer STARTING=1
integer STOPPING=3
integer STOPPED=4

# Set up common variables
#
. metrica_install_common.ksh

# Check environment variables
#
[[ -z "$APPSERVER_DIR" ]] && $ECHO "APPSERVER_DIR not defined" && exit 2
[[ -z "$APPSERVER_MGMT_PORT" ]] && $ECHO "APPSERVER_MGMT_PORT not defined" && exit 3
[[ -z "$APPSERVER_NAME" ]] && $ECHO "APPSERVER_NAME not defined" && exit 4
[[ -z "$METRICA_DIR" ]] && $ECHO "METRICA_DIR not defined" && exit 5

APPSERVER_AUTH_FILE=$APPSERVER_DIR/.bes_auth
EJBPARTITION_NAME=${EJBPARTITION_NAME:-standard}
WEBPARTITION_NAME=${WEBPARTITION_NAME:-standard}
APPSERVER_STD_ARGS="-mgmtport $APPSERVER_MGMT_PORT -server $APPSERVER_NAME"
CONTROL_DIR=$METRICA_DIR/SystemManagement/control
APP_FILE=$CONTROL_DIR/metrica_check_appserver.conf
SERVER_CONF=$APPSERVER_DIR/var/servers/$APPSERVER_NAME/adm/properties/partitions/standard/partition_server.config
REPORT=$METRICA_DIR/logs/checks/metrica_check_appserver.html
WRK0=/tmp/work0$$
if [ ! -d $METRICA_DIR/logs/checks ]; then
	mkdir -p $METRICA_DIR/logs/checks
fi

# Make sure appserver binaries are on path
#
export PATH=$PATH:$APPSERVER_DIR/bin

#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}



#
# Show correct command-line usage
#
usage()
{
show_info
cat <<END

  Usage: $PROGNAME [options]

        options:    [ -h | -help | --help ]
                  | [ -v | -version | --version ]

  This script checks the status of the Application Server.
END
}


 
#
# Show Version
#
version()
{
	show_info
	echo "$PROGNAME version @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_appserver.ksh SysManagement_r4.79.0.9:cset.007910:8:8 01/24/06 @(#)"
	exit 0
}


 
#
# Print Header Information to Standard Out & HTML Report
#
header()
{
# Print Header Information to Standard Out & HTML Report
	$ECHO "
 Application Server Report.

Application Server: $APPSERVER_NAME
OSAGENT Port: $OSAGENT_PORT
App Server Management Port: $APPSERVER_MGMT_PORT

Checking Metrica Application Server: "
cat <<END > $REPORT
<HTML>
<HEAD>
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=windows-1252">
<TITLE>APPLICATION SERVER REPORT</TITLE>
</HEAD>
<BODY LINK="#0000ff" VLINK="#800080" BGCOLOR="#ffffff">
<H2>
<P><A NAME="appserver"><B>Application Server Healthcheck Report </B></P>
</H2>
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Date:&nbsp;&nbsp;&nbsp;</B>&nbsp;&nbsp;&nbsp;$DATEXT</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<P></P>
<TABLE width="40%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>App Server:</B></FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana>$APPSERVER_NAME</FONT></FONT></TD>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>OSAGENT Port:</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana>$OSAGENT_PORT</FONT></FONT></TD>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>AppServer Mgmt Port:</B></FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana>$APPSERVER_MGMT_PORT</FONT></FONT></TD>
</TBODY></TABLE>
END
    #report_message "<B>Checking Metrica Application Server: </B>"
}
  

sub_header()
{
	echo $*
	echo ''
cat <<END >> $REPORT
<B><FONT FACE="Verdana" SIZE=2><P>$*</P></FONT></B>
END

}



footer()
{

# Print Footer Information to Standard Out & HTML Report
cat <<END >> $REPORT
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>*** Metrica Application Server Check Finished ***</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<HR align=left width="100%" noShade SIZE=1>
</BODY>
</HTML>
END
	$ECHO Metrica Application Server Check Complete. 
	$ECHO Report written to $REPORT
	rm -rf $WRK0 >> /dev/null 2>&1
}


#
# Print Message to HTML Report
#
report_message()
{

	 echo "  "$*
	 echo ''
	cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2><DIR><P>$*</P></DIR></FONT>
<P> </P>
END

}



#
# Print Error Detection Message to HTML Report
#
report_error()
{

# Print Error Detection Message to Standard Out & HTML Report
	echo "  "$*
	echo ""
	cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><DIR><B><P>$*</P></B></DIR></FONT>
<P> </P> 
END

}


# Get status of appserver
#
status_server()
{
	sub_header "Checking AppServer status...."
	STATUS=`iastool -ping $APPSERVER_STD_ARGS < $APPSERVER_AUTH_FILE | tail -1`

	if [[ ${STATUS%Starting} != "$STATUS" ]] ; then
		report_message "AppServer: Starting"
		return $STARTING
	elif [[ ${STATUS%Running} != "$STATUS" ]] ; then
		report_message "AppServer: Running"
		return $RUNNING
	else
		report_message "AppServer: Stopped"
		return $STOPPED
	fi
}


# Get status of ejbpartition
#
status_partition()
{
	PNAME=$1
	sub_header "Checking Status of Partition $PNAME...."
	STATUS=`iastool -ping $APPSERVER_STD_ARGS -partition $PNAME < $APPSERVER_AUTH_FILE | tail -1`

	if [[ ${STATUS%Starting} != $STATUS ]]; then
		report_message "Partition: Starting"
		return $STARTING
	elif [[ ${STATUS%Running} != $STATUS ]]; then
		report_message "Partition: Running"
		return $RUNNING
	else
		report_message "Partition: Stopped"
		return $STOPPED
	fi
}


#
# Check Application Server is up
#
# Test App Server first, then partition.
chk_appserver_up()
{
	# Test App Server, then partition(s). Chain commands together so that the first one
	# to run nonzero will stop the chain and end there.
	
	if [[ $EJBPARTITION_NAME = $WEBPARTITION_NAME ]] ; then
		status_server && status_partition $EJBPARTITION_NAME
	else
		status_server && status_partition $EJBPARTITION_NAME && status_partition $WEBPARTITION_NAME
	fi

	# Overall App Server status is given as status of last status call to run.
	# This will be 0 (RUNNING) unless either is not running.
	APPSERVER_STATUS=$?

	case $APPSERVER_STATUS in
			# 0 = RUNNING
		"0")
			$ECHO "Application Server is currently Running."
			#report_message "<B>Application Server is currently Running.</B>"
		;;
			# 1 = STARTING
		"1")
			$ECHO "Application Server is currently Starting. Try later?"
			report_error "-	EXCEPTION ALERT - Application Server is currently Starting. Try later?"
			footer
			$ECHO "*** Application Server Check Failed ***"
			exit 1
		;;
			# 3 = STOPPING
		"3")
			$ECHO "Application Server is currently Stopping."
			report_error "-	EXCEPTION ALERT - Application Server is currently Stopping."
			footer
			$ECHO "*** Application Server Check Failed ***"
			exit 1
		;;
			# 4 = STOPPED
		"4")
			$ECHO "Application Server is currently Stopped."
			report_error "-	EXCEPTION ALERT - Application Server is currently Stopped."
			footer
			$ECHO "*** Application Server Check Failed ***"
			exit 1
		;;
	esac
}

#
#
#
chk_web_partitions()
{

	WEB_TEMP=/tmp/web_temp$$
	sub_header "Checking Web Partitions ..."
	metrica_appserveradmin.ksh -status WEB > $WEB_TEMP

	exec < $WEB_TEMP
	while read LINE
	do
		report_message $LINE
	done

	STOP_CNT=`cat $WEB_TEMP | grep -ic "Stopped"`
	if [ "$STOP_CNT" -gt 0 ]; then
		report_error "- EXCEPTION ALERT - Some or All of the Web partitions are currently Stopped."
	fi

	rm $WEB_TEMP > /dev/null 2>&1

}


#
# Check OSAgent ports are active
#
chk_ports_active()
{
	sub_header "Checking OSAGENT port..."
	netstat -a | grep $OSAGENT_PORT 1>/dev/null 2>&1 && { 
		report_message "OSAGENT: Active"
	} || {
		$ECHO "OSAGENT_PORT is Inactive. Please investigate!"
		report_error "OSAGENT: Inactive. Please investigate!"
	}

	sub_header "Checking APPSERVER_MGMT port..."
	netstat -a | grep $APPSERVER_MGMT_PORT 1>/dev/null 2>&1 && { 
		report_message "APPSERVER_MGMT_PORT: Active"
	} || {
		$ECHO "APPSERVER_MGMT_PORT is Inactive. Please investigate!"
		report_error "APPSERVER_MGMT_PORT: Inactive. Please investigate!"
	}
}	


#
# Check the AppServer JVM Heap Sizes and compare with healthcheck configured values
#
chk_heap_sizes(){

#Get the configured vales for Heap Sizes.                  
	 ARCH=`uname`
	 case $ARCH in
		 HP-UX)
			 MAXHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xmx" | awk '{print $2}' | sed s/-Xmx//1 | sed s/m//1`
			 MINHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xms" | awk '{print $2}' | sed s/-Xms//1 | sed s/m//1`
			 NEWHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xmn" | awk '{print $2}' | sed s/-Xmn//1 | sed s/m//1`
		 ;;
		 SunOS)
			 MAXHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xmx" | awk '{print $2}' | sed s/-Xmx//1 | sed s/m//1`
			 MINHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xms" | awk '{print $2}' | sed s/-Xms//1 | sed s/m//1`
			 NEWHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xmn" | awk '{print $2}' | sed s/-Xmn//1 | sed s/m//1`
		 ;;
	 esac

#Does the Appserver control file exist - if not create it.
	if [ ! -f $APP_FILE ]; then

		touch $APP_FILE
		$ECHO "JVM_MAXIMUM_HEAP_SIZE "$MAXHEAP >> $APP_FILE
		$ECHO "JVM_MINIMUM_HEAP_SIZE "$MINHEAP >> $APP_FILE
		$ECHO "JVM_NEW_HEAP_SIZE "$NEWHEAP >> $APP_FILE

	else

		CONFIG_MAX=`cat $APP_FILE | grep "JVM_MAXIMUM_HEAP_SIZE" | grep -v ".JVM_MAXIMUM_HEAP_SIZE" | awk '{print $2}'`
		CONFIG_MIN=`cat $APP_FILE | grep "JVM_MINIMUM_HEAP_SIZE" | grep -v ".JVM_MINIMUM_HEAP_SIZE" | awk '{print $2}'`
		CONFIG_NEW=`cat $APP_FILE | grep "JVM_NEW_HEAP_SIZE" | grep -v ".JVM_NEW_HEAP_SIZE" | awk '{print $2}'`

	fi


	sub_header "Comparing JVM Heap Sizes with Configured Values ...."

#Insert Table Headers
cat <<END >> $WRK0
<DIR>
<TABLE cellSpacing=2 cellPadding=6 border=3 width="70%">
<TBODY>
<TR align=middle>
<TH colSpan=4>Parameter Comparisons </TH></TR>
<TR>
<TH align=middle>Heap Size Parameter </TH>
<TH align=middle>Actual Value</TH>
<TH align=middle>Expected Value </TH>
END

	EXCEPT_FLAG="N"
	check_value $MAXHEAP $CONFIG_MAX "Maximum"
	check_value $MINHEAP $CONFIG_MIN "Minimum"
	check_value $NEWHEAP $CONFIG_NEW "New"

cat $WRK0 >> $REPORT
cat <<END >> $REPORT
</TBODY>
</TABLE></DIR><BR>
END

# Check if New Heap Size is 1/7 of Maximum Heap

	#RATIO=`echo 'scale=2; '$MAXHEAP'/'$NEWHEAP' | bc | awk '{FS=".";print $1}'`
	#if [ "$RATIO" -ne 7 ]; then
		#report_error "- EXCEPTION ALERT - The JVM New Heap Size Parameter is not 1/7th of the Maximum Heap Size"
		#EXCEPT_FLAG="Y"
	#fi


	#Print Completion message if OK
	if [ "$EXCEPT_FLAG" = "N" ];then
		$ECHO "	All Parameters conform to configured values .... OK"
		report_message "All Parameters conform to configured values .... OK"
	fi

}

check_value(){

	$ECHO "	The JVM "$3" Heap Size is "$1      
	if [ "$1" -ne "$2" ]; then
		report_error "- EXCEPTION ALERT - The JVM "$3" Heap Size Parameter is "$1" when "$2" was expected"
		COLOR="#ff0000"
		EXCEPT_FLAG="Y"
	else
		COLOR="#000000"
	fi
cat <<END >> $WRK0
<TR>
<TD align=middle><FONT COLOR="$COLOR">JVM $3 Heap Size</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$1</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$2</FONT></TD>
</TR>
END

}



#
# Check App Server Logs and write a summary to report.
# 
# This section will need further consideration as we don't have a lot of
# guidance yet as to what constitutes the sort of error that really 
# needs to be reported here.
#
chk_appserver_logs()
{

# Specify all logs which need checking in this list. Could move list to 
# somewhere more accessible and configurable..
# IAS Errors first.

	IASLOGLIST="$APPSERVER_DIR/var/servers/$APPSERVER_NAME/adm/logs/*err*"

	CNT_IASERROR=`cat $IASLOGLIST | grep -ci 'error|stderr'`
	ERROR_IASFILES=`grep -li error $IASLOGLIST`

	if [ "$CNT_PTNERROR" -gt "0" ]
	then
		report_error "-	EXCEPTION ALERT - $CNT_IASERROR IAS Errors have been detected in the Application Server Logs since these files were last cleared down or archived away."
		report_error "-	Examine the following files for more details: " 
		for i in `echo $ERROR_IASFILES`; do
			report_error "$i"
		done
	fi

# Then Partition Errors. I'm advised that these can sometimes be treated as
# less serious. So we just log them as messages.

	PTNLOGLIST="$APPSERVER_DIR/var/servers/$APPSERVER_NAME/adm/logs/partitions/$EJBPARTITION_NAME/*err* $APPSERVER_DIR/var/servers/$APPSERVER_NAME/adm/logs/partitions/$WEBPARTITION_NAME/*err*"

	CNT_PTNERROR=`cat $PTNLOGLIST|sort |uniq | grep -ci 'error|stderr'`
	ERROR_PTNFILES=`grep -li error $PTNLOGLIST`
	
	if [ "$CNT_PTNERROR" -gt "0" ]
	then
		report_message "-	NOTE: - $CNT_IASERROR Partition Errors have been detected in the Application Server Logs since these files were last cleared down or archived away."
		report_message "-	Examine the following files for more details: " 
		for i in `echo $ERROR_PTNFILES`; do
			report_message "$i"
		done
	fi
	
	report_message "Application Server Checks Completed"
	echo '<P ALIGN="CENTER"><HR></P>' >> $REPORT
}


#
# Main Routine
#
main()
{
	header

#Check Application Server Up
	chk_appserver_up

#Check Web Partitions Up
	chk_web_partitions

#Check Management and OSAgent Ports are active
	chk_ports_active

#Check JVM Heap Sizes with values in control file
	chk_heap_sizes

#Check Application Server Log(s) for errors
# PH 13/11/03 - This section is being omitted for the moment as it is 
# felt more investigation is needed as to which errors need reporting.
#	chk_appserver_logs
 
	footer

}



#Preserve Input Parameters
I1=$1
I2=$2
I3=$3
I4=$4
I5=$5
I6=$6
case $1 in
# No parameters specified
	"")
		main
		exit 0
	;;
# Help options
	"-h" | "-help" | "--help")
		usage
		exit 0
	;;

# Version options
	"-v" | "-version" | "--version")
		version
		exit 0
	;;

# Any other option
	*)
		usage
		exit 1
	;;
esac
