#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_extents.ksh SysManagement_r4.79.0.9:cset.007910:5:5 01/24/06 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_check_extents.ksh
# Author:       Barry Deignan
# Description:  Script to list Tables & Indexes exceeding a given number of extents                
#
###############################################################################

#Set Environment
PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9
LOG0=/tmp/maxelog0$$
LOG1=/tmp/maxelog1$$
DBLIST=/tmp/maxedb$$
REPORT=$METRICA_DIR/logs/checks/metrica_check_extents.log

if [ ! -d $METRICA_DIR/logs/checks ]; then
	mkdir -p $METRICA_DIR/logs/checks
fi

LIMIT=$2
CNT0=0

#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}

#
# Show correct command-line usage
#
usage()
{
show_info
cat <<END

  Usage: $PROGNAME [options]

        options:    [ -h | -help | --help ]
                  | [ -v | -version | --version ]
                  | [ -extents <number of extents>  ]

  List Tables & Indexes with a quantity of extents
  equal to or exceeding the passed in number   

END
}
 
version()
{
	show_info
	echo "$PROGNAME version @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_extents.ksh SysManagement_r4.79.0.9:cset.007910:5:5 01/24/06 @(#)"
    exit 0
}
 
clean_up()
{

if [ -f $LOG0 ]
then
	rm $LOG0
fi

if [ -f $LOG1 ]
then
	rm $LOG1
fi

if [ -f $DBLIST ]
then
	rm $DBLIST
fi

}

#
# Main Routine
#
main(){
echo "Checking Databases ......"
 
clean_up

#
# Validate Input Number
if [ "$LIMIT" -gt "0" -a "$LIMIT" -le "250" ]
then
        # Print Report Header
        echo "     TABLE AND INDEX EXCEEDING "$LIMIT" EXTENTS BY DATABASE" > $REPORT
        echo '  ========================================================' >> $REPORT

        # Compile a list of PM Databases
        echo $METRICA_PERFORMANCE_DB >> $DBLIST
        echo $METRICA_ADMINISTRATION_DB >> $DBLIST
        echo $METRICA_SYSTEM_DB >> $DBLIST

        #Read List and Check Extents in each Database
        for a in `cat $DBLIST`
        do
                DB=$a
                header
                getdata
                parseit
                clear
                cat $REPORT
        done
else
        usage
        echo "ERROR: "$LIMIT" is an invalid value for number of extents - must be between 1 and 250"
        exit
fi

clean_up

}

parseit(){
CNT1=0
for b in `cat $LOG1`
do
        CNT0=`expr $CNT0 + 1`

        if [ "$CNT0" = "1" ]
        then
                TABLE=`echo $b | sed s/=//1 | sed s/\,//1`
                CNT1=`expr $CNT1 + 1`
                clear
                echo "Processing "$DB "Please Wait ......"
                echo "Checking Table - "$CNT1 " - "$TABLE 
        fi

        if [ "$CNT0" = "2" ]
        then
                SIZE=`echo $b | sed s/=//1 | sed s/\,//1`
        fi

        if [ "$CNT0" = "3" ]
        then
                EXTENTS=`echo $b | sed s/=//1 | sed s/\,//1`
                if [ "$EXTENTS" -ge "$LIMIT" ] 
                then
                        cat $LOG0 | grep $TABLE | grep $SIZE | grep $EXTENTS >> $REPORT
                fi
                CNT0=0
        fi

done
}


header(){

cat <<END >> $REPORT
=================================================
Database:  $DB                    
=================================================
Table/Index                                       Size(Kb)      No. of Extents
===========                                       ========      ==============
END

}

getdata(){

echo 'select substr(a.tabname,1,40) table_name, (sum(size)*2) table_size, count(*) extent_count from sysextents a where a.dbsname like "'$DB'%" and a.tabname not like " %" and a.tabname not like "sys%" group by a.dbsname, a.tabname order by 3 desc into temp table_extents; select * from table_extents where extent_count >= '$LIMIT';' | dbaccess sysmaster > $LOG0 2>&1

cat $LOG0 | grep -v expression | grep -v Database | grep -v retrieved | grep -v table | grep -v rows> $LOG1

}
  

case $1 in
        # Help options
        "-h" | "-help" | "--help")
                usage
                exit 0
        ;;

        # Version options
        "-v" | "-version" | "--version")
                version
                exit 0
        ;;

        # Get Extents    
        "-extents")
                main
				exit 0
        ;;

        # Any other option
        *)
                usage
                exit 1
        ;;
esac
