#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_gateways.ksh SysManagement_r4.79.0.9:cset.007910:6:6 01/24/06 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_check_gateways.ksh
# Author:       Barry Deignan
# Description:  Monitors the file statistics produced by a
#               productized gateway. Logs informational messages and
#               exceptions based on thresholds in the configuration file.
#
###############################################################################

#Set Environment
PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9
DATEXT=`date +%d-%b-%Y:%H:%M'`


#
# Check environment variables
#
build_environment()
{
	if [[ -z "$METRICA_DIR" ]];then
		echo "- EXCEPTION ALERT - METRICA_DIR not defined"
		echo "- Script Aborted"
		exit 1
	else
		. $METRICA_DIR/setup.ksh
		REPORT=$METRICA_DIR/logs/checks/metrica_check_gateways.html
		if [ ! -d $METRICA_DIR/logs/checks ]; then
			mkdir -p $METRICA_DIR/logs/checks
		fi
		CONF=$METRICA_DIR/SystemManagement/control/metrica_check_gateways.conf
		PERL_SCRIPT=$METRICA_DIR/SystemManagement/bin/metrica_check_gateways.pl

		WRK0=/tmp/work0$$
		WRK1=/tmp/work1$$
		WRK2=/tmp/work2$$
		touch $WRK0 $WRK1 $WRK2 2>/dev/null
	fi
}
 
   

#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}




#
# Show correct command-line usage
#
usage()
{
show_info
cat <<END

  Usage: $PROGNAME [options]

        options:    [ -h | -help | --help ]
                  | [ -v | -version | --version ]

  Monitors the file statistics produced by a
  productized gateway. Logs informational messages and
  exceptions based on thresholds in the configuration file.
  $CONF

END
}
 


version()
{
	show_info
	echo "$PROGNAME version @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_gateways.ksh SysManagement_r4.79.0.9:cset.007910:6:6 01/24/06 @(#)"
}
 

 
header()
{

# Print Header Information to Standard Out & HTML Report
echo 'Checking Gateway File Statistics'      
cat <<END > $REPORT
<HTML>
<HEAD>
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=windows-1252">
<TITLE>GATEWAYS FILE STATISTICS CHECKING REPORT </TITLE>
</HEAD>
<BODY LINK="#0000ff" VLINK="#800080" BGCOLOR="#ffffff">
<H2>
<P><A NAME="gateways"><B>Gateways File Statistics Checking Report </B></P>
</H2>
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Date:&nbsp;&nbsp;&nbsp;</B>&nbsp;&nbsp;&nbsp;$DATEXT</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
END
	report_message ''

}
 
  

sub_header()
{

	echo ''
	echo ''
	echo $*
	echo ''
cat <<END >> $REPORT
<B><FONT FACE="Verdana" SIZE=2><P>$*</P></FONT></B>
END

}



footer()
{

# Print Footer Information to Standard Out & HTML Report
cat <<END >> $REPORT
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>*** Gateways File Statistics Checking Report ***</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<HR align=left width="100%" noShade SIZE=1>
</BODY>
</HTML>
END
	clean_up
}



report_message()
{

	# Print Message to Standard Out & HTML Report
	echo "	"$*
	echo ''
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2><DIR><P>$*</P></DIR></FONT>
<P> </P>
END

}



report_error()
{

	# Print Error Detection Message to Standard Out & HTML Report
	echo "  "$*
	echo ""
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><DIR><B><P>$*</P></B></DIR></FONT>
<P> </P> 
END

}
  

check_conf_file()
{

	#Check for existence of Gateways File Statistics Configuration File
	sub_header "Checking for Gateways File Statistics Configuration file ...."

	if [ -f $CONF ];then
		report_message "File exists ... OK"
	else
		report_error "-	EXCEPTION ALERT - The configuration file "$CONF" does not exist" 
		report_error "-	This file is created as part of the configuration of the healthcheck scripts" 
		report_error "-	and is used to compare actual gateway statistics with configured thresholds" 
		report_error "-	Run "$METRICA_DIR"/SystemManagement/bin/metrica_configure_healthcheck.ksh"
		report_error "-	to configure this file"

		footer
		exit 1
	fi

	#Check for existence of Gateways File Statistics Perl Script
	sub_header "Checking for Gateways File Statistics Perl Script ...."

	if [ -f $PERL_SCRIPT ];then
		report_message "Script exists ... OK"
	else
		report_error "-	EXCEPTION ALERT - The perl script "$PERL_SCRIPT" does not exist" 
		report_error "-	Program aborted"                                                              

		footer
		exit 2
	fi

}
 


run_gateway_check()
{

	sub_header "Checking Gateways ...."

	EXCEPT_FLAG="N"
	ERROR_COUNT=0
	EXCEPT_COUNT=0
	EXCEPT_COUNT0=0
	EXCEPT_COUNT1=0

	# Run Perl Script
	$PERL_SCRIPT -c $CONF -l $WRK0 > $WRK1 2>&1

	# Check for errors in the std out
	ERROR_COUNT=`wc $WRK1 | awk '{print $1}'`

	if [ "$ERROR_COUNT" -gt "0" ]; then
		EXCEPT_FLAG="Y"
		report_error "- EXCEPTION ALERT - Errors detected in the standard output" 
		exec < $WRK1
		while read LINE
		do
			report_error "- "$LINE 
		done
	fi

	# Check for errors in the log file
	EXCEPT_COUNT0=`grep -ci "EXCEPTION" $WRK0`
	EXCEPT_COUNT1=`grep -ci "no file statistics found" $WRK0`
	EXCEPT_COUNT=`expr $EXCEPT_COUNT0 + $EXCEPT_COUNT1`

	if [ "$EXCEPT_COUNT" -gt "0" ]; then
		EXCEPT_FLAG="Y"
		report_error "- EXCEPTION ALERT - Exceptions detected in the log file" 
		exec < $WRK0
		while read LINE
		do
			report_error "- "$LINE 
		done
	fi

	#Print Completion message if OK
	if [ "$EXCEPT_FLAG" = "N" ];then
		exec < $WRK0
		while read LINE
		do
			report_message $LINE 
		done
		report_message "No Gateway threshold violations detected .... OK"
	fi

}



clean_up()
{

	#Remove all temporary files
	rm -f $WRK0 $WRK1 $WRK2 2>/dev/null


}



#
# Main Routine
#
main()
{

	build_environment
	header

	#Check for existence of Configuration File
	check_conf_file

	run_gateway_check

	footer


}


case $1 in
        # Help options
        "-h" | "-help" | "--help")
                usage
                exit 0
        ;;

        # Version options
        "-v" | "-version" | "--version")
                version
                exit 0
        ;;

        # Any other option
        *)
                main
                exit 0
        ;;
esac
