#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_ids.ksh SysManagement_r4.79.0.9:cset.008512:14:14 10/19/07 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_check_ids.ksh
# Author:       Barry Deignan
# Description:  Script to:                 
#                 1. Check online.log for Errors, AF files and Assert Failures.
#                 2. Check for corruption Reserved Pages and Extents.
#                 3. Check for possible capacity thresholds. 
#                 4. Check Table & Index extent counts. 
#                 5. Check Informix Shared Memory Utilisation.
#
###############################################################################

#Set Environment
PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9
HOSTNAME=`uname -n`
WRK0=/tmp/work0$$
WRK1=/tmp/work1$$
WRK2=/tmp/work2$$
DATEXT=`date +%d-%b-%Y:%H:%M'`


#
# Check environment variables
#
build_environment()
{
	if [[ -z "$METRICA_DIR" ]]
	then
		echo "- EXCEPTION ALERT - METRICA_DIR not defined"
		echo "- Script Aborted"
		exit 2
	else
		. $METRICA_DIR/setup.ksh
		if [ ! -d $METRICA_DIR/logs/checks ]; then
			mkdir -p $METRICA_DIR/logs/checks
		fi

		REPORT=$METRICA_DIR/logs/checks/metrica_check_ids.html
		CONF=$METRICA_DIR/SystemManagement/control/metrica_check_ids.conf
	fi
}


 
#
# Validate Input parameters
#
validate()
{

	if [ "$I1" = "-capacity_threshold" -a "$I2" -gt "0" -a "$I2" -lt "100"  -a "$I3" = "-extents" -a "$I4" -gt "0" -a "$I4" -lt "250" -a "$I5" = "-number_of_days_log" -a "$I6" -gt "-1" -a "$I6" -lt "100" ]
	then

		CAPACITY_THRESHOLD=$I2
		EXTENTS=$I4
		NUMDAYS=$I6
	else
		header
		usage
		report_error "- EXCEPTION ALERT - Invalid Arguments"
		report_error "- You entered "$PROGNAME $I1 $I2 $I3 $I4 $I5 $I6
		report_error "- Usage: "$PROGNAME" [options]"
		report_error "-	options:    [ -h | -help | --help ]"
		report_error "-	| [ -v | -version | --version ]"
		report_error '-	| [ -capacity_threshold <%utilisation>  ]'
		report_error '- | [ -extents <number_of_extents>  ]'
		report_error "- | [ -number_of_days_log <numer_of_days>  ]"
		report_error "-	%utilisation  should be between 1 and 99"          
		report_error "-	number of extents should be between 1 and 250"
		report_error "- number of days for online.log should be between 0 and 99"
		footer
		exit 3
	fi

}



informix_up() 
{

	onstat - > /dev/null
	ISUP=`echo $?`

	if [ "$ISUP" = "5" ];then
		OK="Y"
	else
		header
		report_error "- EXCEPTION ALERT - Informix Dynamic Server appears to be down"
		echo $UP
		footer
		exit 4
	fi

}


 
#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}



#
# Show correct command-line usage
#
usage()
{
	show_info
cat <<END

  Usage: $PROGNAME {
			-v | -version | --version |
			-h | -help | --help 
			} | {
		 	[ -capacity_threshold <%utilisation>  ]
			[ -extents <number of extents>  ]
            [ -number_of_days_log <numer_of_days>  ]
		 	}


  Options: 
			-v | -version | --version			Display version information
			-h | -help | --help					Display help information
			-capacity_threshold <%utilisation>	Capacity threshold utilisation
			-extents <number of extents>		Number of extents
            -number_of_days_log <numer_of_days> Number of days previous to current that will be scanned
                                                for errors in the Informix online log 

	 Substitute <%utilisation> with a figure between 1 and 99. 
	 This is the percentage capacity threshold that script uses to determine dbspaces that may be reaching saturation.

	 Substitute <number of extents> with the quantity of extents between 1 and 250.
	 The script will determine a list of tables and indexes that exceed this quantity.

     Substitute <numer_of_days> with the number of days required between 0 and 99.
     Specifying a value of 0 will cause the entire online log to be scanned for errors.

  Script to:
     1. Check online.log for Errors, AF files and Assert Failures.
     2. Check for corruption in Catalogs, Reserved Pages and Extents.
     3. Check for possible capacity thresholds.
     4. Check Table & Index extent counts.
     5. Check Informix Shared Memory Utilisation.


END
}


 
version()
{
	show_info
	echo "$PROGNAME version SysManagement/delivery/utils/bin/Monitoring/metrica_check_ids.ksh SysManagement_r4.79.0.9:cset.008512:14:14 10/19/07"
}


 
header()
{
 
# Print Header Information to Standard Out & HTML Report
	echo 'Checking Informix Dynamic Server '$DATEXT 
cat <<END > $REPORT
<HTML>
<HEAD>
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=windows-1252">
<TITLE>CHECKING INFORMIX DYNAMIC SERVER </TITLE>
</HEAD>
<BODY LINK="#0000ff" VLINK="#800080" BGCOLOR="#ffffff">
<H2>
<P><A NAME="ids"><B>Informix Dynamic Server Check Report </B></P>
</H2>
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Date:&nbsp;&nbsp;&nbsp;</B>&nbsp;&nbsp;&nbsp;$DATEXT</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<P></P>
<TABLE width="40%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Host Server:</B></FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana>$HOSTNAME</FONT></FONT></TD>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Informix Server:</B></FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana>$INFORMIXSERVER</FONT></FONT></TD>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Informix DB Port:</B></FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana>$DB_PORT</FONT></FONT></TD>
</TBODY></TABLE>
END
	report_message ''

}



sub_header()
{

	echo $* 
	echo ''
cat <<END >> $REPORT
<B><FONT FACE="Verdana" SIZE=2><P>$*</P></FONT></B>
END

}


 
footer()
{

# Print Footer Information to Standard Out & HTML Report
cat <<END >> $REPORT
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>*** Informix Dynamic Server Checks Finished ***</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<HR align=left width="100%" noShade SIZE=1>
</BODY>
</HTML>
END
	clean_up
}
 


report_message()
{

# Print Message to Standard Out & HTML Report
	echo "	"$*
	echo ''
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2><DIR><P>$*</P></DIR></FONT>
<P> </P> 
END

}



report_error()
{

	# Print Error Detection Message to Standard Out & HTML Report
	echo "  "$*
	echo ""
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><DIR><B><P>$*</P></B></DIR></FONT>
<P> </P>
END

}


 
chk_online_log()
{

	OLOG=`onstat -c | grep MSGPATH | grep -v ".MSGPATH" | awk '{print $2}'`

	sub_header "Checking the entire Online Log For Errors and Assertion Fail Messages ......"
	report_message "Use metrica_configure_healthcheck.ksh (option 1) to limit the number of days scanned."
 
	CNT_ERROR=`cat $OLOG | grep -ci error`
	CNT_ASSERT=`cat $OLOG | grep -ci "Assert Failed"`
	CNT_OLOG=`expr $CNT_ERROR + $CNT_ASSERT`

	if [ "$CNT_OLOG" -gt "0" ]; then
	     
		report_error "- EXCEPTION ALERT - Errors detected in the Informix online.log file ......"
		report_error "- "$CNT_ERROR" lines with errors."
		report_error "- "$CNT_ASSERT "lines with assert failures."
		report_error "- Examine "$OLOG" for more details" 
	else
		report_message "Online Log Error Checks Completed ... OK"
	fi

	sub_header "Checking that Shared Memory Segments are being freed regularly ......"
 
	CNT_FREE=`tail -1000 $OLOG | grep -ci "Attempting to free unused"`

	if [ "$CNT_FREE" -eq 0 ]; then

		report_error "- EXCEPTION ALERT - Shared Memory Segments are Not being freed on a regular basis ....."
		report_error "- 'onmode -F' should be set up to run from the informix users crontab"
		report_error "- Please refer to the Operations Guide for more information"
	else
		report_message "Check Completed ... OK"
	fi
	 
}

# ADDED BY FRYJ 23/11/05
# This subroutine checks the online logs for the previous N days
# This sub relies on an assistant perl script called metrica_check_ids_date_assist.pl
# This script needs to be present in the $METRICA_DIR/SystemManagement/bin/ dir
#
chk_online_log_numdays()
{
    OLOG=`onstat -c | grep MSGPATH | grep -v ".MSGPATH" | awk '{print $2}'`

	# Use assistant script to calculate the date $NUMDAYS days ago
	# usage for the script is metrica_check_ids_date_assist.pl
	# where n is number of days to subtract from current data
	# returned string is to be used to grep through onlone.log

	# Here we find the position in the online log where that date occurs
	# If we cant find a timestamp for that day, we'll try adding one to NUMDAYS
	# and retry. Do this a max of COUNTER_MAX times then abort and write error.

	COUNTER=$NUMDAYS
	COUNTER_MAX=`expr $NUMDAYS + 5`

	until [[ $OLOG_POS != "" ]]; do
	    NUMDAYS_OLOG_DATE=`$METRICA_DIR/SystemManagement/bin/metrica_check_ids_date_assist.pl $COUNTER`
	    OLOG_POS=`grep -n "$NUMDAYS_OLOG_DATE" $OLOG |awk -F: '{print $1}' | head -1`
		#echo "DATE=$NUMDAYS_OLOG_DATE; COUNTER=$COUNTER; POS=$OLOG_POS" 
	    ADD=1
	    COUNTER=`expr $COUNTER + $ADD`
	    if [[ $COUNTER -gt $COUNTER_MAX ]]; then
	        OLOG_POS="ERROR"
	        break;
	    fi
	done
	# remove the last edded count so that COUNTER represents the number of days of log that will be used
	COUNTER=`expr $COUNTER - $ADD`

	if [[ $OLOG_POS = "ERROR" ]]; then
	    DAYS_DIFF=`expr $COUNTER_MAX - $NUMDAYS`
		sub_header "Checking Online Log For Errors and Assertion Fail Messages from last $NUMDAYS day(s)..."
	    report_error "- ERROR: could not find a timestamp in the online log for $NUMDAYS ago..."
	    report_error "- Searched online.log backwards for $DAYS_DIFF days but could not find date stamps for this period"
		report_error "- This may be because the online.log file has recently been cleared."

        sub_header "Checking that Shared Memory Segments are being freed regularly ......"

        CNT_FREE=`tail -1000 $OLOG | grep -ci "Attempting to free unused"`

        if [ "$CNT_FREE" -eq 0 ]; then

            report_error "- EXCEPTION ALERT - Shared Memory Segments are Not being freed on a regular basis ....."
            report_error "- 'onmode -F' should be set up to run from the informix users crontab"
            report_error "- Please refer to the Operations Guide for more information"
        else
            report_message "Check Completed ... OK"
        fi

		return
	else	
		OLOG_LENGTH=`cat $OLOG | wc -l | sed 's/^ *//'`
		TAIL_LENGTH=`expr $OLOG_LENGTH - $OLOG_POS`
	
   		sub_header "Checking Online Log For Errors and Assertion Fail Messages from last $NUMDAYS day(s)..."
	    if [ "$COUNTER" != "$NUMDAYS" ]; then 
			DAYS_DIFF=`expr $COUNTER - $NUMDAYS`
        	report_message "It was necessary to search back $DAYS_DIFF extra day(s) to find a valid date stamp in the online.log file"
			report_message "This means that the last $COUNTER days in the online log have been searched for errors..."
    	fi

    	CNT_ERROR=`tail -$TAIL_LENGTH $OLOG | grep -ci error`
    	CNT_ASSERT=`tail -$TAIL_LENGTH $OLOG | grep -ci "Assert Failed"`
   		CNT_OLOG=`expr $CNT_ERROR + $CNT_ASSERT`

    	if [ "$CNT_OLOG" -gt "0" ]; then

	        report_error "- EXCEPTION ALERT - Errors detected in the Informix online.log file ......"
	        report_error "- "$CNT_ERROR" lines with errors."
	        report_error "- "$CNT_ASSERT "lines with assert failures."
	        report_error "- Examine "$OLOG" for more details"
	    else
	        report_message "Online Log Error Checks Completed ... OK"
	    fi

	    sub_header "Checking that Shared Memory Segments are being freed regularly ......"

	    CNT_FREE=`tail -1000 $OLOG | grep -ci "Attempting to free unused"`
	
	    if [ "$CNT_FREE" -eq 0 ]; then
	
	        report_error "- EXCEPTION ALERT - Shared Memory Segments are Not being freed on a regular basis ....."
	        report_error "- 'onmode -F' should be set up to run from the informix users crontab"
	        report_error "- Please refer to the Operations Guide for more information"
	    else
	        report_message "Check Completed ... OK"
	    fi
	fi
}


chk_corruption()
{

	sub_header "Checking for Corruption ......"
 
	report_message "Checking Informix Server Extents ....."
	CNT_CE=`oncheck -ce | grep -ci error`

	report_message "Checking Informix Reserved Pages ....."
	CNT_CR=`oncheck -cr | grep -ci error`

	CNT_CURR=`expr $CNT_CE + $CNT_CR`

	if [ "$CNT_CURR" -gt "0" ]; then
        	report_error "- EXCEPTION ALERT - Corruption detected  ......"
        	report_error "- "$CNT_CE" errors found when oncheck -ce was run."
        	report_error "- "$CNT_CR" errors found when oncheck -cr was run."
	else
        	report_message "Corruption Checks Completed ... OK"
	fi

}



chk_cachedreads()
{

	sub_header "Checking % Cached Reads ....."
	 
	CACHED=`onstat -p | head -6 | tail -1 | awk '{print $4}' | awk '{FS=".";print $1}'`

	THRESHOLD="90"
	if [ "$CACHED" -lt "$THRESHOLD" ]; then
        report_error "- EXCEPTION ALERT - The Cached Reads is below "$THRESHOLD"% ......"
        report_error "- Check your BUFFERS setting in the onconfig file "                        
	else
       	report_message $CACHED"% Cached Reads ... OK"
	fi

}



chk_updstats()
{

	sub_header "Checking for Update Stats Script in cron scheduler ....."
	 
	CNT_WRAP=`crontab -l | grep -i metrica_cronwrap.ksh | grep -c updstats`
	CNT_EXST=`crontab -l | grep -ci updstats.pl`
	CNT_PROC=`crontab -l | grep -i updstats.pl | grep -c "\-f"`
	CNT_GENR=`crontab -l | grep -i updstats.pl | grep -c "\-g"`

	CONFORMS="Y"
	PROC="Y"
	GENR="Y"
	if [ "$CNT_WRAP" -eq 0 ]; then
		CONFORMS="N"
		if [ "$CNT_EXST" -eq 0 ]; then
				CONFORMS="N"
		else
			CONFORMS="Y"
			if [ "$CNT_PROC" -eq 0 ]; then
				CONFORMS="N"
				PROC="N"
			fi

			if [ "$CNT_GENR" -eq 0 ]; then
				CONFORMS="N"
				GENR="N"
			fi
		fi
	fi
			

	if [ "$CONFORMS" = "N" ]
	then
        	report_error "- EXCEPTION ALERT - The Update Statistics Script is not configured correctly in the cron  ......"
			if [ "$PROC" = "N" ]; then
        		report_error "- The script is not being run with the '-f' flag to update stats for stored procedures."
        	fi

			if [ "$GENR" = "N" ]; then
        		report_error "- The script is not being run with the '-g' flag to re-generate the sql scripts."
        	fi

			report_error "- Please refer to the Operations Guide for more information"
	else
        	report_message "Update Statistics Script Configuration Check Completed ... OK"
	fi

}



chk_seqscans()
{

	sub_header "Checking the Ratio of Sequential Scans to Transactions"
	 
	ISAMTOT=`onstat -p | head -9 | tail -1 | awk '{print $1}'`
	#echo "ISAMTOT="$ISAMTOT
	SEQSCANS=`onstat -p | head -18 | tail -1 | awk '{print $8}'`
	if [ "$SEQSCANS" -eq 0 ]; then
		SEQSCANS=1
	fi
	#echo "SEQSCANS="$SEQSCANS
	RATIO=`echo "scale=1;"$ISAMTOT"/"$SEQSCANS | bc | awk '{FS=".";print $1}'`
	#echo "RATIO="$RATIO

	if [ "$RATIO" -lt 100 ]; then
        report_error "- EXCEPTION ALERT - The Ratio between Sequential Scans and ISAM Transactions exceeds 1%  ......"
        report_error "- Update Statistics may need to be run and/or the indexing policy reviewed"                        
	else
       	report_message "Sequential Scan to Transaction Ratio Check Completed ... OK"
	fi

}



chk_capacity()
{

	sub_header "Checking Standard Dbspaces that are over "$CAPACITY_THRESHOLD"% Utilised"

	#Get Exempt Dbspaces if any
	EXEMPT='""'
	if [ -f $CONF ]; then
	cat $CONF | grep "EXEMPT_DBSPACE" | sed s/"EXEMPT_DBSPACE"//g > $WRK0
	for a in `cat $WRK0`
	do
		EXEMPT='"'$a'", '$EXEMPT
	done
	EXEMPT=`echo $EXEMPT | sed s/\," "\"\"//`
	report_message "EXEMPT Dbspaces "$EXEMPT
	fi

	#Get Utilisation for Standard Dbspaces

	echo 'SELECT substr(dbs.name,1,20) dbspace, TRUNC((SUM((chk.chksize*2))/1024),2) Size, TRUNC((SUM((chk.nfree*2))/1024),2) Free, (100-(ROUND(((SUM(chk.nfree*2))/(SUM(chk.chksize*2)))*100))) Per_Util FROM sysdbspaces dbs, syschunks chk WHERE dbs.dbsnum=chk.dbsnum AND dbs.is_sbspace="0" AND dbs.name NOT IN ('$EXEMPT') GROUP BY 1 ORDER BY 1 INTO TEMP server_size; UNLOAD TO '$WRK0' SELECT * FROM server_size;' | dbaccess sysmaster > /dev/null 2>&1

	check_dbspace "Standard Dbspace"

	#Get Utilisation for Smart Blob Spaces
	sub_header "Checking Smart Blobspace Usable Spaces that are over "$CAPACITY_THRESHOLD"% Utilised"

	echo 'SELECT substr(dbs.name,1,20) dbspace, TRUNC((SUM((chk.udsize*2))/1024),2) Size, TRUNC((SUM((chk.udfree*2))/1024),2) Free, (100-(ROUND(((SUM(chk.udfree*2))/(SUM(chk.udsize*2)))*100))) Per_Util FROM sysdbspaces dbs, syschunks chk WHERE dbs.dbsnum=chk.dbsnum AND dbs.is_sbspace="1" AND dbs.name NOT IN ('$EXEMPT') GROUP BY 1 ORDER BY 1 INTO TEMP server_size; UNLOAD TO '$WRK0' SELECT * FROM server_size ;' | dbaccess sysmaster > /dev/null 2>&1

	check_dbspace "Smart Blobspace Usable Space"

	#Get Utilisation for Smart Blob Spaces - MetaData Area
	sub_header "Checking Smart Blobspace Metadata Areas that are over "$CAPACITY_THRESHOLD"% Utilised"

	echo 'SELECT substr(dbs.name,1,20) dbspace, TRUNC((SUM((chk.mdsize*2))/1024),2) Size, TRUNC((SUM((chk.nfree*2))/1024),2) Free, (100-(ROUND(((SUM(chk.nfree*2))/(SUM(chk.mdsize*2)))*100))) Per_Util FROM sysdbspaces dbs, syschunks chk WHERE dbs.dbsnum=chk.dbsnum AND dbs.is_sbspace="1" AND dbs.name NOT IN ('$EXEMPT') GROUP BY 1 ORDER BY 1 INTO TEMP server_size; UNLOAD TO '$WRK0' SELECT * FROM server_size ;' | dbaccess sysmaster > /dev/null 2>&1

	check_dbspace "Smart Blobspace MetaData Area"

}



#
# Display Dbspace Table
#
check_dbspace()
{

cat <<END > $WRK2
<DIR>
<TABLE cellSpacing=2 cellPadding=6 border=3 width="80%">
<TBODY>
<TR align=middle>
<TH colSpan=4> $* Utilisation Comparisons </TH></TR>
<TR>
<TH align=middle>Dbspace Name </TH>
<TH align=middle>Total Size (Mb) </TH>
<TH align=middle>Free Space (Mb) </TH>
<TH align=middle>%Utilisation </TH>
END

	cat $WRK0 | grep -v "Database" | grep -v "row" | grep -v per_util | sed '/^$/d' | sed s/\|/" "/g> $WRK1

	EXCEPT_FLAG="N"
   	CNT0=0
   	for a in `cat $WRK1`
   	do
       	CNT0=`expr $CNT0 + 1`

       	if [ "$CNT0" = "1" ]
       	then
           	DBSPACE=$a
       	fi

       	if [ "$CNT0" = "2" ]
       	then
			TOTAL_SIZE=$a
		fi

       	if [ "$CNT0" = "3" ]
       	then
			FREE=$a
		fi

       	if [ "$CNT0" = "4" ]
       	then
			PER_UTIL=$a
			CNT0=0

			if [ "$PER_UTIL" -ge "$CAPACITY_THRESHOLD" ]; then
				report_error "- EXCEPTION_ALERT - The "$*" for "$DBSPACE " is "$PER_UTIL"% Utilised"
				COLOR="#ff0000"
				EXCEPT_FLAG="Y"
			else
				COLOR="#000000"
				echo "	The "$*" for "$DBSPACE " is "$PER_UTIL"% Utilised"
			fi
cat <<END >> $WRK2
<TR>
<TD align=middle><FONT COLOR="$COLOR">$DBSPACE</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$TOTAL_SIZE</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$FREE</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$PER_UTIL %</FONT></TD>
</TR>
END
     	fi
    done

cat $WRK2 >> $REPORT
cat <<END >> $REPORT
</TBODY>
</TABLE></DIR><BR>
END
	if [ "$EXCEPT_FLAG" = "N" ]; then
        report_message $*" Capacity Checks Completed ... OK"
	fi

}



chk_utime()
{

	sub_header "Checking for Utime Columns in Index keys ....."
	UTIME_TABLES=$METRICA_DIR/logs/checks/metrica_utime_tables.log
	clean_up

	echo 'UNLOAD TO '$UTIME_TABLES' DELIMITER ";" SELECT DISTINCT(t.tabname) FROM systables t, syscolumns c, sysxtdtypes x WHERE t.tabid=c.tabid AND c.colname IN ("day", "week_of", "month_of", "edate", "sdate", "timestamp" , "day_date" ) AND x.name="utime" AND c.extended_id=x.extended_id;' | dbaccess $METRICA_PERFORMANCE_DB > /dev/null 2>&1


	CNT_UT=`wc $UTIME_TABLES | awk '{print $1}'`
	if [ "$CNT_UT" -gt "0" ]; then
       	report_error "- EXCEPTION ALERT - Tables/Views detected with Utime Columns in IndexKeys"
		COLOR="#ff0000"

cat <<END > $WRK2
<DIR>
<TABLE cellSpacing=2 cellPadding=6 border=3 width="80%">
<TBODY>
<TH align=left>Table/View Name </TH>
END

		for a in `cat $UTIME_TABLES`
		do
			TAB=`echo $a | awk '{FS=";"; print $1}'`
			echo "	"$TAB
cat <<END >> $WRK2
<TR>
<TD align=left><FONT COLOR="$COLOR">$TAB</FONT></TD>
</TR>
END
		done
		echo ""
cat $WRK2 >> $REPORT
cat <<END >> $REPORT
</TBODY>
</TABLE></DIR><BR>
END
			
	else
     	report_message "Utime Check Completed ... OK"
	fi

}



chk_index()
{

	sub_header "Checking for Index existence on Raw, Summary and Network tables ....."
	I3_TABLES=$METRICA_DIR/logs/checks/metrica_3_index_tables.log
	I0_TABLES=$METRICA_DIR/logs/checks/metrica_0_index_tables.log
	clean_up


	 #Get Exempt Tables if any
	 EXEMPT='""'
	 if [ -f $CONF ]; then
		 cat $CONF | grep "EXEMPT_TABLE" | sed s/"EXEMPT_TABLE"//g > $WRK0
		 for a in `cat $WRK0`
		 do
			 EXEMPT='"'$a'", '$EXEMPT
		 done
		 EXEMPT=`echo $EXEMPT | sed s/\," "\"\"//`
		 report_message "EXEMPT Tables "$EXEMPT
	 fi

	echo 'UNLOAD TO '$WRK0' DELIMITER ";" SELECT t.tabname FROM systables t WHERE t.nindexes < 3 AND t.tabname like "%raw" AND t.tabname NOT IN ('$EXEMPT') AND t.tabtype="T";' | dbaccess $METRICA_PERFORMANCE_DB > /dev/null 2>&1
	cat $WRK0 > $I3_TABLES

	echo 'UNLOAD TO '$WRK0' DELIMITER ";" SELECT t.tabname FROM systables t WHERE t.nindexes < 3 AND t.tabname like "%day" AND t.tabname NOT IN ('$EXEMPT') AND t.tabtype="T";' | dbaccess $METRICA_PERFORMANCE_DB > /dev/null 2>&1
	cat $WRK0 >> $I3_TABLES

	echo 'UNLOAD TO '$WRK0' DELIMITER ";" SELECT t.tabname FROM systables t WHERE t.nindexes < 3 AND t.tabname like "%week" AND t.tabname NOT IN ('$EXEMPT') AND t.tabtype="T";' | dbaccess $METRICA_PERFORMANCE_DB > /dev/null 2>&1
	cat $WRK0 >> $I3_TABLES

	echo 'UNLOAD TO '$WRK0' DELIMITER ";" SELECT t.tabname FROM systables t WHERE t.nindexes < 3 AND t.tabname like "%month" AND t.tabname NOT IN ('$EXEMPT') AND t.tabtype="T";' | dbaccess $METRICA_PERFORMANCE_DB > /dev/null 2>&1
	cat $WRK0 >> $I3_TABLES

	echo 'UNLOAD TO '$WRK0' DELIMITER ";" SELECT t.tabname FROM systables t WHERE t.nindexes = 0 AND t.tabname like "nc_%" AND t.tabname NOT IN ('$EXEMPT') AND t.tabtype="T";' | dbaccess $METRICA_PERFORMANCE_DB > /dev/null 2>&1
	cat $WRK0 | sort > $I0_TABLES


	if [ -f $I3_TABLES ]; then
		CNT_3IT=`wc $I3_TABLES | awk '{print $1}'`
	else
		CNT_3IT=0
	fi

	if [ -f $I0_TABLES ]; then
		CNT_0IT=`wc $I0_TABLES | awk '{print $1}'`
	else
		CNT_0IT=0
	fi

	CNT_IT=`expr $CNT_3IT + $CNT_0IT`

	if [ "$CNT_IT" -gt "0" ]; then
		if [ "$CNT_3IT" -gt "0" ]; then

       		report_error "- EXCEPTION ALERT - Raw/Summary Tables detected with less than 3 Indexes"
			COLOR="#ff0000"

cat <<END > $WRK2
<DIR>
<TABLE cellSpacing=2 cellPadding=6 border=3 width="80%">
<TBODY>
<TH align=left>Table Name </TH>
END

			cat $I3_TABLES | sort > $WRK0
			cat $WRK0 > $I3_TABLES
			for a in `cat $I3_TABLES`
			do
				TAB=`echo $a | awk '{FS=";"; print $1}'`
				echo "	"$TAB
cat <<END >> $WRK2
<TR>
<TD align=left><FONT COLOR="$COLOR">$TAB</FONT></TD>
</TR>
END
			done
			echo ""
cat $WRK2 >> $REPORT
cat <<END >> $REPORT
</TBODY>
</TABLE></DIR><BR>
END
		fi

		if [ "$CNT_0IT" -gt "0" ]; then

       		report_error "- EXCEPTION ALERT - Network Tables detected with no Indexes"
			COLOR="#ff0000"

cat <<END > $WRK2
<DIR>
<TABLE cellSpacing=2 cellPadding=6 border=3 width="80%">
<TBODY>
<TH align=left>Table Name </TH>
END

			for a in `cat $I0_TABLES`
			do
				TAB=`echo $a | awk '{FS=";"; print $1}'`
				echo "	"$TAB
cat <<END >> $WRK2
<TR>
<TD align=left><FONT COLOR="$COLOR">$TAB</FONT></TD>
</TR>
END
			done
			echo ""
cat $WRK2 >> $REPORT
cat <<END >> $REPORT
</TBODY>
</TABLE></DIR><BR>
END
		fi
			
	else
     	report_message "Index Existence Check Completed ... OK"
	fi

}


 

chk_extents()
{

	sub_header "Checking Table & Index Extents ....."
	EXTENTS_LOG=$METRICA_DIR/logs/checks/metrica_check_extents.log
	echo "Database|Table/Index|Size|Extents" > $EXTENTS_LOG
	clean_up

	echo 'UNLOAD TO '$WRK0' SELECT a.dbsname, a.tabname table_name, (sum(size)*2) table_size, count(*) extent_count from sysextents a where a.dbsname IN ("'$METRICA_PERFORMANCE_DB'","'$METRICA_ADMINISTRATION_DB'","'$METRICA_SYSTEM_DB'") AND a.tabname NOT LIKE " %" AND a.tabname NOT LIKE "sys%" GROUP BY a.dbsname, a.tabname HAVING count(*) >= '$EXTENTS' ORDER BY 3 DESC;' | dbaccess sysmaster > /dev/null 2>&1


	CNT_EXT=`wc $WRK0 | awk '{print $1}'`
		if [ "$CNT_EXT" -gt "1" ]
		then
			CNT_IX_IDX_EXT=`grep -c "_ix[0-9]|" $WRK0`
			CNT_UK_IDX_EXT=`grep -c "_uk[0-9]|" $WRK0`
        		CNT_IDX_EXT=`expr $CNT_IX_IDX_EXT + $CNT_UK_IDX_EXT`
        		CNT_TAB_EXT=`expr $CNT_EXT - $CNT_IDX_EXT`
        		report_error "- EXCEPTION ALERT - Tables & Indexes Extents"
        		report_error "- "$CNT_TAB_EXT" Tables detected with at least "$I4" extents"                
        		report_error "- "$CNT_IDX_EXT" Indexes detected with at least "$I4" extents"                
        		report_error "- See details in "$EXTENTS_LOG 
		else
        		report_message "Table & Index Extents Check Completed ... OK"
		fi
		cat $WRK0 >> $EXTENTS_LOG

}


 
chk_shmem()
{

	sub_header "Checking Informix Shared Memory Allocation & Utilisation ....."

	SHMTOTAL=`onstat -c | grep SHMTOTAL | grep -v ".SHMTOTAL" | awk '{print $2}'`
	SHMTOTAL=`expr $SHMTOTAL \* 1000`
	ALLOCATED=`onstat -g seg | grep Total | awk '{print $4}'`
	USED=`onstat -g seg | grep Total | awk '{print $7}'`
	USED=`expr $USED \* 409600`

	if [ "$SHMTOTAL" -ne "0" ]
	then
		ALLOCATION=`echo $ALLOCATED"*100" | bc`
		PER_ALLOCATION=`echo $ALLOCATION/$SHMTOTAL | bc`

		PER_USED=`echo $USED/$ALLOCATED | bc`
		UNIT="%"
		report_message "SHMTOTAL is set to "`expr $SHMTOTAL / 1000`" kilobytes"
	else
		PER_ALLOCATION="0"
		PER_USED="0"
		UNIT="bytes"
		report_message "SHMTOTAL is set to 0 (unlimited)"
	fi

	if [ "$PER_ALLOCATION" -ge "95" -a "$PER_USED" -ge "95" ]
	then
      	report_error "- EXCEPTION_ALERT - Shared Memory Allocated is approaching maximum limit "$SHMTOTAL" bytes"
       	report_error "- Current Allocation is "$PER_ALLOCATION" % and Utilisation is "$PER_USED" %"
	else
		if [ "$SHMTOTAL" -eq "0" ]; then		
			PER_ALLOCATION=$ALLOCATED
			PER_USED=`echo $USED/$ALLOCATED | bc`
		fi
        	report_message "Current Allocation is "$PER_ALLOCATION $UNIT" and Utilisation is "$PER_USED" %"
        	report_message "Informix Shared Memory Check Completed ... OK"
	fi
}



clean_up()
{

	rm -f $WRK0 $WRK1 $WRK2 2>/dev/null

}
 


#
# Main Routine
#
main()
{

	build_environment
	validate
	informix_up
	header

	#Checking Online Log 
	if [[ $NUMDAYS != "0" ]]; then
		chk_online_log_numdays
	else
		chk_online_log
	fi

	#Check for Corruption
	chk_corruption

	#Check % of Cached Reads
	chk_cachedreads

	#Check for Update Stats Script in cron
	chk_updstats

	#Check for Sequential Scans Ratio 
	chk_seqscans

	#Check Server Capacity       
	chk_capacity

	#Check Table & Index extents
	chk_extents

	#Check for utime index columns
	chk_utime

	#Check for index existence on raw, summary and nc tables
	chk_index

	#Check Informix Shared Memory
	chk_shmem
 
	footer


}



#Preserve Input Parameters
I1=$1
I2=$2
I3=$3
I4=$4
I5=$5
I6=$6
case $1 in
        # Help options
        "-h" | "-help" | "--help")
                usage
                exit 0
        ;;

        # Version options
        "-v" | "-version" | "--version")
                version
                exit 0
        ;;

        # Get Extents    
        "-capacity_threshold")
                main
				exit 0
        ;;

        # Any other option
        *)
                usage
                exit 1
        ;;
esac
