#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_informix_mail.ksh SysManagement_r4.79.0.9:cset.007910:5:5 01/24/06 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_check_informix_mail.ksh
# Author:       Stuart Brooks
# Description:  Script to check mail for the informix user as part of the
#               Performance manager healthcheck scripts
#
###############################################################################

#Set Environment
PROGNAME=metrica_check_informix_mail.ksh
HOSTNAME=`uname -n`
VERNO=SysManagement_r4.79.0.9
DATE=`date +%d%b%Y.%H.%M'`
DATEXT=`date +%d-%b-%Y:%H:%M'`
WRK0=/tmp/work0$$
WRK1=/tmp/work1$$
CONF=$METRICA_DIR/SystemManagement/control/metrica_healthcheck.conf

#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}


#
# Show correct command-line usage
#
usage()
{
show_info
cat <<END

  Usage: $PROGNAME [options]

        options:  <health_check_report>
				  | [ -h | -help | --help ]
                  | [ -v | -version | --version ]

				<health_check_report>   pathname to html health check output report

                Script to check mail for the informix user as part of the
                Performance manager healthcheck scripts

END
}

#
# Show version information
#
version()
{
	show_info
	echo "$PROGNAME version @(#) @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_informix_mail.ksh SysManagement_r4.79.0.9:cset.007910:5:5 01/24/06 @(#) "
	exit 0
}

#
# main processing subroutine
#
main()
{
	export MAIL=/usr/mail/informix
	IUM_STATUS=`cat $CONF | grep metrica_check_informix_mail | grep -v .metrica_check_informix_mail | awk '{print $2}'`
	echo "Getting Informix user email statistics..."
	echo ""
	echo "<P><FONT face=Verdana SIZE=2><B><U>Informix Email Statistics</U></B></FONT></FONT>" >> $REPORT
	echo "<P></P>" >> $REPORT
	if [[ -x /usr/bin/mailx ]]; then

		# Check and print the number of messages
		msgcount=`mailx -H  |  wc -l | xargs echo`
		if [ "$msgcount" -gt "0" ]; then

cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><B><P>Informix user has $msgcount mail messages.</P></B></FONT><P> </P>
END

			echo "Informix user has $msgcount new mail messages."
			if [ "$IUM_STATUS" -eq "2" ]; then
				mailx -H > $WRK0
				cat $WRK0
				while read a
				do

cat << END >> $REPORT
<FONT FACE="Verdana SIZE=2">$a</FONT><BR>
END

				done < $WRK0
				echo "" > $WRK0
			fi
		else

cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#008000"><P><B>Informix user has no mail.</B></P></FONT> <P> </P>
END

			echo "Informix user has no mail"
			echo ""
		fi
	else
		if [[ -f /usr/mail/informix ]]; then
			msgcount=`echo "q" | mail -h | head -1 | awk '{print $1}'`

cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><B><P>Informix user has $msgcount mail messages.</P></B></FONT><P> </P>
END
			echo "Informix user has $msgcount mail messages."
			if [ "$IUM_STATUS" -eq "2" ]; then
				echo "q" | mail -h > $WRK0
				cat $WRK0
				sed 's/^	/\&nbsp;\&nbsp;\&nbsp;\&nbsp;/' $WRK0 > $WRK1
				while read a
				do

cat << END >> $REPORT
<FONT FACE="Verdana SIZE=2">$a</FONT><BR>
END

				done < $WRK1
				echo "" > $WRK0
			fi
		else

cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#008000"><P><B>Informix user has no mail.</B></P></FONT> <P> </P>
END

			echo "Informix user has no mail"
			echo ""
		fi
	fi
}

case $1 in
		# Help options
		"-h" | "-help" | "--help")
				usage
				exit 0
		;;

		# Version options
		"-v" | "-version" | "--version")
				version
				exit 0
		;;

		# Any other option
		*)
			REPORT=$1
			if [[ -f $REPORT ]]; then
				main
			else
				echo "Health Check Report not specified or invalid....exiting"
				exit 1
			fi
		;;
esac
