#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_kernel.ksh SysManagement_r4.79.0.9:cset.007910:6:6 01/24/06 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_check_kernel.ksh
# Author:       Barry Deignan
# Description:  Script to compare some of the actual Server Kernel parameters   
#               with pre-configured settings for same.                           
#
###############################################################################

#Set Environment
PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9
DATEXT=`date +%d-%b-%Y:%H:%M'`


#
# Check environment variables
#
build_environment()
{
	if [[ -z "$METRICA_DIR" ]];then
		echo "- EXCEPTION ALERT - METRICA_DIR not defined"
		echo "- Script Aborted"
		exit 1
	else
		. $METRICA_DIR/setup.ksh
		REPORT=$METRICA_DIR/logs/checks/metrica_check_kernel.html
		if [ ! -d $METRICA_DIR/logs/checks ]; then
			mkdir -p $METRICA_DIR/logs/checks
		fi

		CONF=$METRICA_DIR/SystemManagement/control/metrica_check_kernel.conf
		ARCH=`uname`
		WRK0=/tmp/work0$$
		WRK1=/tmp/work1$$
		WRK2=/tmp/work2$$
		touch $WRK0 $WRK1 $WRK2 2>/dev/null
	fi
}
 
   

#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}




#
# Show correct command-line usage
#
usage()
{
show_info
cat <<END

  Usage: $PROGNAME [options]

        options:    [ -h | -help | --help ]
                  | [ -v | -version | --version ]

  Script to compare some of the actual Server Kernel parameters   
  with pre-configured settings for same.                           

END
}
 


version()
{
	show_info
	echo "$PROGNAME version @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_kernel.ksh SysManagement_r4.79.0.9:cset.007910:6:6 01/24/06 @(#)"
	exit 0
}
 

 
header()
{

# Print Header Information to Standard Out & HTML Report
echo 'Server Kernel Parameters Monitor'      
cat <<END > $REPORT
<HTML>
<HEAD>
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=windows-1252">
<TITLE>SERVER KERNEL PARAMETERS MONITOR </TITLE>
</HEAD>
<BODY LINK="#0000ff" VLINK="#800080" BGCOLOR="#ffffff">
<H2>
<P><A NAME="kernel"><B>Server Kernel Parameters Monitor Report</B></P>
</H2>
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Date:&nbsp;&nbsp;&nbsp;</B>&nbsp;&nbsp;&nbsp;$DATEXT</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
END
	report_message ''

}
 
  

sub_header()
{

	echo ''
	echo ''
	echo $*
	echo ''
cat <<END >> $REPORT
<B><FONT FACE="Verdana" SIZE=2><P>$*</P></FONT></B>
END

}



footer()
{

# Print Footer Information to Standard Out & HTML Report
cat <<END >> $REPORT
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>*** Server Kernel Parameters Check Finished ***</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<HR align=left width="100%" noShade SIZE=1>
</BODY>
</HTML>
END
	clean_up
}



report_message()
{

	# Print Message to Standard Out & HTML Report
	echo "	"$*
	echo ''
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2><DIR><P>$*</P></DIR></FONT>
<P> </P>
END

}



report_error()
{

	# Print Error Detection Message to Standard Out & HTML Report
	echo "  "$*
	echo ""
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><DIR><B><P>$*</P></B></DIR></FONT>
<P> </P> 
END

}
  

check_conf_file()
{

	#Check for existence of Server Kernel Parameters Configuration File
	sub_header "Checking for Server Kernel Parameters Configuration file ...."

	if [ -f $CONF ];then
		CONF_FLAG="Y"
		report_message "File exists ... OK"
	else
		report_error "-	EXCEPTION ALERT - The configuration file "$CONF" does not exist" 
		report_error "-	This file is created as part of the configuration of the healthcheck scripts" 
		report_error "-	and is used to compare actual server kernel parameters with configured values" 
		report_error "-	Run "$METRICA_DIR"/SystemManagement/bin/metrica_configure_healthcheck.ksh"
		report_error "-	to configure this file"

		footer
		exit 1
	fi

}
 


compare_values()
{

	sub_header "Comparing Parameter Values ...."

# Insert Table Headers
cat <<END >> $WRK0
<DIR>
<TABLE cellSpacing=2 cellPadding=6 border=3 width="70%">
<TBODY>
<TR align=middle>
<TH colSpan=4>Kernel Parameter Comparisons </TH></TR>
<TR>
<TH align=middle>Parameter </TH>
<TH align=middle>Actual Value</TH>
<TH align=middle>Expected Value </TH>
END
	EXCEPT_FLAG="N"
	cat $CONF | grep -v ^$ | sed s/=/"	"/1 > $WRK1
	cat $WRK1 | awk '{print $1}' > $WRK2
	for a in `cat $WRK2`
	do
		case $ARCH in
			HP-UX)
				ACTUAL_VALUE=`cat /stand/system | grep $a | grep -v "."$a | awk '{print $2}'`
				;;
			SunOS)
				ACTUAL_VALUE=`cat /etc/system | grep $a | grep -v "."set | awk '{print $2}' | sed s/${a}//1 | sed s/=//1`
				;;
		esac

		CONFIG_VALUE=`cat $WRK1 | grep $a | grep -v "."$a | awk '{print $2}'`

		if [ "$ACTUAL_VALUE" != "$CONFIG_VALUE" ]; then
			report_error "- EXCEPTION ALERT - The Kernel Parameter "$a" is "$ACTUAL_VALUE" when "$CONFIG_VALUE" was expected" 
			COLOR="#ff0000"
			EXCEPT_FLAG="Y"
		else
			COLOR="#000000"
		fi

cat <<END >> $WRK0
<TR>
<TD align=middle><FONT COLOR="$COLOR">$a</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$ACTUAL_VALUE</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$CONFIG_VALUE</FONT></TD>
</TR>
END
	done
cat $WRK0 >> $REPORT
cat <<END >> $REPORT
</TBODY>
</TABLE></DIR><BR>
END

	#Print Completion message if OK
	if [ "$EXCEPT_FLAG" = "N" ];then
		report_message "All Kernel Parameters conform to configured thresholds .... OK"
	fi

}




clean_up()
{

	#Remove all temporary files
	rm -f $WRK0 $WRK1 $WRK2 2>/dev/null


}



#
# Main Routine
#
main()
{

	build_environment
	header

	#Check for existence of Configuration File
	check_conf_file

	compare_values

	footer


}


case $1 in
        # Help options
        "-h" | "-help" | "--help")
                usage
                exit 0
        ;;

        # Version options
        "-v" | "-version" | "--version")
                version
                exit 0
        ;;

        # Any other option
        *)
                main
                exit 0
        ;;
esac
