#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_loader.ksh SysManagement_r4.79.0.9:cset.008517:12:12 11/01/07 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_check_loader.ksh
# Author:       Barry Deignan
# Description:  Script to check on a per ruleset basis if the associated loader(s)
#               exceeds and/or falls below a pre-defined threshold for any of the
#               following metrics:
#                 1. The lif file loading rate.
#                 2. The number of bad files being generated.
#                 3. The number of lif files in backlog.
#
###############################################################################

#Set Environment
PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9
DATEXT=`date +%d-%b-%Y:%H:%M'`


#
# Check environment variables
#
build_environment()
{
	if [[ -z "$METRICA_DIR" ]];then
		echo "- EXCEPTION ALERT - METRICA_DIR not defined"
		echo "- Script Aborted"
		exit 1
	else
		. $METRICA_DIR/setup.ksh
		REPORT=$METRICA_DIR/logs/checks/metrica_check_loader.html
		if [ ! -d $METRICA_DIR/logs/checks ]; then
			mkdir -p $METRICA_DIR/logs/checks
		fi

		CONF=$METRICA_DIR/SystemManagement/control/metrica_check_loader.conf
		PIDFILE=$METRICA_DIR/logs/metrica_loader.pid
		WRK0=/tmp/work0$$
		WRK1=/tmp/work1$$
		TABLES=/tmp/tables$$
		ALL_RULESETS=/tmp/all_rulesets$$
		ALL_RULESET_PIDS=/tmp/all_ruleset_pids$$
		DISTINCT_RULESETS=/tmp/distinct_rulesets$$
		LOADER_PIDS=/tmp/loader_pids$$
		DIRPATH=/tmp/dirpaths$$
		touch $WRK0 $WRK1 $ALL_RULESETS $ALL_RULESET_PIDS $DISTINCT_RULESETS 2>/dev/null
		touch $LOADER_PIDS $DIRPATH 2>/dev/null
	fi
}
 
   

#
# Check that Informix is online
#
informix_up() 
{

	onstat - > /dev/null
	ISUP=`echo $?`

	if [ "$ISUP" = "5" ];then
		OK="Y"
	else
		header
		report_error "- EXCEPTION ALERT - Informix Dynamic Server appears to be down"
		footer
		exit 1
	fi

}
 



#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}




#
# Show correct command-line usage
#
usage()
{
show_info
cat <<END

  Usage: $PROGNAME [options]

        options:    [ -h | -help | --help ]
                  | [ -v | -version | --version ]

  Script to check on a per ruleset basis if the associated loader(s)
  exceeds and/or falls below a pre-defined threshold for any of the
  following metrics:
     1. The lif file loading rate.
     2. The number of bad files being generated.
     3. The number of lif files in backlog.

END
}
 


version()
{
	show_info
	echo "SysManagement/delivery/utils/bin/Monitoring/metrica_check_loader.ksh SysManagement_r4.79.0.9:cset.008517:12:12 11/01/07"
	exit 0
}
 

 
header()
{

# Print Header Information to Standard Out & HTML Report
echo 'Metrica Loader Data Management Check '
cat <<END > $REPORT
<HTML>
<HEAD>
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=windows-1252">
<TITLE>METRICA LOADER DATA MANAGEMENT CHECK </TITLE>
</HEAD>
<BODY LINK="#0000ff" VLINK="#800080" BGCOLOR="#ffffff">
<H2>
<P><A NAME="loader"><B>Metrica Loader Data Management Report </B></P>
</H2>
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Date:&nbsp;&nbsp;&nbsp;</B>&nbsp;&nbsp;&nbsp;$DATEXT</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
END
	report_message ''

}
 
  

sub_header()
{

	echo ''
	echo ''
	echo $*
	echo ''
cat <<END >> $REPORT
<B><FONT FACE="Verdana" SIZE=2><P>$*</P></FONT></B>
END

}



footer()
{

# Print Footer Information to Standard Out & HTML Report
cat <<END >> $REPORT
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>*** Metrica Loader Data Management Check Finished ***</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<HR align=left width="100%" noShade SIZE=1>
</BODY>
</HTML>
END
	clean_up
}



report_message()
{

	# Print Message to Standard Out & HTML Report
	echo "	"$*
	echo ''
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2><DIR><P>$*</P></DIR></FONT>
<P> </P>
END

}



report_error()
{

	# Print Error Detection Message to Standard Out & HTML Report
	echo "  "$*
	echo ""
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><DIR><B><P>$*</P></B></DIR></FONT>
<P> </P> 
END

}
  


#
# Check that configuration file exists
#
check_conf_file()
{

	#Check for existence of Loader/Ruleset Configuration File
	sub_header "Checking for Loader Data Management Configuration file ...."

	if [ -f $CONF ];then
		CONF_FLAG="Y"
		report_message "File exists ... OK"
	else
		report_error "-	EXCEPTION ALERT - The configuration file "$CONF" does not exist" 
		report_error "-	This file is created as part of the configuration of the healthcheck scripts" 
		report_error "-	and is used to compare actual loader performance with expected thresholds" 
		report_error "-	and should contain a line for each Tech Pack ruleset with the following"
		report_error "-	information:"
		report_error "-	#Ruleset_Id	Loaders	Lifs/Hour	Bad_Threshold	Backlog_Threshold" 
		report_error "-	E.g. "
		report_error "-	1030111021 	4	1000	50	4000" 
	fi

}
  


#
# Calculate current MBs per Day being loaded to Raw Tables
#
calc_mbday()
{

	sub_header "Calculating MBs per Day being loaded to Raw Tables ...."

	#Get list of tables and their sizes
	 echo 'UNLOAD TO '$TABLES' DELIMITER "|" SELECT  a.tabname, TRUNC(b.npdata*2, 2) FROM systables a, sysmaster:sysptnhdr b WHERE a.partnum = b.partnum AND a.tabname LIKE "%_raw" ORDER BY a.tabname;' | dbaccess $METRICA_PERFORMANCE_DB >> /dev/null 2>&1

	MB_DAY=0
	for a in `cat $TABLES`
	do
		TAB=`echo $a | awk '{FS="|";print $1}'`
		TAB_SIZE=`echo $a | awk '{FS="|";print $2}'`

		#Count Number of distinct days in the table
		echo 'UNLOAD TO '$WRK0' delimiter " " SELECT count(distinct day) FROM '$TAB';' | dbaccess $METRICA_PERFORMANCE_DB >> /dev/null 2>&1
		DAYS=`cat $WRK0 | awk '{FS=".";print $1}'`

		if [ "$TAB_SIZE" -eq 0 -o "$DAYS" -eq 0 ]; then
			TAB_MB=0
		else
			TAB_MB=`echo 'scale = 2; '$TAB_SIZE'/'$DAYS | bc`
		fi

		MB_DAY=`echo 'scale = 2; '$TAB_MB'+'$MB_DAY | bc`

	done

	if [ "$MB_DAY" -eq 0 ]; then
		MB_DAY=1
	fi

	MB_DAY=`echo 'scale = 2; '$MB_DAY'/1000' | bc`
	report_message "An average of "$MB_DAY" MBs loaded per day "

}



#
# Determine loader processes 
#
get_running(){

	#Create a list of distinct rulesets that have loaders running
	sub_header "Creating a list of distinct rulesets that have loaders running ...."

	ps -ef | grep metrica_load | grep -v grep | awk '{print $2}' > $WRK0
	if [ -f $PIDFILE ]; then
		cat $PIDFILE | sed s/"PID "/PID_/g > $WRK1
	fi

	for z in `cat $WRK0`
	do
		PID="PID_"$z
		RS=`cat $WRK1 | grep $PID | awk '{print $2}' | sed s/,//`

		if [ "$RS" != "" ]; then
			PS_OUT=`ps -p $z | grep $z`
			if [[ "$PS_OUT" != "" ]]
       			then
				echo $RS >> $ALL_RULESETS
				echo $RS" "$z >> $ALL_RULESET_PIDS
			fi
		fi
	done

	LOADERS=`wc $ALL_RULESETS | awk '{print $1}'`
	
	if [ "$LOADERS" -eq "0" ];then
        report_error "- EXCEPTION ALERT - There are no loaders running"
        footer
        exit 1
	fi

	#Ensure list is distinct
	for a in `cat $ALL_RULESETS`
	do
        EXIST=`grep -c $a $DISTINCT_RULESETS`
        if [ "$EXIST" -eq "0" ];then
        	echo $a >> $DISTINCT_RULESETS
        fi
	done

	COUNT_RULESETS=`wc $DISTINCT_RULESETS | awk '{print $1}'`
	COUNT_LOADERS=`wc $ALL_RULESETS | awk '{print $1}'`

	report_message "There are "$COUNT_LOADERS" Loader(s) running for "$COUNT_RULESETS" Ruleset(s)"
	report_message "Calculating Loader Performance per Ruleset .... "

}

 

#
# Check that all configured Rulesets have loaders running
#
check_all_running()
{

	#Check that all rulesets in configuration file have loaders running

	if [ -f $CONF ];then

		sub_header "Checking that all configured Rulesets have Loaders running ...."

		cat $CONF | grep -v "#" | grep -vi "ruleset" | awk '{print $1}' > $WRK0
		ERROR_FLAG="N"
		for a in `cat $WRK0`
		do
			EXIST=`grep -c $a $ALL_RULESETS`
			if [ "$EXIST" -eq "0" ];then
				report_error "- EXCEPTION ALERT - No loaders running for Ruleset "$a
				ERROR_FLAG="Y"
			else
				report_message "There are "$EXIST" loaders running for Ruleset "$a
			fi
		done

		if [ "$ERROR_FLAG" = "N" ]; then
			report_message "All Rulesets have loaders running .... OK"
		fi
	fi	
}



#
# Calculate number of Good and Bad files
#
get_good_bad()
{

	sub_header "Ruleset "$b" ...." 

	TOTAL_LIFS_PER_HOUR=0
	TOTAL_BAD_FILES=0
	#Get the Loader Process Ids For this Ruleset
	grep "$b " $ALL_RULESET_PIDS | awk '{print $2}' > $LOADER_PIDS

	for c in `cat $LOADER_PIDS`
	do
		# Get log file containing this process id
		# For performance, find all the files in the NGN_LOG directory modified in the last day,
		# excluding compressed files and the summariser and services logs,
		# and ignore subdirectories.

		LOADER_LOG=`find $NGN_LOG/* -type f -mtime -1 ! -name "*.Z" ! -name "*summ*" ! -name "*service*" -exec grep -l "($c)" {} \; -o -type d -prune`

		#Get number of Successful and failed load messages for metrica_load Process Id
		if [ -z $LOADER_LOG ]; then
			TOTAL_LIFS=0
			BAD_FILES=0
		else
			TOTAL_LIFS=`grep -c "($c): FILE_PROCESSING_SUCCESSFULLY" $LOADER_LOG`
			BAD_FILES=`grep -c "($c): FILE_PROCESSING_FAILED" $LOADER_LOG`
		fi

		# Get UDT time for loader start                                  
	 
		if [ "$TOTAL_LIFS" -gt "0" ];then

			if [ "$TOTAL_LIFS" -eq "1" ];then
				#Get Successful load messages for each metrica_load Process Id
				grep "($c): ISI_MSG_OPEN_FOUND_IS" $LOADER_LOG | head -1 > $WRK0

				#Get File Open Time for Start Time
    				get_utime 
    				START_TIME=$UTIME
			else	
				#Get Start Time
    				grep "($c): FILE_PROCESSING_SUCCESSFULLY" $LOADER_LOG | head -1 > $WRK0
    				get_utime 
    				START_TIME=$UTIME
			fi

			#Get End Time
    			grep "($c): FILE_PROCESSING_SUCCESSFULLY" $LOADER_LOG | tail -1 > $WRK0
    			get_utime 
    			FINISH_TIME=$UTIME

			#Get Lapsed Time & Calculate Lifs per Hour
    			ELAPSED_TIME=`expr $FINISH_TIME - $START_TIME `
			TOTAL_LIFS=`expr $TOTAL_LIFS \* 1000`

			#AVOID DIV by 0 ERRORS if ELAPSED TIME is less than 1 minute
			if [ "$ELAPSED_TIME" -lt "60" ]; then
    				LIFS_PER_SECS=`expr $TOTAL_LIFS / $ELAPSED_TIME`
    				LIFS_PER_HOUR=`expr $LIFS_PER_SECS \* 3600`
			else

    				MINUTES=`expr $ELAPSED_TIME / 60 `
    				LIFS_PER_MINS=`expr $TOTAL_LIFS / $MINUTES`
    				LIFS_PER_HOUR=`expr $LIFS_PER_MINS \* 60`
			fi
    			LIFS_PER_HOUR=`expr $LIFS_PER_HOUR / 1000 `
	
		else
			LIFS_PER_HOUR=0
		fi

		# Accumulate Totals
		TOTAL_LIFS_PER_HOUR=`expr $TOTAL_LIFS_PER_HOUR + $LIFS_PER_HOUR`
		TOTAL_BAD_FILES=`expr $TOTAL_BAD_FILES + $BAD_FILES`

		#echo "Ruleset" $b
		#echo "Process ID" $c
		#echo "Total Lifs" $TOTAL_LIFS
		#echo "lifs per Hour" $LIFS_PER_HOUR
		#echo "Bad Files" $BAD_FILES

	done

	RULESET_ID=$b
	RULESET_LOADERS=`wc $LOADER_PIDS | awk '{print $1}'`
	report_message ""
	#echo "Total Lifs per hour" $TOTAL_LIFS_PER_HOUR
	#echo "Total bad files" $TOTAL_BAD_FILES
}

 

#
# Convert times to UTC for calculations
#
get_utime()
{

	#Convert day in Loader Log File to UDT (number of seconds since 1970)
	CNT0=0
	for y in `cat $WRK0`
	do
        CNT0=`expr $CNT0 + 1 `
        if [ "$CNT0" -eq "2" ];then
                MONTH=$y
        fi

        if [ "$CNT0" -eq "3" ];then
            DAY=$y
            BITS=`echo $DAY | wc | awk '{print $3}'`
            if [ "$BITS" -eq "2" ];then
                #PAD with a leading zero
                DAY="0"$DAY
            fi
        fi

        if [ "$CNT0" -eq "4" ];then
            TIME=$y
            HOUR=`echo $TIME | awk '{FS=":";print $1}'`
            BITS=`echo $HOUR | wc | awk '{print $3}'`
            if [ "$BITS" -eq "2" ];then
                #PAD with a leading zero
                HOUR="0"$HOUR
            fi
            MINS=`echo $TIME | awk '{FS=":";print $2}'`
            SECS=`echo $TIME | awk '{FS=":";print $3}'`
        fi

        if [ "$CNT0" -eq "5" ];then
            YEAR=$y
        fi
	done

	if [ -f $METRICA_DIR/SystemManagement/bin/metrica_date_converter.pl ];then
		UTIME=`$METRICA_DIR/SystemManagement/bin/metrica_date_converter.pl -time $DAY$MONTH$YEAR$HOUR":"$MINS":"$SECS | awk '{print $5}'`
	else
		report_error "- EXCEPTION ALERT - Cannot Proceed - Script Aborted"
		report_error "- The metrica_date_converter.pl script does not exist in"
		report_error "- "$METRICA_DIR"/SystemManagement/bin"
		footer
		exit
	fi
}



#
# Determine the spool directory for a Ruleset
#
get_dir_path()
{

	# Get the path where Lifs to load reside
	echo 'UNLOAD TO '$DIRPATH' SELECT dir_path FROM lc_ruleset WHERE ruleset_id="'$b'";' | dbaccess $METRICA_ADMINISTRATION_DB > /dev/null 2>&1

	LIF_DIRECTORY=`cat $DIRPATH | sed s/\|//1`

	#Check that Lif Directory is either a directory or a link
	if [ -d $LIF_DIRECTORY -o -h $LIF_DIRECTORY ];then
		report_message "Spool Directory is: "$LIF_DIRECTORY
		ACTUAL_BACKLOG=`ls $LIF_DIRECTORY | grep -v INUSE | grep -v good | grep -v bad | wc | awk '{print $1}'`
	else
		report_error "- EXCEPTION ALERT - Spool Directory "$LIF_DIRECTORY "does not exist "
		ACTUAL_BACKLOG=0
	fi

}



#
# Compare Calculations with configured thresholds
#
compare_thresholds()
{


	LOADER_COLOR="#000000"
	LIF_COLOR="#000000"
	BAD_COLOR="#000000"
	BACKLOG_COLOR="#000000"
	EXCEPT_FLAG="N"
	if [ "$CONF_FLAG" = "Y" ];then

	#Check that Ruleset is in configuration file
		RULESET_IN=`cat $CONF | grep $RULESET_ID | grep -v "."$RULESET_ID | wc | awk '{print $1}'`
		if [ "$RULESET_IN" -eq "0" ];then
			report_error "-	EXCEPTION ALERT - There is no entry in "$CONF" for Ruleset Id "$RULESET_ID
			EXCEPT_FLAG="Y"
			MISCONFIG_FLAG="Y"
		elif [ "$RULESET_IN" -gt "1" ];then
			report_error "-	EXCEPTION ALERT - There are multiple entries in "$CONF" for Ruleset Id "$RULESET_ID
			EXCEPT_FLAG="Y"
			MISCONFIG_FLAG="Y"
		else 
			CONF_RI_ERROR="N"
			MISCONFIG_FLAG="N"


			NUM_LOADERS=`cat $CONF | grep $RULESET_ID | grep -v "."$RULESET_ID | awk '{print $2}'`
			check_conf_element $NUM_LOADERS $RULESET_LOADERS
			NUM_LOADERS=$CONF_VAR

			LIF_LOAD_THRESHOLD=`cat $CONF | grep $RULESET_ID | grep -v "."$RULESET_ID | awk '{print $3}'`
			check_conf_element $LIF_LOAD_THRESHOLD $TOTAL_LIFS_PER_HOUR
			LIF_LOAD_THRESHOLD=$CONF_VAR

			BAD_THRESHOLD=`cat $CONF | grep $RULESET_ID | grep -v "."$RULESET_ID | awk '{print $4}'`
			check_conf_element $BAD_THRESHOLD $TOTAL_BAD_FILES
			BAD_THRESHOLD=$CONF_VAR

			BACKLOG_THRESHOLD=`cat $CONF | grep $RULESET_ID | grep -v "."$RULESET_ID | awk '{print $5}'`
			check_conf_element $BACKLOG_THRESHOLD $ACTUAL_BACKLOG
			BACKLOG_THRESHOLD=$CONF_VAR

			if [ "$CONF_RI_ERROR" = "Y" ];then
				report_error "- EXCEPTION ALERT - There seems to be a problem with the entry in "$CONF
				report_error "- for Ruleset Id "$RULESET_ID". It should have the following format:"
				report_error "- Ruleset_Id  #Loaders    Lifs/Hour   Bad_Threshold   Backlog_Threshold"
				report_error "- E.g. "
				report_error "- "$RULESET_ID"   4   1000    50  4000"
			fi

			if [ "$RULESET_LOADERS" -ne "$NUM_LOADERS" ];then
				report_error "- EXCEPTION ALERT - There are "$RULESET_LOADERS" loaders running when "$NUM_LOADERS" were expected"
				LOADER_COLOR="#ff0000"
				EXCEPT_FLAG="Y"
			fi
		
			PERCENT_LIF=`expr $TOTAL_LIFS_PER_HOUR \* 100`
			PERCENT_LIF=`expr $PERCENT_LIF / $LIF_LOAD_THRESHOLD `
			if [ "$PERCENT_LIF" -lt "70" ];then
				report_error "- EXCEPTION ALERT - The LIF file loading rate is "$TOTAL_LIFS_PER_HOUR" per hour when "$LIF_LOAD_THRESHOLD" is expected"
				LIF_COLOR="#ff0000"
				EXCEPT_FLAG="Y"
			fi

			if [ "$TOTAL_BAD_FILES" -gt "$BAD_THRESHOLD" ];then
				report_error "- EXCEPTION ALERT - There are "$TOTAL_BAD_FILES" Lif files rejected when no more than "$BAD_THRESHOLD" were expected" 
				BAD_COLOR="#ff0000"
				EXCEPT_FLAG="Y"
			fi

			if [ "$ACTUAL_BACKLOG" -gt "$BACKLOG_THRESHOLD" ];then
				report_error "- EXCEPTION ALERT - There is a backlog of "$ACTUAL_BACKLOG" Lif files in "$LIF_DIRECTORY" when no more than "$BACKLOG_THRESHOLD" were expected" 
				BACKLOG_COLOR="#ff0000"
				EXCEPT_FLAG="Y"
			fi
		fi
	fi

	#Reset Thresholds for Display purposes
	if [ "$EXCEPT_FLAG" = "Y" -a "$MISCONFIG_FLAG" = "Y" ];then
		NUM_LOADERS=" "
		LIF_LOAD_THRESHOLD=" "
		BAD_THRESHOLD=" "
		BACKLOG_THRESHOLD=" "
	fi
	
	echo "	Number of Loaders\t" "=\t" $RULESET_LOADERS
	echo "	Lifs Loading Per Hour\t" "=\t" $TOTAL_LIFS_PER_HOUR
	echo "	Number of Bad Files\t" "=\t" $TOTAL_BAD_FILES 
	echo "	Current Backlog\t" "\t" "=\t" $ACTUAL_BACKLOG 
cat <<END >> $REPORT
<DIR>
<TABLE cellSpacing=2 cellPadding=6 border=3 width="70%">
<TBODY>
<TR align=middle>
<TH colSpan=4>Ruleset $RULESET_ID Loader Statistics </TH></TR>
<TR>
<TH align=middle> </TH>
<TH align=middle>Loaders </TH>
<TH align=middle>Avg. Lifs per Hour </TH>
<TH align=middle>Number of Bad Files </TH>
<TH align=middle>Current Backlog </TH></TR>
<TR>
<TD align=middle><B>Actual</B> </TD>
<TD align=middle><FONT COLOR="$LOADER_COLOR">$RULESET_LOADERS </FONT></TD>
<TD align=middle><FONT COLOR="$LIF_COLOR">$TOTAL_LIFS_PER_HOUR</FONT></TD>
<TD align=middle><FONT COLOR="$BAD_COLOR">$TOTAL_BAD_FILES</FONT></TD>
<TD align=middle><FONT COLOR="$BACKLOG_COLOR">$ACTUAL_BACKLOG</FONT></TD>
</TR>
<TR>
<TD align=middle><B>Thresholds</B> </TD>
<TD align=middle><FONT COLOR="$LOADER_COLOR">$NUM_LOADERS </FONT></TD>
<TD align=middle><FONT COLOR="$LIF_COLOR">$LIF_LOAD_THRESHOLD</FONT></TD>
<TD align=middle><FONT COLOR="$BAD_COLOR">$BAD_THRESHOLD</FONT></TD>
<TD align=middle><FONT COLOR="$BACKLOG_COLOR">$BACKLOG_THRESHOLD</FONT></TD>
</TR>
</TBODY>
</TABLE></DIR><BR>
END

	#Print Completion message if OK
	if [ "$EXCEPT_FLAG" = "N" -a "$CONF_FLAG" = "Y" -a "$CONF_RI_ERROR" = "N" ];then
		report_message "Ruleset "$RULESET_ID" conforms to configured thresholds .... OK"
	fi

}



#
# Validate the Configuration file thresholds
#
check_conf_element()
{

	ALPHA="N"
	[[ $1 != +([0-9]) ]] && ALPHA="Y"
	if [ "$1" = "" -o "$ALPHA" = "Y" ];then
		CONF_RI_ERROR="Y"
		MISCONFIG_FLAG="Y"
		EXCEPT_FLAG="Y"
		CONF_VAR=$2
	else
		CONF_VAR=$1
	fi

}



#
# Remove temporary files
#
clean_up()
{

	#Remove all temporary files
	rm -f $WRK0 $WRK1 $TABLES $ALL_RULESETS $ALL_RULESET_PIDS $DISTINCT_RULESETS 2>/dev/null
	rm -f $LOADER_PIDS $DIRPATH 2>/dev/null

}



#
# Main Routine
#
main()
{

	build_environment
	informix_up
	header

	#Check for existence of Configuration File
	check_conf_file

	#Calculate current MBs per day being loaded to raw tables
	# Removed as might be a bottleneck on some deployments
	#calc_mbday

	#Determine Ruleset & loaders running on system
	get_running

	#Check that all configured loaders are running
	check_all_running

	for b in `cat $DISTINCT_RULESETS`
	do
        get_good_bad
        get_dir_path
		compare_thresholds
	done

	footer


}


case $1 in
        # Help options
        "-h" | "-help" | "--help")
                usage
                exit 0
        ;;

        # Version options
        "-v" | "-version" | "--version")
                version
                exit 0
        ;;

        # Any other option
        *)
                main
                exit 0
        ;;
esac
