#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_onconfig.ksh SysManagement_r4.79.0.9:cset.007910:7:7 01/24/06 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_check_onconfig.ksh
# Author:       Barry Deignan
# Description:  Script to compare some of the actual Informix Onconfig parameters   
#               with pre-configured settings for same.                           
#
###############################################################################

#Set Environment
PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9
DATEXT=`date +%d-%b-%Y:%H:%M'`


#
# Check environment variables
#
build_environment()
{
	if [[ -z "$METRICA_DIR" ]];then
		echo "- EXCEPTION ALERT - METRICA_DIR not defined"
		echo "- Script Aborted"
		exit 1
	else
		. $METRICA_DIR/setup.ksh
		REPORT=$METRICA_DIR/logs/checks/metrica_check_onconfig.html
		if [ ! -d $METRICA_DIR/logs/checks ]; then
			mkdir -p $METRICA_DIR/logs/checks
		fi
		CONF=$METRICA_DIR/SystemManagement/control/metrica_check_onconfig.conf
		WRK0=/tmp/work0$$
		WRK1=/tmp/work1$$
		ALL_RULESETS=/tmp/all_rulesets$$
		DISTINCT_RULESETS=/tmp/distinct_rulesets$$
		GOOD_LOG=/tmp/good_log$$
		BAD_LOG=/tmp/bad_log$$
		LOADER_LOGS=/tmp/loader_logs$$
		LOADER_PIDS=/tmp/loader_pids$$
		DIRPATH=/tmp/dirpaths$$
		if [ ! -f $WRK0 ];then
			touch $WRK0
		fi
		if [ ! -f $WRK1 ];then
			touch $WRK1
		fi
	fi
}
 
   

informix_up() 
{

	onstat - > /dev/null
	ISUP=`echo $?`

	if [ "$ISUP" = "5" ];then
		OK="Y"
	else
		header
		report_error "- EXCEPTION ALERT - Informix Dynamic Server appears to be down"
		footer
		exit 1
	fi

}
 



#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}




#
# Show correct command-line usage
#
usage()
{
show_info
cat <<END

  Usage: $PROGNAME [options]

        options:    [ -h | -help | --help ]
                  | [ -v | -version | --version ]

  Script to compare some of the actual Informix Onconfig parameters   
  with pre-configured settings for same.                           

END
}
 


version()
{
	show_info
	echo "$PROGNAME version @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_onconfig.ksh SysManagement_r4.79.0.9:cset.007910:7:7 01/24/06 @(#)"
	exit 0
}
 

 
header()
{

# Print Header Information to Standard Out & HTML Report
echo 'Informix Configuration Parameters Monitor '
cat <<END > $REPORT
<HTML>
<HEAD>
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=windows-1252">
<TITLE>INFORMIX CONFIGURATION PARAMETERS MONITOR </TITLE>
</HEAD>
<BODY LINK="#0000ff" VLINK="#800080" BGCOLOR="#ffffff">
<H2>
<P><A NAME="onconfig"><B>Informix Onconfig Configuration Parameters Monitor Report </B></P>
</H2>
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Date:&nbsp;&nbsp;&nbsp;</B>&nbsp;&nbsp;&nbsp;$DATEXT</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
END
	report_message ''

}
 
  

sub_header()
{

	echo ''
	echo ''
	echo $*
	echo ''
cat <<END >> $REPORT
<B><FONT FACE="Verdana" SIZE=2><P>$*</P></FONT></B>
END

}



footer()
{

# Print Footer Information to Standard Out & HTML Report
cat <<END >> $REPORT
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>*** Informix Configuration Parameters Check Finished ***</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<HR align=left width="100%" noShade SIZE=1>
</BODY>
</HTML>
END
	clean_up
}



report_message()
{

	# Print Message to Standard Out & HTML Report
	echo "	"$*
	echo ''
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2><DIR><P>$*</P></DIR></FONT>
<P> </P>
END

}



report_error()
{

	# Print Error Detection Message to Standard Out & HTML Report
	echo "  "$*
	echo ""
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><DIR><B><P>$*</P></B></DIR></FONT>
<P> </P> 
END

}
  

check_conf_file()
{

	#Check for existence of Informix Configuration Parameters Configuration File
	sub_header "Checking for Informix Configuration Parameters Configuration file ...."

	if [ -f $CONF ];then
		CONF_FLAG="Y"
		report_message "File exists ... OK"
	else
		report_error "-	EXCEPTION ALERT - The configuration file "$CONF" does not exist" 
		report_error "-	This file is created as part of the configuration of the healthcheck scripts" 
		report_error "-	and is used to compare actual onconfig parameters with configured values" 
		report_error "-	Run "$METRICA_DIR"/SystemManagement/bin/metrica_configure_healthcheck.ksh"
		report_error "-	to configure this file"

		footer
		exit 1
	fi

}



check_dumpdir()
{

	sub_header "Checking DUMPDIR ...."
	ACTUAL_VALUE=`onstat -c | grep DUMPDIR | grep -v .DUMPDIR | awk '{print $2}'`

	if [ "$ACTUAL_VALUE" = "/tmp" ]; then
		TEMP="Y"
		EXCEPT="Y"
	fi
		
	if [ ! -d "$ACTUAL_VALUE" ]; then
		NOTEXIST="Y"
		EXCEPT="Y"

	else

		#Determine if there is enough space in DUMPDIR to accomodate a dump
		SHMEM_BYTES=`onstat -g seg | grep Total | awk '{print $4}'`
		SHMEM_KBS=`echo $SHMEM_BYTES"/1000" | bc`
		ARCH=`uname`
		case $ARCH in
			HP-UX)
				AVAIL_KBS=`df -k $ACTUAL_VALUE | grep -i free | awk '{print $1}'`
			;;
			SunOS)
				AVAIL_KBS=`df -b $ACTUAL_VALUE | grep -v Filesystem | awk '{print $2}'`
			;;
		esac


		if [ "$SHMEM_KBS" -gt "$AVAIL_KBS" ]; then
			NOSPACE="Y"
			EXCEPT="Y"
		fi
	fi

	if [ "$EXCEPT" = "Y" ]; then
cat << END > $WRK0
DUMPDIR specifies a directory in which the database server dumps shared
memory, gcore files, or messages from a failed assertion. Because shared
memory can be large, DUMPDIR should be set to a file system with a 
significant amount of space available.
END
		report_message `cat $WRK0`
		clean_up

		if [ "$TEMP" = "Y" ]; then
			report_error "- EXCEPTION ALERT - The DUMPDIR is set to /tmp"  
		fi
		
		if [ "$NOTEXIST" = "Y" ]; then
			report_error "- EXCEPTION ALERT - The DUMPDIR directory "$ACTUAL_VALUE" does not exist"
		fi
		
		if [ "$NOSPACE" = "Y" ]; then
			report_error "- EXCEPTION ALERT - The DUMPDIR directory "$ACTUAL_VALUE" does not have sufficient space available"
		fi
		
	else
		report_message "DUMPDIR configured  .... OK"
	fi
 
}


compare_values()
{

	sub_header "Comparing Parameter Values ...."

#Insert Table Headers
cat <<END >> $WRK0
<DIR>
<TABLE cellSpacing=2 cellPadding=6 border=3 width="70%">
<TBODY>
<TR align=middle>
<TH colSpan=4>Parameter Comparisons </TH></TR>
<TR>
<TH align=middle>Parameter </TH>
<TH align=middle>Actual Value</TH>
<TH align=middle>Expected Value </TH>
END
	EXCEPT_FLAG="N"
	cat $CONF | grep -v ^$ | grep -v VPCLASS | awk '{print $1}' > $WRK1
	for a in `cat $WRK1`
	do
		
		ACTUAL_VALUE=`onstat -c | grep $a | grep -v "."$a | awk '{print $2}'`
		CONFIG_VALUE=`cat $CONF | grep $a | grep -v "."$a | awk '{print $2}'`

		if [ "$ACTUAL_VALUE" != "$CONFIG_VALUE" ]; then
			report_error "- EXCEPTION ALERT - The Onconfig Parameter "$a" is "$ACTUAL_VALUE" when "$CONFIG_VALUE" was expected" 
			COLOR="#ff0000"
			EXCEPT_FLAG="Y"
		else
			COLOR="#000000"
		fi

cat <<END >> $WRK0
<TR>
<TD align=middle><FONT COLOR="$COLOR">$a</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$ACTUAL_VALUE</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$CONFIG_VALUE</FONT></TD>
</TR>
END
	done

	cat $CONF | grep -v ^$ | grep VPCLASS > $WRK1
	exec < $WRK1
	while read line
        do
                CONFIG_VALUE=`echo $line | awk '{print $2}'`
		CLASS=`echo $CONFIG_VALUE | awk '{FS=",";print $1}'`
                ACTUAL_VALUE=`onstat -c | grep VPCLASS | grep -v "."VPCLASS | grep $CLASS | awk '{print $2}'`

                if [ "$ACTUAL_VALUE" != "$CONFIG_VALUE" ]; then
                        report_error "- EXCEPTION ALERT - The Onconfig Parameter VPCLASS is "$ACTUAL_VALUE" when "$CONFIG_VALUE"
was expected"
                        COLOR="#ff0000"
                        EXCEPT_FLAG="Y"
                else
                        COLOR="#000000"
                fi

cat <<END >> $WRK0
<TR>
<TD align=middle><FONT COLOR="$COLOR">VPCLASS</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$ACTUAL_VALUE</FONT></TD>
<TD align=middle><FONT COLOR="$COLOR">$CONFIG_VALUE</FONT></TD>
</TR>
END
        done

cat $WRK0 >> $REPORT
cat <<END >> $REPORT
</TBODY>
</TABLE></DIR><BR>
END

	#Print Completion message if OK
	if [ "$EXCEPT_FLAG" = "N" ];then
		report_message "All Parameters conform to configured thresholds .... OK"
	fi

}




clean_up()
{

	#Remove all temporary files
	if [ -f $WRK0 ];then
		rm $WRK0
	fi
	if [ -f $WRK1 ];then
		rm $WRK1
	fi

}



#
# Main Routine
#
main()
{

	build_environment
	informix_up
	header

	#Check for existence of Configuration File
	check_conf_file

	check_dumpdir

	compare_values

	footer


}


case $1 in
        # Help options
        "-h" | "-help" | "--help")
                usage
                exit 0
        ;;

        # Version options
        "-v" | "-version" | "--version")
                version
                exit 0
        ;;

        # Any other option
        *)
                main
                exit 0
        ;;
esac
