#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_summariser.ksh SysManagement_r4.79.0.9:cset.008208:10:10 10/13/06 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_check_summariser.ksh
# Author:       Barry Deignan
# Description:  Script to determines the status of the summariser.                
#				It then:
#				 1. Calculates the total duration of successful summaries for the current period. 
#				 2. Calculates the number successful summaries.
#				 3. Calculates the number of failed summaries.
#				 4. Calculates the number of summaries currently queued.
#				If a configuration file (metrica_check_summariser.conf) exists the script will
#				 1. Compare the number of failed summaries with a configured threshold.
#				 2. Compare the number of queued summaries with a configured threshold.
#
###############################################################################

#Set Environment
PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9
DATE=`date +%d%b%Y.%H.%M'`
DATEXT=`date +%d-%b-%Y:%H:%M`
DMY=`date +%d%b%Y`
HMS=`date +%H:%M:%S`
CONVDAT=$DMY$HMS


#
# Check environment variables
#
build_environment()
{

	if [[ -z "$METRICA_DIR" ]];then
		echo "- EXCEPTION ALERT - METRICA_DIR not defined"
		echo "- Script Aborted"
		exit 1
	else
		. $METRICA_DIR/setup.ksh
		REPORT=$METRICA_DIR/logs/checks/metrica_check_summariser.html
		if [ ! -d $METRICA_DIR/logs/checks ]; then
			mkdir -p $METRICA_DIR/logs/checks
		fi

		CONF=$METRICA_DIR/SystemManagement/control/metrica_check_summariser.conf
		WRK0=/tmp/work0$$
		WRK1=/tmp/work1$$
		WRK2=/tmp/work2$$
		FAILED_SUMMARIES=$METRICA_DIR/logs/checks/failed_summaries.$DATE.log
		ALL_REASONS=/tmp/all_reasons$$
		DISTINCT_REASONS=/tmp/distinct_reasons$$
		touch $WRK0 $WRK1 $WRK2 $ALL_REASONS $DISTINCT_REASONS 2>/dev/null
		APPSERVER_AUTH_FILE=$APPSERVER_DIR/.bes_auth
		APPSERVER_STD_ARGS="-mgmtport $APPSERVER_MGMT_PORT -server $APPSERVER_NAME"
	fi
}
 
   

#
# Check that Informix is online
#
informix_up() 
{

	onstat - > /dev/null
	ISUP=`echo $?`

	if [ "$ISUP" = "5" ];then
		OK="Y"
	else
		header
		report_error "- EXCEPTION ALERT - Informix Dynamic Server appears to be down"
		footer
		exit 1
	fi

}
 



#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}




#
# Show correct command-line usage
#
usage()
{
show_info
cat <<END

  Usage: $PROGNAME [options]

        options:    [ -h | -help | --help ]
                  | [ -v | -version | --version ]

  Script to determines the status of the summariser.                
	It then:
	1. Calculates the total duration of successful summaries for the current period. 
	2. Calculates the number successful summaries.
	3. Calculates the number of failed summaries.
	4. Calculates the number of summaries currently queued.
  If a configuration file (metrica_check_summariser.conf) exists the script will
	1. Compare the number of failed summaries with a configured threshold.
	2. Compare the number of queued summaries with a configured threshold.

END
}
 


version()
{
	show_info
	echo "$PROGNAME version @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_check_summariser.ksh SysManagement_r4.79.0.9:cset.008208:10:10 10/13/06 @(#)"
	exit 0
}
 

 
header()
{

# Print Header Information to Standard Out & HTML Report
echo 'Metrica Summariser Execution Check '
cat <<END > $REPORT
<HTML>
<HEAD>
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=windows-1252">
<TITLE>METRICA SUMMARISER HEALTH CHECK </TITLE>
</HEAD>
<BODY LINK="#0000ff" VLINK="#800080" BGCOLOR="#ffffff">
<H2>
<P><A NAME="summary"><B>Metrica Summariser Health Check Report </B></P>
</H2>
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Date:&nbsp;&nbsp;&nbsp;</B>&nbsp;&nbsp;&nbsp;$DATEXT</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
END
	report_message ''

}
 
  

sub_header()
{

	echo ''
	echo ''
	echo $*
	echo ''
cat <<END >> $REPORT
<B><FONT FACE="Verdana" SIZE=2><P>$*</P></FONT></B>
END

}



footer()
{

# Print Footer Information to Standard Out & HTML Report
cat <<END >> $REPORT
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>*** Metrica Summariser Health Check Finished ***</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<HR align=left width="100%" noShade SIZE=1>
</BODY>
</HTML>
END
	clean_up
}



report_message()
{

	# Print Message to Standard Out & HTML Report
	echo "	"$*
	echo ''
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2><DIR><P>$*</P></DIR></FONT>
<P> </P>
END

}



report_error()
{

	# Print Error Detection Message to Standard Out & HTML Report
	echo "  "$*
	echo ""
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><DIR><B><P>$*</P></B></DIR></FONT>
<P> </P> 
END

}
  


#
# Check that configuration file exists
#
check_conf_file()
{

	#Check for existence of Summariser Configuration File
	sub_header "Checking for Summariser Health Check Configuration file ...."

	FAILED_THRESHOLD=0
	QUEUED_THRESHOLD=0
	if [ -f $CONF ];then
		CONF_FLAG="Y"
		report_message "File exists ... OK"

		FAILED_THRESHOLD=`cat $CONF | grep FAILED | grep -v ".FAILED" | awk '{print $2}'`
		check_conf_element $FAILED_THRESHOLD
		FAILED_THRESHOLD=$CONF_VAR

		QUEUED_THRESHOLD=`cat $CONF | grep QUEUED | grep -v ".QUEUED" | awk '{print $2}'`
		check_conf_element $QUEUED_THRESHOLD
		QUEUED_THRESHOLD=$CONF_VAR

	else
		report_error "-	EXCEPTION ALERT - The configuration file "$CONF" does not exist" 
		report_error "-	This file is created as part of the configuration of the healthcheck scripts" 
		report_error "-	and is used to compare actual Summariser performance with expected thresholds." 
		report_error "-	Run the metrica_configure_healthcheck.ksh script to configure"         
	fi

}
 


#
# Determine Status of Summariser
#
get_status(){

	sub_header "Determining Status of Summariser  ...."

	APP_STATUS=`iastool -ping $APPSERVER_STD_ARGS < $APPSERVER_AUTH_FILE | tail -1 | grep -c "Running"`
	EJB_STATUS=`iastool -ping $APPSERVER_STD_ARGS -partition $EJBPARTITION_NAME < $APPSERVER_AUTH_FILE | tail -1 | grep -c "Running"`
	WEB_STATUS=`iastool -ping $APPSERVER_STD_ARGS -partition $WEBPARTITION_NAME < $APPSERVER_AUTH_FILE | tail -1 | grep -c "Running"`
	if [ "$APP_STATUS" -eq "1" -a "$EJB_STATUS" -eq "1" -a "$WEB_STATUS" -eq "1" ] ; then

		metrica_summariseradmin.ksh -status > $WRK0
		CNT0=`grep -ic "summariser Running" $WRK0`

		if [ "$CNT0" -eq "1" ]; then
			report_message "The Summariser is running ... OK"
		else
       		report_error "- EXCEPTION ALERT - The Summariser is stopped"
       		footer
       		exit 1
		fi
	else
		report_error "-	EXCEPTION ALERT - The AppServer and Summariser are stopped cannot proceed"
		footer
		exit 2
	fi

}

 

#
# Get the value of Concurrent Tasks and Compare with Number of CPUs
#
get_tasks()
{

#	echo "Determining Number of CPUs  ..."
	ARCH=`uname`
	case $ARCH in
		HP-UX)
			CPUS=`/etc/ioscan -k -C processor | grep processor | wc -l`
		;;
		SunOS)
			CPUS=`/usr/bin/mpstat | grep -v CPU | wc | awk '{print $1}'`
		;;
	esac

#	echo "Determining Number of Concurrent Tasks  ..."
	echo 'UNLOAD TO '$WRK0' DELIMITER ";" SELECT value FROM propertyservice_property WHERE property_name = "ConcurrentTasks";' | dbaccess $METRICA_ADMINISTRATION_DB > /dev/null 2>&1          

	CC_TASKS=`cat $WRK0 | awk '{FS=";";print $1}'`
	sub_header "Checking the value of Concurrent Tasks against available CPUs ...."

	report_message "Concurrent Tasks: "$CC_TASKS 
	report_message "Quantity of CPUs: "$CPUS 

	if [ "$CC_TASKS" -gt "$CPUS" ]; then
		report_error "- EXCEPTION ALERT - Concurrent Tasks is greater than the number of CPUs " 
	else
		report_message "Concurrest Tasks  ... OK"                                                         
	fi

}
		
 

#
# Get the Period Status for all Summaries
#
get_period_status()
{

	#Get Yesterdays date in UTC

	get_utime $CONVDAT

	sub_header "Determining status for all Summaries for the Preceding Period "$DAY $MON $DAT $YEAR" ...."

	metrica_summ.ksh -periodstatus -time $YESTERDAY_UTIME -unformatted -noheader | grep -v "QUEUED" > $WRK0
	metrica_summ.ksh -periodstatus -unformatted | grep "QUEUED" > $WRK2

	#Initialise Counters
	DURATION=0
	SUMM_DURATION=0
	QTY_SUCCESS=0
	QTY_EXECUTING=0
	QTY_QUEUED=0
	QTY_FAILED=0
	HOURS=0
	MINUTES=0
	SECONDS=0
	HOURS_IN_SECS=0
	MINUTES_IN_SECS=0
	LEFTOVER=0
	FIRST="Y"
	EARLIEST=9999999999
	LATEST=0

	#Calculate Successes, Executing, Queued, Failures

	QTY_QUEUED=`wc $WRK2 | awk '{print $1}'`

	cat $WRK0 | sort > $WRK1
	exec < $WRK1
	while read LINE
	do
		SUMMARY=`echo $LINE | awk '{FS="|";print $1}'`
		STATUS=`echo $LINE | awk '{FS="|";print $2}'`

		if [ "$STATUS" = "SUCCESS" ]; then
			if [ "$FIRST" = "Y" ]; then
				FIRST="N"

				printf "%-50s\t%-15s\t%-15s\n" "SUMMARY TABLE" "START TIME" "DURATION (Secs)"
				UL1="================================================="
				UL2="=============="
				printf "%-50s\t%-15s\t%-15s\n" $UL1 $UL2 $UL2 
cat <<END >> $REPORT
<DIR>
<TABLE cellSpacing=2 cellPadding=6 border=3 width="80%">
<TBODY>
<TR align=middle>
<TH colSpan=4>Successfull Summaries By Duration </TH></TR>
<TR>
<TH align=left width="50%">Summary Table </TH>
<TH align=left width="25%">Start Time </TH>
<TH align=middle width="25%">Duration (Secs)</TH></TR>
END
			fi

			SUMM_DURATION=`echo $LINE | awk '{FS="|";print $6}'`
			START=`echo $LINE | awk '{FS="|";print $5}'`
			START_DATE=`echo $START | awk '{print $1}' | sed s/-//g`
			START_TIME=`echo $START | awk '{print $2}'`

			get_utime $START_DATE$START_TIME 

			if [ "$EARLIEST" -gt "$YESTERDAY_UTIME" ];then
				EARLIEST=$YESTERDAY_UTIME
			fi

			DURATION_UTIME=`expr $YESTERDAY_UTIME + $SUMM_DURATION `

			if [ "$LATEST" -lt "$DURATION_UTIME" ];then
				LATEST=$DURATION_UTIME
			fi

			#report_message $SUMMARY "Start Time = "$START_TIME "Duration = "$SUMM_DURATION
			printf "%-50s\t%-15s\t%-15s\n" $SUMMARY $START_TIME $SUMM_DURATION 
cat <<END >> $REPORT
<TR>
<TD align=left width="50%">$SUMMARY</FONT></TD>
<TD align=left width="25%">$START_TIME</FONT></TD>
<TD align=left width="25%">$SUMM_DURATION</FONT></TD>
</TR>
END
			DURATION=`expr $DURATION + $SUMM_DURATION `
			QTY_SUCCESS=`expr $QTY_SUCCESS + 1 `
		fi

		if [ "$STATUS" = "EXECUTING" ]; then
			QTY_EXECUTING=`expr $QTY_EXECUTING + 1 `
		fi

		#if [ "$STATUS" = "QUEUED" ]; then
			#QTY_QUEUED=`expr $QTY_QUEUED + 1 `
		#fi

		if [ "$STATUS" = "FAILED" ]; then
			QTY_FAILED=`expr $QTY_FAILED + 1 `
			REASON=`echo $LINE | awk '{FS="|";print $7}' | sed s/" "/_/g`
			INFORMATION=`echo $LINE | awk '{FS="|";print $8}' | sed s/" "/_/g`
			if [ "$INFORMATION" = "" ]; then
				INFORMATION="No_Information"
			fi
			echo $LINE >> $FAILED_SUMMARIES
			echo $REASON"|"$INFORMATION"	" >> $ALL_REASONS
		fi
	done

	if [ "$FIRST" = "N" ];then
cat <<END >> $REPORT
</TBODY>
</TABLE></DIR><BR>
END
	fi

	#Calculate Duration in Hours, Minutes and Seconds
	DURATION=`expr $LATEST - $EARLIEST`
	if [ "$DURATION" -gt "0" ]; then
		HOURS=`expr $DURATION / 3600 `
		HOURS_IN_SECS=`expr $HOURS \* 3600 `
		LEFTOVER=`expr $DURATION - $HOURS_IN_SECS `

		MINUTES=`expr $LEFTOVER / 60 `
		MINUTES_IN_SECS=`expr $MINUTES \* 60 `

		SECONDS=`expr $LEFTOVER - $MINUTES_IN_SECS `

		report_message $QTY_SUCCESS" Summaries executed successfully in "$HOURS" Hrs "$MINUTES" Mins "$SECONDS" Secs"
	fi

	#Report Executing
	report_message $QTY_EXECUTING" Summaries are currently executing"

	#Report Queued
	if [ "$QTY_QUEUED" -gt "$QUEUED_THRESHOLD" ]; then
		report_error "-	EXCEPTION ALERT - "$QTY_QUEUED" Summaries are currently queued - Threshold is "$QUEUED_THRESHOLD
	else
		report_message $QTY_QUEUED" Summaries are currently queued - Threshold is "$QUEUED_THRESHOLD
	fi

	#Report Failed
	if [ "$QTY_FAILED" -gt "$FAILED_THRESHOLD" ]; then

		report_error "- EXCEPTION ALERT - "$QTY_FAILED" Summaries failed execution - Threshold is "$FAILED_THRESHOLD

		#List failed summaries to screen
		cut -f 1 -d'|' $FAILED_SUMMARIES | while read SUMMARY_ID
		do
			report_error $SUMMARY_ID
		done

cat <<END >> $REPORT
<DIR>
<TABLE cellSpacing=2 cellPadding=6 border=3 width="80%">
<TBODY>
<TR align=middle>
<TH colSpan=4>Summary Failures By Reason </TH></TR>
<TR>
<TH align=middle width="15%">Summaries Failed </TH>
<TH align=middle>Reason Code</TH></TR>
END
		#Ensure list is distinct
		exec < $ALL_REASONS
		while read LINE
		do
			EXIST=`grep -c $LINE $DISTINCT_REASONS`
			if [ "$EXIST" -eq "0" ];then
				echo $LINE >> $DISTINCT_REASONS
			 fi
		done

		exec < $DISTINCT_REASONS 
		while read LINE
		do
			QTY_REASON=`grep -c $LINE $ALL_REASONS`
cat <<END >> $REPORT
<TR>
<TD align=middle><FONT COLOR="#ff0000">$QTY_REASON</FONT></TD>
<TD align=left><FONT COLOR="#ff0000">$LINE</FONT></TD>
</TR>
END
			#report_error "-	"$QTY_REASON" Summaries failed due to: "$LINE
		done
cat <<END >> $REPORT
</TBODY>
</TABLE></DIR><BR>
END
	else
		report_message $QTY_FAILED" Summaries failed execution - Threshold is "$FAILED_THRESHOLD
	fi

}

 

#
# Convert times to UTC for calculations
#
get_utime()
{

	#Convert day in Loader Log File to UDT (number of seconds since 1970)

	if [ -f $METRICA_DIR/SystemManagement/bin/metrica_date_converter.pl ];then
		UTIME=`$METRICA_DIR/SystemManagement/bin/metrica_date_converter.pl -time $1 | awk '{print $5}'`
		YESTERDAY_UTIME=`expr $UTIME - 86400 `
#YESTERDAY_UTIME=`expr $UTIME - 864000 `
		$METRICA_DIR/SystemManagement/bin/metrica_date_converter.pl -utime $YESTERDAY_UTIME > /tmp/utime$$ 
		DAY=`cat /tmp/utime$$ | awk '{print $10}'`
		MON=`cat /tmp/utime$$ | awk '{print $11}'`
		DAT=`cat /tmp/utime$$ | awk '{print $12}'`
		YEAR=`cat /tmp/utime$$ | awk '{print $14}'`
		rm -f /tmp/utime$$ > /dev/null
	else
		report_error "- EXCEPTION ALERT - Cannot Proceed - Script Aborted"
		report_error "- The metrica_date_converter.pl script does not exist in"
		report_error "- "$METRICA_DIR"/SystemManagement/bin"
		footer
		exit
	fi
}




#
# Validate the Configuration file thresholds
#
check_conf_element()
{

	ALPHA="N"
	[[ $1 != +([0-9]) ]] && ALPHA="Y"
	if [ "$1" = "" -o "$ALPHA" = "Y" ];then
		CONF_VAR=0
	else
		CONF_VAR=$1
	fi

}



#
# Remove temporary files
#
clean_up()
{

	#Remove all temporary files
	rm -f $WRK0 $WRK1 $WRK2 $ALL_REASONS $DISTINCT_REASONS 2>/dev/null

}



#
# Main Routine
#
main()
{

	build_environment
	informix_up
	header

	#Check for existence of Configuration File
	check_conf_file

	#Determine Status Of the Summariser           
	get_status

	#Check the value of ConcurrentTasks
	get_tasks

	#Check the status of all configured summaries for yesterdays run
	get_period_status

	footer


}


case $1 in
        # Help options
        "-h" | "-help" | "--help")
                usage
                exit 0
        ;;

        # Version options
        "-v" | "-version" | "--version")
                version
                exit 0
        ;;

        # Any other option
        *)
                main
                exit 0
        ;;
esac
