#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_configure_healthcheck.ksh SysManagement_r4.79.0.9:cset.008510:14:14 10/19/07 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_configure_healthcheck.ksh
# Authors:      Barry Deignan, James Fry, Stuart Brooks
# Description:  Script to configure control files for the following:                 
#                 1. Informix Dynamic Server Checking Tool.                     
#                 2. Informix Configuration Parameters Monitor.                
#                 3. Server Kernel Parameters Monitor.                
#                 4. Server File System Capacity Checking Tool.                
#                 5. Metrica Loader Data Management Checking Script.                    
#                 6. Gateways File Statistics Checking Script.
#                 7. AppServer Checking Script.
#                 8. Summariser Checking Script.
#                 9. Configure Report E-mail Recipients.
#                10. Configure Report Destinations.
#                11. Metrica Unix Mail Check
#                12. Informix Unix Mail Check
#
###############################################################################

#Set Environment
PROGNAME=`basename $0`
VERNO=SysManagement_r4.79.0.9
WRK0=/tmp/work0$$
WRK1=/tmp/work1$$
WRK2=/tmp/work2$$
HOSTNAME=`uname -n`
DATEXT=`date +%d-%b-%Y:%H:%M'`




#
# Check environment variables
#
build_environment()
{
	if [[ -z "$METRICA_DIR" ]]
	then
		echo "- EXCEPTION ALERT - METRICA_DIR not defined"
		echo "- Script Aborted"
		exit 1
	else
		. $METRICA_DIR/setup.ksh
		. $METRICA_DIR/utils/bin/metrica_install_common.ksh

		#Check if Control Directory Exists
		CONTROL_DIR=$METRICA_DIR/SystemManagement/control
		if [ ! -d $CONTROL_DIR ]; then
			mkdir -p $CONTROL_DIR
		fi

		#Check if Control Directory Exists
		LOGS_DIR=$METRICA_DIR/logs/checks
		if [ ! -d $LOGS_DIR ]; then
			mkdir -p $LOGS_DIR
		fi

		#Check if Main Healthcheck Control File exists
		CONTROL_FILE=$CONTROL_DIR/metrica_healthcheck.conf
		ARCH=`uname`
cat <<END > $WRK0
		metrica_check_ids
		metrica_check_onconfig
		metrica_check_kernel
		metrica_check_filesystems
		metrica_check_loader
		metrica_check_gateways
		metrica_check_appserver
		metrica_check_summariser
		metrica_check_metrica_mail
		metrica_check_informix_mail
END
		if [ ! -f $CONTROL_FILE ]; then
			for a in `cat $WRK0`
			do
				if [ "$a" -eq "metrica_check_metrica_mail" ]; then
					$ECHO "metrica_check_metrica_mail	1" >> $CONTROL_FILE
				elif [ "$a" -eq "metrica_check_informix_mail" ]; then
					$ECHO "metrica_check_informix_mail	0" >> $CONTROL_FILE
				else
					$ECHO $a"	Disabled" >> $CONTROL_FILE
				fi
			done
			$ECHO "MAILTO $LOGNAME@$HOSTNAME" >> $CONTROL_FILE
		else
			for a in `cat $WRK0`
			do
				EXIST=`cat $CONTROL_FILE | grep $a | grep -v "."$a | wc | awk '{print $1}'`
				if [ "$EXIST" -eq "0" ];then
					if [ "$a" -eq "metrica_check_metrica_mail" ]; then
						$ECHO "metrica_check_metrica_mail	1" >> $CONTROL_FILE
					elif [ "$a" -eq "metrica_check_informix_mail" ]; then
						$ECHO "metrica_check_informix_mail	0" >> $CONTROL_FILE
					else
						$ECHO $a"	Disabled" >> $CONTROL_FILE
					fi
				fi
			done
		fi


		IDS_FILE=$CONTROL_DIR/metrica_check_ids.conf
		ICP_FILE=$CONTROL_DIR/metrica_check_onconfig.conf
		SKP_FILE=$CONTROL_DIR/metrica_check_kernel.conf
		SFS_FILE=$CONTROL_DIR/metrica_check_filesystems.conf
		LDM_FILE=$CONTROL_DIR/metrica_check_loader.conf
		GFS_FILE=$CONTROL_DIR/metrica_check_gateways.conf
		APP_FILE=$CONTROL_DIR/metrica_check_appserver.conf
		SUM_FILE=$CONTROL_DIR/metrica_check_summariser.conf

	fi
}


 
 
#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}



#
# Show correct command-line usage
#
usage()
{
	show_info
cat <<END

  Usage: $PROGNAME [options]

        options:    [ -h | -help | --help ]
                  | [ -v | -version | --version ]

  Script to configure control files for the following:
     1. Informix Dynamic Server Check Tool.
     2. Informix Configuration Parameters Monitor.
     3. Server Kernel Parameters Monitor.
     4. Server File System Capacity Checking Tool.
     5. Metrica Loader Data Management Check Script.
     6. Gateways File Statistics Health Check Script.
     7. AppServer Health Check Script.
     8. Summariser Health Check Script.
     9. Configure Report E-mail Recipients.
    10. Configure copy location(s) for HTML results file.
    11. Configure Metrica Unix Mail Check.
    12. Configure Informix Unix Mail Check.

END
}


 
version()
{
	show_info
 	echo "$PROGNAME SysManagement/delivery/utils/bin/Monitoring/metrica_configure_healthcheck.ksh SysManagement_r4.79.0.9:cset.008510:14:14 10/19/07"
	exit 0
}



#
# Establish if user wants to quit program or return to menu
#
quit_return()
{
	
	if [ "$1" = "q" ]; then
		clean_up
		completion
		exit 0
	fi

	if [ "$1" = "r" ]; then
		clean_up
		     
		case "$2" in
			 "configure_eml")
				 EML="Y";;
			 "configure_cpy")
			     CPY="Y";;
			 "main")
				 RERUN=Y;;
		esac
		clear
		$2
	fi

}


#
# Display install layers
#
get_config_choice()
{

	#Get Enabled status for each section
	IDS_STATUS=`cat $CONTROL_FILE | grep metrica_check_ids | grep -v .metrica_check_ids | awk '{print $2}'`
	ICP_STATUS=`cat $CONTROL_FILE | grep metrica_check_onconfig | grep -v .metrica_check_onconfig | awk '{print $2}'`
	SKP_STATUS=`cat $CONTROL_FILE | grep metrica_check_kernel | grep -v .metrica_check_kernel | awk '{print $2}'`
	SFS_STATUS=`cat $CONTROL_FILE | grep metrica_check_filesystems | grep -v .metrica_check_filesystems | awk '{print $2}'`
	LDM_STATUS=`cat $CONTROL_FILE | grep metrica_check_loader | grep -v .metrica_check_loader | awk '{print $2}'`
	GFS_STATUS=`cat $CONTROL_FILE | grep metrica_check_gateways | grep -v .metrica_check_gateways | awk '{print $2}'`
	APP_STATUS=`cat $CONTROL_FILE | grep metrica_check_appserver | grep -v .metrica_check_appserver | awk '{print $2}'`
	SUM_STATUS=`cat $CONTROL_FILE | grep metrica_check_summariser | grep -v .metrica_check_summariser | awk '{print $2}'`
	EML_STATUS=`cat $CONTROL_FILE | grep MAILTO | grep -v .MAILTO | wc | awk '{print $1}'`
    CPY_STATUS=`cat $CONTROL_FILE | grep COPYTO | grep -v .COPYTO | wc | awk '{print $1}'`

	MUM_STATUS=`cat $CONTROL_FILE | grep metrica_check_metrica_mail | grep -v .metrica_check_metrica_mail | awk '{print $2}'`
	case $MUM_STATUS in
		"0")
			MUM_STATUS_DESC="New Mail Count OFF, Mail Headers OFF"
			;;
		"1")
			MUM_STATUS_DESC="New Mail Count ON, Mail Headers OFF"
			;;
		"2")
			MUM_STATUS_DESC="New Mail Count ON, Mail Headers ON"
			;;
		*)
			MUM_STATUS=0
			MUM_STATUS_DESC="New Mail Count OFF, Mail Headers OFF"
			;;
	esac

	IUM_STATUS=`cat $CONTROL_FILE | grep metrica_check_informix_mail | grep -v .metrica_check_informix_mail | awk '{print $2}'`
	case $IUM_STATUS in
		"0")
			IUM_STATUS_DESC="New Mail Count OFF, Mail Headers OFF"
			;;
		"1")
			IUM_STATUS_DESC="New Mail Count ON, Mail Headers OFF"
			;;
		"2")
			IUM_STATUS_DESC="New Mail Count ON, Mail Headers ON"
			;;
		*)
			IUM_STATUS=0
			IUM_STATUS_DESC="New Mail Count OFF, Mail Headers OFF"
			;;
	esac
	clear
cat <<END
 
Performance Manager Healthcheck Configuration Tool
===================================================

   Select Health Check Section to Enable/Disable/Configure:

     Option                                                     Status
     =======                                                    =======

     1) Informix Dynamic Server Checking Tool.                  $IDS_STATUS
     2) Informix Configuration Parameters Monitor.              $ICP_STATUS
     3) Server Kernel Parameters Monitor.                       $SKP_STATUS
     4) Server File System Capacity Checking Tool.              $SFS_STATUS
     5) Metrica Loader Data Management Checking Script.         $LDM_STATUS
     6) Gateways File Statistics Checking Script.               $GFS_STATUS
     7) Appserver Checking Script.                              $APP_STATUS
     8) Summariser Checking Script.                             $SUM_STATUS
     9) Configure Report E-mail Recipients.                     $EML_STATUS Account(s) Configured
    10) Configure Additional HTML file locations.               $CPY_STATUS Location(s) Configured
    11) Metrica Unix Mail Check                                 $MUM_STATUS_DESC
    12) Informix Unix Mail Check                                $IUM_STATUS_DESC
    13) All Above
    14) Quit (q to quit)

END

	$ECHO -n "Enter your choice [1-14] : "
	read ANS

	case "$ANS" in 
		"1") 
			IDS="Y";;
		"2") 
			ICP="Y";;
		"3") 
			SKP="Y";;
		"4") 
			SFS="Y";;
		"5") 
			LDM="Y";;
		"6") 
			GFS="Y";;
		"7") 
			APP="Y";;
		"8") 
			SUM="Y";;
		"9") 
			EML="Y";;
        "10") 
            CPY="Y";;
		"11") 
			MUM="Y";;
		"12")
			IUM="Y";;
		"13")
			IDS="Y"
			ICP="Y"
			SKP="Y"
			SFS="Y"
			LDM="Y"
			GFS="Y"
			APP="Y"
			SUM="Y"
			EML="Y"
			CPY="Y"
			MUM="Y"
			IUM="Y";;
		"14") 
			completion
			m_exit 0 ;; 
		"q") 
			completion
			m_exit 0 ;; 
		*) 
			$ECHO $ANS"is an Invalid Selection" 
			clean_up
			get_config_choice ;;
	esac 
}


 
#
# Enable/Disable Healthcheck Option
#                                   
enable_selection()
{
	$ECHO ""
	STATUS=`cat $CONTROL_FILE | grep $1 | grep -v "."$1 | awk '{print $2}'`
	if [ "$STATUS" = "Enabled" ]; then
		$ECHO "This option is currently enabled. "
		$ECHO -n "Do you want to disable it [Y/N] -default [N] (q to Quit r to Return to menu): "
		read DISABLE

		quit_return $DISABLE "main"

		if [ "$DISABLE" = "" ]; then
			DISABLE="N"
		fi

		if [ "$DISABLE" = "Y" -o "$DISABLE" = "y" ]; then
			cat $CONTROL_FILE | grep -v $1 > $WRK0
			$ECHO $1"	Disabled" >> $WRK0
			cat $WRK0 > $CONTROL_FILE
			$ECHO "Status changed to disabled"
		fi
	else
		$ECHO "This option is currently disabled."
		$ECHO -n "Do you want to enable it [Y/N] -default [Y] (q to Quit r to Return to menu): "
		read DISABLE

		quit_return $DISABLE "main"

		if [ "$DISABLE" = "" ]; then
			DISABLE="Y"
		fi

		if [ "$DISABLE" = "Y" -o "$DISABLE" = "y" ]; then
			cat $CONTROL_FILE | grep -v $1 > $WRK0
			$ECHO $1"	Enabled" >> $WRK0
			cat $WRK0 > $CONTROL_FILE
			$ECHO "Status changed to enabled"
		fi
	fi
	STATUS=`cat $CONTROL_FILE | grep $1 | grep -v "."$1 | awk '{print $2}'`
		

}

clean_up()
{

	#Reset Menu choices
	IDS="N"
	ICP="N"
	SKP="N"
	SFS="N"
	LDM="N"
	GFS="N"
	APP="N"
	SUM="N"
	EML="N"
    CPY="N"
    MUM="N"
    IUM="N"
	rm -f $WRK0 $WRK1 $WRK2 2>/dev/null
}
 


configure_ids()
{

	if [ "$IDS" = "Y" ]; then
		clear
		$ECHO ""
		$ECHO "Configure the Informix Dynamic Server Check Script "
		$ECHO "================================================== "

		enable_selection metrica_check_ids
		IDS_STATUS=$STATUS

		configure_ids_menu 
	fi

}



configure_ids_menu()
{

	touch $IDS_FILE
	CAP_CNT=`grep -c "CAPACITY_THRESHOLD" $IDS_FILE`
	if [ "$CAP_CNT" -eq "0" ];then
		$ECHO "CAPACITY_THRESHOLD 95" >> $IDS_FILE
	fi

	EXT_CNT=`grep -c "EXTENTS" $IDS_FILE`
	if [ "$EXT_CNT" -eq "0" ];then
		$ECHO "EXTENTS 50" >> $IDS_FILE
	fi

	NUMDAYS_CNT=`grep -c "NUMBER_DAYS_LOG" $IDS_FILE`
	if [ "NUMDAYS_CNT" -eq "0" ];then
		$ECHO "NUMBER_DAYS_LOG 0"  >> $IDS_FILE
	fi

	IDS_CNT=`wc $IDS_FILE | grep -v "#" | grep -v ^$ | awk '{print $1}'`

	if [ "$IDS_STATUS" = "Enabled" ]; then
		if [ "$IDS_CNT" -gt "0" ]; then
			$ECHO "The configuration file "$IDS_FILE" already exists"  
			$ECHO ""
			cat $IDS_FILE | grep -v "#" | grep -v "EXEMPT" | grep -v ^$ > $WRK0
			display_conf_2 "Parameter" "Value" $WRK0

			EXEMPT=`grep -c "EXEMPT_DBSPACE" $IDS_FILE`
			if [ "$EXEMPT" -gt "0" ]; then
				$ECHO ""
				$ECHO "The following Dbspaces are exempt from capacity check"
				cat $IDS_FILE | grep -v "#" | grep "EXEMPT_DBSPACE" | grep -v ^$ | sed s/"EXEMPT_DBSPACE"//1 > $WRK0
				cat $WRK0
			fi

			EXEMPT=`grep -c "EXEMPT_TABLE" $IDS_FILE`
			if [ "$EXEMPT" -gt "0" ]; then
				$ECHO ""
				$ECHO "The following Tables are exempt from Index count check"
				cat $IDS_FILE | grep -v "#" | grep "EXEMPT_TABLE" | grep -v ^$ | sed s/"EXEMPT_TABLE"//1 > $WRK0
				cat $WRK0
			fi

		fi

cat <<END

   Select the action you require:                           

     Option                                                               
     =======                                                             

     1)  Modify the Capacity Threshold for Dbspace Utilisation.                                  
     2)  Modify the number of Table & Index Extents Threshold.                                  
     3)  Add a Dbspace to be exempt from Threshold Check.
     4)  Remove a Dbspace to be exempt from Threshold Check.
     5)  Add a Table to be exempt from Index Count Check.
     6)  Remove a Table to be exempt from Index Count Check.
     7)  Set number of days of online.log to check for errors.
     8)  Continue.                                           
     9)  Return to main menu                                                  
     10) Quit (q to Quit)                          

END

		$ECHO -n "Enter your choice [1-10] : "
		read CHOICE

		case "$CHOICE" in 
			"1") 
				modify_capacity_threshold;;
			"2") 
				modify_extents;;
			"3") 
				add_exempt_dbspace;;
			"4") 
				remove_exempt_dbspace;;
			"5") 
				add_exempt_table;;
			"6") 
				remove_exempt_table;;
			"7")
				modify_numbdays;;
			"8") 
				OK="Y";;
			"9") 
				quit_return "r" "main";;
			"10") 
				quit_return "q";;
			"r") 
				quit_return "r" "main";;
			"q") 
				quit_return "q";;
			*) 
				$ECHO $CHOICE" is an Invalid Choice - Try Again"
				configure_ids_menu;;
		esac
	fi

}



modify_capacity_threshold()
{

	#Get Capacity Threshold

	$ECHO ""
	$ECHO "The Capacity Threshold is the % Dbspace Utilisation at which the script should alert"
	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do

		$ECHO ""
		$ECHO "Enter the Capacity Threshold required  "                  
		$ECHO -n "[between 1 and 99] - default [95] (q to Quit r to Return to menu) : "                  
		read CAPACITY_THRESHOLD

		quit_return $CAPACITY_THRESHOLD "configure_ids_menu"

		if [ "$CAPACITY_THRESHOLD" = "" ]; then
			CAPACITY_THRESHOLD="95"
		fi

		validate_number $CAPACITY_THRESHOLD 1 99
	done
	cat $IDS_FILE | grep -v "CAPACITY_THRESHOLD" > $WRK0
	$ECHO "CAPACITY_THRESHOLD	"$CAPACITY_THRESHOLD >> $WRK0
	cat $WRK0 > $IDS_FILE
	configure_ids_menu

}

modify_numbdays()
{

    #Get number of days online.log to be checked

    $ECHO ""
    OK_FLAG="N"
    while [ "$OK_FLAG" = "N" ]
    do

        $ECHO ""
        $ECHO "Enter the number of days in the past that the online.log informix log should be checked for errors."
		$ECHO "If a value of '0' is entered, the whole online.log will be checked"
        $ECHO -n "Enter number [between 0 and 99] - default [0] (q to Quit r to Return to menu) : "
        read NUMBER_DAYS_LOG

        quit_return $NUMBER_DAYS_LOG "configure_ids_menu"

        if [ "$NUMBER_DAYS_LOG" = "" ]; then
            NUMBER_DAYS_LOG="0"
        fi

        validate_number $NUMBER_DAYS_LOG 0 99
    done
    cat $IDS_FILE | grep -v "NUMBER_DAYS_LOG" > $WRK0
    $ECHO "NUMBER_DAYS_LOG   "$NUMBER_DAYS_LOG >> $WRK0
    cat $WRK0 > $IDS_FILE
    configure_ids_menu

}


modify_extents()
{

	#Get Number of Extents

	$ECHO ""
	$ECHO "The Number of Extents is compared to the actual number of extents allocated to each table and index"
	$ECHO "in the Informix Server Databases - if any exceed this figure an exception alert is raised"                
	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do

		$ECHO ""
		$ECHO -n "Enter the Number of Extents required [between 1 and 250] - default [50] : "
		read EXTENTS

		quit_return $EXTENTS "configure_ids_menu"

		if [ "$EXTENTS" = "" ]; then
			EXTENTS="50"
		fi

		validate_number $EXTENTS 1 250
	done
	cat $IDS_FILE | grep -v "EXTENTS" > $WRK0
	$ECHO "EXTENTS	"$EXTENTS >> $WRK0
	cat $WRK0 > $IDS_FILE
	configure_ids_menu
}



#
# Add an Exempt Dbspace                   
#
add_exempt_dbspace()
{

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		$ECHO ""
		$ECHO -n "Enter a Dbspace to exempt e.g logspace (q to Quit r to Return to menu) : "
		read DBSPACE

		if [ "$DBSPACE" = "" ]; then
			DBSPACE="NULL"
		fi

		quit_return $DBSPACE "configure_ids_menu"

		cat $IDS_FILE | grep "EXEMPT_DBSPACE" | sed s/" "/=/g > $WRK0

		DBSPACE_PATTERN="EXEMPT_DBSPACE="$DBSPACE"="
		EXISTS=`cat $WRK0 | grep $DBSPACE_PATTERN | wc | awk '{print $1}'`

		$ECHO 'UNLOAD TO '$WRK1' SELECT count(*) FROM sysdbspaces where name="'$DBSPACE'";' | dbaccess sysmaster > /dev/null 2>&1
		ISREAL=`cat $WRK1 | sed s/\.0\|//`

		if [ "$DBSPACE" = "NULL" -o "$EXISTS" -gt "0" -o "$ISREAL" -eq "0" ]; then
			$ECHO "Try Again - "$DBSPACE" is not a valid Dbspace or already configured"
		else		
			OK_FLAG="Y"
			$ECHO "Adding Dbspace "$DBSPACE" to exemption list"
			$ECHO "EXEMPT_DBSPACE "$DBSPACE" " >> $IDS_FILE
		fi
	done
	configure_ids_menu
}



#
# Remove an Exempt Dbspace                   
#
remove_exempt_dbspace()
{

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		$ECHO ""
		$ECHO -n "Enter a Dbspace to remove from the exemption list (q to Quit r to Return to menu) : "
		read DBSPACE

		if [ "$DBSPACE" = "" ]; then
			DBSPACE="NULL"
		fi

		quit_return $DBSPACE "configure_ids_menu"

		cat $IDS_FILE | grep "EXEMPT_DBSPACE" | sed s/" "/=/g > $WRK0

		DBSPACE_PATTERN="EXEMPT_DBSPACE="$DBSPACE"="
		EXISTS=`cat $WRK0 | grep $DBSPACE_PATTERN | wc | awk '{print $1}'`
		if [ "$DBSPACE" = "NULL" -o "$EXISTS" -eq "0" ]; then
			$ECHO "Try Again - "$DBSPACE" is not a valid Dbspace or is not on the exemption list"
		else		
			OK_FLAG="Y"
			$ECHO "Removing Dbspace "$DBSPACE" from exemption list"
			cat $IDS_FILE | grep -v "EXEMPT_DBSPACE" > $WRK1
			cat $WRK0 | grep -v $DBSPACE_PATTERN | sed s/=/" "/g >> $WRK1
			cat $WRK1 > $IDS_FILE
		fi
	done
	configure_ids_menu

}



#
# Add an Exempt Table                     
#
add_exempt_table()
{

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		$ECHO ""
		$ECHO -n "Enter a Table to exempt from index count check (q to Quit r to Return to menu) : "
		read TABLE

		if [ "$TABLE" = "" ]; then
			TABLE="NULL"
		fi

		quit_return $TABLE "configure_ids_menu"

		cat $IDS_FILE | grep "EXEMPT_TABLE" | sed s/" "/=/g > $WRK0

		TABLE_PATTERN="EXEMPT_TABLE="$TABLE"="
		EXISTS=`cat $WRK0 | grep $TABLE_PATTERN | wc | awk '{print $1}'`

		$ECHO 'UNLOAD TO '$WRK1' SELECT count(*) FROM systables where tabname="'$TABLE'";' | dbaccess $METRICA_PERFORMANCE_DB > /dev/null 2>&1
		ISREAL=`cat $WRK1 | sed s/\.0\|//`

		if [ "$TABLE" = "NULL" -o "$EXISTS" -gt "0" -o "$ISREAL" -eq "0" ]; then
			$ECHO "Try Again - "$TABLE" is not a valid table or already configured"
		else		
			OK_FLAG="Y"
			$ECHO "Adding Table "$TABLE" to exemption list"
			$ECHO "EXEMPT_TABLE "$TABLE" " >> $IDS_FILE
		fi
	done
	configure_ids_menu
}



#
# Remove an Exempt Dbspace                   
#
remove_exempt_table()
{

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		$ECHO ""
		$ECHO -n "Enter a Table to remove from the exemption list (q to Quit r to Return to menu) : "
		read TABLE

		if [ "$TABLE" = "" ]; then
			TABLE="NULL"
		fi

		quit_return $TABLE "configure_ids_menu"

		cat $IDS_FILE | grep "EXEMPT_TABLE" | sed s/" "/=/g > $WRK0

		TABLE_PATTERN="EXEMPT_TABLE="$TABLE"="
		EXISTS=`cat $WRK0 | grep $TABLE_PATTERN | wc | awk '{print $1}'`
		if [ "$TABLE" = "NULL" -o "$EXISTS" -eq "0" ]; then
			$ECHO "Try Again - "$TABLE" is not a valid table or is not on the exemption list"
		else		
			OK_FLAG="Y"
			$ECHO "Removing Table "$TABLE" from exemption list"
			cat $IDS_FILE | grep -v "EXEMPT_TABLE" > $WRK1
			cat $WRK0 | grep -v $TABLE_PATTERN | sed s/=/" "/g >> $WRK1
			cat $WRK1 > $IDS_FILE
		fi
	done
	configure_ids_menu

}




configure_icp()
{

	if [ "$ICP" = "Y" ]; then
		clear
		$ECHO ""
		$ECHO "Configure the Informix Configuration Parameters Comparison Tool "
		$ECHO "================================================================= "

		enable_selection metrica_check_onconfig
		ICP_STATUS=$STATUS

		check_config_exist $ICP_FILE "2"
		if [ "$CONTINUE" = "Y" ]; then

			#Get BUFFERS              
			get_onconfig_parm "BUFFERS" "100" "2147483647"

			#Get SHMVIRTSIZE              
			get_onconfig_parm "SHMVIRTSIZE" "1024" "4096000"

			#Get SHMADD              
			get_onconfig_parm "SHMADD" "1024" "524288"

			#Get DS_TOTAL_MEMORY              
			get_onconfig_parm "DS_TOTAL_MEMORY" "1024" "4096000"

			#Get LRUS              
			get_onconfig_parm "LRUS" "1" "511"

			#Get LRU_MAX_DIRTY              
			get_onconfig_parm "LRU_MAX_DIRTY" "0" "100"

			#Get LRU_MIN_DIRTY              
			get_onconfig_parm "LRU_MIN_DIRTY" "0" "100"

			#Get CLEANERS              
			get_onconfig_parm "CLEANERS" "1" "512"

			#Get PHYSBUFF              
			get_onconfig_parm "PHYSBUFF" "16" "128"

			#Get LOGBUFF              
			get_onconfig_parm "LOGBUFF" "16" "128"

			#Get STACKSIZE            
			get_onconfig_parm "STACKSIZE" "64" "128"

			#Get RESIDENT            
			get_onconfig_parm "RESIDENT" "0" "2"

			#Get LOCKS            
			get_onconfig_parm "LOCKS" "2000" "8000000"

			#Get RA_PAGES            
			MAX=`onstat -c | grep BUFFERS | grep -v .BUFFERS | awk '{print $2}'`
			get_onconfig_parm "RA_PAGES" "8" $MAX 

			#Get RA_THRESHOLD            
			MAX=`onstat -c | grep RA_PAGES | grep -v .RA_PAGES | awk '{print $2}'`
			MAX=`expr $MAX - 1`
			get_onconfig_parm "RA_THRESHOLD" "0" $MAX 

			#Get DD_HASHSIZE            
			get_onconfig_parm "DD_HASHSIZE" "61" "127"

			#Get DD_HASHMAX            
			get_onconfig_parm "DD_HASHMAX" "31" "127"

			#Get DS_POOLSIZE            
			get_onconfig_parm "DS_POOLSIZE" "61" "127"

			#Get DS_HASHSIZE            
			get_onconfig_parm "DS_HASHSIZE" "31" "127"

			#Get PC_POOLSIZE            
			get_onconfig_parm "PC_POOLSIZE" "700" "1400"

			#Get PC_HASHSIZE            
			get_onconfig_parm "PC_HASHSIZE" "61" "127"

			#Get VPCLASS
			get_vpclass_parm "VPCLASS" "cpu"
			get_vpclass_parm "VPCLASS" "aio"
			get_vpclass_parm "VPCLASS" "adc"

			$ECHO ""  
			$ECHO "Configuration of Informix Parameters Comparison Tool completed"  
		else
			$ECHO ""  
			$ECHO "Configuration of Informix Parameters Comparison Tool cancelled"  
		fi
	fi

}



#
# Configure the HP Server Kernel Parameters Comparison Tool
#
configure_skp_hp()
{

	if [ "$SKP" = "Y" ]; then
		clear
		$ECHO ""
		$ECHO "Configure the Server Kernel Parameters Comparison Tool "
		$ECHO "====================================================== "

		enable_selection metrica_check_kernel
		SKP_STATUS=$STATUS

		check_config_exist $SKP_FILE
		if [ "$CONTINUE" = "Y" ]; then

		# Generate a list of SUN Kernel Parameters
cat <<END > $WRK0
	shmmax
	maxdsiz
	max_thread_proc
END

			for a in `cat $WRK0`
			do
				get_hp_kernel_parm $a 
			done

			$ECHO ""  
			$ECHO "Configuration of Server Kernel Parameters Comparison Tool completed"  
		else
			$ECHO ""  
			$ECHO "Configuration of Server Kernel Parameters Comparison Tool cancelled"  
		fi
	fi

}



#
# Configure the Sun Server Kernel Parameters Comparison Tool
#
configure_skp_sun()
{

	if [ "$SKP" = "Y" ]; then
		clear
		$ECHO ""
		$ECHO "Configure the Server Kernel Parameters Comparison Tool "
		$ECHO "====================================================== "

		enable_selection metrica_check_kernel
		SKP_STATUS=$STATUS

		check_config_exist $SKP_FILE
		if [ "$CONTINUE" = "Y" ]; then

		# Generate a list of SUN Kernel Parameters
cat <<END > $WRK0
	semsys:seminfo_semmap
	semsys:seminfo_semmni
	semsys:seminfo_semmns
	semsys:seminfo_semmnu
	semsys:seminfo_semume
	semsys:seminfo_semmsl
	shmsys:shminfo_shmmin
	shmsys:shminfo_shmmni
	shmsys:shminfo_shmseg
END

			for a in `cat $WRK0`
			do
				get_sun_kernel_parm $a "Y"
			done

			#Get Kernel Parameter shmsys:shminfo_shmmax
			get_sun_kernel_parm "shmsys:shminfo_shmmax" "N" 

			$ECHO ""  
			$ECHO "Configuration of Server Kernel Parameters Comparison Tool completed"  
		else
			$ECHO ""  
			$ECHO "Configuration of Server Kernel Parameters Comparison Tool cancelled"  
		fi
	fi

}



#
# Configure Server File System Checking Script
#
configure_sfs()
{

	if [ "$SFS" = "Y" ]; then
		clear
		$ECHO ""
		$ECHO "Configure the Server File System Capacity Checking Script "
		$ECHO "========================================================== "

		enable_selection metrica_check_filesystems
		SFS_STATUS=$STATUS

		configure_sfs_menu
	fi

}



configure_sfs_menu()
{

	touch $SFS_FILE
	SFS_CNT=`wc $SFS_FILE | grep -v "#" | grep -v ^$ | awk '{print $1}'`

	if [ "$SFS_STATUS" = "Enabled" ]; then
		if [ "$SFS_CNT" -gt "0" ]; then
			$ECHO "The following filesystems have already been configured"
			printf "%-30s\t%-30s\n" "FileSystem" "Capacity Threshold" 
			printf "%-30s\t%-30s\n" "==========" "==================" 
			cat $SFS_FILE | grep -v "#" | grep -v ^$ > $WRK0            

			CNT0="0"
			for a in `cat $WRK0`
			do
				CNT0=`expr $CNT0 + 1 `

				if [ "$CNT0" -eq "1" ]; then
					DISP_FS=$a
				fi

				if [ "$CNT0" -eq "2" ]; then
					DISP_CP=$a
					CNT0="0"
					printf "%-36s\t%2.0f\t%-1s\n" $DISP_FS $DISP_CP "%" 
				fi
			done
		fi

cat <<END

   Select the action you require:                           

     Option                                                               
     =======                                                             

     1)  Add a FileSystem to the configuration file.                                             
     2)  Remove a FileSystem from configuration file.       
     3)  Continue.                                           
     4)  Return to main menu                                                  
     5)  Quit (q to Quit)                          

END

		$ECHO -n "Enter your choice [1-5] : "
		read CHOICE

		case "$CHOICE" in 
			"1") 
				add_fs;;
			"2") 
				remove_fs;;
			"3") 
				OK="Y";;
			"4") 
				quit_return "r" "main";;
			"5") 
				quit_return "q";;
			"r") 
				quit_return "r" "main";;
			"q") 
				quit_return "q";;
			*) 
				$ECHO $CHOICE" is an Invalid Choice - Try Again"
				configure_sfs_menu;;
		esac
	fi

}



#
# Configure Metrica Loader Data Management Checking Script
#
configure_ldm()
{

	if [ "$LDM" = "Y" ]; then
		clear
		$ECHO ""
		$ECHO "Configure the Metrica Loader Data Management Checking Script"
		$ECHO "============================================================= "

		enable_selection metrica_check_loader
		LDM_STATUS=$STATUS

		configure_ldm_menu

	fi

}



configure_ldm_menu()
{

	touch $LDM_FILE
	LDM_CNT=`wc $LDM_FILE | grep -v "#" | grep -v ^$ | awk '{print $1}'`

	if [ "$LDM_STATUS" = "Enabled" ]; then
		if [ "$LDM_CNT" -gt "0" ]; then
			$ECHO "The following Rulesets have already been configured"
			$ECHO ""
			cat $LDM_FILE | grep -v "#" | grep -v ^$ > $WRK0
			display_conf_5 "Ruleset_Id" "Loaders" "Lifs/Hour" "Bad_Files" "Backlog" $WRK0
			$ECHO ""
		fi

cat <<END

   Select the action you require:                           

     Option                                                               
     =======                                                             

     1)  Add a Ruleset to the configuration file.                                             
     2)  Remove a Ruleset from configuration file.       
     3)  Continue.                                           
     4)  Return to main menu                                                  
     5)  Quit (q to Quit)                          

END

		$ECHO -n "Enter your choice [1-5] : "
		read CHOICE

		case "$CHOICE" in 
			"1") 
				add_ruleset;;
			"2") 
				remove_ruleset;;
			"3") 
				OK="Y";;
			"4") 
				quit_return "r" "main";;
			"5") 
				quit_return "q";;
			"r") 
				quit_return "r" "main";;
			"q") 
				quit_return "q";;
			*) 
				$ECHO $CHOICE" is an Invalid Choice - Try Again"
				configure_ldm_menu;;
		esac
	fi

}

#
# Configure Gateway File Statistics Checking Script
#
configure_gfs()
{

	if [ "$GFS" = "Y" ]; then
		clear
		$ECHO ""
		$ECHO "Configure the Gateway File Statistics Checking Script"       
		$ECHO "====================================================== "

		enable_selection metrica_check_gateways
		GFS_STATUS=$STATUS

		check_config_exist $GFS_FILE "0" "APPEND_ASK"
		if [ "$CONTINUE" = "Y" ]; then

			add_gateway

			ADD_MORE_FLAG="Y"
			while [ "$ADD_MORE_FLAG" = "Y" ]
			do
				$ECHO ""
				$ECHO "So far you have added the following to the configuration file ....."
				$ECHO ""
				cat $GFS_FILE 
				$ECHO ""
				$ECHO -n "Do you want to add another ? [Y/N] -default [Y] (q to Quit r to Return to menu) : "
				read ANS

				quit_return $ANS "main"

				if [ "$ANS" = "" ]; then
					ANS="Y"
				fi

				if [ "$ANS" = "Y" -o "$ANS" = "y" ]; then
					add_gateway
				else
					ADD_MORE_FLAG="N"
				fi
			done

			$ECHO ""  
			$ECHO "Configuration of Gateway File Statistics Checking Script completed"  
		else
			$ECHO ""  
			$ECHO "Configuration of Gateway File Statistics Checking Script cancelled"  
		fi
	fi


}



#
# Configure AppServer Checking Script
#
configure_app()
{

	if [ "$APP" = "Y" ]; then
		clear
		$ECHO ""
		$ECHO "Configure the AppServer Check Script "
		$ECHO "======================================= "

		enable_selection metrica_check_appserver
		APP_STATUS=$STATUS

		configure_app_menu

		$ECHO ""  
		$ECHO "Configuration of AppServer Checking Script completed"  
	fi
}



configure_app_menu()
{

	touch $APP_FILE
	SERVER_CONF=$APPSERVER_DIR/var/servers/$APPSERVER_NAME/adm/properties/partitions/standard/partition_server.config

	get_max_heap
	get_min_heap
	get_new_heap
	ARCH=`uname`
	#case $ARCH in
		#HP-UX)
			#MAXHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xmx" | awk '{print $2}' | sed s/-Xmx//1 | sed s/m//1`
			#MINHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xms" | awk '{print $2}' | sed s/-Xms//1 | sed s/m//1`
			#NEWHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xmn" | awk '{print $2}' | sed s/-Xmn//1 | sed s/m//1`
		#;;
		#SunOS)
			#MAXHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xmx" | awk '{print $2}' | sed s/-Xmx//1 | sed s/m//1`
			#MINHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xms" | awk '{print $2}' | sed s/-Xms//1 | sed s/m//1`
			#NEWHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -XX:MaxNewSize" | awk '{print $2}' | awk '{FS="=";print $2}' | sed s/m//1`
		#;;
	#esac

	MAX_CNT=`grep -c "JVM_MAXIMUM_HEAP_SIZE" $APP_FILE`
	if [ "$MAX_CNT" -eq "0" ];then
		$ECHO "JVM_MAXIMUM_HEAP_SIZE "$MAXHEAP >> $APP_FILE
	fi

	MIN_CNT=`grep -c "JVM_MINIMUM_HEAP_SIZE" $APP_FILE`
	if [ "$MIN_CNT" -eq "0" ];then
		$ECHO "JVM_MINIMUM_HEAP_SIZE "$MINHEAP >> $APP_FILE
	fi

	NEW_CNT=`grep -c "JVM_NEW_HEAP_SIZE" $APP_FILE`
	if [ "$NEW_CNT" -eq "0" ];then
		$ECHO "JVM_NEW_HEAP_SIZE "$NEWHEAP >> $APP_FILE
	fi

	APP_CNT=`wc $APP_FILE | grep -v "#" | grep -v ^$ | awk '{print $1}'`

	if [ "$APP_STATUS" = "Enabled" ]; then
		if [ "$APP_CNT" -gt "0" ]; then
			$ECHO "The configuration file "$APP_FILE" already exists"  
			$ECHO ""
			cat $APP_FILE | grep -v "#" | grep -v ^$ > $WRK0
			display_conf_2 "Parameter" "Value" $WRK0

			fi


cat <<END

   Select the action you require:                           

     Option                                                               
     =======                                                             

     1)  Modify the Maximum Heapsize comparison value.                                   
     2)  Modify the Minimum Heapsize comparison value.                                   
     3)  Modify the New Heapsize comparison value.                                   
     4)  Continue.                                           
     5)  Return to main menu                                                  
     6)  Quit (q to Quit)                          

END

		$ECHO -n "Enter your choice [1-6] : "
		read CHOICE

		case "$CHOICE" in 
			"1") 
				modify_heap "MAXIMUM";;
			"2") 
				modify_heap "MINIMUM";;
			"3") 
				modify_heap "NEW";;
			"4") 
				OK="Y";;
			"5") 
				quit_return "r" "main";;
			"6") 
				quit_return "q";;
			"r") 
				quit_return "r" "main";;
			"q") 
				quit_return "q";;
			*) 
				$ECHO $CHOICE" is an Invalid Choice - Try Again"
				configure_app_menu;;
		esac
	fi

}



get_max_heap() 
{

	MAXHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xmx" | awk '{print $2}' | sed s/-Xmx//1 | sed s/m//1`
	if [ "$MAXHEAP" = "" ]; then
		MAXHEAP=0
	fi

}



get_min_heap() 
{

	MINHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xms" | awk '{print $2}' | sed s/-Xms//1 | sed s/m//1`
	if [ "$MINHEAP" = "" ]; then
		MINHEAP=0
	fi

}



get_new_heap() 
{

	NEWHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -XX:MaxNewSize" | awk '{print $2}' | awk '{FS="=";print $2}' | sed s/m//1`
	if [ "$NEWHEAP" = "" ]; then
		NEWHEAP=`cat $SERVER_CONF | grep -v ".vmparam" | grep "vmparam -Xmn" | awk '{print $2}' | sed s/-Xmn//1 | sed s/m//1`
	fi 

	if [ "$NEWHEAP" = "" ]; then
		NEWHEAP=0
	fi

}

 

modify_heap()
{

	case "$1" in
		"MAXIMUM")
			DEFAULT=$MAXHEAP ;;
		"MINIMUM")
			DEFAULT=$MINHEAP ;;
		"NEW")
			DEFAULT=$NEWHEAP ;;
	esac

	#Get Heap Size            

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do

		$ECHO ""
		$ECHO "Enter the JVM_"$1"_HEAP_SIZE comparison value"
		$ECHO -n "default ["$DEFAULT"] (q to Quit r to Return to menu) : "                  
		read VALUE

		quit_return $VALUE "configure_app_menu"

		if [ "$VALUE" = "" ]; then
			VALUE=$DEFAULT
		fi

		validate_number $VALUE 1 16384
	done
	cat $APP_FILE | grep -v $1 > $WRK0
	$ECHO "JVM_"$1"_HEAP_SIZE" $VALUE >> $WRK0
	cat $WRK0 > $APP_FILE
	configure_app_menu

}



#
# Configure Summariser Checking Script
#
configure_sum()
{

	if [ "$SUM" = "Y" ]; then
		clear
		$ECHO ""
		$ECHO "Configure the Summariser Check Script "
		$ECHO "======================================= "

		enable_selection metrica_check_summariser
		SUM_STATUS=$STATUS

		check_config_exist $SUM_FILE "2"
		if [ "$CONTINUE" = "Y" ]; then

			configure_sum_menu

			$ECHO ""  
			$ECHO "Configuration of Summariser Checking Script completed"  
		else
			$ECHO ""  
			$ECHO "Configuration of Summariser Checking Script cancelled"  
		fi
	fi
}


configure_sum_menu()
{


	if [ "$SUM_STATUS" = "Enabled" ]; then


cat <<END

   Select the action you require:                           

     Option                                                               
     =======                                                             

     1)  Modify the summary thresholds
     2)  Continue.
     3)  Return to main menu                                                  
     4)  Quit (q to Quit)                          

END

		$ECHO -n "Enter your choice [1-4] : "
		read CHOICE

		case "$CHOICE" in 
			"1") 
				add_summary_thresholds;;
			"2") 
				OK="Y";;
			"3") 
				quit_return "r" "main";;
			"4") 
				quit_return "q";;
			"r") 
				quit_return "r" "main";;
			"q") 
				quit_return "q";;
			*) 
				$ECHO $CHOICE" is an Invalid Choice - Try Again"
				configure_sum_menu;;
		esac
	fi

}


#
# Configure Report E-mail Recipients                 
#
configure_eml()
{

	if [ "$EML" = "Y" ]; then
		$ECHO ""
		$ECHO "Configure the Report E-mail Recipients."       
		$ECHO "========================================= "

		cat $CONTROL_FILE | grep MAILTO | grep -v .MAILTO | awk '{print $2}' > $WRK0
		EML_STATUS=`cat $CONTROL_FILE | grep MAILTO | grep -v .MAILTO | wc | awk '{print $1}'`

		CONTINUE="N"
		if [ "$EML_STATUS" -gt "0" ]; then
			$ECHO "The following accounts have already been configured"
			for a in `cat $WRK0`
			do
				$ECHO "		"$a
			done
		fi

cat <<END

   Select the action you require:                           

     Option                                                               
     =======                                                             

     1)  Add an E-mail recipient.                                             
     2)  Remove an E-mail recipient.
     3)  Continue.
     4)  Return to main menu                                                  
     5)  Quit (q to Quit)                          

END

		$ECHO -n "Enter your choice [1-5] : "
		read ANS

		case "$ANS" in 
			"1") 
				add_eml;;
			"2") 
				remove_eml;;
			"3") 
				OK="Y";;
			"4") 
				quit_return "r" "main";;
			"5") 
				quit_return "q";;
			"r") 
				quit_return "r" "main";;
			"q") 
				quit_return "q";;
			*) 
				$ECHO $ANS" is an Invalid Choice - Try Again"
				configure_eml;;
		esac
	fi

}



#
# Add an E-mail recipient
#
add_eml()
{
	
	CONTINUE="Y"
	while [ "$CONTINUE" = "Y" ]
	do
		$ECHO ""
		$ECHO -n "Enter the email address to add (q to Quit r to Return to menu): "                        
		read EMAIL

		quit_return $EMAIL "configure_eml"

		if [ "$EMAIL" = "" ]; then
			EMAIL="NULL"
		fi

		VALID=`echo $EMAIL | grep -c "@"`
		EXIST=`cat $CONTROL_FILE | grep -c $EMAIL`

		if [ "$EMAIL" = "" -o "$VALID" -eq "0" -o "$EXIST" -ge "1" ]; then
			$ECHO "Try Again - "$EMAIL" is not a valid email address or is already configured"
			CONTINUE="Y"
		else
			$ECHO "Adding E-mail address ... "$EMAIL 
			$ECHO "MAILTO "$EMAIL >> $CONTROL_FILE
			CONTINUE="N"
			configure_eml
		fi
	done

}



#
# remove an E-mail recipient
#
remove_eml()
{
	
	EML_STATUS=`cat $CONTROL_FILE | grep MAILTO | grep -v .MAILTO | wc | awk '{print $1}'`
	CONTINUE="Y"
	while [ "$CONTINUE" = "Y" ]
	do

		if [ "$EML_STATUS" -gt "0" ]; then
		
			$ECHO ""
			$ECHO -n "Enter the email address to remove (q to Quit r to Return to menu): "                        
			read EMAIL

			quit_return $EMAIL "configure_eml"

			if [ "$EMAIL" = "" ]; then
				EMAIL="NULL"
			fi

			VALID=`echo $EMAIL | grep -c "@"`
			EXIST=`cat $CONTROL_FILE | grep -c $EMAIL`

			if [ "$EMAIL" = "" -o "$VALID" -eq "0" -o "$EXIST" -eq "0" ]; then
				$ECHO "Try Again - "$EMAIL" is not a valid email address or does not exist"
				CONTINUE="Y"
			else
				$ECHO "Removing E-mail address ... "$EMAIL 
				cat $CONTROL_FILE | grep -v $EMAIL > $WRK0
				cat $WRK0 > $CONTROL_FILE
				CONTINUE="N"
				configure_eml
			fi
		else
			$ECHO ""
			$ECHO "There is currently no E-mail account configured."
			CONTINUE="N"
			configure_eml
		fi
	done
	
}

#
# Configure Report HTML File copy locations
#
configure_cpy()
{

    if [ "$CPY" = "Y" ]; then
        $ECHO ""
        $ECHO "Configure Additional Report HTML File Locations"
        $ECHO "==============================================="

        cat $CONTROL_FILE | grep COPYTO | grep -v .COPYTO | awk '{print $2}' > $WRK0
        CPY_STATUS=`cat $CONTROL_FILE | grep COPYTO | grep -v .COPYTO | wc | awk '{print $1}'`

        CONTINUE="N"
        if [ "$CPY_STATUS" -gt "0" ]; then
            $ECHO "The following file location have already been configured"
            for a in `cat $WRK0`
            do
                $ECHO "     "$a
            done
        fi

cat <<END

   Select the action you require:

     Option
     =======

     1)  Add a new report file location.
     2)  Remove a report file location.
     3)  Continue.
     4)  Return to main menu
     5)  Quit (q to Quit)

END

        $ECHO -n "Enter your choice [1-5] : "
        read ANS

        case "$ANS" in
            "1")
                add_cpy;;
            "2")
                remove_cpy;;
			"3") 
				OK="Y";;
            "4")
                quit_return "r" "main";;
            "5")
                quit_return "q";;
            "r")
                quit_return "r" "main";;
            "q")
                quit_return "q";;
            *)
                $ECHO $ANS" is an Invalid Choice - Try Again"
                configure_cpy;;
        esac
    fi

}

#
# Add a report file location
#
add_cpy()
{

    CONTINUE="Y"
    while [ "$CONTINUE" = "Y" ]
    do
        $ECHO ""
        $ECHO -n "Enter the file location (dir) to add (q to Quit r to Return to menu): "
        read COPYDIR

        quit_return $COPYDIR "configure_cpy"

        if [ "$COPYDIR" = "" ]; then
            COPYDIR="NULL"
        fi
        
		if [[ -d $COPYDIR && -w $COPYDIR ]]; then
	        VALID="1"
		else
			VALID="0"
		fi
	EXIST=`cat $CONTROL_FILE | grep -c "COPYTO *$COPYDIR$"`

        if [ "$COPYDIR" = "" -o "$VALID" -eq "0" -o "$EXIST" -ge "1" ]; then
            $ECHO "Try Again - directory "$COPYDIR" does not exist, is not writeable or is already configured"
            CONTINUE="Y"
        else
            $ECHO "Adding directory... "$COPYDIR
            $ECHO "COPYTO "$COPYDIR >> $CONTROL_FILE
            CONTINUE="N"
            configure_cpy
        fi
    done

}

#
# remove a file report location
#
remove_cpy()
{

    CPY_STATUS=`cat $CONTROL_FILE | grep COPYTO | grep -v .COPYTO | wc | awk '{print $1}'`
    CONTINUE="Y"
    while [ "$CONTINUE" = "Y" ]
    do
        if [ "$CPY_STATUS" -gt "0" ]; then

            $ECHO ""
            $ECHO -n "Enter the file location directory to remove (q to Quit r to Return to menu): "
            read COPYDIR 

            quit_return $COPYDIR "configure_cpy"

            if [ "$COPYDIR" = "" ]; then
                COPYDIR="NULL"
            fi

            EXIST=`cat $CONTROL_FILE | grep -c $COPYDIR`

            if [ "$COPYDIR" = "" -o "$EXIST" -eq "0" ]; then
                $ECHO "Try Again - "$COPYDIR" is not valid or is not configured"
                CONTINUE="Y"
            else
                $ECHO "Removing file location directory... "$COPYDIR
                cat $CONTROL_FILE | grep -v $COPYDIR > $WRK0
                cat $WRK0 > $CONTROL_FILE
                CONTINUE="N"
                configure_cpy
            fi
        else
            $ECHO ""
            $ECHO "There are no directories configured to remove"
            CONTINUE="N"
            configure_cpy
        fi

    done

}



#
# Add a FilesSystem to Configuration File
#
add_fs()
{

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		$ECHO ""
		$ECHO -n "Enter a File System (q to Quit r to Return to menu) : "
		read FILESYSTEM

		if [ "$FILESYSTEM" = "" ]; then
			FILESYSTEM="NULL"
		fi

		quit_return $FILESYSTEM "configure_sfs_menu"

		if [ "$FILESYSTEM" != "" ]; then
			EXISTS=`cat $SFS_FILE | grep $FILESYSTEM" " | grep -v "."$FILESYSTEM" " | wc | awk '{print $1}'`
			if [ ! -d $FILESYSTEM -o "$EXISTS" -gt "0" ]; then
				$ECHO "Try Again - "$FILESYSTEM" is not a valid File System or already configured"
			else		
				OK_FLAG="Y"
			fi
		else
			OK_FLAG="Y"
		fi
	done

	if [ "$FILESYSTEM" != "" ]; then

		OK_FLAG="N"
		while [ "$OK_FLAG" = "N" ]
		do
			$ECHO ""
			$ECHO -n "Enter the %capacity threshold at which you want to be alerted [1 - 100] - default [99] : "
			read CAPACITY

			if [ "$CAPACITY" = "" ];then
				CAPACITY="99"
			fi

			validate_number $CAPACITY "1" "100"
		done

		$ECHO ""
		$ECHO "You have configured the healthcheck script to raise an exception alert when"
		$ECHO $FILESYSTEM" reaches "$CAPACITY"% utilisation"
		$ECHO $FILESYSTEM" "$CAPACITY >> $SFS_FILE
		configure_sfs_menu
	fi

}



#
# remove a Filesystem       
#
remove_fs()
{
	
	SFS_CNT=`wc $SFS_FILE | awk '{print $1}'`
	CONTINUE="Y"
	while [ "$CONTINUE" = "Y" ]
	do

		if [ "$SFS_CNT" -gt "0" ]; then
		
			$ECHO ""
			$ECHO -n "Enter the FileSystem to remove (q to Quit r to Return to menu) : "                        
			read FILESYSTEM

			quit_return $FILESYSTEM "configure_sfs_menu"

			if [ "$FILESYSTEM" = "" ]; then
				FILESYSTEM="NULL"
			fi

			EXIST=`cat $SFS_FILE | grep -c $FILESYSTEM`

			if [ "$FILESYSTEM" = "NULL" -o "$EXIST" -eq "0" ]; then
				$ECHO "Try Again - "$FILESYSTEM" is not a valid FileSystem or does not exist"
				CONTINUE="Y"
			else
				$ECHO "Removing FileSystem ... "$FILESYSTEM 
				cat $SFS_FILE | grep -v $FILESYSTEM > $WRK0
				cat $WRK0 > $SFS_FILE
				CONTINUE="N"
				configure_sfs_menu
			fi
		else
			$ECHO ""
			$ECHO "There are no FileSystems configured to remove"                                        
			CONTINUE="N"
			configure_sfs_menu
		fi
	done
	
}





#
# Add a Ruleset to Loader Data Management Configuration File
#
add_ruleset()
{

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		$ECHO ""
		$ECHO -n "Enter a Ruleset ID  (q to Quit r to Return to menu): "
		read RULESET_ID

		quit_return $RULESET_ID "main"

 		if [ "$RULESET_ID" = "" ];then
			RULESET_ID="0"
		fi

		validate_number $RULESET_ID "1" "2147483647"
	done

	if [ "$RULESET_ID" != "" ]; then

		#Loaders
		OK_FLAG="N"
		while [ "$OK_FLAG" = "N" ]
		do
			$ECHO ""
			$ECHO -n "Enter the number of metrica_load processes expected for this Ruleset (q to Quit r to Return to menu): "
			read LOADERS

			quit_return $LOADERS "main"

			if [ "$LOADERS" = "" ];then
				LOADERS="0"
			fi

			validate_number $LOADERS "1" "100"
		done

		#Lifs/Hour
		OK_FLAG="N"
		while [ "$OK_FLAG" = "N" ]
		do
			$ECHO ""
			$ECHO -n "Enter the number of Lifs/Hour expected for this Ruleset (q to Quit r to Return to menu): "
			read LIFS

			quit_return $LIFS "main"

			if [ "$LIFS" = "" ];then
				LIFS="0"
			fi

			validate_number $LIFS "1" "1000000"
		done

		#Bad Files Threshold
		OK_FLAG="N"
		while [ "$OK_FLAG" = "N" ]
		do
			$ECHO ""
			$ECHO -n "Enter the Bad File Threshold for this Ruleset (q to Quit r to Return to menu): "
			read BADS

			quit_return $BADS "main"

			if [ "$BADS" = "" ];then
				BADS="0"
			fi

			validate_number $LIFS "0" "1000000"
		done

		#Backlog Threshold
		OK_FLAG="N"
		while [ "$OK_FLAG" = "N" ]
		do
			$ECHO ""
			$ECHO -n "Enter the Backlog Threshold for this Ruleset (q to Quit r to Return to menu): "
			read BACKLOG

			quit_return $BACKLOG "main"

			if [ "$BACKLOG" = "" ];then
				BACKLOG="0"
			fi

			validate_number $LIFS "1" "1000000"
		done

		$ECHO ""
		$ECHO "You have configured the healthcheck script to raise an exception alert when"
		$ECHO "Ruleset Id "$RULESET_ID" : "
		$ECHO "-	has anything other than "$LOADERS" metrica_load processes running"
		$ECHO "-	is not processing within 90% of "$LIFS" Lif files per/hour"
		$ECHO "-	has rejected any more than "$BADS" Lif files"
		$ECHO "-	has a backlog of Lif files exceeding "$BACKLOG
		$ECHO $RULESET_ID"	"$LOADERS"	"$LIFS"	"$BADS"	"$BACKLOG >> $LDM_FILE
		configure_ldm_menu
	fi

}



#
# Remove a Ruleset       
#
remove_ruleset()
{
	
	LDM_CNT=`wc $LDM_FILE | awk '{print $1}'`
	CONTINUE="Y"
	while [ "$CONTINUE" = "Y" ]
	do

		if [ "$LDM_CNT" -gt "0" ]; then
		
			$ECHO ""
			$ECHO -n "Enter the Ruleset Id to remove (q to Quit r to Return to menu) : "                        
			read RULESET_ID

			quit_return $RULESET_ID "configure_ldm_menu"

			if [ "$RULESET_ID" = "" ]; then
				RULESET_ID="NULL"
			fi

			EXIST="N"
			CNT0="0"
			for a in `cat $LDM_FILE`
			do
				CNT0=`expr $CNT0 + 1 `
				if [ "$CNT0" -eq "1" -a "$a" = "$RULESET_ID" ]; then
					EXIST="Y"
				fi

				if [ "$CNT0" -eq "5" ]; then
					CNT0="0"
				fi
			done

			if [ "$RULESET_ID" = "NULL" -o "$EXIST" = "N" ]; then
				$ECHO "Try Again - "$RULESET_ID" is not a valid Ruleset Id or does not exist"
				CONTINUE="Y"
			else
				RULESET_ID=$RULESET_ID" "
				$ECHO "Removing Ruleset Id ... "$RULESET_ID 
				cat $LDM_FILE | grep -v $RULESET_ID > $WRK0
				cat $WRK0 > $LDM_FILE
				CONTINUE="N"
				configure_ldm_menu
			fi
		else
			$ECHO ""
			$ECHO "There are no Ruleset Ids configured to remove"                                        
			CONTINUE="N"
			configure_ldm_menu
		fi
	done
	
}



#
# Add a Gateway Definition to Gateways Configuration File
#
add_gateway()
{

	$ECHO ""
	$ECHO -n "Enter a Gateway Name (q to Quit r to Return to menu):"
	read GATEWAY_NAME

	quit_return $GATEWAY_NAME "main"

	if [ "$GATEWAY_NAME" != "" ]; then
		$ECHO "GATEWAY,"$GATEWAY_NAME > $WRK0

		#THRESHOLD
		FIRST_TIME="Y"
		MORE_THRESHOLDS="Y"
		while [ "$MORE_THRESHOLDS" = "Y" ]
		do
			if [ "$FIRST_TIME" = "Y" ]; then
				FIRST_TIME="N"
				get_gateway_threshold
				get_gateway_operand
				get_gateway_value

				$ECHO "THRESHOLD,"$THRESHOLD","$OPERAND","$VALUE >> $WRK0
			fi

			$ECHO ""
			$ECHO "So far you have added the following threshold(s) to the configuration file ....."
			$ECHO ""
			cat $WRK0 
			$ECHO ""
			$ECHO -n "Do you want to add another ? [Y/N] -default [Y] : "
			read ANS

			if [ "$ANS" = "" ]; then
				ANS="Y"
			fi

			if [ "$ANS" = "Y" -o "$ANS" = "y" ]; then
				get_gateway_threshold
				get_gateway_operand
				get_gateway_value

				$ECHO "THRESHOLD,"$THRESHOLD","$OPERAND","$VALUE >> $WRK0
			else
					MORE_THRESHOLDS="N"
			fi

		done

		OK_FLAG="N"
		while [ "$OK_FLAG" = "N" ]
		do
			$ECHO ""
			$ECHO "Enter the full path to the file statistics directory" 
			$ECHO -n "for Gateway "$GATEWAY_NAME" (q to Quit r to Return to menu) : " 
			read FILE_STATS_DIR

			quit_return $FILE_STATS_DIR "main"

			if [ "$FILE_STATS_DIR" = "" -o ! -d $FILE_STATS_DIR ];then
				$ECHO "Try again "$FILE_STATS_DIR" is not a valid directory"
				OK_FLAG="N"
			else
				OK_FLAG="Y"
			fi

		done

		cat $WRK0 >> $GFS_FILE
		$ECHO "FILE_STATS_DIR,"$FILE_STATS_DIR >> $GFS_FILE
	fi

}
 


get_gateway_threshold()
{

	VALID_FLAG="N"
	while [ "$VALID_FLAG" = "N" ]
	do
	VALID_FLAG="Y"
cat <<END
	Select a threshold  type for Gateway $GATEWAY_NAME 

	  1)  rawFileSizeTotal - the total size of the raw input files parsed
	  2)  numberFilesParsedOK - the number of raw files parsed successfully
	  3)  numberFilesParsedBad - the number of raw files which were bad (could not be parsed)
	  4)  numberFilesParsedBadZeroSize - the number of raw files which were bad due to zero size
	  5)  numberLIFFiles - the total number of LIF files produced
	  6)  lifFileSizeTotal - the total size of LIF files produced
	  7)  totalParsingTime - the total time in seconds spent parsing the data.
	  8)  backlogMeasurement_0_30 - the number of raw files in backlog from 0 to 30 minutes
	  9)  backlogMeasurement_30_60 - the number of raw files in backlog from 30 to 60 minutes 
	 10)  backlogMeasurement_60_90  - the number of raw files in backlog from 60 to 90 minutes
	 11)  backlogMeasurement_90_120 - the number of raw files in backlog from 90 to 120 minutes
	 12)  backlogMeasurement_120_180  - the number of raw files in backlog from 120 to 180 minutes
	 13)  backlogMeasurement_180_240  - the number of raw files in backlog from 180 to 240 minutes
	 14)  backlogMeasurement_240_1440  - the number of raw files in backlog from 240 to 1440 minutes
	 15)  backlogMeasurementGreater_1440  - the number of raw files in backlog greater than 1440 minutes
	 16)  Return to main menu
	 17)  Quit (q to Quit)

END
		$ECHO -n "Enter your choice [1-17] : "
		read CHOICE

		case "$CHOICE" in 
			"1") 
				THRESHOLD="rawFileSizeTotal";;
			"2") 
				THRESHOLD="numberFilesParsedOK";;
			"3") 
				THRESHOLD="numberFilesParsedBad";;
			"4") 
				THRESHOLD="numberFilesParsedBadZeroSize";;
			"5") 
				THRESHOLD="numberLIFFiles";;
			"6") 
				THRESHOLD="lifFileSizeTotal";;
			"7") 
				THRESHOLD="totalParsingTime";;
			"8") 
				THRESHOLD="backlogMeasurement_0_30";;
			"9") 
				THRESHOLD="backlogMeasurement_30_60";;
			"10") 
				THRESHOLD="backlogMeasurement_60_90";;
			"11") 
				THRESHOLD="backlogMeasurement_90_120";;
			"12") 
				THRESHOLD="backlogMeasurement_120_180";;
			"13") 
				THRESHOLD="backlogMeasurement_180_240";;
			"14") 
				THRESHOLD="backlogMeasurement_240_1440";;
			"15") 
				THRESHOLD="backlogMeasurementGreater_1440";;
			"16")
				quit_return "r" "main";;
			"17")
				quit_return "q";;
			"r")
				quit_return "r" "main";;
			"q")
				quit_return "q";;
			  *) 
				VALID_FLAG="N"
				$ECHO "Invalid Selection" ;sleep 1 ;;
		esac 

	done

}



#
# Get Gateway Operand
#
get_gateway_operand()
{

	VALID_FLAG="N"
	while [ "$VALID_FLAG" = "N" ]
	do
	VALID_FLAG="Y"
cat <<END
	Select an operand for Threshold type $THRESHOLD 

	  1)  GT - greater than                                                 
	  2)  LT - less than                                                    
	  3)  EQ - equal to                                                                        
	  4)  NE - not equal to                                                                      
	  5)  Return to main menu
	  6)  Quit (q to Quit)

END
		$ECHO -n "Enter your choice [1-6] : "
		read CHOICE

		case "$CHOICE" in 
			"1") 
				OPERAND="GT";;
			"2") 
				OPERAND="LT";;
			"3") 
				OPERAND="EQ";;
			"4") 
				OPERAND="NE";;
			"5")
				quit_return "r" "main";;
			"6")
				quit_return "q";;
			"r")
				quit_return "r" "main";;
			"q")
				quit_return "q";;
			*) 
			VALID_FLAG="N"
			$ECHO "Invalid Selection" ;sleep 1 ;;
		esac 

	done

}



#
# Get Gateway Value
#
get_gateway_value()
{

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		$ECHO ""
		$ECHO "Enter the value to be compared for threshold "
		$ECHO -n $THRESHOLD" [0 - 100000] - default [0] (q to Quit r to Return to menu) : "
		read VALUE

		quit_return $VALUE "main"

		if [ "$VALUE" = "" ];then
			VALUE="0"
		fi

		validate_number $VALUE "0" "100000"
	done

}



#
# Add Summariser Thresholds
#
add_summary_thresholds()
{

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		$ECHO ""
		$ECHO "Enter a number for Failed Summaries threshold  "
		$ECHO -n "over which you want to be alerted [0 - 100] - default [0] (q to Quit r to Return to menu): "
		read FAILED_THRESHOLD

		quit_return $FAILED_THRESHOLD "main"

		if [ "$FAILED_THRESHOLD" = "" ];then
			FAILED_THRESHOLD="0"
		fi

		validate_number $FAILED_THRESHOLD "0" "100"
	done

	$ECHO ""
	$ECHO "You have configured the healthcheck script to raise an exception alert when"
	$ECHO "The number of Failed Summaries exceeds "$FAILED_THRESHOLD

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		$ECHO ""
		$ECHO "Enter a number for Queued Summaries threshold  "
		$ECHO -n "over which you want to be alerted [0 - 100] - default [0] (q to Quit r to Return to menu): "
		read QUEUED_THRESHOLD

		quit_return $QUEUED_THRESHOLD "main"

		if [ "$QUEUED_THRESHOLD" = "" ];then
			QUEUED_THRESHOLD="0"
		fi

		validate_number $QUEUED_THRESHOLD "0" "100"
	done

	$ECHO ""
	$ECHO "You have configured the healthcheck script to raise an exception alert when"
	$ECHO "The number of Queued Summaries exceeds "$QUEUED_THRESHOLD
	$ECHO "FAILED "$FAILED_THRESHOLD > $SUM_FILE
	$ECHO "QUEUED "$QUEUED_THRESHOLD >> $SUM_FILE

}



#
# Configure Metrica Unix Mail Check
#
configure_mum()
{

	if [ "$MUM" = "Y" ]; then
		$ECHO ""
		$ECHO "Configure the Metrica Unix Mail Check."       
		$ECHO "===================================== "
		$ECHO ""
		$ECHO "The current status of the Metrica Unix Mail Check is $MUM_STATUS_DESC"

		CONTINUE="N"

cat <<END

   Select the action you require:                           

     Option                                                               
     =======                                                             

     1)  Set New Mail Count OFF, Mail Headers OFF
     2)  Set New Mail Count  ON, Mail Headers OFF
     3)  Set New Mail Count  ON, Mail Headers  ON
     4)  Continue.
     5)  Return to main menu                                                  
     6)  Quit (q to Quit)                          

END

		$ECHO -n "Enter your choice [1-6] : "
		read ANS

		case "$ANS" in 
			"1") 
		        if [ "$MUM_STATUS" != "0" ]; then
		            cat $CONTROL_FILE | grep -v metrica_check_metrica_mail > $WRK0
		            $ECHO "metrica_check_metrica_mail	0" >> $WRK0
		            cat $WRK0 > $CONTROL_FILE
					MUM_STATUS=0
					MUM_STATUS_DESC="New Mail Count OFF, Mail Headers OFF"
					$ECHO "Status changed to $MUM_STATUS_DESC"; sleep 2
				else
					$ECHO "Status remains as $MUM_STATUS_DESC"; sleep 2
				fi
				configure_mum;;
			"2") 
		        if [ "$MUM_STATUS" != "1" ]; then
		            cat $CONTROL_FILE | grep -v metrica_check_metrica_mail > $WRK0
		            $ECHO "metrica_check_metrica_mail	1" >> $WRK0
		            cat $WRK0 > $CONTROL_FILE
					MUM_STATUS=1
					MUM_STATUS_DESC="New Mail Count ON, Mail Headers OFF"
					$ECHO "Status changed to $MUM_STATUS_DESC"; sleep 2
				else
					$ECHO "Status remains as $MUM_STATUS_DESC"; sleep 2
				fi
				configure_mum;;
			"3") 
		        if [ "$MUM_STATUS" != "2" ]; then
		            cat $CONTROL_FILE | grep -v metrica_check_metrica_mail > $WRK0
		            $ECHO "metrica_check_metrica_mail	2" >> $WRK0
		            cat $WRK0 > $CONTROL_FILE
					MUM_STATUS=1
					MUM_STATUS_DESC="New Mail Count ON, Mail Headers ON"
					$ECHO "Status changed to $MUM_STATUS_DESC"; sleep 2
				else
					$ECHO "Status remains as $MUM_STATUS_DESC"; sleep 2
				fi
				configure_mum;;
			"4") 
				OK="Y";;
			"5") 
				quit_return "r" "main";;
			"6") 
				quit_return "q";;
			"r") 
				quit_return "r" "main";;
			"q") 
				quit_return "q";;
			*) 
				$ECHO $ANS" is an Invalid Choice - Try Again"
				configure_mum;;
		esac
	fi
}



#
# Configure Informix Unix Mail Check
#
configure_ium()
{

	if [ "$IUM" = "Y" ]; then
		IDS_PERMS=`ls -l $METRICA_DIR/SystemManagement/bin/metrica_check_informix_mail.ksh | awk '{print $1}'`
		if [ "$IDS_PERMS" != "-rwsr-xr-x" ]; then
			chown informix:informix $METRICA_DIR/SystemManagement/bin/metrica_check_informix_mail.ksh 2> /dev/null
			if [ ! -f /var/mail/informix ]; then
				touch /var/mail/informix
				chmod 660 /var/mail/informix
				chown informix:mail /var/mail/informix
			fi
			$ECHO "Please enter informix user password"
			su informix -c "chmod 4755 $METRICA_DIR/SystemManagement/bin/metrica_check_informix_mail.ksh ; chmod o+r /var/mail/informix"
			RC=$?
		else
			RC=0
		fi
		if [ "$RC" = "0" ]; then
			$ECHO ""
			$ECHO "Configure the Informix Unix Mail Check."       
			$ECHO "====================================== "
			$ECHO ""
			$ECHO "The current status of the Informix Unix Mail Check is $IUM_STATUS_DESC"

cat <<END

   Select the action you require:                           

     Option                                                               
     =======                                                             

     1)  Set New Mail Count OFF, Mail Headers OFF
     2)  Set New Mail Count  ON, Mail Headers OFF
     3)  Set New Mail Count  ON, Mail Headers  ON
     4)  Continue.
     5)  Return to main menu                                                  
     6)  Quit (q to Quit)                          

END

			$ECHO -n "Enter your choice [1-6] : "
			read ANS

			case "$ANS" in 
				"1") 
			        if [ "$IUM_STATUS" != "0" ]; then
			            cat $CONTROL_FILE | grep -v metrica_check_informix_mail > $WRK0
			            $ECHO "metrica_check_informix_mail	0" >> $WRK0
			            cat $WRK0 > $CONTROL_FILE
						IUM_STATUS=0
						IUM_STATUS_DESC="New Mail Count OFF, Mail Headers OFF"
						$ECHO "Status changed to $IUM_STATUS_DESC"; sleep 2
					else
						$ECHO "Status remains as $IUM_STATUS_DESC"; sleep 2
					fi
					configure_ium ;;
				"2") 
			        if [ "$IUM_STATUS" != "1" ]; then
			            cat $CONTROL_FILE | grep -v metrica_check_informix_mail > $WRK0
			            $ECHO "metrica_check_informix_mail	1" >> $WRK0
			            cat $WRK0 > $CONTROL_FILE
						IUM_STATUS=1
						IUM_STATUS_DESC="New Mail Count ON, Mail Headers OFF"
						$ECHO "Status changed to $IUM_STATUS_DESC"; sleep 2
					else
						$ECHO "Status remains as $IUM_STATUS_DESC"; sleep 2
					fi
					configure_ium ;;
				"3") 
			        if [ "$IUM_STATUS" != "2" ]; then
			            cat $CONTROL_FILE | grep -v metrica_check_informix_mail > $WRK0
			            $ECHO "metrica_check_informix_mail	2" >> $WRK0
			            cat $WRK0 > $CONTROL_FILE
						IUM_STATUS=2
						IUM_STATUS_DESC="New Mail Count ON, Mail Headers ON"
						$ECHO "Status changed to $IUM_STATUS_DESC"; sleep 2
					else
						$ECHO "Status remains as $IUM_STATUS_DESC"; sleep 2
					fi
					configure_ium ;;
				"4") 
					OK="Y";;
				"5") 
					quit_return "r" "main";;
				"6") 
					quit_return "q";;
				"r") 
					quit_return "r" "main";;
				"q") 
					quit_return "q";;
				*) 
					$ECHO $ANS" is an Invalid Choice - Try Again"
					configure_ium;;
			esac
		else
			$ECHO "Invalid password"; sleep 2
		fi
	fi

}


#
# Check that configuration File exists
#
check_config_exist()
{

	#Only continue if selection is enabled
	CONTINUE="N"
	if [ "$STATUS" = "Enabled" ]; then
		if [ -f $1 ]; then
			$ECHO "The configuration file "$1" already exists"

			if [ "$2" -eq "2" ]; then
				display_conf_2 "Parameter" "Value" $1
			elif [ "$2" -eq "5" ]; then
				display_conf_5 "Ruleset_Id" "Loaders" "Lifs/Hour" "Bad_Files" "Backlog" $1
			else
				cat $1
			fi

			$ECHO -n "Do you want to overwrite it [Y/N] - default [N] (q to Quit r to Return to menu): "                        
			read OVERWRITE

			quit_return $OVERWRITE "main"

			if [ "$OVERWRITE" = "" ]; then
				OVERWRITE="N"
			fi

			if [ "$OVERWRITE" = "Y" -o "$OVERWRITE" = "y" ]; then
				$ECHO "" > $1
				OVERWRITE="Y"
				CONTINUE="Y"
			fi

			if [ "$3" = "APPEND_ASK" -a "$OVERWRITE" != "Y" ]; then
				$ECHO -n "Do you want to append to the existing file [Y/N] - default [Y] (q to Quit r to Return to menu): "
				read APPEND

				quit_return $APPEND "main"

				if [ "$APPEND" = "" ]; then
					APPEND="Y"
				fi

				if [ "$APPEND" = "Y" -o "$APPEND" = "y" ]; then
					CONTINUE="Y"
				fi
			fi
	 	else
		 	touch $1
		 	CONTINUE="Y"
	 	fi
	fi

}



#
# Display Configuration Files with 2 elements
#
display_conf_2()
{

	printf "%-20s\t%-20s\n" $1 $2 
	printf "%-20s\t%-20s\n" "=================" "============"
	cat $3 | grep -v "#" | grep -v ^$ > $WRK1

	CNT0="0"
	for a in `cat $WRK1`
	do
		CNT0=`expr $CNT0 + 1 `

		if [ "$CNT0" -eq "1" ]; then
			DISP_1=$a
		fi

		if [ "$CNT0" -eq "2" ]; then
			DISP_2=$a
			CNT0="0"
			printf "%-20s\t%-20s\n" $DISP_1 $DISP_2 
		fi
	done

}



#
# Display Configuration Files with 5 elements
#
display_conf_5()
{

	printf "%-13s\t%-13s\t%-13s\t%-13s\t%-13s\n" $1 $2 $3 $4 $5 
	UL="============"
	printf "%-13s\t%-13s\t%-13s\t%-13s\t%-13s\n" $UL $UL $UL $UL $UL 
	cat $6 | grep -v "#" | grep -v ^$ > $WRK1

	CNT0="0"
	for a in `cat $WRK1`
	do
		CNT0=`expr $CNT0 + 1 `

		if [ "$CNT0" -eq "1" ]; then
			DISP_1=$a
		fi

		if [ "$CNT0" -eq "2" ]; then
			DISP_2=$a
		fi

		if [ "$CNT0" -eq "3" ]; then
			DISP_3=$a
		fi

		if [ "$CNT0" -eq "4" ]; then
			DISP_4=$a
		fi

		if [ "$CNT0" -eq "5" ]; then
			DISP_5=$a
			CNT0="0"
			printf "%-13s\t%-13s\t%-13s\t%-13s\t%-13s\n" $DISP_1 $DISP_2 $DISP_3 $DISP_4 $DISP_5 
		#	printf "%-36s\t%2.0f\t%-1s\n" $DISP_1 $DISP_2 $4
		fi
	done

}

validate_number()
{

	ALPHA="N"
	[[ $1 != +([0-9]) ]] && ALPHA="Y"
	if [ "$ALPHA" = "Y" ];then
		$ECHO "Try Again - "$1" is an invalid value must be an integer between "$2" and "$3
		OK_FLAG="N"
	elif [ "$1" -lt "$2" -o "$1" -gt "$3" ];then
		$ECHO "Try Again - "$1" is an invalid value must be an integer between "$2" and "$3
		OK_FLAG="N"
	else
		OK_FLAG="Y"
	fi
}



get_onconfig_parm()
{


	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
	DEFAULT=`onstat -c | grep $1 | grep -v "."$1 | awk '{print $2}'`
	$ECHO ""
	$ECHO -n "Enter the comparison value for "$1" ["$2" through "$3"] - default ["$DEFAULT"] : "
	read PARAMETER

	if [ "$PARAMETER" = "" ]; then
		PARAMETER=$DEFAULT
	fi

	validate_number $PARAMETER $2 $3
	done
	$ECHO $1"	"$PARAMETER >> $ICP_FILE

}



get_vpclass_parm()
{

	#Get VPCLASS
	DEFAULT=`onstat -c | grep $1 | grep -v "."$1 | grep $2 | awk '{print $2}'`

	if [ "$DEFAULT" = "" ]; then

		#Determining Number of CPUs
		ARCH=`uname`
			case $ARCH in
				HP-UX)
					CPUS=`/etc/ioscan -k -C processor | grep processor | wc -l`
				;;
				SunOS)
					CPUS=`/usr/bin/mpstat | grep -v CPU | wc | awk '{print $1}'`
				;;
			esac

		if [ "$CPUS" -gt 1 ]; then
			CPUS=`expr $CPUS - 1`
		fi

		DEFAULT=$2",num="$CPUS",noage"
	fi

	$ECHO ""
	$ECHO -n "Enter the VPCLASS parameter specification comparison value for the "$2" class - default ["$DEFAULT"] : "
	read PARAMETER

	if [ "$PARAMETER" = "" ]; then
	PARAMETER=$DEFAULT
	fi

	$ECHO $1"   "$PARAMETER >> $ICP_FILE


}
  


#
# Routine to Get Current value and validate entered
#
get_hp_kernel_parm()
{

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		DEFAULT=`cat /stand/system | grep $1 | grep -v "."$1 | awk '{print $2}'`
		if [ "$DEFAULT" = "" ]; then
			$ECHO "The Kernel Parameter "$1" is not set on this system"
			DEFAULT="1"
		fi

		$ECHO ""
		$ECHO -n "Enter the comparison value for "$1" - default ["$DEFAULT"] : "
		read PARAMETER

		if [ "$PARAMETER" = "" ]; then
			PARAMETER=$DEFAULT
		fi

			OK_FLAG="Y"
	done
	$ECHO "Kernel Parameter "$1"="$PARAMETER 
	$ECHO $1" "$PARAMETER >> $SKP_FILE

}



#
# Routine to Get Current value and validate entered
#
get_sun_kernel_parm()
{

	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do
		DEFAULT=`cat /etc/system | grep $1 | grep -v "."set | awk '{print $2}' | sed s/${1}//1 | sed s/=//1`
		if [ "$DEFAULT" = "" ]; then
			$ECHO "The Kernel Parameter "$1" is not set on this system"
			DEFAULT="1"
		fi

		$ECHO ""
		$ECHO -n "Enter the comparison value for "$1" - default ["$DEFAULT"] : "
		read PARAMETER

		if [ "$PARAMETER" = "" ]; then
			PARAMETER=$DEFAULT
		fi

		if [ "$2" = "Y" ]; then
			validate_number $PARAMETER "0" "2147483647"
		else
			OK_FLAG="Y"
		fi

	done
	$ECHO "Kernel Parameter "$1"="$PARAMETER 
	$ECHO $1"="$PARAMETER >> $SKP_FILE

}


modify_metrica_mail()
{

	#Get Capacity Threshold

	$ECHO ""
	$ECHO "The Capacity Threshold is the % Dbspace Utilisation at which the script should alert"
	OK_FLAG="N"
	while [ "$OK_FLAG" = "N" ]
	do

		$ECHO ""
		$ECHO "Enter the Capacity Threshold required  "                  
		$ECHO -n "[between 1 and 99] - default [95] (q to Quit r to Return to menu) : "                  
		read CAPACITY_THRESHOLD

		quit_return $CAPACITY_THRESHOLD "configure_ids_menu"

		if [ "$CAPACITY_THRESHOLD" = "" ]; then
			CAPACITY_THRESHOLD="95"
		fi

		validate_number $CAPACITY_THRESHOLD 1 99
	done
	cat $IDS_FILE | grep -v "CAPACITY_THRESHOLD" > $WRK0
	$ECHO "CAPACITY_THRESHOLD	"$CAPACITY_THRESHOLD >> $WRK0
	cat $WRK0 > $IDS_FILE
	configure_ids_menu

}

#
# Print Completion messages
#
completion()
{

	PROG=$METRICA_DIR/SystemManagement/bin/metrica_healthcheck.ksh
	LOG=$METRICA_DIR/logs/checks/metrica_healthcheck.log
	CRONTAB=/var/spool/cron/crontabs/$LOGNAME

cat <<END 
Component                                               Status
==========                                              =======
Informix Dynamic Server Checking Tool.                  $IDS_STATUS
Informix Configuration Parameters Monitor.              $ICP_STATUS
Server Kernel Parameters Monitor.                       $SKP_STATUS
Server File System Capacity Checking Tool.              $SFS_STATUS
Metrica Loader Data Management Checking Script.         $LDM_STATUS
Gateways File Statistics Checking Script.               $GFS_STATUS
Appserver Checking Script.                              $APP_STATUS
Summariser Checking Script.                             $SUM_STATUS
Metrica Unix Mail Check                                 $MUM_STATUS_DESC
Informix Unix Mail Check                                $IUM_STATUS_DESC

The report will be mailed to:  
END
cat $CONTROL_FILE | grep MAILTO | sed s/MAILTO/"		"/g
cat <<END 

The metrica_healthcheck.ksh script can be run from the command line 
or alternatively from the cron scheduler.
A line to run this job must be added to the crontab file for user $LOGNAME such as
30 6-17 * * * METRICA_DIR=$METRICA_DIR; export METRICA_DIR; $PROG > $LOG 2>&1

Performance Manager Health Check Configuration Complete.
END

}



#
# Main Routine
#
main()
{

	clean_up
	build_environment

	while [ "$RERUN" = "Y" ]
	do

	#Display Menu and get choice
	get_config_choice

	#Configure Informix Dynamic Server Health Check Script
	configure_ids

	#Configure Informix Configuration Parameters Comparison Tool      
	configure_icp

	#Configure Server Kernel Parameters Comparison Tool    
	case $ARCH in
		HP-UX)
				configure_skp_hp
			;;
			SunOS)
				configure_skp_sun
			;;
	esac

	#Configure Server File System Capacity Check Tool    
	configure_sfs

	#Configure Metrica Loader Data Management Check Script
	configure_ldm

	#Configure Gateway Fileamic Server Health Check Script
	configure_gfs

	#Configure AppServer Health Check Script
	configure_app

	#Configure Summariser Health Check Script
	configure_sum

	#Configure Report E-mail Recipients       
	clear
	configure_eml

	#Configure Report Destination Directory
    clear
	configure_cpy

	#Configure Metrica Unix Mail Check
    clear
	configure_mum

	#Configure Informix Mail Check
    clear
	configure_ium

	clean_up

	done

}



case $1 in
        # Help options
        "-h" | "-help" | "--help")
                usage
                exit 0
        ;;

        # Version options
        "-v" | "-version" | "--version")
                version
                exit 0
        ;;

        # Any other option
        *)
				RERUN="Y"
                main
                exit 0
        ;;
esac
