#!/usr/bin/perl
############################################################
#
# File: metrica_cpu_speed.pl
#
# Description: determines the clock frequency in MHz
# of a Sun Solaris or a HP-UX UNIX server                              
#
# Author: Barry Deignan
#
# Version: @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_cpu_speed.pl SysManagement_r4.79.0.9:aset.008275:1:1 02/23/05 @(#)
#
############################################################
use Time::localtime;
use Getopt::Long;

# version info from TC
my $version = "@(#) SysManagement/delivery/utils/bin/Monitoring/metrica_cpu_speed.pl SysManagement_r4.79.0.9:aset.008275:1:1 02/23/05 @(#)";

$Getopt::Long::ignorecase=1;
$Getopt::Long::autoabbrev=1;

my ($opt_l, $opt_c, $opt_v, $opt_h, $opt_d);

# handle both long and short options from the command line
my $result = GetOptions
     ("" => \$opt_n,
	 "v!" => \$opt_v,
     "version!" => \$opt_v,
     "h!" => \$opt_h,
     "help!" => \$opt_h);

if ($opt_h){
    print_help();
    exit;
}

if ($opt_v){
    print_version();
    exit;
}

# The main body of work, check the architecture          
main();


############################################################
#
# sub: main
#
# desc: The main functionality of the script, check the     
# architecture of the server and in turn determine the CPU   
# clock speed                                       
#
############################################################
sub main{

    # load the configuration
	$arch=`uname`;
	if ( $arch =~ /HP-UX/ ) {
		get_hp(); }
	elsif ( $arch =~ /SunOS/ ) { 
		get_sun(); }
	else {
    	print_help(); }

}


############################################################
#
# sub: get_hp
#
# desc: Determine CPU clock speed for HP-UX             
#
############################################################
sub get_hp{

	local($PSTAT, $PSTAT_PROCESSOR) = (239, 10);        # pstat_getprocessor
	local($struct_pst_processor) = ("L30");
	local($cpu_info, $cpu_ticks);

	$cpu_info = "\0" x 120;
	syscall($PSTAT, $PSTAT_PROCESSOR, $cpu_info, length($cpu_info), 1, 0);

	($cpu_ticks) = (unpack($struct_pst_processor, $cpu_info))[26];

	print $cpu_ticks/10000 . "\n";

}

############################################################
#
# sub: get_sun
#
# desc: Determine the CPU clock speed for SUN      
#
############################################################
sub get_sun{
    
	$cpu_ticks=`/usr/sbin/psrinfo -v | grep operates | tail -1 | awk '{print \$6}'`;
	print $cpu_ticks;
    
}

############################################################
#
# sub: usage
#
# desc: Prints the usage information 
#
############################################################
sub usage{
    return "usage is metrica_cpu_speed.pl " . 
        "(-h|-v) \n";
}

############################################################
#
# sub; print_help
#
# desc: Prints out the help information
#
############################################################
sub print_help {
    my @HELP_MESSAGE = 
        (
         "metrica_cpu_speed.pl is a healthcheck tool used to determine the ",
         "CPU clock speed in MHz for both HP-UX and SUN Solaris servers    ", 
         );
    
    print "\n" . join ("\n",@HELP_MESSAGE,"","") ;
    print usage();
}

############################################################
#
# sub: print_version
#
# desc: Prints out the version information, which must be
# setup by setting "keyword expansion on" in Truechange.
############################################################
sub print_version {
    print "metrica_cpu_speed.pl version @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_cpu_speed.pl SysManagement_r4.79.0.9:aset.008275:1:1 02/23/05 @(#)\n";
}
