#!/bin/ksh
# @(#) SysManagement/delivery/utils/bin/Monitoring/metrica_healthcheck.ksh SysManagement_r4.79.0.9:cset.008544:15:15 12/19/07 @(#)
###############################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
# Title:        metrica_healthcheck.ksh
# Author:       Barry Deignan
# Description:  Script to run a suite of Performance manager healthcheck scripts    
#               and mail the resulting report to configured admin personnel.                
#
###############################################################################

#Set Environment
PROGNAME=`basename $0`
HOSTNAME=`uname -n`
VERNO=SysManagement_r4.79.0.9
DATE=`date +%d%b%Y.%H.%M'`
DATEXT=`date +%d-%b-%Y:%H:%M'`
REPORT=/tmp/metrica_healthcheck_$DATE.html
LOCK=/tmp/metrica_healthcheck.lock




#
# Check environment variables
#
build_environment()
{
	if [ ! -d "$METRICA_DIR" -o ! -f $METRICA_DIR"/setup.ksh" ];then
		REPORT=/tmp/metrica_healthcheck_$DATE.html
		header
		echo "- EXCEPTION ALERT - METRICA_DIR not defined or setup.ksh not found"
		echo "- Script Aborted"
		footer
		exit 1
	else
		. $METRICA_DIR/setup.ksh
		REPORT=$METRICA_DIR/logs/checks/metrica_healthcheck_$DATE.html
		REPORT_DIR=$METRICA_DIR/logs/checks
		CSV_FILE=$REPORT_DIR/metrica_healthcheck.csv
		if [ ! -d $REPORT_DIR ]; then
			mkdir -p $REPORT_DIR
		fi
		SCRIPT_DIR=$METRICA_DIR/SystemManagement/bin

		CONF=$METRICA_DIR/SystemManagement/control/metrica_healthcheck.conf
		IDS_CONF=$METRICA_DIR/SystemManagement/control/metrica_check_ids.conf
		EXCEPTION_COUNT=0
		WRK0=/tmp/work0$$
		WRK1=/tmp/work1$$
		if [ ! -f $WRK0 ];then
			touch $WRK0
		fi
		if [ ! -f $WRK1 ];then
			touch $WRK1
		fi
	fi
}



#
# Check if the script is already running
#
check_running()
{

	REMOVE_LOCK="Y"
	ps -ef | grep -v grep | grep -v $$ | /usr/xpg4/bin/grep -q metrica_healthcheck
	RC=$?
	if [ "$RC" -eq "0" ] && [ -f $LOCK ]; then
		REPORT=/tmp/metrica_healthcheck_$DATE.html
		header
		echo "- EXCEPTION ALERT - The Performance Manager Health Check process is already running"
		echo "- and the "$LOCK" file exists"
		echo "- Script Aborted"
		REMOVE_LOCK="N"
		footer
		exit 2
	else
		touch $LOCK
	fi
}
		
 
   

informix_up() 
{

	onstat - > /dev/null 
	ISUP=`echo $?`

	if [ "$ISUP" = "5" ];then
		OK="Y"
		IDS_VERSION=`onstat -g pos | grep oninit | awk '{print $11}'`
	else
		REPORT=/tmp/metrica_healthcheck_$DATE.html
		header
		report_error "- EXCEPTION ALERT - Informix Dynamic Server appears to be down"
		footer
		exit 3
	fi

}
 

check_metrica_user()
{

	if [ $(whoami) != "metrica" ]; then
		REPORT=/tmp/metrica_healthcheck_$DATE.html
		header
		echo "- EXCEPTION ALERT - The health check must be run by the metrica user"
		echo "- Script Aborted"
		footer
		exit 0
	fi

}


#
# Print file description and copyright information
#
show_info()
{
cat <<END

Metrica Performance Manager installed subsystem information script
Copyright  2005 Vallent Software Systems

END
}




#
# Show correct command-line usage
#
usage()
{
show_info
cat <<END

  Usage: $PROGNAME [options]

        options:    [ -h | -help | --help ]
                  | [ -v | -version | --version ]

  Script to run a suite of Performance manager healthcheck scripts    
  and mail the resulting report to configured admin personnel.                


END
}
 


version()
{
	show_info
 	echo "$PROGNAME SysManagement/delivery/utils/bin/Monitoring/metrica_healthcheck.ksh SysManagement_r4.79.0.9:cset.008544:15:15 12/19/07"
	exit 0
}
 

 
#
#
#
get_stats()
{

	echo "Determining CPU and Memory Stats ..."
	ARCH=`uname`
		case $ARCH in
				HP-UX)
						get_hp_stats
						;;
				SunOS)
						get_sun_stats
						;;
		esac

}



#
# Get CPU utilisation, Memory Size and Memory Free for HP
#
get_hp_stats()
{

	MODEL=`model`
	OS=$ARCH" "`uname -r`

	CPU_LABEL1="CPUs:"
	CPUS=`/etc/ioscan -k -C processor | grep processor | wc -l`
	if [ -f $METRICA_DIR/SystemManagement/bin/metrica_cpu_speed.pl ]; then
		SPEED=`$METRICA_DIR/SystemManagement/bin/metrica_cpu_speed.pl`
		CPUS=$CPUS" x "$SPEED" MHz"
	fi

	CPU_LABEL2="Current CPU Util:"
	IDLE=`/usr/bin/sar -u 1 5 | grep "Average" | awk '{print $5}'`
	UTILISATION=`expr 100 - $IDLE`

	MEMORY_LABEL1="Memory Size:"
	MEMORY_LABEL2="Memory Free:"
	cat /var/adm/syslog/syslog.log | grep "Physical" > $WRK0
	MEMORY_SIZE=`tail -1 $WRK0 | awk '{print $7}'`
	UNITS=`tail -1 $WRK0 | awk '{print $8}' | sed s/,//1`
	/usr/bin/vmstat 1 5 > $WRK0
	FREE_MEMORY=`tail -1 $WRK0 | awk '{print $5}'`
	FREE_MEMORY=`expr $FREE_MEMORY \* 4 `

}



#
# Get CPU utilisation, Memory Size and Memory Free for SUN
#
get_sun_stats()
{

	MODEL=`uname -p`" "`uname -i`	
	OS=$ARCH" "`uname -r`

	CPU_LABEL1="CPUs:"
	CPUS=`/usr/bin/mpstat | grep -v CPU | wc | awk '{print $1}'`
	if [ -f $METRICA_DIR/SystemManagement/bin/metrica_cpu_speed.pl ]; then
		SPEED=`$METRICA_DIR/SystemManagement/bin/metrica_cpu_speed.pl`
		CPUS=$CPUS" x "$SPEED" MHz"
	fi

	CPU_LABEL2="Current CPU Util:"
	IDLE=`/usr/bin/sar -u 1 5 | grep "Average" | awk '{print $5}'`
	UTILISATION=`expr 100 - $IDLE`

	MEMORY_LABEL1="Memory Size:"
	MEMORY_LABEL2="Memory Free:"
	/usr/sbin/prtconf | grep -i "memory size" > $WRK0
	MEMORY_SIZE=`cat $WRK0 | awk '{print $3}'`
	UNITS=`cat $WRK0 | awk '{print $4}'`
	/usr/bin/vmstat free 1 5 > $WRK0
	FREE_MEMORY=`tail -1 $WRK0 | awk '{print $5}'`
	FREE_MEMORY=`expr $FREE_MEMORY / 1024 `
	echo "" > $WRK0

}

#
# Get stats for metrica users email. 


#
get_mail_stats_metrica()
{
	MUM_STATUS=`cat $CONF | grep metrica_check_metrica_mail | grep -v .metrica_check_metrica_mail | awk '{print $2}'`

	if [ "$MUM_STATUS" -eq "1" ] || [ "$MUM_STATUS" -eq "2" ]; then
		echo "Getting Metrica user email statistics..."
		echo ""
		echo "<P><FONT face=Verdana SIZE=2><B><U>Metrica Email Statistics</U></B></FONT></FONT>" >> $REPORT
		echo "<P></P>" >> $REPORT
		if [[ -x /usr/bin/mailx ]]; then

		# Check and print the number of messages
			msgcount=`mailx -H  |  wc -l | xargs echo`
			if [ "$msgcount" -gt "0" ]; then

cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><B><P>Metrica user has $msgcount mail messages.</P></B></FONT><P> </P>
END

				echo "Metrica user has $msgcount mail messages."
				if [ "$MUM_STATUS" -eq "2" ]; then
					mailx -H > $WRK0
					cat $WRK0
					while read a 
					do 

cat << END >> $REPORT
<FONT FACE="Verdana SIZE=2">$a</FONT><BR>
END

					done < $WRK0
					echo "" > $WRK0
				fi
			else

cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#008000"><P><B>Metrica user has no mail.</B></P></FONT> <P> </P>
END

				echo "Metrica user has no mail"
				echo ""
			fi
		else
			if [[ -f /usr/mail/metrica ]]; then
				msgcount=`echo "q" | mail -h | head -1 | awk '{print $1}'`

cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><B><P>Metrica user has $msgcount mail messages.</P></B></FONT><P> </P>
END

				echo "Metrica user has $msgcount mail messages."
				if [ "$MUM_STATUS" -eq "2" ]; then
					echo "q" | mail -h > $WRK0
					cat $WRK0
					sed 's/^    /\&nbsp;\&nbsp;\&nbsp;\&nbsp;/' $WRK0 > $WRK1
					while read a
					do

cat << END >> $REPORT
<FONT FACE="Verdana SIZE=2">$a</FONT><BR>
END

					done < $WRK1
					echo "" > $WRK0
				fi
			else

cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#008000"><P><B>Metrica user has no mail.</B></P></FONT> <P> </P>
END

				echo "Metrica user has no mail"
				echo ""
			fi
		fi
	fi
}

#
# Get stats for informix users email.
#
get_mail_stats_informix()
{
	IUM_STATUS=`cat $CONF | grep metrica_check_informix_mail | grep -v .metrica_check_informix_mail | awk '{print $2}'`

	if [ "$IUM_STATUS" -eq "1" ] || [ "$IUM_STATUS" -eq "2" ]; then
		if [ -x $SCRIPT_DIR/metrica_check_informix_mail.ksh ]; then	
			chmod og+w $REPORT
			$SCRIPT_DIR/metrica_check_informix_mail.ksh $REPORT
			chmod og-w $REPORT
		else
			echo "" > $WRK0
			echo "" > $WRK1

cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><B><P>Informix mail check script (metrica_check_informix_mail.ksh) is missing.</P></B></FONT> <P> </P>
END

		exception_message "-    EXCEPTION ALERT - The Script "$SCRIPT_DIR"/metrica_check_informix_mail.ksh does not exist or is not executable"
		fi
	fi
}


header()
{

#Create CSV file if it does not already exist
	if [ ! -f $CSV_FILE ]; then
		echo "Date,Hostname,Informix Server, AppServer Name,Number of CPUs,CPU Utilisation,Size of Memory,Free Memory,Informix,Onconfig,Kernel,Loader,AppServer,FileSystems,Gateways,Summariser" > $CSV_FILE
	fi
	CSV_OUT=$DATEXT","$HOSTNAME","$INFORMIXSERVER","$APPSERVER_NAME","$CPUS","$UTILISATION","$MEMORY_SIZE","$FREE_MEMORY

	#Get version for Appserver, Perl and Java
	PERL_VERSION=`perl -v | head -2 | tail -1 | awk '{print $4}' | sed s/v//1`
	JAVA_VERSION=`iastool -info | grep "java.version" | awk '{print $3}'`
	grep "Borland Enterprise" $APPSERVER_DIR/startup.log > $WRK0 2>&1
	if [ -f $WRK0 ]; then
		BES_VERSION=`tail -1 $WRK0`
		rm -rf $WRK0 > /dev/null
	else
		BES_VERSION="Unknown"          
fi

	# Display Server and Software Version Information
cat << END

	MODEL:			$MODEL
	OS:			$OS
	CPUs:			$CPUS
	CPU Util:		$UTILISATION % 
	Memory Size:		$MEMORY_SIZE $UNITS
	Memory Free:		$FREE_MEMORY $UNITS

	Informix Server:	$INFORMIXSERVER
	Informix Version:	$IDS_VERSION   

	BES Server:		$APPSERVER_NAME
	BES Version:		$BES_VERSION

	Java Version:		$JAVA_VERSION
	Perl Version:		$PERL_VERSION

END

# Print Header Information to Standard Out & HTML Report
cat <<END > $REPORT
<HTML>
<HEAD>
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=windows-1252">
<TITLE>PERFORMANCE MANAGER HEALTHCHECK REPORT </TITLE>
</HEAD>
<A NAME="top">
<BODY LINK="#0000ff" VLINK="#800080" BGCOLOR="#ffffff">
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana>Vallent Software Systems</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana>Healthcheck Report</FONT></FONT></TD></TR>
</TBODY></TABLE>
<HR>
<H1>
<P><B>Performance Manager Healthcheck Report </B></P>
</H1>
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Date:&nbsp;&nbsp;&nbsp;</B>&nbsp;&nbsp;&nbsp;$DATEXT</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana></FONT></FONT></TD></TR>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>To:&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</B>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;$MAILTO</FONT></FONT></TD>
</TR>
</TBODY></TABLE>
<P></P>
<TABLE width="88%">
<TBODY>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>Host Server:</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$HOSTNAME</FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>Model:</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$MODEL</FONT></FONT></TD>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B> </B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana> </FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>Operating System:</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$OS</FONT></FONT></TD>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>$CPU_LABEL1</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$CPUS</FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>$CPU_LABEL2</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$UTILISATION%</FONT></FONT></TD>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>$MEMORY_LABEL1</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$MEMORY_SIZE $UNITS</FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>$MEMORY_LABEL2</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$FREE_MEMORY $UNITS</FONT></FONT></TD>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>Informix Server:</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$INFORMIXSERVER</FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>Informix Version:</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$IDS_VERSION</FONT></FONT></TD>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>Application Server :</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$APPSERVER_NAME</FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>AppServer Version:</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$BES_VERSION</FONT></FONT></TD>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>Perl Version:</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$PERL_VERSION</FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B>Java Version:</B></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana>$JAVA_VERSION</FONT></FONT></TD>
</TBODY></TABLE>
END
}



report_links()
{
echo "<P><FONT face=Verdana SIZE=2><B><U>Healthcheck Results</U></B></FONT></FONT>" >> $REPORT
cat <<END >> $REPORT
<P>The following suite of checks have been run against the Performance Manager Application on $HOSTNAME. </P>
<P>These have resulted in $EXCEPTION_COUNT EXCEPTION ALERTS being raised.</P>
<P>Please click on the link to each individual report for details.</P>
<TABLE width="88%">
<TBODY>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><U>REPORT</U></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><U>STATUS</U></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><U>REPORT</U></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><U>STATUS</U></FONT></FONT></TD>
</TR>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><A HREF="#ids">Informix Server</A></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana COLOR=$IDS_COLOR><B>$IDS_STATUS</FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><A HREF="#onconfig">Onconfig Parameters</A></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana COLOR=$ICP_COLOR><B>$ICP_STATUS</FONT></FONT></TD>
</TR>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><A HREF="#loader">Loader Processes</A></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana COLOR=$LDM_COLOR><B>$LDM_STATUS</FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><A HREF="#appserver">Appserver Status</A></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana COLOR=$APP_COLOR><B>$APP_STATUS</FONT></FONT></TD>
</TR>
<TR>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><A HREF="#kernel">Kernel Parameters</A></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana COLOR=$SKP_COLOR><B>$SKP_STATUS</FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><A HREF="#filesystem">Filesystem Capacity</A></FONT></FONT></TD>
<TD WIDTH="22%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana COLOR=$SFS_COLOR><B>$SFS_STATUS</FONT></FONT></TD>
</TR>
<TR>
<TD WIDTH="25%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><A HREF="#gateways">Gateways Statistics</A></FONT></FONT></TD>
<TD WIDTH="25%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana COLOR=$GFS_COLOR><B>$GFS_STATUS</FONT></FONT></TD>
<TD WIDTH="25%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana><B><A HREF="#summary">Summariser Report</A></FONT></FONT></TD>
<TD WIDTH="25%" VALIGN="MIDDLE"><FONT size=-1><FONT face=Verdana COLOR=$SUM_COLOR><B>$SUM_STATUS</FONT></FONT></TD>
</TR>
</TBODY></TABLE>
<HR>
<P>&nbsp;</P>
END
	report_message ''

}
 
  

sub_header()
{

	echo ''
	echo ''
	echo $*
	echo ''
cat <<END >> $REPORT
<B><FONT FACE="Verdana" SIZE=2 ><P>$*</P></FONT></B>
END

}



footer()
{

# Print Footer Information to Standard Out & HTML Report
cat <<END >> $REPORT
</BODY>
</HTML>
END
	mail_report
	copy_report
	clean_up
	echo $CSV_OUT >> $CSV_FILE
}



report_message()
{

	# Print Message to Standard Out & HTML Report
	echo "	"$*
	echo ''
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 ><P>$*</P></FONT>
<P> </P>
END

}



report_error()
{

	# Print Error Detection Message to Standard Out & HTML Report
	echo "  "$*
	echo ""
cat <<END >> $REPORT
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><DIR><B><P>$*</P></B></DIR></FONT>
<P> </P> 
END

}
  

check_conf_file()
{

	#Check for existence of Healthcheck Configuration File
	#sub_header "Checking for Healthcheck Configuration file ...."

	if [ -f $CONF ];then
		CONF_FLAG="Y"
	# Get mail recipients
	MAILTO=""
	cat $CONF | grep "MAILTO" | grep -v ".MAILTO" | awk '{print $2}' > $WRK1
	for a in `cat $WRK1`
	do
		MAILTO=$MAILTO$a"; "
	done


	else
		report_error "-	EXCEPTION ALERT - The configuration file "$CONF" does not exist" 
		report_error "-	This file is created as part of the configuration of the healthcheck scripts" 
		report_error "-	and is used to control which scripts are enabled and who receives this report" 
		report_error "-	Run "$METRICA_DIR"/SystemManagement/bin/metrica_configure_healthcheck.ksh"
		report_error "-	to configure this file"

		footer
		exit 4
	fi

}
 



clean_up()
{

	#Remove all temporary files
	rm -f $WRK0 $WRK1 2>/dev/null

	#Remove Lock File
	if [ "$REMOVE_LOCK" != "N" ]; then
		rm -f $LOCK 2>/dev/null
	fi

}



#
# Check program is enabled
#
check_enable()
{


	PROG=`echo $1 | sed s/\.ksh//`
	ENABLED=`cat $CONF | grep $PROG | grep -v "."$PROG | awk '{print $2}'`

}



#
# RUN program IDS
#
run_program_ids()
{

	IDS_COLOR="#008000"
	EXCEPT_COUNT=0

	PROGRAM=metrica_check_ids.ksh
	check_enable $PROGRAM

	if [ "$ENABLED" = "Enabled" ]; then
		if [ -f $IDS_CONF ]; then
			CAPACITY_THRESHOLD=`cat $IDS_CONF | grep CAPACITY | grep -v ".CAPACITY" | awk '{print $2}'`
			EXTENTS=`cat $IDS_CONF | grep EXTENTS | grep -v ".EXTENTS" | awk '{print $2}'`
			NUMBER_OF_DAYS_LOG=`cat $IDS_CONF | grep NUMBER_DAYS_LOG | grep -v ".NUMBER_DAYS_LOG" | awk '{print $2}'`
		else
			CAPACITY_THRESHOLD="90"
			EXTENTS="100"
			NUMBER_OF_DAYS_LOG="0"
		fi

		#Check that script exists and is executable
		if [ -x $SCRIPT_DIR/$1 ]; then

			$SCRIPT_DIR/$PROGRAM -capacity_threshold $CAPACITY_THRESHOLD -extents $EXTENTS -number_of_days_log $NUMBER_OF_DAYS_LOG
			cat $REPORT_DIR/$PROG.html >> $WRK0

			EXCEPT_COUNT=`grep -c "EXCEPTION" $REPORT_DIR/$PROG.html`
			EXCEPTION_COUNT=`expr $EXCEPTION_COUNT + $EXCEPT_COUNT`
			if [ "$EXCEPT_COUNT" -gt "0" ]; then
				IDS_COLOR="#ff0000"
			fi

			IDS_STATUS="("$EXCEPT_COUNT" Exceptions)"
			CSV_OUT=$CSV_OUT","$EXCEPT_COUNT

		else
			exception_message "-	EXCEPTION ALERT - The Script "$SCRIPT_DIR"/"$PROGRAM" does not exist or is not executable" 
			IDS_STATUS="(1 Exceptions)"
			CSV_OUT=$CSV_OUT",1"
			IDS_COLOR="#ff0000"
		fi
	else
		#exception_message "This health check component is disabled - Run metrica_configure_healthcheck.ksh to enable it"
		IDS_STATUS="(Not Configured)"
		CSV_OUT=$CSV_OUT","$ENABLED
		IDS_COLOR="#ff0000"

	fi
}


#
# RUN program
#
run_program()
{

	COLOR="#008000"
	EXCEPT_COUNT=0

	check_enable $1
	if [ "$ENABLED" = "Enabled" ]; then

		#Check that script exists and is executable
		if [ -x $SCRIPT_DIR/$1 ]; then

			$SCRIPT_DIR/$1
			cat $REPORT_DIR/$PROG.html >> $WRK0

			EXCEPT_COUNT=`grep -c "EXCEPTION" $REPORT_DIR/$PROG.html`
			EXCEPTION_COUNT=`expr $EXCEPTION_COUNT + $EXCEPT_COUNT`
			if [ "$EXCEPT_COUNT" -gt "0" ]; then
				COLOR="#ff0000"
			fi

			STATUS="("$EXCEPT_COUNT" Exceptions)"
			CSV_OUT=$CSV_OUT","$EXCEPT_COUNT
		else 
			exception_message "-	EXCEPTION ALERT - The Script "$SCRIPT_DIR"/"$PROGRAM" does not exist or is not executable" 
			STATUS="(1 Exceptions)"
			CSV_OUT=$CSV_OUT",1"
			COLOR="#ff0000"
		fi
	else
		#exception_message "This health check component is disabled - Run metrica_configure_healthcheck.ksh to enable it"
		STATUS="(Not Configured)"
		CSV_OUT=$CSV_OUT","$ENABLED
		COLOR="#ff0000"
	fi

}



#
# Mail report
#
mail_report()
{

	echo ""
	echo "The Health Check has completed and the report is saved as: "
	echo $REPORT
	echo ""
    NUMRECIPIENTS=`cat $CONF | grep -c "MAILTO"`
	# Mail report if Exception Count > zero
	if [[ "$EXCEPTION_COUNT" -gt "0" && "$NUMRECIPIENTS" -gt "0" ]]; then

		# Remove semi colons from MAILTO
		MAILTO=`echo $MAILTO | sed s/\;//g`
		echo "Mailing Report to: " 
		for a in `echo $MAILTO`
		do
			echo "		"$a 
		done

cat <<END > $WRK0
From: $LOGNAME
To: $MAILTO                
Subject: Performance Manager Healthcheck Report on $HOSTNAME
Mime-Version: 1.0
Content-Type: text/html
END
		cat $REPORT >> $WRK0 
																					 
		/usr/lib/sendmail -t -oi < $WRK0     
	fi
	if [ "$NUMRECIPIENTS" -lt "1" ]; then
        echo "No email recipients for the metrica healthcheck have been defined."
        echo "Please use the metrica_configure_healthcheck tool to add recipients."
        echo ""
    fi
}

#
# Copy the report HTML file to specified location(s)
#
copy_report()
{
	NUMDIRS=`cat $CONF | grep -c "COPYTO"`
	if [ "$NUMDIRS" -gt "0" ]; then
		echo ""
		echo "The saved heathcheck report, $REPORT, has been copied to the following locations:"
		echo ""

	    # Get copy directories
	    COPYTO=""
	    cat $CONF | grep "COPYTO" | grep -v ".COPYTO" | awk '{print $2}' > $WRK1
	    for a in `cat $WRK1`
	    do
	        COPYTO=$a
			echo "$a"
			cp $REPORT $a/metrica_healthcheck.html
	    done
		echo ""
	else
		echo "No additional file copy locations have been defined."
		echo ""
	fi
}

exception_message()
{

# Print a message if program is disabled or missing or not executable
cat <<END >> $WRK0
<H2>
<P><A NAME="$LINK"><B>$TITLE</B></P>
</H2>
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>Date:&nbsp;&nbsp;&nbsp;</B>&nbsp;&nbsp;&nbsp;$DATEXT</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<FONT FACE="Verdana" SIZE=2 COLOR="#ff0000"><DIR><B><P>$*</P></B></DIR></FONT>
<P> </P>
<TABLE width="100%">
<TBODY>
<TR>
<TD align=left><FONT size=-1><FONT face=Verdana><B>*** $TITLE Finished ***</FONT></FONT></TD>
<TD align=right><FONT size=-1><FONT face=Verdana><A HREF="#top">Back to Top</A></FONT></FONT></TD></TR>
</TBODY></TABLE>
<HR align=left width="100%" noShade SIZE=1>
END

}



#
# Main Routine
#
main()
{

	#Build the environment if METRICA_DIR is set
	build_environment

	#Check if script already running
	check_running

	#Check that the script is being run by the metrica user
	check_metrica_user

	#Check that Informix is online                  
	informix_up

	#Check for existence of Configuration File
	echo 'Metrica Performance Manager Healthcheck '
	check_conf_file
	get_stats
	header
	get_mail_stats_metrica
	get_mail_stats_informix

	PROGRAM=metrica_check_ids.ksh
	TITLE="Informix Dynamic Server Check Report"
	LINK="ids"
	run_program_ids $PROGRAM 

	PROGRAM=metrica_check_onconfig.ksh
	TITLE="Informix Onconfig Configuration Parameters Monitor Report"
	LINK="onconfig"
	run_program $PROGRAM
	ICP_STATUS=$STATUS
	ICP_COLOR=$COLOR

	PROGRAM=metrica_check_kernel.ksh
	TITLE="Server Kernel Parameters Monitor Report"
	LINK="kernel"
	run_program $PROGRAM
	SKP_STATUS=$STATUS
	SKP_COLOR=$COLOR

	PROGRAM=metrica_check_loader.ksh
	TITLE="Metrica Loader Data Management Report"
	LINK="loader"
	run_program $PROGRAM
	LDM_STATUS=$STATUS
	LDM_COLOR=$COLOR

	PROGRAM=metrica_check_appserver.ksh
	TITLE="Application Server Healthcheck Report"
	LINK="appserver"
	run_program $PROGRAM
	APP_STATUS=$STATUS
	APP_COLOR=$COLOR

	PROGRAM=metrica_check_filesystems.ksh
	TITLE="Server File System Capacity Report"
	LINK="filesystem"
	run_program $PROGRAM
	SFS_STATUS=$STATUS
	SFS_COLOR=$COLOR

	PROGRAM=metrica_check_gateways.ksh
	TITLE="Gateways File Statistics Checking Report"
	LINK="gateways"
	run_program $PROGRAM
	GFS_STATUS=$STATUS
	GFS_COLOR=$COLOR

	PROGRAM=metrica_check_summariser.ksh
	TITLE="Metrica Summariser Health Check Report"
	LINK="summary"
	run_program $PROGRAM
	SUM_STATUS=$STATUS
	SUM_COLOR=$COLOR


	#Compile Report and mail it
	report_links
	cat $WRK0 >> $REPORT
	footer
                      
}


case $1 in
        # Help options
        "-h" | "-help" | "--help")
                usage
                exit 0
        ;;

        # Version options
        "-v" | "-version" | "--version")
                version
                exit 0
        ;;

        # Any other option
        *)
                main
                exit 0
        ;;
esac
