#!/usr/bin/perl
# @(#) SysManagement/delivery/utils/bin/Monitoring/updstats.pl SysManagement_r4.79.0.9:cset.007910:3:3 01/24/06 @(#)
###############################################################################
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2002-2005.
#
# Title:         updstats.pl
# Author:        Tony Chow
# Description:   Generic Update Stats script
# Version:       2.2.0.3
# Last Modified: 8 Oct, 2004
#
###############################################################################

# Usage: updstats.pl [ -f ] [ -g ]
# -f : run update statistics for procedure <procname>
# -g : re-generate the update statistics SQL files
#      if the SQL file doesn't exist, the script will generate it
#
# will perform the following update statistics statements
# 1) update statistics medium for table <tabname> distributions only
#    if there is a varcounter column:
#    update statistics medium for table <tabname> (all index col) distributions only
# 2) update statistics high for table <tabname> (1st index col)
# 3) update statistics low for table <tabname> (all other index columns)
# 4) if there is no index:
#    update statistics medium for table <tabname>
# 5) with -f flag:
#    update statistics for procedure <procname>
#
# This script generates multiple SQL files for update statistics - 1 for administration
# database, 1 for system database, and several for performance database.  It creates
# multiple files for raw and day tables and 1 for week, month, and the rest of tables
# in order to process them simultaneously.  It will generate a log file for each SQL
# file and these log files contain the status of each execution.  If there is error 
# during the execution, you can check the line # versus the SQL file and determine which
# statement causes the error.
# This script will also generate a log file called updstats.log which includes the execution
# time for each SQL file. 
# All SQL and log files are created in $METRICA_DIR/logs/updstats.
# Number of threads for day and raw table is specified by the MAX_THREAD variable.

use Env ;
$ENV{'PDQPRIORITY'} = "50";

use Getopt::Std;
getopts('fg');

$MAX_THREAD=10;
$WDIR="${METRICA_DIR}/logs/updstats/";
if ( ! -d $WDIR ) {
    system "mkdir $WDIR";
}
chdir "${WDIR}" or die "Can't cd to ${WDIR}: $!\n";
$LOG_MAIN="updstats.log";



sub strip_space {
    my @out = @_ ;
    for (@out) {
        s/^\s+// ;
        s/\s+$// ;
    }
    return wantarray ? @out : $out[0] ;
}

sub chomp_values {
    my @chomp_array = grep(!/^$/, @_);
}

sub get_colname {
    my $retval=`/usr/bin/echo "OUTPUT TO PIPE 'cat' WITHOUT HEADINGS select colname from syscolumns where tabid=@_[0] and colno=@_[1];" | $INFORMIXDIR/bin/dbaccess $dbn 2>/dev/null`;
    $retval =~ tr/\000\012//d;
    return $retval
}

sub col_exist {
    my ($lst, $k_id) = @_;
    $retval=0;
    foreach $id (@$lst) {
        if ( $id eq $k_id ) {
            $retval=1;
            last;
        }
    }
    return $retval;
}

sub get_idxcol {
    $tabid=@_[0] ;
    @indices = chomp_values(`/usr/bin/echo "OUTPUT TO PIPE 'cat' WITHOUT HEADINGS select indexkeys from sysindices where tabid=$tabid;" | $INFORMIXDIR/bin/dbaccess $dbn 2>/dev/null`);
    @idx_1st = () ;
    @idx_all = () ;
    @idn_1st = () ;
    @idn_all = () ;
    $sql_high = "" ;
    $sql_med = "" ;
    $sql_low = "" ;
    foreach $index (@indices) {
        $flag1=1;
        @index_keys = split("," , $index) ;
        foreach $key_id (@index_keys) {
            $key_name="";
            $key_id=abs(strip_space(strip_space($key_id))) ;

            if ( $flag1 == 1 ) {
                if ( col_exist(\@idx_1st, $key_id) == 0 ) {
                    $key_name=get_colname($tabid, $key_id);
                    push (@idx_1st, $key_id) ;
                    push (@idn_1st, $key_name) ;
                    $sql_high=$sql_high."update statistics high for table '$owner'.$tablename (${key_name});\n" ;
                }
                $flag1=0;
            }

            if ( col_exist(\@idx_all, $key_id) == 0 ) {
		if ( $key_name eq "" ) {
                    $key_name=get_colname($tabid, $key_id);
		}
                push (@idx_all, $key_id) ;
                push (@idn_all, $key_name) ;
                $sql_med=$sql_med."update statistics medium for table '$owner'.$tablename (${key_name}) distributions only;\n" ;
            }
        }
    }

    for $key_name (@idn_all) {
        if ( col_exist(\@idn_1st, $key_name) == 0 ) {
            $sql_low=$sql_low."update statistics low for table '$owner'.$tablename (${key_name});\n" ;
	}
    }

    $cnt=`/usr/bin/echo "OUTPUT TO PIPE 'cat' WITHOUT HEADINGS select count(*) from syscolumns where tabid=$tabid and coltype=40;" | $INFORMIXDIR/bin/dbaccess $dbn 2>/dev/null`;
    if ( $cnt > 0 ) {
        print SQL_FILE $sql_med;
    } else {
        print SQL_FILE "update statistics medium for table '$owner'.$tablename distributions only;\n" ;
    }
    print SQL_FILE $sql_high;
    if ( $sql_low ne "" ) {
        print SQL_FILE $sql_low;
    } elsif ( @idx_all == 0 ) {
        print SQL_FILE "update statistics medium for table '$owner'.$tablename;\n" ;
    }
}

sub process_tables {
    $flag = "@_[0]" ;
    $TAB_TEMP="/tmp/${dbn}_tab.tmp";
    $s="unload to $TAB_TEMP delimiter '|' select tabid,owner,tabname from systables where tabid>99 and tabtype='T'";
    if ( $flag ne "" ) {
	if ( $flag eq "_other" ) {
            $s.=" and tabname not matches '*_raw' and tabname not matches '*_day' and tabname not matches '*_week' and tabname not matches '*_month'";
        } else {
            $s.=" and tabname matches '*${flag}'";
        }
    }

    `/usr/bin/echo "$s" | $INFORMIXDIR/bin/dbaccess $dbn 2>/dev/null`;
    open TABTEMP, "<$TAB_TEMP" or die "ERROR: Cannot open $TAB_TEMP";
    while ( $line = <TABTEMP> ) {
        ($tabid, $owner, $tablename) = split /\|/, $line;
        get_idxcol($tabid);
    }
    close TABTEMP;
}

sub process_db {
    $flag = "@_[0]" ;
    $FILENAME = "updstats_${dbn}${flag}" ;
    $SQL_FILE = $FILENAME.".sql" ;
    if ( ( $opt_g ) || ( ! -r $SQL_FILE ) ) {
        open (SQL_FILE, ">$SQL_FILE") || die "Cannot open log file: $!" ;
        process_tables($flag) ;
        close(SQL_FILE) ;
    }

    if ( ( $flag eq "_raw" ) || ( $flag eq "_day" ) ) {
        $NO_LINE=`wc -l ${SQL_FILE}`;
        $NO_LINE=${NO_LINE}/${MAX_THREAD}+1;
	`split -l ${NO_LINE} ${SQL_FILE} ${FILENAME}`;
	@filelist=`ls ${FILENAME}??`;
	foreach $filename (@filelist) {
            $filename =~ tr/\000\012//d;
	    $LOG_FILE=${filename}.".log";
	    $SQL_FIL1=${filename}.".sql";
	    system "( echo 'Starting ${SQL_FIL1}: '`date` >>$LOG_MAIN; mv $filename $SQL_FIL1; dbaccess ${dbn} ${SQL_FIL1} ; echo 'Finish ${LOG_FILE}: '`date` >>$LOG_MAIN ) >${LOG_FILE} 2>&1 &";
        }
    } else {
	$LOG_FILE=$FILENAME.".log";
        system "( echo 'Starting ${SQL_FILE}: '`date` >>$LOG_MAIN; dbaccess ${dbn} $SQL_FILE ; echo 'Finish ${LOG_FILE}: '`date` >>$LOG_MAIN ) >$LOG_FILE 2>&1 &";
    }
}

sub process_procedures {
    $SPL_TEMP="/tmp/updstats_spl.tmp";
    $SPL_FILE = "updstats_spl.sql" ;
    $LOG_FILE = "updstats_spl.log" ;
    open (SPL_FILE, ">$SPL_FILE") || die "Cannot open log file: $!" ;

    @dbl=( $ENV{'METRICA_PERFORMANCE_DB'}, $ENV{'METRICA_ADMINISTRATION_DB'}, $ENV{'METRICA_SYSTEM_DB'} );
    foreach $dbn (@dbl) {
        print SPL_FILE "database ${dbn};\n";
        `/usr/bin/echo 'unload to $SPL_TEMP delimiter "|" select distinct owner,procname from sysprocedures;' | $INFORMIXDIR/bin/dbaccess $dbn 2>/dev/null`;
        open SPLTEMP, "<$SPL_TEMP" or die "ERROR: Cannot open $SPL_TEMP";
        while ( $line = <SPLTEMP> ) {
            ($owner, $procname) = split /\|/, $line;
            print SPL_FILE "update statistics for procedure '$owner'.$procname;\n" ;
        }
    }
    close SPLTEMP;
    close SPL_FILE;
    system "( echo 'Start ${SPL_FILE}: '`date` >>$LOG_MAIN; dbaccess - $SPL_FILE ; echo 'Finish ${LOG_FILE}: '`date` >>$LOG_MAIN ) >$LOG_FILE 2>&1 &";
}

##########################################################################################################


system "echo 'Check log files in ${WDIR} for details.\nProgram Start: '`date` >$LOG_MAIN";
$dbn=$ENV{'METRICA_PERFORMANCE_DB'};
process_db("_raw");
process_db("_day");
process_db("_week");
process_db("_month");
process_db("_other");
$dbn=$ENV{'METRICA_ADMINISTRATION_DB'};
process_db("");
$dbn=$ENV{'METRICA_SYSTEM_DB'};
process_db("");
if ( $opt_f ) {
    process_procedures();
}
