#!/bin/ksh
# @(#) uiweb/delivery/bin/metrica_report.ksh uiweb_r4.79.0.8:cset.042061:41:41 06/13/07 @(#)
######################################################################
#
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#              Vallent Software Systems UK
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005
#
# Title:        metrica__report.ksh
# Author:       Elesh Mistry
# Description:  CLI Report Generation script
# Version    uiweb_r4.79.0.8
#
######################################################################

# Display Metrica Banner
#
display_metrica_banner()
{
cat <<END
####################################################################
#
#                     Metrica Installation
#
#                       COPYRIGHT NOTICE
#
#         This file remains the intellectual property of:
#
#                  Vallent Software Systems UK
#                        Spencer House
#                        23 Sheen Road
#                    Richmond, Surrey, UK
#
# All rights reserved. Copyright Vallent Software Systems UK, 2005.
#
####################################################################

END
}

# Common exit handler
#
m_exit()
{
  rm -f $tmp_file
  if [ "$1" = "0" ]; then
    exit 0
  fi
  $ECHO
  $ECHO -n "$PROGNAME exited with errors"
  if [ "$2" != "" ]; then
    $ECHO -n ": $2"
  fi
  if [ "$1" != "" ]; then
    $ECHO " ($1)"
  fi
  $ECHO
  exit $1
}

# Show correct command-line usage
#
usage() {

		PROGNAME=`basename $0`

		echo
		echo "Usage: "$PROGNAME {
		echo "             -h | -help | --help |"
		echo "             -v | -version | --version"
		echo "       } | {"
		echo "             -generate"
		echo "             -reportfile <report_xml_file>"
		echo ""
		echo "             [-task <taskname>]"
		echo ""
		echo "             [-file     [-outputdir <outdir>]"
		echo "                        [-outputformat {formatted_csv [-remove_report_headers]|bulk_csv|html}]]"
		echo ""
		echo "             [-email    [-notifyonly]"
		echo "                        [-outputformat {formatted_csv [-remove_report_headers]|bulk_csv|html}]"
		echo "                        {-addresslist <address_list> |"
		echo "                         -addressfile <address_file>}]"
		echo ""
		echo "             [-reportstore]"
		echo "       }"
		echo
		echo
		echo "Options:"
		echo
		echo "     -v | -version | --version     Display version information"
		echo "     -h | -help | --help           Display command help"
		echo "     -generate                     Mandatory: Option to generate the report. By default a Bulk CSV file is generated."
		echo "     -reportfile                   Mandatory: Parameter to indicate the configuration file to be processed"
		echo "     -task                         Optional: To indicate the grouping of the generated report."
		echo "     -file                         Optional: To indicate that the files generated should be written to disk"
		echo "     -outputdir                    Optional: To indicate where the generated files will be placed"
		echo "     -outputformat                 Optional: To indicate the format of the generated files/email attachments"
		echo "     -remove_report_headers        Optional: To indicate that the Formatted CSV output will not include report headers"
		echo "     -email                        Optional: To indicate that an email is to be sent"
		echo "     -notifyonly                   Optional: To indicate the email is NOT to contain the generated report"
		echo "     -addresslist                  Optional: List of email addresses"
		echo "     -addressfile                  Optional: File containing email addresses"
		echo "                   (Note: If -email option is used, then -addresslist AND/OR -addressfile must also be used)"
		echo "     -reportstore                  Optional: To indicate the report is to be saved in the report store"
		echo
	}

###############################################################################
#
# Function    : print_component_version
#
# Description : Prints the version of the component
#
# Parameters  : none
#
###############################################################################
print_component_version() {

  . $METRICA_DIR/CS/bin/metrica_version.ksh
  print_version "Reporting Uiweb" "uiweb_r4.79.0.8:cset.042061:41:41"

}


# Setup Environment
#
setup_environment()
{

echo "Setting up environment..."

if [ -z "$OSAGENT_PORT" ]; then
   echo "\$OSAGENT_PORT is not set\n"
   exit 1;
fi

if [ -z "$APPSERVER_DIR" ] ; then
   echo "\$APPSERVER_DIR is not set\n"
   exit 1;
fi

if [ -z "$METRICA_DIR" ]; then
   echo "\$METRICA_DIR is not set\n"
   exit 1;
fi

  CLASSPATH=$METRICA_DIR/ext/lib/xerces.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/vbjorb.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/asrt.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/vbsec.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/lm.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/xmlrt.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/servlet.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/jcelements.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/jcschart.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/ifxjdbc.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/ifxjdbcx.jar

  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/reportlibraryservice.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/rlscommandlinetool.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/commandlineutils.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/xmlutils.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/datastoreutils.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/reportserverstores.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/reportserver.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/reportingutil.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/reportuserdetails.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/reportparameterdetails.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/reporting_reportdefinition.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/reportservicesecuritybackdoor.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Reporting/lib/evaluationlibrary.jar

  CLASSPATH=$CLASSPATH:$METRICA_DIR/GroupingService/lib/groupingservice.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/GroupingService/lib/gscommon.jar

  CLASSPATH=$CLASSPATH:$METRICA_DIR/Security/lib/emailselection.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Security/lib/login.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Security/lib/securitymanager.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Security/lib/securityservice.jar

  CLASSPATH=$CLASSPATH:$METRICA_DIR/Uiweb/lib/uiweb.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Uiweb/lib/cli.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/Uiweb/lib/uiweb_reportconfiguration.jar

  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/jaxb-api.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/jaxb-impl.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/jaxb-libs.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/relaxngDatatype.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/namespace.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/ext/lib/xsdlib.jar

  CLASSPATH=$CLASSPATH:$METRICA_DIR/MetaLayer/lib/rmqcommon.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/MetaLayer/lib/mlcommon.jar

  CLASSPATH=$CLASSPATH:$METRICA_DIR/DBServices/lib/queryservice_util.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/DBServices/lib/queryservice_requestmanager.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/DBServices/lib/queryservice_xmlejb.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/DBServices/lib/queryservice_logging.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/DBServices/lib/queryservice_connectionmanager.jar

  CLASSPATH=$CLASSPATH:$METRICA_DIR/DBElements/lib/metricaudt.jar

  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/aliasmanager.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/commonutils.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/exceptionutils.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/ejbhomefactory.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/logservice.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/propertyservice_common.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/propertyservice_client.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/propertyservice_cache.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/propertyservice_persistence.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/propertyservice_cmdlinetool.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/adc_gui.jar
  CLASSPATH=$CLASSPATH:$METRICA_DIR/CS/lib/viewer.jar

	export CLASSPATH
}

#------------------------- Main Command-line Processing ----------------------

case "$1" in
  # Help options
  "-h"|"-help"|"--help"|"-?")
    usage
	m_exit 0
  ;;

  # Version options
  "-v"|"-version"|"--version")
    print_component_version
	m_exit 0
  ;;
  "" )
    print_component_version
    usage
    m_exit 0
    ;;
esac

echo "Running Metrica Scheduled Report Script ..."

setup_environment

EVENT_LOG=$METRICA_DIR/logs/metrica_report.log
ERROR_LOG=$METRICA_DIR/logs/metrica_report_error.log

echo "Calling Java - Starting CLI Report Manager ..."

exit_code="java -DORBagentPort=$OSAGENT_PORT -Djava.awt.headless=true -Xms20m -Xmx100m com.adc.metrica.reporting.uiweb.cli.CLIReportManager $@ -error $ERROR_LOG -event $EVENT_LOG"

$exit_code >$EVENT_LOG 2>$ERROR_LOG
if (( $? != 0 ))
    then
    	echo "CLI report generation failed. See $ERROR_LOG for details."
        exit 5
fi

echo "CLI report generation complete."
