#!/usr/bin/perl
# @(#) uiweb/delivery/bin/metrica_report_failover.pl uiweb_r4.79.0.8:cset.041119:7:7 12/06/05 @(#)
######################################################################################################
#                   COPYRIGHT NOTICE
#
# This file remains the intellectual property of:
#
#                 Vallent Software Systems UK 
#                    Spencer House
#                    23 Sheen Road
#                 Richmond, Surrey, UK
#
# Copyright @ 2005 Vallent Software Systems UK. All rights reserved. 
#
# Title:            metrica_report_failover.pl
# Author:           Brian Heavey, Micheal Spillane
#
# Date              01/12/2005
# Description:      This script used to rerun anyreport that fails on its original execution.
#                   If it fails a second time an error will be reported.
#                   This script should be run from the crontab. An entry should be inserted  
#                   from the crontab to run on a daily basis
#
# Usage:            metrica_report_failover.pl {
#                     -v | -version | --version |
#                     -h | -help | --help 
#          } | {   
#                     Refer to documentation of metrica_report.ksh script for further 
#                     details as arguments to metrica_report_failover.pl are passed 
#                     directly to metrica_report.ksh.
#          } 
#
######################################################################################################

#---------------------------------------------------------------------------
#       CORE PERL LIBRARIES
#---------------------------------------------------------------------------
use strict;                 # require declarations
use Getopt::Long;           # for option processing
use File::Basename;         # parse the file minus extensions


#---------------------------------------------------------------------------
#       GLOBAL VARIABLES
#---------------------------------------------------------------------------

my %param  = ();                      # Commandline options

# Information command line variables
my ($opt_help, $opt_h, $opt_version, $opt_v);

# Environment variables
my $PRODUCT       = "Metrica Performance Manager";
my $COPYRIGHT      = "Copyright  2005 Vallent Software Systems UK. All rights reserved";
# For reference REPOSITORY_STR = "uiweb_r4.79.0.8:cset.041119:7:7"
my $REPOSITORY_STR = "uiweb_r4.79.0.8:cset.041119:7:7";  # This value is set by repository

#---------------------------------------------------------------------------
#       CONSTANTS
#---------------------------------------------------------------------------

# Return Codes
my $EXIT_RC_SUCCESS="0";
my $EXIT_RC_SYNTAX= "1";

# Getopt settings
$Getopt::Long::autoabbrev=0; # turn off auto-abbreviation
$Getopt::Long::ignorecase=0; # turn on case sensitivity
$Getopt::Long::passthrough=1; #  suppresses errors in GetOptions for unknown arguments 


############################################################
#
# Global objects
#
my $VERSION       = new MetricaVersion($REPOSITORY_STR);

#---------------------------------------------------------------------------

# If there are no arguments then display the usage
my @params = @ARGV;
my $number_args = scalar @params;
if ($number_args < 1) {
  usage();
  exit $EXIT_RC_SUCCESS;
}

# Process switches from the command line

my $result=0;
$result =
GetOptions(
 "version!"     => \$opt_version,
 "v!"           => \$opt_v,
 "help!"        => \$opt_help,
 "h!"           => \$opt_h,
 );
 

# Process commands
process();

###############################################################################
#
# Subroutine  : runReport
#
# Description : Calls metrica_report.ksh script and performs retries if it fails
#
# Parameters  : None
#
###############################################################################
 
sub runReport 
{
  my @code = @_;
  my $success = 0;
  my $attempts = "2";

  # until all retries have gone.
  while (1)
  {    
    my $rc = 0xffff & system( "@code");
    if ($rc){
         $success = 0;
    }
    else {
         $success = 1;
    }
 
    # return if we're sucessful
    if ($success) {
    
         # Uncomment the following line if you wish to log successes
         # failover_log("\: AUDIT \: metrica_report_failover.pl \| Report Success: @code \n");
         
         return 0;
    }

    if( $attempts > 1 ) {
         # Uncomment the following line if you wish to log retries
         # failover_log("\: WARNING \: metrica_report_failover.pl \| Retrying Report : @code \n");
    }
    
    # we've used up a try
    $attempts--;  
    last if $attempts < 1;
  }

  # Did not succeed, return error.
  failover_log("\: ERROR \: metrica_report_failover.pl \| Report Failure : @code \n");
  return 1;
}

###############################################################################
#
# Subroutine  : process
#
# Description : Switches according to command line args.
#
# Parameters  : None
#
###############################################################################
sub process {

  if (defined $opt_version) {
    printf "%s\n%s\n%s\n", $PRODUCT, $COPYRIGHT, $VERSION->version;
    exit $EXIT_RC_SUCCESS;
  }
  elsif (defined $opt_v) {
    printf "%s\n%s\n%s\n", $PRODUCT, $COPYRIGHT, $VERSION->version;
    exit $EXIT_RC_SUCCESS;
  }
  elsif (defined $opt_help) {
    usage();
    exit $EXIT_RC_SUCCESS;
  }
  elsif (defined $opt_h) {
    usage();
    exit $EXIT_RC_SUCCESS;
  }
  else {
    my @args = @ARGV;
    
    my $cmd = "metrica_report.ksh @args > /dev/null 2>&1";
    # runReport incorporates a retry mechanism
    runReport ("$cmd"); 
    exit $EXIT_RC_SYNTAX;
  }
}


sub failover_log 
{
  print time(), "@_";
}

###############################################################################
#
# Subroutine  : usage
#
# Description : Prints usage
#
# Parameters  : None
#
###############################################################################
sub usage {

  # Show script name without directory
  my $file_exe = basename($0);

# Quit with usage
print <<"EOUSAGE";

Usage: $file_exe {
          -v | -version | --version |
          -h | -help | --help 
} | {   
          Refer to documentation of metrica_report.ksh script for further 
          details as arguments to metrica_report_failover.pl are passed 
          directly to metrica_report.ksh.
} 

EOUSAGE

}


############################################################
#
# MetricaVersion package
#
package MetricaVersion;

#-----------------------------------------------------------
#
# constructor method
#
# Parameters:
#  $repositoryStr : This is the string returned by the repository 
#                   uiweb_r4.60.0.latest:cset.041113:3:3 variable
#
# Returns:
#  Object reference
#
sub new {
   my $class = shift;
   my ($repositoryStr) = @_;
   my $self = {}; bless $self, $class;

   #
   # Translation hash for the package state
   #
   my %state;
   $state{r} = "Released product";
   $state{d} = "Development";
   $state{a} = "Development testing";
   $state{b} = "Functional testing";

   #
   # Analyse the version string from the repository
   #  uiweb_r4.60.0.latest:cset.041113:3:3
   #
   if ($repositoryStr =~ /(\w+)_(\w)(\d+)\.(\d+)\.(\d+)\.([^:]+):/) {
      $self->{subsystem} = $1;
      $self->{state}     = $state{$2};
      $self->{major}     = $3;
      $self->{minor}     = $4;
      $self->{patch}     = $5;
      $self->{checkpt}   = $6;
   } 
   else {
      $self->{invalidStr} = $repositoryStr;
   }

   return $self;
}

#-----------------------------------------------------------
#
# version method
#
# Parameters:
#  none
#
# Returns:
#  Correctly formatted version string
#
sub version {
   my $self = shift;

   if (!defined($self->{invalidStr})) {
      return sprintf "%s Version %s.%s Patch %s (Checkpoint: %s, State: %s)", $self->{subsystem}, $self->{major}, $self->{minor}, $self->{patch}, $self->{checkpt}, $self->{state}; 
   }
   else {
      die "Version unknown :  $self->{invalidStr}";
   }
}

